@@ -3,30 +3,30 @@
 *                          G e n e r i c   A r r a y                            *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXArray.h,v 1.1 1999/09/14 13:32:47 jeroen Exp $                         *
+* $Id: FXArray.h,v 1.9 2001/03/07 06:04:25 jeroen Exp $                         *
 ********************************************************************************/
 #ifndef FXARRAY_H
 #define FXARRAY_H
 
 
 
-/*************************  D e f i n i t i o n  *****************************/
+/*************************  D e f i n i t i o n  *******************************/
 
 
 template<class TYPE>
@@ -37,16 +37,19 @@ class FXArray {
   FXint   grow;                // Grow amount
 public:
   FXArray();
-  FXint no() const;
-  TYPE* data() const;
-  FXint size() const;
+  FXArray(const FXArray<TYPE>& orig);
+  FXArray<TYPE>& operator=(const FXArray<TYPE> &orig);
+  inline FXint no() const;
+  inline TYPE* data() const;
+  inline FXint size() const;
   void size(FXint n);
   FXint inc() const;
   void inc(FXint n);
-  const TYPE& operator [](FXint i) const;
-  TYPE& operator [](FXint i);
+  inline const TYPE& operator [](FXint i) const;
+  inline TYPE& operator [](FXint i);
   void insert(FXint pos,const TYPE& p);
   void append(const TYPE& p);
+  void prepend(const TYPE& p);
   void remove(FXint pos);
   void extract(const TYPE& p);
   FXint find(const TYPE& p);
@@ -71,9 +74,21 @@ FXArray<TYPE>::FXArray(){
   }
 
 
+// Copy construct
+template<class TYPE>
+FXArray<TYPE>::FXArray(const FXArray<TYPE>& orig){
+  number=orig.number;
+  total=orig.total;
+  grow=orig.grow;
+  allocElms(list,total);
+  constructElms(list,number);
+  copyElms(list,orig.list,number);
+  }
+
+
 // Return number of elements
 template<class TYPE>
-int FXArray<TYPE>::no() const {
+FXint FXArray<TYPE>::no() const {
   return number;
   }
 
@@ -87,29 +102,29 @@ TYPE* FXArray<TYPE>::data() const {
 
 // Return size of list
 template<class TYPE>
-int FXArray<TYPE>::size() const {
+FXint FXArray<TYPE>::size() const {
   return total; 
   }
 
 
 // Return grow delta
 template<class TYPE>
-int FXArray<TYPE>::inc() const {
+FXint FXArray<TYPE>::inc() const {
   return grow;
   }
 
 
 // Set grow delta
 template<class TYPE>
-void FXArray<TYPE>::inc(int g){
+void FXArray<TYPE>::inc(FXint g){
   FXASSERT(g>=0);
-  gw=g;
+  grow=g;
   }
 
 
 // Return element rvalue
 template<class TYPE>
-const TYPE& FXArray<TYPE>::operator[](int i) const {
+const TYPE& FXArray<TYPE>::operator[](FXint i) const {
   FXASSERT(0<=i&&i<number); 
   return list[i]; 
   }
@@ -117,12 +132,29 @@ const TYPE& FXArray<TYPE>::operator[](int i) const {
 
 // Return element lvalue
 template<class TYPE>
-TYPE& FXArray<TYPE>::operator[](int i){ 
+TYPE& FXArray<TYPE>::operator[](FXint i){ 
   FXASSERT(0<=i&&i<number); 
   return list[i]; 
   }
 
 
+// Assign an array
+template<class TYPE>
+FXArray<TYPE>& FXArray<TYPE>::operator=(const FXArray<TYPE>& orig){
+  if(this!=&orig){
+    destructElms(list,number);
+    freeElms(list);
+    number=orig.number;
+    total=orig.total;
+    grow=orig.grow;
+    allocElms(list,total);
+    constructElms(list,number);
+    copyElms(list,orig.list,number);
+    }
+  return *this;
+  }
+
+
 // Set new size
 template<class TYPE>
 void FXArray<TYPE>::size(FXint n){
@@ -138,7 +170,7 @@ void FXArray<TYPE>::size(FXint n){
     else{
       if(n>total){
         TYPE *ptr;
-        int s=total+grow;
+        FXint s=total+grow;
         if(grow==0) s=total+total;
         if(s<n) s=n;
         allocElms(ptr,s);
@@ -160,7 +192,7 @@ void FXArray<TYPE>::size(FXint n){
 // Insert element anywhere
 template<class TYPE>
 void FXArray<TYPE>::insert(FXint pos,const TYPE& p){
-  int s=number-pos;
+  FXint s=number-pos;
   size(number+1);
   moveElms(&list[pos+1],&list[pos],s);
   list[pos]=p;
@@ -170,17 +202,27 @@ void FXArray<TYPE>::insert(FXint pos,const TYPE& p){
 // Append element at end
 template<class TYPE>
 void FXArray<TYPE>::append(const TYPE& p){
-  int s=number;
-  size(s+1);
+  FXint s=number;
+  size(number+1);
   list[s]=p;
   }
 
 
+// Prepend element at begin
+template<class TYPE>
+void FXArray<TYPE>::prepend(const TYPE& p){
+  FXint s=number;
+  size(number+1);
+  moveElms(&list[1],&list[0],s);
+  list[0]=p;
+  }
+
+
 // Remove element at pos
 template<class TYPE>
 void FXArray<TYPE>::remove(FXint pos){
-  int s=number;
-  moveElms(&list[pos],&list[pos+1],s-pos);            // Uses assignment operator
+  FXint s=number;
+  moveElms(&list[pos],&list[pos+1],s-pos-1);          // Uses assignment operator
   size(s-1);
   }
 
@@ -188,7 +230,7 @@ void FXArray<TYPE>::remove(FXint pos){
 // Find element, -1 if not found
 template<class TYPE>
 FXint FXArray<TYPE>::find(const TYPE& p){
-  int s=number;
+  FXint s=number;
   while(s!=0 && !(list[s-1]==p)) s--;
   return s-1;
   }
