@@ -3,30 +3,30 @@
 *                            B M P   I m a g e   O b j e c t                    *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXBMPImage.h,v 1.2 1999/09/20 22:12:56 jeroen Exp $                      *
+* $Id: FXBMPImage.h,v 1.4 2001/01/25 19:48:27 jeroen Exp $                      *
 ********************************************************************************/
 #ifndef FXBMPIMAGE_H
 #define FXBMPIMAGE_H
 
 
 
-// Image class
+/// Microsoft Bitmap image
 class FXAPI FXBMPImage : public FXImage {
   FXDECLARE(FXBMPImage)
 protected:
@@ -35,11 +35,27 @@ private:
   FXBMPImage(const FXBMPImage&);
   FXBMPImage &operator=(const FXBMPImage&);
 public:
+  
+  /// Construct image from memory stream formatted in Microsoft bitmap format
   FXBMPImage(FXApp* a,const void *pix=NULL,FXuint opts=0,FXint w=1,FXint h=1);
+
+  /// Save pixels into stream in Microsoft bitmap format
   virtual void savePixels(FXStream& store) const;
+  
+  /// Load pixels from stream in Microsoft bitmap format
   virtual void loadPixels(FXStream& store);
+  
+  /// Destroy icon
   virtual ~FXBMPImage();
   };
+
   
+/// Load a bmp file from a stream
+extern FXAPI FXbool fxloadBMP(FXStream& store,FXuchar*& data,FXColor& transp,FXint& width,FXint& height);
+
+
+/// Save a bmp file to a stream
+extern FXAPI FXbool fxsaveBMP(FXStream& store,const FXuchar *data,FXColor transp,FXint width,FXint height);
+
     
 #endif
