@@ -3,53 +3,49 @@
 *                          C a n v a s   W i d g e t                            *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXCanvas.h,v 1.3 1999/11/05 06:57:13 jeroen Exp $                        *
+* $Id: FXCanvas.h,v 1.11 2001/08/23 16:24:51 jeroen Exp $                       *
 ********************************************************************************/
 #ifndef FXCANVAS_H
 #define FXCANVAS_H
 
 
 
-// Canvas, an area drawn by another object
+/// Canvas, an area drawn by another object
 class FXAPI FXCanvas : public FXWindow {
   FXDECLARE(FXCanvas)
 protected:
-  FXCanvas(){}
+  FXCanvas();
 private:
   FXCanvas(const FXCanvas&);
   FXCanvas &operator=(const FXCanvas&);
 public:
   long onPaint(FXObject*,FXSelector,void*);
   long onMotion(FXObject*,FXSelector,void*);
-  long onLeftBtnPress(FXObject*,FXSelector,void*);
-  long onLeftBtnRelease(FXObject*,FXSelector,void*);
-  long onMiddleBtnPress(FXObject*,FXSelector,void*);
-  long onMiddleBtnRelease(FXObject*,FXSelector,void*);
-  long onRightBtnPress(FXObject*,FXSelector,void*);
-  long onRightBtnRelease(FXObject*,FXSelector,void*);
   long onKeyPress(FXObject*,FXSelector,void*);
   long onKeyRelease(FXObject*,FXSelector,void*);
 public:
+
+  /// Construct new drawing canvas widget
   FXCanvas(FXComposite* p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=FRAME_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
-  virtual void move(FXint x,FXint y);
-  virtual void resize(FXint w,FXint h);
-  virtual void position(FXint x,FXint y,FXint w,FXint h);
+
+  /// Return TRUE because canvas can receive focus
+  virtual FXbool canFocus() const;
   };
 
 
