@@ -1,25 +1,25 @@
 /********************************************************************************
 *                                                                               *
-*      D e v i c e   C o n t e x t   D r a w i n g   o n   a   W i n d o w      *
+*  D e v i c e   C o n t e x t   F o r   W i n d o w s   a n d   I m a g e s    *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1999 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1999,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXDCWindow.h,v 1.3 1999/09/23 16:07:20 gui Exp $                     *
+* $Id: FXDCWindow.h,v 1.24 2001/08/23 16:24:51 jeroen Exp $                     *
 ********************************************************************************/
 #ifndef FXDCWINDOW_H
 #define FXDCWINDOW_H
@@ -34,165 +34,187 @@ class FXFont;
 class FXVisual;
 
 
-//  Device Context for Window
+
+/**
+* Window Device Context
+*
+* The Window Device Context allows drawing into an FXDrawable, such as an
+* on-screen window (FXWindow and derivatives) or an off-screen image (FXImage
+* and its derivatives).
+* Because certain hardware resources are locked down, only one FXDCWindow may be
+* locked on a drawable at any one time.
+*/
 class FXAPI FXDCWindow : public FXDC {
-  FXDECLARE(FXDCWindow)
 protected:
-  FXVisual   *visual;
-  FXRectangle clip;
-#ifndef FX_NATIVE_WIN32
-  FXuint      flags;
-  GC          gc;
+  FXDrawable *surface;        // Drawable surface
+  FXVisual   *visual;         // Visual of drawable
+  FXRectangle rect;           // Paint rectangle inside drawable
+#ifndef WIN32
+  FXuint      flags;          // GC Flags
+  FXPixel     devfg;          // Device foreground pixel value
+  FXPixel     devbg;          // Device background pixel value
+  void*       gc;             // GC
 #else
-  HDC         dc;
+  FXID        oldpalette;
+  FXID        oldbrush;
+  FXID        oldpen;
+  FXPixel     devfg;          // Device foreground pixel value
+  FXPixel     devbg;          // Device background pixel value
+  FXID        dc;
   FXbool      needsNewBrush;
-  LOGBRUSH   *brush;
   FXbool      needsNewPen;
-  EXTLOGPEN  *pen;
   FXbool      needsPath;
 #endif
-protected:
-  FXDCWindow(){}
-#ifdef FX_NATIVE_WIN32
+private:
+#ifdef WIN32
   void updateBrush();
   void updatePen();
-  void saveDC(HPEN& hPen,HBRUSH& hBrush,HFONT& hFont,RECT& rect,FXuint& text,FXuint& back,FXint& fillmode);
-  void restoreDC(const HPEN& hPen,const HBRUSH& hBrush,const HFONT& hFont,const RECT& rect,FXuint text,FXuint back,FXint fillmode);
 #endif
 private:
+  FXDCWindow();
   FXDCWindow(const FXDCWindow&);
   FXDCWindow &operator=(const FXDCWindow&);
 public:
-  
-  // Construct for painting in response to expose;
-  // This sets the clip rectangle to the exposed rectangle
+
+  /// Construct for painting in response to expose;
+  /// This sets the clip rectangle to the exposed rectangle
   FXDCWindow(FXDrawable* drawable,FXEvent* event);
 
-  // Construct for normal drawing; 
-  // This sets clip rectangle to the whole drawable
+  /// Construct for normal drawing;
+  /// This sets clip rectangle to the whole drawable
   FXDCWindow(FXDrawable* drawable);
 
-  // Begin locks in a drawable surface
+  /// Begin locks in a drawable surface
   void begin(FXDrawable *drawable);
-  
-  // End unlock the drawable surface 
+
+  /// End unlock the drawable surface
   void end();
-  
-  // Draw points
+
+  /// Read back pixel
+  virtual FXColor readPixel(FXint x,FXint y);
+
+  /// Draw points
   virtual void drawPoint(FXint x,FXint y);
   virtual void drawPoints(const FXPoint* points,FXuint npoints);
   virtual void drawPointsRel(const FXPoint* points,FXuint npoints);
 
-  // Draw lines
+  /// Draw lines
   virtual void drawLine(FXint x1,FXint y1,FXint x2,FXint y2);
   virtual void drawLines(const FXPoint* points,FXuint npoints);
   virtual void drawLinesRel(const FXPoint* points,FXuint npoints);
   virtual void drawLineSegments(const FXSegment* segments,FXuint nsegments);
-  
-  // Draw rectangles
+
+  /// Draw rectangles
   virtual void drawRectangle(FXint x,FXint y,FXint w,FXint h);
   virtual void drawRectangles(const FXRectangle* rectangles,FXuint nrectangles);
 
-  // Draw arcs
+  /// Draw arcs
   virtual void drawArc(FXint x,FXint y,FXint w,FXint h,FXint ang1,FXint ang2);
   virtual void drawArcs(const FXArc* arcs,FXuint narcs);
 
-  // Filled rectangles
+  /// Filled rectangles
   virtual void fillRectangle(FXint x,FXint y,FXint w,FXint h);
   virtual void fillRectangles(const FXRectangle* rectangles,FXuint nrectangles);
 
-  // Draw arcs
+  /// Draw arcs
   virtual void fillArc(FXint x,FXint y,FXint w,FXint h,FXint ang1,FXint ang2);
   virtual void fillArcs(const FXArc* arcs,FXuint narcs);
 
-  // Filled polygon
+  /// Filled polygon
   virtual void fillPolygon(const FXPoint* points,FXuint npoints);
   virtual void fillConcavePolygon(const FXPoint* points,FXuint npoints);
   virtual void fillComplexPolygon(const FXPoint* points,FXuint npoints);
 
-  // Filled polygon with relative points
+  /// Filled polygon with relative points
   virtual void fillPolygonRel(const FXPoint* points,FXuint npoints);
   virtual void fillConcavePolygonRel(const FXPoint* points,FXuint npoints);
   virtual void fillComplexPolygonRel(const FXPoint* points,FXuint npoints);
 
-  // Draw hashed box
+  /// Draw hashed box
   virtual void drawHashBox(FXint x,FXint y,FXint w,FXint h,FXint b=1);
-  
-  // Draw area from source
+
+  /// Draw area from source
   virtual void drawArea(const FXDrawable* source,FXint sx,FXint sy,FXint sw,FXint sh,FXint dx,FXint dy);
 
-  // Draw image 
+  /// Draw image
   virtual void drawImage(const FXImage* image,FXint dx,FXint dy);
 
-  // Draw bitmap
+  /// Draw bitmap
   virtual void drawBitmap(const FXBitmap* bitmap,FXint dx,FXint dy);
 
-  // Draw icon 
+  /// Draw icon
   virtual void drawIcon(const FXIcon* icon,FXint dx,FXint dy);
   virtual void drawIconShaded(const FXIcon* icon,FXint dx,FXint dy);
+  virtual void drawIconSunken(const FXIcon* icon,FXint dx,FXint dy);
 
-  // Draw string
+  /// Draw string
   virtual void drawText(FXint x,FXint y,const FXchar* string,FXuint length);
   virtual void drawImageText(FXint x,FXint y,const FXchar* string,FXuint length);
 
-  // Set foreground/background drawing color
+  /// Set foreground/background drawing color
   virtual void setForeground(FXColor clr);
   virtual void setBackground(FXColor clr);
 
-  // Set dash pattern
-  virtual void setDashes(FXuint dashoffset,const FXchar *dashlist,FXuint n);
-  
-  // Set line width  
+  /// Set dash pattern
+  virtual void setDashes(FXuint dashoffset,const FXchar *dashpattern,FXuint dashlength);
+
+  /// Set line width
   virtual void setLineWidth(FXuint linewidth=0);
-  
-  // Set line cap style
+
+  /// Set line cap style
   virtual void setLineCap(FXCapStyle capstyle=CAP_BUTT);
-  
-  // Set line join style
+
+  /// Set line join style
   virtual void setLineJoin(FXJoinStyle joinstyle=JOIN_MITER);
-  
-  // Set line style
+
+  /// Set line style
   virtual void setLineStyle(FXLineStyle linestyle=LINE_SOLID);
-  
-  // Set fill style
+
+  /// Set fill style
   virtual void setFillStyle(FXFillStyle fillstyle=FILL_SOLID);
-  
-  // Set fill rule
+
+  /// Set fill rule
   virtual void setFillRule(FXFillRule fillrule=RULE_EVEN_ODD);
-  
-  // Set blit function
+
+  /// Set blit function
   virtual void setFunction(FXFunction func=BLT_SRC);
-  
-  // Set the tile
+
+  /// Set the tile
   virtual void setTile(FXImage* tile,FXint dx=0,FXint dy=0);
-  
-  // Set the stipple pattern
+
+  /// Set the stipple pattern
   virtual void setStipple(FXBitmap *stipple,FXint dx=0,FXint dy=0);
-  
-  // Set the stipple pattern
+
+  /// Set the stipple pattern
   virtual void setStipple(FXStipplePattern stipple,FXint dx=0,FXint dy=0);
-  
-  // Set clip rectangle
+
+  /// Set clip region
+  virtual void setClipRegion(const FXRegion& region);
+
+  /// Set clip rectangle
   virtual void setClipRectangle(FXint x,FXint y,FXint w,FXint h);
-  
-  // Clear clipping
+
+  /// Set clip rectangle
+  virtual void setClipRectangle(const FXRectangle& rectangle);
+
+  /// Clear clipping
   virtual void clearClipRectangle();
-  
-  // Set clip mask
+
+  /// Set clip mask
   virtual void setClipMask(FXBitmap* mask,FXint dx=0,FXint dy=0);
-  
-  // Clear clip mask
+
+  /// Clear clip mask
   virtual void clearClipMask();
-  
-  // Set font to draw text with
+
+  /// Set font to draw text with
   virtual void setTextFont(FXFont *fnt);
-  
-  // Clip against child windows
+
+  /// Clip against child windows
   virtual void clipChildren(FXbool yes);
 
-  // Cleanup
+  /// Destructor
   virtual ~FXDCWindow();
   };
- 
-  
+
+
 #endif
