@@ -3,29 +3,38 @@
 *                        F o n t   S e l e c t i o n   B o x                    *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1999 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1999,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXFontSelector.h,v 1.3 1999/10/13 06:48:16 jeroen Exp $                  *
+* $Id: FXFontSelector.h,v 1.9 2001/03/23 07:02:41 jeroen Exp $                  *
 ********************************************************************************/
 #ifndef FXFONTSELECTOR_H
 #define FXFONTSELECTOR_H
 
 
-// File selection widget
+class FXTextField;
+class FXList;
+class FXComboBox;
+class FXLabel;
+class FXButton;
+class FXCheckButton;
+class FXFont;
+
+
+/// Font selection widget
 class FXAPI FXFontSelector : public FXPacker {
   FXDECLARE(FXFontSelector)
 protected:
@@ -47,7 +56,6 @@ protected:
   FXLabel       *preview;
   FXFont        *previewfont;
   FXFontDesc     selected;
-  FXuint         hints;
 protected:
   FXFontSelector(){}
   void listFontFaces();
@@ -55,7 +63,6 @@ protected:
   void listSlants();
   void listFontSizes();
   void previewFont();
-  void updateControls();
 private:
   FXFontSelector(const FXFontSelector&);
   FXFontSelector &operator=(const FXFontSelector&);
@@ -67,8 +74,11 @@ public:
   long onCmdSize(FXObject*,FXSelector,void*);
   long onCmdSizeText(FXObject*,FXSelector,void*);
   long onCmdCharset(FXObject*,FXSelector,void*);
+  long onUpdCharset(FXObject*,FXSelector,void*);
   long onCmdSetWidth(FXObject*,FXSelector,void*);
+  long onUpdSetWidth(FXObject*,FXSelector,void*);
   long onCmdPitch(FXObject*,FXSelector,void*);
+  long onUpdPitch(FXObject*,FXSelector,void*);
   long onCmdScalable(FXObject*,FXSelector,void*);
   long onUpdScalable(FXObject*,FXSelector,void*);
   long onCmdAllFonts(FXObject*,FXSelector,void*);
@@ -89,14 +99,32 @@ public:
     ID_LAST
     };
 public:
+
+  /// Constructor
   FXFontSelector(FXComposite *p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
+
+  /// Create server-side resources
   virtual void create();
+
+  /// Return a pointer to the "Accept" button
   FXButton *acceptButton() const { return accept; }
+
+  /// Return a pointer to the "Cancel" button
   FXButton *cancelButton() const { return cancel; }
-  virtual void save(FXStream& store) const;
-  virtual void load(FXStream& store);
+
+  /// Set font selection
   void setFontSelection(const FXFontDesc& fontdesc);
+
+  /// Get font selection
   void getFontSelection(FXFontDesc& fontdesc) const;
+
+  /// Save to a stream
+  virtual void save(FXStream& store) const;
+
+  /// Load from a stream
+  virtual void load(FXStream& store);
+
+  /// Destructor
   virtual ~FXFontSelector();
   };
 
