@@ -3,29 +3,30 @@
 *                      O p e n G L   V i e w e r   W i d g e t                  *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXGLViewer.h,v 1.9 1999/11/09 23:58:37 jeroen Exp $                      *
+* $Id: FXGLViewer.h,v 1.48 2001/05/23 20:23:33 jeroen Exp $                     *
 ********************************************************************************/
 #ifndef FXGLVIEWER_H
 #define FXGLVIEWER_H
 
 
 class FXDCPrint;
+class FXGLObject;
 
 
 // Pick tolerance
@@ -33,17 +34,18 @@ class FXDCPrint;
 
 
 // GL Viewer options
-enum FXGLViewerOption {
-  VIEWER_LOCKED   = 0x00008000,     // Mouse based view manipulation is locked
-  VIEWER_LIGHTING = 0x00010000,     // Lighting is on
-  VIEWER_FOG      = 0x00020000,     // Fog mode on
-  VIEWER_DITHER   = 0x00040000      // Dithering
+enum {
+  VIEWER_LOCKED      = 0x00008000,    /// Mouse based view manipulation is locked
+  VIEWER_LIGHTING    = 0x00010000,    /// Lighting is on
+  VIEWER_FOG         = 0x00020000,    /// Fog mode on
+  VIEWER_DITHER      = 0x00040000     /// Dithering
   };
 
-  
-/*******************************  Viewer  Structs  *****************************/
-  
 
+/*******************************  Viewer  Structs  *****************************/
+
+
+/// OpenGL Viewer Viewport
 struct FXViewport {
   FXint      w,h;               // Viewport dimensions
   FXdouble   left,right;        // World box
@@ -51,6 +53,8 @@ struct FXViewport {
   FXdouble   hither,yon;
   };
 
+
+// OpenGL Light Source
 struct FXAPI FXLight {
   FXHVec     ambient;           // Ambient light color
   FXHVec     diffuse;           // Diffuse light color
@@ -64,6 +68,8 @@ struct FXAPI FXLight {
   FXfloat    q_attn;            // Quadratic attenuation factor
   };
 
+
+// OpenGL Material Description
 struct FXAPI FXMaterial {
   FXHVec     ambient;           // Ambient material color
   FXHVec     diffuse;           // Diffuse material color
@@ -72,15 +78,15 @@ struct FXAPI FXMaterial {
   FXfloat    shininess;         // Specular shininess
   };
 
-  
+
 // Feedback buffer sort routine
 typedef FXbool (*FXZSortFunc)(FXfloat*& buffer,FXint& used,FXint& size);
 
 
 /********************************  Viewer  Class  ******************************/
 
-  
-// Canvas, an area drawn by another object
+
+/// Canvas, an area drawn by another object
 class FXAPI FXGLViewer : public FXGLCanvas {
   FXDECLARE(FXGLViewer)
   friend class FXGLObject;
@@ -88,11 +94,8 @@ protected:
   FXViewport      wvt;              // Window viewport transform
   FXHMat          transform;        // Current transformation matrix
   FXHMat          itransform;       // Inverse of current transformation matrix
-  FXdouble        screenmin;        // Minimum screen dimension
-  FXdouble        screenmax;        // Maximum screen dimension
   FXuint          projection;       // Projection mode
   FXQuat          rotation;         // Viewer orientation
-  FXdouble        aspect;           // Aspect ratio
   FXdouble        fov;              // Field of view
   FXdouble        zoom;             // Zoom factor
   FXdouble        offset;           // Offset for lines on surfaces
@@ -119,24 +122,33 @@ protected:
   FXGLObject     *selection;        // Current object
   FXZSortFunc     zsortfunc;        // Routine to sort feedback buffer
   FXGLObject     *scene;            // What we're looking at
+public:
+
+  // Common DND types
+  static FXDragType objectType;     // GL Object type
 
 protected:
-  
+
   // Mouse actions when in viewing window
   enum {
+    HOVERING,                       // Hovering mouse w/o doing anything
     PICKING,                        // Pick mode
-    TRANSLATING,                    // Translating camera
     ROTATING,                       // Rotating camera around target
+    POSTING,                        // Posting right-mouse menu
+    TRANSLATING,                    // Translating camera
     ZOOMING,                        // Zooming
     FOVING,                         // Change field-of-view
     DRAGGING,                       // Dragging objects
     TRUCKING,                       // Trucking camera
-    LASSOING,                       // Lassoing rectangle
-    GYRATING                        // Rotation of camera around eye
+    GYRATING,                       // Rotation of camera around eye
+    DO_LASSOSELECT,                 // Lasso select when mouse pressed
+    LASSOSELECT,                    // Anchor of lasso rectangle
+    DO_LASSOZOOM,                   // Zoom when mouse pressed
+    LASSOZOOM                       // Zoom rectangle
     };
 
 protected:
-  FXGLViewer(){}
+  FXGLViewer();
   void glsetup();
   void updateProjection();
   void updateTransform();
@@ -145,21 +157,26 @@ protected:
   void drawWorld(FXViewport& wv);
   void drawAnti(FXViewport& wv);
   void drawLasso(FXint x0,FXint y0,FXint x1,FXint y1);
-  FXint selectHits(FXuint *hits,FXint x,FXint y,FXint w,FXint h,FXint maxhits);
+  FXint selectHits(FXuint*& hits,FXint& nhits,FXint x,FXint y,FXint w,FXint h);
   FXint renderFeedback(FXfloat *buffer,FXint x,FXint y,FXint w,FXint h,FXint maxbuffer);
   void drawFeedback(FXDCPrint& pdc,const FXfloat* buffer,FXint used);
   virtual FXGLObject* processHits(FXuint *pickbuffer,FXint nhits);
+  void setOp(FXuint o);
   virtual void layout();
 private:
   FXGLViewer(const FXGLViewer&);
   FXGLViewer &operator=(const FXGLViewer&);
+  void initialize();
 public:
+
+  // Events
   long onPaint(FXObject*,FXSelector,void*);
-  long onUpdate(FXObject*,FXSelector,void*);
   long onEnter(FXObject*,FXSelector,void*);
   long onLeave(FXObject*,FXSelector,void*);
   long onMotion(FXObject*,FXSelector,void*);
+  long onMouseWheel(FXObject*,FXSelector,void*);
   long onChanged(FXObject*,FXSelector,void*);
+  long onPick(FXObject*,FXSelector,void*);
   long onClicked(FXObject*,FXSelector,void*);
   long onDoubleClicked(FXObject*,FXSelector,void*);
   long onTripleClicked(FXObject*,FXSelector,void*);
@@ -179,6 +196,11 @@ public:
   long onKeyRelease(FXObject*,FXSelector,void*);
   long onFocusIn(FXObject*,FXSelector,void*);
   long onFocusOut(FXObject*,FXSelector,void*);
+  long onClipboardLost(FXObject*,FXSelector,void*);
+  long onClipboardGained(FXObject*,FXSelector,void*);
+  long onClipboardRequest(FXObject*,FXSelector,void*);
+
+  // Commands
   long onCmdPerspective(FXObject*,FXSelector,void*);
   long onUpdPerspective(FXObject*,FXSelector,void*);
   long onCmdParallel(FXObject*,FXSelector,void*);
@@ -206,6 +228,10 @@ public:
   long onQueryTip(FXObject*,FXSelector,void*);
   long onCmdXYZDial(FXObject*,FXSelector,void*);
   long onUpdXYZDial(FXObject*,FXSelector,void*);
+  long onCmdRollPitchYaw(FXObject*,FXSelector,void*);
+  long onUpdRollPitchYaw(FXObject*,FXSelector,void*);
+  long onCmdXYZScale(FXObject*,FXSelector,void*);
+  long onUpdXYZScale(FXObject*,FXSelector,void*);
   long onUpdCurrent(FXObject*,FXSelector,void*);
   long onCmdCutSel(FXObject*,FXSelector,void*);
   long onCmdCopySel(FXObject*,FXSelector,void*);
@@ -238,21 +264,24 @@ public:
   long onUpdTurbo(FXObject*,FXSelector,void*);
   long onCmdPrintImage(FXObject*,FXSelector,void*);
   long onCmdPrintVector(FXObject*,FXSelector,void*);
+  long onCmdLassoZoom(FXObject*,FXSelector,void*);
+  long onCmdLassoSelect(FXObject*,FXSelector,void*);
+  virtual long onDefault(FXObject*,FXSelector,void*);
 
 public:
-  
+
   // Projection modes
   enum {
-    PARALLEL,
-    PERSPECTIVE
+    PARALLEL,		  // Parallel projection
+    PERSPECTIVE		  // Perspective projection
     };
 
-  // Precompiled display lists
+  // Precompiled display lists [DEPRECATED]
   enum {
     OFFSETPROJECTION  = 1,
     SURFACEPROJECTION = 2
     };
-    
+
   // Messages
   enum {
     ID_PERSPECTIVE=FXGLCanvas::ID_LAST,
@@ -266,7 +295,7 @@ public:
     ID_RESETVIEW,
     ID_FITVIEW,
     ID_TIPTIMER,
-    ID_BACK_COLOR,    
+    ID_BACK_COLOR,
     ID_AMBIENT_COLOR,
     ID_LIGHT_AMBIENT,
     ID_LIGHT_DIFFUSE,
@@ -275,9 +304,15 @@ public:
     ID_TURBO,
     ID_FOG,
     ID_DITHER,
+    ID_SCALE_X,
+    ID_SCALE_Y,
+    ID_SCALE_Z,
     ID_DIAL_X,
     ID_DIAL_Y,
     ID_DIAL_Z,
+    ID_ROLL,
+    ID_PITCH,
+    ID_YAW,
     ID_FOV,
     ID_ZOOM,
     ID_LOCK,
@@ -287,80 +322,240 @@ public:
     ID_DELETE_SEL,
     ID_PRINT_IMAGE,
     ID_PRINT_VECTOR,
+    ID_LASSO_ZOOM,
+    ID_LASSO_SELECT,
     ID_LAST
     };
 
 public:
+
+  // Common DND type names
+  static const FXchar objectTypeName[];
+
+public:
+
+  /// Construct GL viewer widget
   FXGLViewer(FXComposite* p,FXGLVisual *vis,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
-  virtual long handle(FXObject* sender,FXSelector key,void* data);
+
+  /// Construct GL viewer widget sharing display list with another GL viewer
+  FXGLViewer(FXComposite* p,FXGLVisual *vis,FXGLViewer* sharegroup,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
+
+  /// Create all of the server-side resources for this window
   virtual void create();
-  virtual void resize(FXint w,FXint h);
-  virtual void position(FXint x,FXint y,FXint w,FXint h);
+
+  /// Detach server-side resources
+  virtual void detach();
+
+  /// Return size of pixel in world coordinates
   FXdouble worldPix() const { return worldpx; }
+
+  /// Return size of pixel in model coordinates
   FXdouble modelPix() const { return modelpx; }
+
+  /// Return a NULL-terminated list of all objects in the given rectangle, or NULL
   FXGLObject** lasso(FXint x1,FXint y1,FXint x2,FXint y2);
+
+  /// Return a NULL-terminated list of all objects in the given rectangle, or NULL
   virtual FXGLObject** select(FXint x,FXint y,FXint w,FXint h);
+
+  /// Perform a pick operation, returning the object at the given x,y position, or NULL
   virtual FXGLObject* pick(FXint x,FXint y);
+
+  /// Change the model bounding box; this adjusts the viewer
   FXbool setBounds(const FXRange& box);
+
+  /// Fit viewer to the given bounding box
   FXbool fitToBounds(const FXRange& box);
+
+  /// Return the viewer's viewport
   void getViewport(FXViewport& v) const;
-  void eyeToScreen(FXint& sx,FXint& sy,const FXVec& e);
+
+  /// Translate eye-coordinate to screen coordinate
+  void eyeToScreen(FXint& sx,FXint& sy,FXVec e);
+
+  /// Translate screen coordinate to eye coordinate at the given depth
   FXVec screenToEye(FXint sx,FXint sy,FXfloat eyez=0.0);
+
+  /// Translate screen coordinate to eye coordinate at the target point depth
   FXVec screenToTarget(FXint sx,FXint sy);
-  FXVec worldToEye(const FXVec& w);
-  FXfloat worldToEyeZ(const FXVec& w);
-  FXVec eyeToWorld(const FXVec& e); 
+
+  /// Translate world coordinate to eye coordinate
+  FXVec worldToEye(FXVec w);
+
+  /// Translate world coordinate to eye coordinate depth
+  FXfloat worldToEyeZ(FXVec w);
+
+  /// Translate eye coordinate to eye coordinate
+  FXVec eyeToWorld(FXVec e);
+
+  /// Calculate world coordinate vector from screen movement
   FXVec worldVector(FXint fx,FXint fy,FXint tx,FXint ty);
+
+  ///  Change default object material setting
   void setMaterial(const FXMaterial &mtl);
+
+  /// Return default object material setting
   void getMaterial(FXMaterial &mtl) const;
+
+  /// Change camera field of view angle (in degrees)
   void setFieldOfView(FXdouble fv);
+
+  /// Return camera field of view angle
   FXdouble getFieldOfView() const { return fov; }
+
+  /// Change camera zoom factor
   void setZoom(FXdouble zm);
+
+  /// Return camera zoom factor
   FXdouble getZoom() const { return zoom; }
+
+  /// Change target point distance
   void setDistance(FXdouble ed);
+
+  /// Return target point distance
   FXdouble getDistance() const { return distance; }
-  void setOrientation(const FXQuat& rot);
-  const FXQuat& getOrientation() const { return rotation; }
-  void setCenter(const FXVec& cntr);
+
+  /// Change unequal model scaling factors
+  void setScale(FXVec s);
+
+  /// Return current scaling factors
+  FXVec getScale() const { return scale; }
+
+  /// Change camera orientation from quaternion
+  void setOrientation(FXQuat rot);
+
+  /// Return current camera orientation quaternion
+  FXQuat getOrientation() const { return rotation; }
+
+  /// Change object center (tranlation)
+  void setCenter(FXVec cntr);
+
+  /// Return object center
   const FXVec& getCenter() const { return center; }
-  void translate(const FXVec& vec);
+
+  /// Translate object center
+  void translate(FXVec vec);
+
+  /// Return boresight vector
   FXbool getBoreVector(FXint sx,FXint sy,FXVec& point,FXVec& dir);
+
+  /// Return eyesight vector
   FXVec getEyeVector() const;
+
+  /// Return eye position
   FXVec getEyePosition() const;
-  virtual FXbool canFocus() const;
+
+  /// Change help text
   void setHelpText(const FXString& text);
+
+  /// Return help text
   FXString getHelpText() const { return help; }
+
+  /// Change tip text
   void setTipText(const FXString&  text);
+
+  /// Return tip text
   FXString getTipText() const { return tip; }
-  const FXHMat& getTransform() const { return transform; }
-  const FXHMat& getInvTransform() const { return itransform; }
-  void setScene(FXGLObject* sc);
-  FXGLObject* getScene() const { return scene; }
-  void setSelection(FXGLObject* sel);
-  FXGLObject* getSelection() const { return selection; }
-  void setProjection(FXuint proj);
-  FXuint getProjection() const { return projection; }
-  void setViewLock(FXbool lock=TRUE);
-  FXbool getViewLock() const;
-  void setBackgroundColor(const FXHVec& clr);
-  FXHVec getBackgroundColor() const { return background; }
-  void setAmbientColor(const FXHVec& clr);
-  FXHVec getAmbientColor() const { return ambient; }
-  FXbool readPixels(FXuchar*& buffer,FXint x,FXint y,FXint w,FXint h);
-  FXbool readFeedback(FXfloat*& buffer,FXint& used,FXint& size,FXint x,FXint y,FXint w,FXint h);
-  void setZSortFunc(FXZSortFunc func){ zsortfunc=func; }
-  FXZSortFunc getZSortFunc() const { return zsortfunc; }
+
+  /// Change line offset
   void setOffset(FXdouble offs);
+
+  /// Return line offset
   FXdouble getOffset() const { return offset; }
+
+  /// Return the current transformation matrix
+  const FXHMat& getTransform() const { return transform; }
+
+  /// Return the inverse of the current transformation matrix
+  const FXHMat& getInvTransform() const { return itransform; }
+
+  /// Change the scene, i.e. the object being displayed.
+  void setScene(FXGLObject* sc);
+
+  /// Return the current scene object
+  FXGLObject* getScene() const { return scene; }
+
+  /// Change selection
+  void setSelection(FXGLObject* sel);
+
+  /// Return selection
+  FXGLObject* getSelection() const { return selection; }
+
+  /// Change the projection mode, PERSPECTIVE or PARALLEL
+  void setProjection(FXuint proj);
+
+  /// Return the projection mode
+  FXuint getProjection() const { return projection; }
+
+  /// Lock the viewer, i.e. prevent mouse-based viewing operations
+  void setViewLock(FXbool lock=TRUE);
+
+  /// Return viewer lock status
+  FXbool getViewLock() const;
+
+  /// Change window background color
+  void setBackgroundColor(const FXHVec& clr);
+
+  /// Return window background color
+  FXHVec getBackgroundColor() const { return background; }
+
+  /// Change global ambient light color
+  void setAmbientColor(const FXHVec& clr);
+
+  /// Return global ambient light color
+  FXHVec getAmbientColor() const { return ambient; }
+
+  /// Read the pixels off the screen as R,G,B tuples.
+  FXbool readPixels(FXuchar*& buffer,FXint x,FXint y,FXint w,FXint h);
+
+  /// Read the feedback buffer containing the current scene, returning used and allocated size
+  FXbool readFeedback(FXfloat*& buffer,FXint& used,FXint& size,FXint x,FXint y,FXint w,FXint h);
+
+  /**
+  * Change hidden-surface feedback buffer sorting algorithm.
+  * This can be used for move/draw printed output depth sorting.
+  */
+  void setZSortFunc(FXZSortFunc func){ zsortfunc=func; }
+
+  /// Return hidden surface sorting function.
+  FXZSortFunc getZSortFunc() const { return zsortfunc; }
+
+  /**
+  * Change the maximum hits, i.e. the maximum size of the pick buffer.
+  * When set to less than or equal to zero, picking is essentially turned off.
+  */
   void setMaxHits(FXint maxh) { maxhits=maxh; }
+
+  /// Return maximum pickbuffer size
   FXint getMaxHits() const { return maxhits; }
+
+  /**
+  * When drawing a GL object, if doesTurbo() is true, the object
+  * may choose to perform a reduced complexity drawing as the user is
+  * interactively manipulating; another update will be done later when
+  * the full complexity drawing can be performed again.
+  */
   FXbool doesTurbo() const { return doesturbo; }
+
+  /// Return turbo mode setting
   FXbool getTurboMode() const { return turbomode; }
+
+  /// Set turbo mode
   void setTurboMode(FXbool turbo=TRUE);
+
+  /// Return light source settings
   void getLight(FXLight& lite) const;
+
+  /// Change light source settings
   void setLight(const FXLight& lite);
+
+  /// Save viewer to a stream
   virtual void save(FXStream& store) const;
+
+  /// Load viewer from a stream
   virtual void load(FXStream& store);
+
+  /// Destructor
   virtual ~FXGLViewer();
   };
 
