@@ -3,30 +3,38 @@
 *                            V i s u a l   C l a s s                            *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1999 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1999,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXGLVisual.h,v 1.2 1999/09/20 22:12:56 jeroen Exp $                      *
+* $Id: FXGLVisual.h,v 1.12 2001/08/23 16:24:51 jeroen Exp $                     *
 ********************************************************************************/
 #ifndef FXGLVISUAL_H
 #define FXGLVISUAL_H
 
 
+class FXFont;
+class FXWindow;
+class FXImage;
+class FXIcon;
+class FXBitmap;
+class FXDCWindow;
+class FXGLCanvas;
 
-// Visual describes pixel format of a drawable
+
+/// Visual describes pixel format of a drawable
 class FXAPI FXGLVisual : public FXVisual {
   FXDECLARE(FXGLVisual)
   friend class FXWindow;
@@ -37,7 +45,7 @@ class FXAPI FXGLVisual : public FXVisual {
   friend class FXGLCanvas;
 protected:
   FXint        redSize;             // Desired #bits for red
-  FXint        greenSize;           // Desired #bits for green  
+  FXint        greenSize;           // Desired #bits for green
   FXint        blueSize;            // Desired #bits for blue
   FXint        alphaSize;           // Desired #bits for alpha
   FXint        depthSize;           // Desired #bits for Z
@@ -48,7 +56,7 @@ protected:
   FXint        accumAlphaSize;      // Desired #bits for accum alpha
 protected:
   FXGLVisual();
-#ifdef FX_NATIVE_WIN32
+#ifdef WIN32
   void makeOpenGLPalette();
 #endif
 private:
@@ -56,13 +64,19 @@ private:
   FXGLVisual &operator=(const FXGLVisual&);
 public:
 
-  // Construct default visual
+  /// Construct default visual
   FXGLVisual(FXApp* a,FXuint flags);
 
-  // Initialize
-  virtual void init();
+  /// Create visual
+  virtual void create();
 
-  // Get sizes for bit-planes
+  /// Detach visual
+  virtual void detach();
+
+  /// Destroy visual
+  virtual void destroy();
+
+  /// Get sizes for bit-planes
   FXint getRedSize() const { return redSize; }
   FXint getGreenSize() const { return greenSize; }
   FXint getBlueSize() const { return blueSize; }
@@ -74,7 +88,7 @@ public:
   FXint getAccumBlueSize() const { return accumBlueSize; }
   FXint getAccumAlphaSize() const { return accumAlphaSize; }
 
-  // Set sizes for bit-planes
+  /// Set sizes for bit-planes
   void setRedSize(FXint rs){ redSize=rs; }
   void setGreenSize(FXint gs){ greenSize=gs; }
   void setBlueSize(FXint bs){ blueSize=bs; }
@@ -86,7 +100,7 @@ public:
   void setAccumBlueSize(FXint bs){ accumBlueSize=bs; }
   void setAccumAlphaSize(FXint as){ accumAlphaSize=as; }
 
-  // Get ACTUAL sizes for bit-planes
+  /// Get ACTUAL sizes for bit-planes
   FXint getActualRedSize() const;
   FXint getActualGreenSize() const;
   FXint getActualBlueSize() const;
@@ -98,24 +112,29 @@ public:
   FXint getActualAccumBlueSize() const;
   FXint getActualAccumAlphaSize() const;
 
-  // Is it double buffer
+  /// Is it double buffered?
   FXbool isDoubleBuffer() const;
 
-  // Is it stereo
+  /// Is it stereo?
   FXbool isStereo() const;
 
-  // Is it hardware-accelerated?
+  /// Is it hardware-accelerated?
   FXbool isAccelerated() const;
 
-  // Save
+  /// Save visual info to a stream
   virtual void save(FXStream& store) const;
 
-  // Load
+  /// Load visual info to a stream
   virtual void load(FXStream& store);
 
-  // Destroy
+  /// Destructor
   virtual ~FXGLVisual();
   };
-  
+
+
+/// Create a display list of bitmaps from font glyphs in a font
+extern FXAPI void glUseFXFont(FXFont* font,int first,int count,int list);
+
+
 
 #endif
