@@ -3,39 +3,40 @@
 *                      M e n u   C a s c a d e   W i d g e t                    *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXMenuCascade.h,v 1.2 1999/09/20 22:12:56 jeroen Exp $                   *
+* $Id: FXMenuCascade.h,v 1.12 2001/07/31 05:34:48 jeroen Exp $                  *
 ********************************************************************************/
 #ifndef FXMENUCASCADE_H
 #define FXMENUCASCADE_H
 
 
-class FXMenuPane;
+class FXPopup;
+struct FXTimer;
 
 
-// Cascade menu item
+/// Cascade menu item
 class FXAPI FXMenuCascade : public FXMenuCaption {
   FXDECLARE(FXMenuCascade)
 protected:
-  FXMenuPane *pane;
-  FXTimer*    timer;
+  FXPopup *pane;
+  FXTimer *timer;
 protected:
-  FXMenuCascade(){}
+  FXMenuCascade();
   void drawTriangle(FXDCWindow& dc,FXint l,FXint t,FXint r,FXint b);
 private:
   FXMenuCascade(const FXMenuCascade&);
@@ -45,8 +46,8 @@ public:
   long onEnter(FXObject*,FXSelector,void*);
   long onLeave(FXObject*,FXSelector,void*);
   long onTimeout(FXObject*,FXSelector,void*);
-  long onLeftBtnPress(FXObject*,FXSelector,void*);
-  long onLeftBtnRelease(FXObject*,FXSelector,void*);
+  long onButtonPress(FXObject*,FXSelector,void*);
+  long onButtonRelease(FXObject*,FXSelector,void*);
   long onKeyPress(FXObject*,FXSelector,void*);
   long onKeyRelease(FXObject*,FXSelector,void*);
   long onHotKeyPress(FXObject*,FXSelector,void*);
@@ -59,18 +60,43 @@ public:
     ID_LAST
     };
 public:
-  FXMenuCascade(FXComposite* p,const FXString& text,FXIcon* ic=NULL,FXMenuPane* pup=NULL,FXuint opts=0);
+  /// Constructor
+  FXMenuCascade(FXComposite* p,const FXString& text,FXIcon* ic=NULL,FXPopup* pup=NULL,FXuint opts=0);
+
+  /// Create server-side resources
   virtual void create();
+
+  /// Detach server-side resources
   virtual void detach();
+
+  /// Destroy server-side resources
   virtual void destroy();
+
+  /// Yes it can receive the focus
   virtual FXbool canFocus() const;
+
+  /// Move the focus to this window
   virtual void setFocus();
+
+  /// Remove the focus from this window
   virtual void killFocus();
-  void setPopup(FXMenuPane *pup){ pane = pup; }
-  FXMenuPane* getPopup() const { return pane; }
+
+  /// Set popup menu to pop up
+  void setMenu(FXPopup *pup){ pane = pup; }
+
+  /// Return popup menu
+  FXPopup* getMenu() const { return pane; }
+
+  /// True if this menu or is popup logically contains the mouse
   virtual FXbool contains(FXint parentx,FXint parenty) const;
+
+  /// Save menu to a stream
   virtual void save(FXStream& store) const;
+
+  /// Load menu from a stream
   virtual void load(FXStream& store);
+
+  /// Destructor
   virtual ~FXMenuCascade();
   };
 
