@@ -3,23 +3,23 @@
 *                             O p t i o n   M e n u                             *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXOptionMenu.h,v 1.4 1999/11/09 23:58:37 jeroen Exp $                    *
+* $Id: FXOptionMenu.h,v 1.12 2001/07/06 19:33:05 jeroen Exp $                   *
 ********************************************************************************/
 #ifndef FXOPTIONMENU_H
 #define FXOPTIONMENU_H
@@ -28,12 +28,9 @@
 class FXPopup;
 
 
-// Option
+/// Option Menu Button
 class FXAPI FXOption : public FXLabel {
   FXDECLARE(FXOption)
-protected:
-  FXString tip;
-  FXString help;
 protected:
   FXOption(){}
 private:
@@ -49,26 +46,33 @@ public:
   long onKeyRelease(FXObject*,FXSelector,void*);
   long onHotKeyPress(FXObject*,FXSelector,void*);
   long onHotKeyRelease(FXObject*,FXSelector,void*);
-  long onQueryHelp(FXObject*,FXSelector,void*);
 public:
+
+  /// Constructor
   FXOption(FXComposite* p,const FXString& text,FXIcon* ic=NULL,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=JUSTIFY_NORMAL|ICON_BEFORE_TEXT|MENUBUTTON_DOWN,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD);
+
+  /// Return default width
   virtual FXint getDefaultWidth();
+
+  /// Return default height
   virtual FXint getDefaultHeight();
+
+  /// Returns true because a menu button can receive focus
   virtual FXbool canFocus() const;
+
+  /// Set focus to this window
   virtual void setFocus();
+
+  /// Remove the focus from this window
   virtual void killFocus();
-  void setHelpText(const FXString& text);
-  FXString getHelpText() const { return help; }
-  void setTipText(const FXString& text);
-  FXString getTipText() const { return tip; }
-  virtual void save(FXStream& store) const;
-  virtual void load(FXStream& store);
+
+  /// Destructor
   virtual ~FXOption();
   };
 
 
-  
-// Option Menu button
+
+/// Option Menu
 class FXAPI FXOptionMenu : public FXLabel {
   FXDECLARE(FXOptionMenu)
 protected:
@@ -93,25 +97,66 @@ public:
   long onCmdUnpost(FXObject*,FXSelector,void*);
   long onQueryHelp(FXObject*,FXSelector,void*);
   long onQueryTip(FXObject*,FXSelector,void*);
+  long onCmdSetValue(FXObject*,FXSelector,void*);
+  long onCmdSetIntValue(FXObject*,FXSelector,void*);
+  long onCmdGetIntValue(FXObject*,FXSelector,void*);
 public:
+
+  /// Constructor
   FXOptionMenu(FXComposite* p,FXPopup* pup=NULL,FXuint opts=JUSTIFY_NORMAL|ICON_BEFORE_TEXT,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD);
+
+  /// Create server-side resources
   virtual void create();
+
+  /// Detach server-side resources
   virtual void detach();
+
+  /// Delete server-side resources
   virtual void destroy();
+
+  /// Remove the focus from this window
   virtual void killFocus();
+
+  /// Return default width
   virtual FXint getDefaultWidth();
+
+  /// Return default height
   virtual FXint getDefaultHeight();
+
+  /// Return TRUE if the position is logically in the pane
   virtual FXbool contains(FXint parentx,FXint parenty) const;
+
+  /// Set the current option
   void setCurrent(FXOption *win);
+
+  /// Return the current option
   FXOption* getCurrent() const { return current; }
+
+  /// Set the current option number
   void setCurrentNo(FXint no);
+
+  /// Get the current option number
   FXint getCurrentNo() const;
+
+  /// Set the pane which will be popped up
   void setPopup(FXPopup *pup);
+
+  /// Return the pane which is poppup up
   FXPopup* getPopup() const { return pane; }
+
+  /// Returns true because a option menu can receive focus
   virtual FXbool canFocus() const;
+
+  /// Return TRUE if popped up
   FXbool isPopped() const;
+
+  /// Save option menu to a stream
   virtual void save(FXStream& store) const;
+
+  /// Load option menu from a stream
   virtual void load(FXStream& store);
+
+  /// Destructor
   virtual ~FXOptionMenu();
   };
 
