@@ -3,36 +3,39 @@
 *                      P r o g r e s s B a r   W i d g e t                      *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXProgressBar.h,v 1.3 1999/10/20 07:19:55 jeroen Exp $                   *
+* $Id: FXProgressBar.h,v 1.13 2001/07/10 02:23:08 jeroen Exp $                  *
 ********************************************************************************/
 #ifndef FXPROGRESSBAR_H
 #define FXPROGRESSBAR_H
 
 
-// Progress bar styles
-enum FXProgressBarStyle {
-  PROGRESSBAR_PERCENTAGE = 0x00008000,    // Show percentage done
-  PROGRESSBAR_VERTICAL   = 0x00010000     // Vertical display
+/// Progress bar styles
+enum {
+  PROGRESSBAR_HORIZONTAL = 0,             /// Horizontal display
+  PROGRESSBAR_VERTICAL   = 0x00008000,    /// Vertical display
+  PROGRESSBAR_PERCENTAGE = 0x00010000,    /// Show percentage done
+  PROGRESSBAR_DIAL       = 0x00020000,    /// Show as a dial instead of bar
+  PROGRESSBAR_NORMAL     = FRAME_SUNKEN|FRAME_THICK
   };
 
 
-// Progress bar widget
+/// Progress bar widget
 class FXAPI FXProgressBar : public FXFrame {
   FXDECLARE(FXProgressBar)
 protected:
@@ -55,32 +58,93 @@ public:
   long onCmdSetIntValue(FXObject*,FXSelector,void*);
   long onCmdGetIntValue(FXObject*,FXSelector,void*);
 public:
-  FXProgressBar(FXComposite* p,FXObject* target=NULL,FXSelector sel=0,FXuint opts=(FRAME_SUNKEN|FRAME_THICK),FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD);
+
+  /// Construct progress bar
+  FXProgressBar(FXComposite* p,FXObject* target=NULL,FXSelector sel=0,FXuint opts=PROGRESSBAR_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD);
+
+  /// Create server-side resources
   virtual void create();
+
+  /// Detach server-side resources
   virtual void detach();
+
+  /// Return default width
   virtual FXint getDefaultWidth();
+
+  /// Return default height
   virtual FXint getDefaultHeight();
+
+  /// Change the amount of progress
   void setProgress(FXuint value);
+
+  /// Get current progress
   FXuint getProgress() const { return progress; }
+
+  /// Set total amount of progress
   void setTotal(FXuint value);
+
+  /// Return total amount of progrss
   FXuint getTotal() const { return total; }
+
+  /// Increment progress by given amount
   void increment(FXuint value);
+
+  /// Hide progress percentage
   void hideNumber();
+
+  /// Show progress percentage
   void showNumber();
+
+  /// Change progress bar width
   void setBarSize(FXint size);
+
+  /// Return progress bar width
   FXint getBarSize() const { return barsize; }
+
+  /// Change backgroundcolor
   void setBarBGColor(FXColor clr);
+
+  /// Return background color
   FXColor getBarBGColor() const { return barBGColor; }
+
+  /// Change bar color
   void setBarColor(FXColor clr);
+
+  /// Return bar color
   FXColor getBarColor() const { return barColor; }
+
+  /// Change text color
   void setTextColor(FXColor clr);
+
+  /// Return text color
   FXColor getTextColor() const { return textNumColor; }
+
+  /// Change alternate text color shown when bar under text
   void setTextAltColor(FXColor clr);
+
+  /// Return alternate text color
   FXColor getTextAltColor() const { return textAltColor; }
+
+  /// Set the text font
   void setFont(FXFont *fnt);
+
+  /// Get the text font
   FXFont* getFont() const { return font; }
+
+  /// Change progress bar style
+  void setBarStyle(FXuint style);
+
+  /// Return current progress bar style
+  FXuint getBarStyle() const;
+
+  /// Save progress bar to a stream
+  virtual void save(FXStream& store) const;
+
+  /// Load progress bar from a stream
+  virtual void load(FXStream& store);
+
+  /// Destructor
   virtual ~FXProgressBar();
   };
-    
 
 #endif
