@@ -3,30 +3,30 @@
 *                     S h e l l   W i n d o w   W i d g e t                     *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXShell.h,v 1.10 1999/11/15 17:14:50 jeroen Exp $                         *
+* $Id: FXShell.h,v 1.16 2001/04/16 21:13:58 jeroen Exp $                        *
 ********************************************************************************/
 #ifndef FXSHELL_H
 #define FXSHELL_H
 
 
 
-// A child of the Root window
+/// A child of the Root window
 class FXAPI FXShell : public FXComposite {
   FXDECLARE(FXShell)
 protected:
@@ -37,16 +37,24 @@ private:
   FXShell(const FXShell&);
   FXShell &operator=(const FXShell&);
 public:
-  long onMap(FXObject*,FXSelector,void*);
-  long onUnmap(FXObject*,FXSelector,void*);
   long onConfigure(FXObject*,FXSelector,void*);
   long onFocusNext(FXObject*,FXSelector,void*);
   long onFocusPrev(FXObject*,FXSelector,void*);
 public:
+
+  /// Create server-side resources
   virtual void create();
+
+  /// Mark this window's layout as dirty
   virtual void recalc();
+
+  /// Move the focus to this window
   virtual void setFocus();
+
+  /// Remove the focus from this window
   virtual void killFocus();
+
+  /// Destroy shell
   virtual ~FXShell();
   };
 
