@@ -3,34 +3,39 @@
 *                 S h u t t e r   C o n t a i n e r   W i d g e t               *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXShutter.h,v 1.4 1999/11/09 23:58:37 jeroen Exp $                      *
+* $Id: FXShutter.h,v 1.13 2001/08/23 16:24:51 jeroen Exp $                      *
 ********************************************************************************/
 #ifndef FXSHUTTER_H
 #define FXSHUTTER_H
 
+
+struct FXTimer;
 class FXShutter;
+class FXButton;
+class FXScrollWindow;
+class FXShutterItem;
 
 
-// Shutter Control
+/// Shutter Control
 class FXAPI FXShutterItem : public FXVerticalFrame {
   FXDECLARE(FXShutterItem)
-  friend class FXShutter;  
+  friend class FXShutter;
 protected:
   FXButton         *button;
   FXScrollWindow   *scrollWindow;
@@ -50,17 +55,36 @@ public:
     ID_LAST
     };
 public:
-  FXShutterItem(FXShutter *p,const char* text=NULL,FXIcon* icon=NULL,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_SPACING,FXint pr=DEFAULT_SPACING,FXint pt=DEFAULT_SPACING,FXint pb=DEFAULT_SPACING,FXint hs=DEFAULT_SPACING,FXint vs=DEFAULT_SPACING);
+  /// Constructor
+  FXShutterItem(FXShutter *p,const FXString& text=NULL,FXIcon* icon=NULL,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_SPACING,FXint pr=DEFAULT_SPACING,FXint pt=DEFAULT_SPACING,FXint pb=DEFAULT_SPACING,FXint hs=DEFAULT_SPACING,FXint vs=DEFAULT_SPACING);
+
+  /// Return a pointer to the button for this item
   FXButton* getButton() const { return button; }
+
+  /// Return a pointer to the contents for this item
   FXVerticalFrame* getContent() const { return content; }
+
+  /// Set the status line help text for this item
+  void setHelpText(const FXString& text);
+
+  /// Get the status line help text for this item
+  FXString getHelpText() const;
+
+  /// Set the tool tip message for this item
+  void setTipText(const FXString&  text);
+
+  /// Get the tool tip message for this item
+  FXString getTipText() const;
+
+  /// Destructor
   virtual ~FXShutterItem();
   };
 
 
-// Shutter Control
+/// Shutter Control
 class FXAPI FXShutter : public FXVerticalFrame {
   FXDECLARE(FXShutter)
-  friend class FXShutterItem;  
+  friend class FXShutterItem;
 protected:
   FXint          current;               // Item currently open
   FXint          closing;               // Item closing down
@@ -93,9 +117,16 @@ public:
     ID_LAST
     };
 public:
+  /// Constructor
   FXShutter(FXComposite *p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_SPACING,FXint pr=DEFAULT_SPACING,FXint pt=DEFAULT_SPACING,FXint pb=DEFAULT_SPACING,FXint hs=DEFAULT_SPACING,FXint vs=DEFAULT_SPACING);
+
+  /// Set the currently displayed item (panel = 0, 1, 2, ..., npanels-1)
   virtual void setCurrent(FXint panel);
+
+  /// Return the index of the currently displayed item
   FXint getCurrent() const { return current; }
+
+  /// Destructor
   virtual ~FXShutter();
   };
 
