@@ -3,23 +3,23 @@
 *                            T a b l e   W i d g e t                            *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1999 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1999,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXTable.h,v 1.21 1999/11/09 23:58:38 jeroen Exp $                        *
+* $Id: FXTable.h,v 1.64 2001/06/18 21:15:44 jeroen Exp $                        *
 ********************************************************************************/
 #ifndef FXTABLE_H
 #define FXTABLE_H
@@ -27,34 +27,39 @@
 
 //////////////////////////////  UNDER DEVELOPMENT  //////////////////////////////
 
+struct FXTimer;
+class FXIcon;
+class FXFont;
+class FXTable;
 
-// Default cell margin
+
+/// Default cell margin
 #define DEFAULT_MARGIN  2
 
 
 
-// List options
-enum FXTableStyle {
-  TABLE_HOR_GRIDLINES   = 0x00100000,   // Horizontal grid lines
-  TABLE_VER_GRIDLINES   = 0x00200000    // Vertical grid lines
+/// Table options
+enum {
+  TABLE_COL_SIZABLE = 0x00100000,   /// Columns are resizable
+  TABLE_ROW_SIZABLE = 0x00200000    /// Rows are resizable
   };
 
 
-// Position in table
+/// Position in table
 struct FXTablePos {
   FXint  row;
   FXint  col;
   };
-  
 
-// Range of table cells
+
+/// Range of table cells
 struct FXTableRange {
   FXTablePos fm;
   FXTablePos to;
   };
 
-  
-// Item in table
+
+/// Item in table
 class FXAPI FXTableItem : public FXObject {
   FXDECLARE(FXTableItem)
   friend class FXTable;
@@ -66,36 +71,52 @@ protected:
 protected:
   FXTableItem():icon(NULL),data(NULL),state(0){}
   virtual void draw(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
-  void drawFocus(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
-  void drawRaised(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
-  void drawSunken(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
-//  void drawTextFragment(FXDCWindow& dc,FXString& string,FXint x,FXint y,FXuint fm,FXuint to);
-//   void drawTextRange(FXString& string,FXint x,FXint y,FXint w,FXint h,FXuint mode,FXuint fm,FXuint to,FXuint si,FXuint ei,FXint scroll);
+  virtual void drawButtonDown(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
+  virtual void drawButtonUp(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
+  virtual void drawNormal(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
 protected:
   enum{
-    SELECTED  = 1,
-    MARKED    = 2,
-    FOCUS     = 4,
-    DISABLED  = 8,
-    DRAGGABLE = 16,
-    RIGHT     = 32,
-    LEFT      = 64,
-    TOP       = 128,
-    BOTTOM    = 256,
-    BUTTON    = 512,
-    PRESSED   = 1024
+    SELECTED   = 0x0001,
+    FOCUS      = 0x0002,
+    DISABLED   = 0x0004,
+    DRAGGABLE  = 0x0008,
+    BUTTON     = 0x0010,
+    PRESSED    = 0x0020
     };
+public:
+  enum{
+    RIGHT      = 0x0040,      /// Align on right 
+    LEFT       = 0x0080,      /// Align on left
+    TOP        = 0x0100,      /// Align on top
+    BOTTOM     = 0x0200,      /// Align on bottom
+    LBORDER    = 0x0400,      /// Draw left border
+    RBORDER    = 0x0800,      /// Draw right border
+    TBORDER    = 0x1000,      /// Draw top border
+    BBORDER    = 0x2000,      /// Draw bottom border
+    BACKGROUND = 0x4000       /// Draw background
+    }; 
 public:
   FXTableItem(const FXString& text,FXIcon* ic=NULL,void* ptr=NULL):label(text),icon(ic),data(ptr),state(FXTableItem::RIGHT){}
-  void setText(const FXString& txt){ label=txt; }
+  virtual void setText(const FXString& txt){ label=txt; }
   FXString getText() const { return label; }
-  void setIcon(FXIcon* icn){ icon=icn; }
+  virtual void setIcon(FXIcon* icn){ icon=icn; }
   FXIcon* getIcon() const { return icon; }
   void setData(void* ptr){ data=ptr; }
   void* getData() const { return data; }
+  virtual void setFocus(FXbool focus);
   FXbool hasFocus() const { return (state&FOCUS)!=0; }
+  virtual void setSelected(FXbool selected);
   FXbool isSelected() const { return (state&SELECTED)!=0; }
+  virtual void setEnabled(FXbool enabled);
   FXbool isEnabled() const { return (state&DISABLED)==0; }
+  virtual void setDraggable(FXbool draggable);
+  FXbool isDraggable() const { return (state&DRAGGABLE)!=0; }
+  void setJustify(FXuint justify);
+  FXuint getJustify() const { return state&(RIGHT|LEFT|TOP|BOTTOM); }
+  void setBorders(FXuint borders);
+  FXuint getBorders() const { return state&(LBORDER|RBORDER|TBORDER|BBORDER); }
+  void setOpaque(FXbool opaque);
+  FXbool getOpaque() const { return (state&BACKGROUND)!=0; }
   virtual FXint getWidth(const FXTable* table) const;
   virtual FXint getHeight(const FXTable* table) const;
   virtual void create();
@@ -106,15 +127,15 @@ public:
   virtual ~FXTableItem(){}
   };
 
-  
 
-// Table Widget
+
+/// Table Widget
 class FXAPI FXTable : public FXScrollArea {
   FXDECLARE(FXTable)
 protected:
   FXTableItem **cells;                  // Cells
-  FXint        *col_x;                  // Column starts [if non-uniform]
-  FXint        *row_y;                  // Row starts [if non-uniform]
+  FXint        *col_x;                  // Vertical grid line positions
+  FXint        *row_y;                  // Horizontal grid line positions
   FXFont       *font;                   // Font
   FXint         nrows;                  // Logically allocated rows
   FXint         ncols;                  // Logically allocated columns
@@ -132,11 +153,15 @@ protected:
   FXColor       selbackColor;           // Select background color
   FXColor       seltextColor;           // Select text color
   FXColor       gridColor;              // Grid line color
+  FXColor       cellBorderColor;        // Cell border color
+  FXint         cellBorderWidth;        // Cell border width
   FXColor       cellBackColor[2][2];    // Row/Column even/odd background color
   FXint         defColumnWidth;         // Default column width [if uniform columns]
   FXint         defRowHeight;           // Default row height [if uniform rows]
   FXint         leading_rows;           // Leading fixed rows
   FXint         leading_cols;           // Leading fixed columns
+  FXint         scrolling_rows;         // Scrolling rows
+  FXint         scrolling_cols;         // Scrolling columns
   FXint         trailing_rows;          // Trailing fixed rows
   FXint         trailing_cols;          // Trailing fixed columns
   FXint         scrollable_left;        // Left side of scrollable part of table
@@ -153,7 +178,11 @@ protected:
   FXint         cellcursor;             // Cursor position in cell
   FXint         cellanchor;             // Anchor position in cell
   FXint         cellscroll;             // Scolled amount in cell
+  FXbool        hgrid;                  // Horizontal grid lines shown
+  FXbool        vgrid;                  // Vertical grid lines shown
   FXuchar       mode;                   // Mode we're in
+  FXint         grabx;                  // Grab point x
+  FXint         graby;                  // Grab point y
   FXTimer      *blinker;                // Blink timer
   FXString      help;
 protected:
@@ -162,46 +191,56 @@ protected:
   FXint yOfRow(FXint r) const;
   FXint xOfCol(FXint c) const;
   void drawCursor(FXuint state);
-  FXbool restore(FXint rlo,FXint rhi,FXint clo,FXint chi);
-  static FXint bsearch(FXint *array,FXint l,FXint h,FXint value);
-  FXbool mark(FXint rlo,FXint rhi,FXint clo,FXint chi,FXuint sel);
-  void drawTableRange(FXDC& dc,FXint xlo,FXint xhi,FXint ylo,FXint yhi,FXint xoff,FXint yoff,FXint rlo,FXint rhi,FXint clo,FXint chi);
+  FXRectangle cellRect(FXint r,FXint c) const;
+  virtual void drawFocus(FXDC& dc,FXint x,FXint y,FXint w,FXint h);
+  virtual void drawCell(FXDC& dc,FXint xlo,FXint xhi,FXint ylo,FXint yhi,FXint xoff,FXint yoff,FXint sr,FXint er,FXint sc,FXint ec);
+  virtual void drawRange(FXDC& dc,FXint xlo,FXint xhi,FXint ylo,FXint yhi,FXint xoff,FXint yoff,FXint rlo,FXint rhi,FXint clo,FXint chi);
   virtual FXTableItem* createItem(const FXString& text,FXIcon* icon,void* ptr);
+  FXint startRow(FXint row,FXint col) const;
+  FXint startCol(FXint row,FXint col) const;
+  FXint endRow(FXint row,FXint col) const;
+  FXint endCol(FXint row,FXint col) const;
+protected:
+  enum {
+    MOUSE_NONE,
+    MOUSE_SCROLL,
+    MOUSE_DRAG,
+    MOUSE_SELECT,
+    MOUSE_COL_SELECT,
+    MOUSE_ROW_SELECT,
+    MOUSE_COL_SIZE,
+    MOUSE_ROW_SIZE
+    };
 private:
   FXTable(const FXTable&);
   FXTable& operator=(const FXTable&);
 public:
   long onPaint(FXObject*,FXSelector,void*);
+  long onFocusIn(FXObject*,FXSelector,void*);
+  long onFocusOut(FXObject*,FXSelector,void*);
+  long onMotion(FXObject*,FXSelector,void*);
   long onKeyPress(FXObject*,FXSelector,void*);
   long onKeyRelease(FXObject*,FXSelector,void*);
   long onLeftBtnPress(FXObject*,FXSelector,void*);
   long onLeftBtnRelease(FXObject*,FXSelector,void*);
+  long onRightBtnPress(FXObject*,FXSelector,void*);
+  long onRightBtnRelease(FXObject*,FXSelector,void*);
   long onUngrabbed(FXObject*,FXSelector,void*);
-  long onActivate(FXObject*,FXSelector,void*);
-  long onDeactivate(FXObject*,FXSelector,void*);
-  long onSelected(FXObject*,FXSelector,void*);
-  long onDeselected(FXObject*,FXSelector,void*);
-  long onInserted(FXObject*,FXSelector,void*);
-  long onDeleted(FXObject*,FXSelector,void*);
-  long onFocusIn(FXObject*,FXSelector,void*);
-  long onFocusOut(FXObject*,FXSelector,void*);
   long onBlink(FXObject*,FXSelector,void*);
   long onSelectionLost(FXObject*,FXSelector,void*);
   long onSelectionGained(FXObject*,FXSelector,void*);
   long onAutoScroll(FXObject*,FXSelector,void*);
-  long onMotion(FXObject*,FXSelector,void*);
-  long onChanged(FXObject*,FXSelector,void*);
   long onCommand(FXObject*,FXSelector,void*);
   long onClicked(FXObject*,FXSelector,void*);
   long onDoubleClicked(FXObject*,FXSelector,void*);
   long onTripleClicked(FXObject*,FXSelector,void*);
-  
+
   // Visual characteristics
   long onCmdHorzGrid(FXObject*,FXSelector,void*);
   long onUpdHorzGrid(FXObject*,FXSelector,void*);
   long onCmdVertGrid(FXObject*,FXSelector,void*);
   long onUpdVertGrid(FXObject*,FXSelector,void*);
-  
+
   // Row/Column manipulations
   long onCmdDeleteColumn(FXObject*,FXSelector,void*);
   long onUpdDeleteColumn(FXObject*,FXSelector,void*);
@@ -209,12 +248,18 @@ public:
   long onUpdDeleteRow(FXObject*,FXSelector,void*);
   long onCmdInsertColumn(FXObject*,FXSelector,void*);
   long onCmdInsertRow(FXObject*,FXSelector,void*);
-  
+
   // Movement
   long onCmdMoveRight(FXObject*,FXSelector,void*);
   long onCmdMoveLeft(FXObject*,FXSelector,void*);
   long onCmdMoveUp(FXObject*,FXSelector,void*);
   long onCmdMoveDown(FXObject*,FXSelector,void*);
+  long onCmdMoveHome(FXObject*,FXSelector,void*);
+  long onCmdMoveEnd(FXObject*,FXSelector,void*);
+  long onCmdMoveTop(FXObject*,FXSelector,void*);
+  long onCmdMoveBottom(FXObject*,FXSelector,void*);
+  long onCmdMovePageDown(FXObject*,FXSelector,void*);
+  long onCmdMovePageUp(FXObject*,FXSelector,void*);
 
   // Mark and extend
   long onCmdMark(FXObject*,FXSelector,void*);
@@ -226,9 +271,9 @@ public:
   long onCmdSelectColumn(FXObject*,FXSelector,void*);
   long onCmdSelectAll(FXObject*,FXSelector,void*);
   long onCmdDeselectAll(FXObject*,FXSelector,void*);
+
 public:
 
-  // Messages
   enum {
     ID_HORZ_GRID=FXScrollArea::ID_LAST,
     ID_VERT_GRID,
@@ -245,13 +290,22 @@ public:
     ID_MOVE_RIGHT,
     ID_MOVE_UP,
     ID_MOVE_DOWN,
+    ID_MOVE_HOME,
+    ID_MOVE_END,
+    ID_MOVE_TOP,
+    ID_MOVE_BOTTOM,
+    ID_MOVE_PAGEDOWN,
+    ID_MOVE_PAGEUP,
     ID_MARK,
     ID_EXTEND,
+    ID_CUT_SEL,
+    ID_COPY_SEL,
+    ID_PASTE_SEL,
     ID_LAST
     };
 
 public:
-  
+
   /// Make new table with nr visible rows and nc visible columns; the table
   /// is initially empty, i.e. contains no cells (nrows=0, ncols=0)
   FXTable(FXComposite *p,FXint nr,FXint nc,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_MARGIN,FXint pr=DEFAULT_MARGIN,FXint pt=DEFAULT_MARGIN,FXint pb=DEFAULT_MARGIN);
@@ -259,28 +313,40 @@ public:
   virtual void detach();
   virtual void recalc();
   virtual FXbool canFocus() const;
-  
+
   /// Change visible rows/columns
   void setVisibleRows(FXint nvrows);
   FXint getVisibleRows() const { return visiblerows; }
   void setVisibleCols(FXint nvcols);
   FXint getVisibleCols() const { return visiblecols; }
-  
+
+  /// Is horizontal grid shown
+  FXbool isHorzGridShown() const { return hgrid; }
+
+  /// Is vertical grid shown
+  FXbool isVertGridShown() const { return vgrid; }
+
+  /// Show or hide horizontal grid
+  void showHorzGrid(FXbool on=TRUE);
+
+  /// Show or hide vertical grid
+  void showVertGrid(FXbool on=TRUE);
+
   /// Report default size, which is determined based on the
   /// visible rows/columns and the default width/height
   virtual FXint getDefaultWidth();
   virtual FXint getDefaultHeight();
-  
+
   /// Compute content size
   virtual FXint getContentWidth();
   virtual FXint getContentHeight();
-  
+
   /// Scroll contents
   virtual void moveContents(FXint x,FXint y);
-  
+
   /// Resize the table content to nr rows and nc columns
-  void setTableSize(FXint nr,FXint nc);
-  
+  void setTableSize(FXint nr,FXint nc,FXbool notify=FALSE);
+
   /// Get number of rows
   FXint getNumRows() const { return nrows; }
 
@@ -300,38 +366,53 @@ public:
   /// Change table style
   FXuint getTableStyle() const;
   void setTableStyle(FXuint style);
-  
-  /// Change leading/trailing fixed rows/columns
-  void setLeadingRows(FXint leadrows);
+
+  /// Get/set leading rows
+  virtual void setLeadingRows(FXint leadrows);
   FXint getLeadingRows() const { return leading_rows; }
-  void setLeadingCols(FXint leadcols);
+
+  /// Get/set leading columns
+  virtual void setLeadingCols(FXint leadcols);
   FXint getLeadingCols() const { return leading_cols; }
-  void setTrailingRows(FXint trailrows);
+
+  /// Get/set trailing rows
+  virtual void setTrailingRows(FXint trailrows);
   FXint getTrailingRows() const { return trailing_rows; }
-  void setTrailingCols(FXint trailcols);
+
+  /// Get/set trailing columns
+  virtual void setTrailingCols(FXint trailcols);
   FXint getTrailingCols() const { return trailing_cols; }
 
   /// Determine row containing y; returns -1 if y outside of table
   FXint rowAtY(FXint y) const;
-  
+
   /// Determine column containing x; returns -1 if x outside of table
   FXint colAtX(FXint x) const;
-  
+
+  /// Return the item at the given index
+  FXTableItem *getItem(FXint row,FXint col) const;
+
+  /// Replace the item with a [possibly subclassed] item
+  void setItem(FXint row,FXint col,FXTableItem* item);
+
   /// Insert new row
-  void insertRows(FXint row,FXint nr=1);
-  
+  virtual void insertRows(FXint row,FXint nr=1,FXbool notify=FALSE);
+
   /// Insert new column
-  void insertColumns(FXint col,FXint nc=1);
-  
-  /// Delete row
-  void deleteRows(FXint row,FXint nr=1);
-  
-  /// Delete column
-  void deleteColumns(FXint col,FXint nc=1);
-  
+  virtual void insertColumns(FXint col,FXint nc=1,FXbool notify=FALSE);
+
+  /// Remove rows of cells
+  virtual void removeRows(FXint row,FXint nr=1,FXbool notify=FALSE);
+
+  /// Remove column of cells
+  virtual void removeColumns(FXint col,FXint nc=1,FXbool notify=FALSE);
+
+  /// Remove single cell
+  virtual void removeItem(FXint row,FXint col,FXbool notify=FALSE);
+
   /// Scroll to make cell at r,c fully visible
   void makePositionVisible(FXint r,FXint c);
-  
+
   /// Change column width
   void setColumnWidth(FXint col,FXint cwidth);
   FXint getColumnWidth(FXint col) const;
@@ -339,7 +420,7 @@ public:
   /// Change row height
   void setRowHeight(FXint row,FXint rheight);
   FXint getRowHeight(FXint row) const;
-  
+
   /// Change default column width
   void setDefColumnWidth(FXint cwidth);
   FXint getDefColumnWidth() const { return defColumnWidth; }
@@ -351,33 +432,36 @@ public:
   /// Modify cell text
   void setItemText(FXint r,FXint c,const FXString& text);
   FXString getItemText(FXint r,FXint c) const;
-  
+
   /// Modify cell icon
   void setItemIcon(FXint r,FXint c,FXIcon* icon);
   FXIcon* getItemIcon(FXint r,FXint c) const;
-  
+
   /// Modify cell user-data
   void setItemData(FXint r,FXint c,void* ptr);
   void* getItemData(FXint r,FXint c) const;
-  
+
   /// Is cell selected, current, visible, enabled
   FXbool isItemSelected(FXint r,FXint c) const;
   FXbool isItemCurrent(FXint r,FXint c) const;
   FXbool isItemVisible(FXint r,FXint c) const;
   FXbool isItemEnabled(FXint r,FXint c) const;
-  
-  /// Repaint cell
+
+  /// Repaint cells between grid lines sr,er and grid lines sc,ec
+  void updateRange(FXint sr,FXint er,FXint sc,FXint ec);
+
+  /// Repaint cell at r,c
   void updateItem(FXint r,FXint c);
   
   /// Enable, disable, select, deselect, toggle cell
   FXbool enableItem(FXint r,FXint c);
   FXbool disableItem(FXint r,FXint c);
-  FXbool selectItem(FXint r,FXint c);
-  FXbool deselectItem(FXint r,FXint c);
-  FXbool toggleItem(FXint r,FXint c);
-  
+  FXbool selectItem(FXint r,FXint c,FXbool notify=FALSE);
+  FXbool deselectItem(FXint r,FXint c,FXbool notify=FALSE);
+  FXbool toggleItem(FXint r,FXint c,FXbool notify=FALSE);
+
   /// Change current cell
-  void setCurrentItem(FXint r,FXint c);
+  void setCurrentItem(FXint r,FXint c,FXbool notify=FALSE);
   FXint getCurrentRow() const { return current.row; }
   FXint getCurrentColumn() const { return current.col; }
 
@@ -386,36 +470,53 @@ public:
   FXint getAnchorRow() const { return anchor.row; }
   FXint getAnchorColumn() const { return anchor.col; }
 
-  /// Extend/kill selection
-  FXbool extendSelection(FXint r,FXint c);
-  FXbool killSelection();
-  
+  /// Select range
+  FXbool selectRange(FXint sr,FXint er,FXint sc,FXint ec,FXbool notify=FALSE);
+
+  /// Extend selection
+  FXbool extendSelection(FXint r,FXint c,FXbool notify=FALSE);
+
+  /// Kill selection
+  FXbool killSelection(FXbool notify=FALSE);
+
   /// Change font
   void setFont(FXFont* fnt);
   FXFont* getFont() const { return font; }
 
-  /// Change colors of various aspects
-  void setTextColor(FXColor clr);
+  /// Obtain colors of various parts
   FXColor getTextColor() const { return textColor; }
-  void setBaseColor(FXColor clr);
   FXColor getBaseColor() const { return baseColor; }
-  void setHiliteColor(FXColor clr);
   FXColor getHiliteColor() const { return hiliteColor; }
-  void setShadowColor(FXColor clr);
   FXColor getShadowColor() const { return shadowColor; }
-  void setBorderColor(FXColor clr);
   FXColor getBorderColor() const { return borderColor; }
-  void setSelBackColor(FXColor clr);
   FXColor getSelBackColor() const { return selbackColor; }
-  void setSelTextColor(FXColor clr);
   FXColor getSelTextColor() const { return seltextColor; }
-  void setGridColor(FXColor clr);
   FXColor getGridColor() const { return gridColor; }
+  FXColor getCellBorderColor() const { return cellBorderColor; }
+
+  /// Change colors of various parts
+  void setTextColor(FXColor clr);
+  void setBaseColor(FXColor clr);
+  void setHiliteColor(FXColor clr);
+  void setShadowColor(FXColor clr);
+  void setBorderColor(FXColor clr);
+  void setSelBackColor(FXColor clr);
+  void setSelTextColor(FXColor clr);
+  void setGridColor(FXColor clr);
+  void setCellBorderColor(FXColor clr);
 
   /// Change cell background color for even/odd rows/columns
   void setCellColor(FXint r,FXint c,FXColor clr);
+
+  /// Obtain cell background color for even/odd rows/columns
   FXColor getCellColor(FXint r,FXint c) const;
-  
+
+  /// Change cell border width
+  void setCellBorderWidth(FXint borderwidth);
+
+  /// Return cell border width
+  FXint getCellBorderWidth() const { return cellBorderWidth; }
+
   /// Change help text
   void setHelpText(const FXString& text);
   FXString getHelpText() const { return help; }
@@ -423,7 +524,7 @@ public:
   /// Serialize
   virtual void save(FXStream& store) const;
   virtual void load(FXStream& store);
-  
+
   virtual ~FXTable();
   };
 
