@@ -3,29 +3,29 @@
 *      A d d i t i o n a l   F O X   I n c l u d e   F i l e   F o r   3 D      *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: fx3d.h,v 1.1 1999/09/14 13:32:48 jeroen Exp $                           *
+* $Id: fx3d.h,v 1.4 2001/01/25 19:48:27 jeroen Exp $                            *
 ********************************************************************************/
 #ifndef FX3D_H
 #define FX3D_H
 
 // OpenGL includes
-#ifdef FX_NATIVE_WIN32
+#ifdef WIN32
 #ifndef STRICT
 #define STRICT
 #endif
@@ -47,6 +47,7 @@
 #include "FXDQuat.h"
 #include "FXDHMat.h"
 #include "FXGLVisual.h"
+#include "FXGLContext.h"
 #include "FXGLCanvas.h"
 #include "FXGLViewer.h"
 #include "FXGLObject.h"
