@@ -1,9 +1,18 @@
 #ifndef FXVER_H
 #define FXVER_H
 
+
 // FOX version
 #define FOX_MAJOR  0
 #define FOX_MINOR  99
-#define FOX_LEVEL  87
+#define FOX_LEVEL  175
+
+
+
+// FOX byte order
+#ifndef FOX_BIGENDIAN
+#define FOX_BIGENDIAN 0
+#endif
+
 
 #endif
