@@ -3,32 +3,38 @@
 *                          C o l o r   S e l e c t o r                          *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXColorSelector.cpp,v 1.9 1999/11/12 22:07:18 jeroen Exp $               *
+* $Id: FXColorSelector.cpp,v 1.27 2001/08/29 21:43:16 jeroen Exp $              *
 ********************************************************************************/
 #include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "fxkeys.h"
+#include "fxpriv.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXObjectList.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXApp.h"
@@ -55,7 +61,7 @@
 #include "FXShell.h"
 #include "FXScrollbar.h"
 #include "FXSlider.h"
-#include "FXScrollWindow.h"
+#include "FXScrollArea.h"
 #include "FXList.h"
 #include "FXColorSelector.h"
 
@@ -64,9 +70,9 @@
   Notes:
   - Custom colors in the twentyfour wells are saved to registry for next time you
     run the program.
-  
+
   - HSV Model:
-  
+
       green          yellow
             o-----o
            /       \
@@ -76,49 +82,56 @@
            \       /
             o-----o
       blue           magenta
-      
+
   - Perhaps this panel should send color change messages to active
     colorwell widget?
-    
+
   - Still need a way to pick a color from the screen:- look at WindowMaker.
 */
 
 /*******************************************************************************/
 
-  
+
 // Map
 FXDEFMAP(FXColorSelector) FXColorSelectorMap[]={
   FXMAPFUNC(SEL_CHANGED,FXColorSelector::ID_COLOR,FXColorSelector::onChgColor),
   FXMAPFUNC(SEL_COMMAND,FXColorSelector::ID_COLOR,FXColorSelector::onCmdColor),
-  FXMAPFUNCS(SEL_UPDATE,FXColorSelector::ID_RGB_RED_TEXT,FXColorSelector::ID_RGB_ALPHA_TEXT,FXColorSelector::onUpdRGBText),
-  FXMAPFUNCS(SEL_UPDATE,FXColorSelector::ID_HSV_HUE_TEXT,FXColorSelector::ID_HSV_ALPHA_TEXT,FXColorSelector::onUpdHSVText),
-  FXMAPFUNCS(SEL_UPDATE,FXColorSelector::ID_CMY_CYAN_TEXT,FXColorSelector::ID_CMY_ALPHA_TEXT,FXColorSelector::onUpdCMYText),
-  FXMAPFUNCS(SEL_UPDATE,FXColorSelector::ID_RGB_RED_SLIDER,FXColorSelector::ID_RGB_ALPHA_SLIDER,FXColorSelector::onUpdRGBSlider),
-  FXMAPFUNCS(SEL_UPDATE,FXColorSelector::ID_HSV_HUE_SLIDER,FXColorSelector::ID_HSV_ALPHA_SLIDER,FXColorSelector::onUpdHSVSlider),
-  FXMAPFUNCS(SEL_UPDATE,FXColorSelector::ID_CMY_CYAN_SLIDER,FXColorSelector::ID_CMY_ALPHA_SLIDER,FXColorSelector::onUpdCMYSlider),
+  FXMAPFUNC(SEL_UPDATE,FXColorSelector::ID_ALPHA_SLIDER,FXColorSelector::onUpdAlphaSlider),
+  FXMAPFUNC(SEL_UPDATE,FXColorSelector::ID_ALPHA_TEXT,FXColorSelector::onUpdAlphaText),
+  FXMAPFUNC(SEL_UPDATE,FXColorSelector::ID_ALPHA_LABEL,FXColorSelector::onUpdAlphaLabel),
+  FXMAPFUNC(SEL_CHANGED,FXColorSelector::ID_ALPHA_SLIDER,FXColorSelector::onCmdAlphaSlider),
+  FXMAPFUNC(SEL_COMMAND,FXColorSelector::ID_ALPHA_SLIDER,FXColorSelector::onCmdAlphaSlider),
+  FXMAPFUNC(SEL_COMMAND,FXColorSelector::ID_ALPHA_TEXT,FXColorSelector::onCmdAlphaText),
+  FXMAPFUNCS(SEL_UPDATE,FXColorSelector::ID_RGB_RED_TEXT,FXColorSelector::ID_RGB_BLUE_TEXT,FXColorSelector::onUpdRGBText),
+  FXMAPFUNCS(SEL_UPDATE,FXColorSelector::ID_HSV_HUE_TEXT,FXColorSelector::ID_HSV_VALUE_TEXT,FXColorSelector::onUpdHSVText),
+  FXMAPFUNCS(SEL_UPDATE,FXColorSelector::ID_CMY_CYAN_TEXT,FXColorSelector::ID_CMY_YELLOW_TEXT,FXColorSelector::onUpdCMYText),
+  FXMAPFUNCS(SEL_COMMAND,FXColorSelector::ID_RGB_RED_TEXT,FXColorSelector::ID_RGB_BLUE_TEXT,FXColorSelector::onCmdRGBText),
+  FXMAPFUNCS(SEL_COMMAND,FXColorSelector::ID_HSV_HUE_TEXT,FXColorSelector::ID_HSV_VALUE_TEXT,FXColorSelector::onCmdHSVText),
+  FXMAPFUNCS(SEL_COMMAND,FXColorSelector::ID_CMY_CYAN_TEXT,FXColorSelector::ID_CMY_YELLOW_TEXT,FXColorSelector::onCmdCMYText),
+  FXMAPFUNCS(SEL_UPDATE,FXColorSelector::ID_RGB_RED_SLIDER,FXColorSelector::ID_RGB_BLUE_SLIDER,FXColorSelector::onUpdRGBSlider),
+  FXMAPFUNCS(SEL_UPDATE,FXColorSelector::ID_HSV_HUE_SLIDER,FXColorSelector::ID_HSV_VALUE_SLIDER,FXColorSelector::onUpdHSVSlider),
+  FXMAPFUNCS(SEL_UPDATE,FXColorSelector::ID_CMY_CYAN_SLIDER,FXColorSelector::ID_CMY_YELLOW_SLIDER,FXColorSelector::onUpdCMYSlider),
+  FXMAPFUNCS(SEL_CHANGED,FXColorSelector::ID_RGB_RED_SLIDER,FXColorSelector::ID_RGB_BLUE_SLIDER,FXColorSelector::onCmdRGBSlider),
+  FXMAPFUNCS(SEL_COMMAND,FXColorSelector::ID_RGB_RED_SLIDER,FXColorSelector::ID_RGB_BLUE_SLIDER,FXColorSelector::onCmdRGBSlider),
+  FXMAPFUNCS(SEL_CHANGED,FXColorSelector::ID_HSV_HUE_SLIDER,FXColorSelector::ID_HSV_VALUE_SLIDER,FXColorSelector::onCmdHSVSlider),
+  FXMAPFUNCS(SEL_COMMAND,FXColorSelector::ID_HSV_HUE_SLIDER,FXColorSelector::ID_HSV_VALUE_SLIDER,FXColorSelector::onCmdHSVSlider),
+  FXMAPFUNCS(SEL_CHANGED,FXColorSelector::ID_CMY_CYAN_SLIDER,FXColorSelector::ID_CMY_YELLOW_SLIDER,FXColorSelector::onCmdCMYSlider),
+  FXMAPFUNCS(SEL_COMMAND,FXColorSelector::ID_CMY_CYAN_SLIDER,FXColorSelector::ID_CMY_YELLOW_SLIDER,FXColorSelector::onCmdCMYSlider),
   FXMAPFUNC(SEL_COMMAND,FXColorSelector::ID_WELL_CHANGED,FXColorSelector::onCmdWell),
   FXMAPFUNC(SEL_CHANGED,FXColorSelector::ID_WELL_CHANGED,FXColorSelector::onChgWell),
-  FXMAPFUNCS(SEL_CHANGED,FXColorSelector::ID_RGB_RED_SLIDER,FXColorSelector::ID_RGB_ALPHA_SLIDER,FXColorSelector::onCmdRGBSlider),
-  FXMAPFUNCS(SEL_COMMAND,FXColorSelector::ID_RGB_RED_SLIDER,FXColorSelector::ID_RGB_ALPHA_SLIDER,FXColorSelector::onCmdRGBSlider),
-  FXMAPFUNCS(SEL_CHANGED,FXColorSelector::ID_HSV_HUE_SLIDER,FXColorSelector::ID_HSV_ALPHA_SLIDER,FXColorSelector::onCmdHSVSlider),
-  FXMAPFUNCS(SEL_COMMAND,FXColorSelector::ID_HSV_HUE_SLIDER,FXColorSelector::ID_HSV_ALPHA_SLIDER,FXColorSelector::onCmdHSVSlider),
-  FXMAPFUNCS(SEL_CHANGED,FXColorSelector::ID_CMY_CYAN_SLIDER,FXColorSelector::ID_CMY_ALPHA_SLIDER,FXColorSelector::onCmdCMYSlider),
-  FXMAPFUNCS(SEL_COMMAND,FXColorSelector::ID_CMY_CYAN_SLIDER,FXColorSelector::ID_CMY_ALPHA_SLIDER,FXColorSelector::onCmdCMYSlider),
-  FXMAPFUNCS(SEL_COMMAND,FXColorSelector::ID_RGB_RED_TEXT,FXColorSelector::ID_RGB_ALPHA_TEXT,FXColorSelector::onCmdRGBText),
-  FXMAPFUNCS(SEL_COMMAND,FXColorSelector::ID_HSV_HUE_TEXT,FXColorSelector::ID_HSV_ALPHA_TEXT,FXColorSelector::onCmdHSVText),
-  FXMAPFUNCS(SEL_COMMAND,FXColorSelector::ID_CMY_CYAN_TEXT,FXColorSelector::ID_CMY_ALPHA_TEXT,FXColorSelector::onCmdCMYText),
   FXMAPFUNCS(SEL_COMMAND,FXColorSelector::ID_CUSTOM_FIRST,FXColorSelector::ID_CUSTOM_LAST,FXColorSelector::onCmdCustomWell),
   FXMAPFUNCS(SEL_CHANGED,FXColorSelector::ID_CUSTOM_FIRST,FXColorSelector::ID_CUSTOM_LAST,FXColorSelector::onChgCustomWell),
   FXMAPFUNC(SEL_COMMAND,FXColorSelector::ID_COLOR_LIST,FXColorSelector::onCmdList),
   FXMAPFUNC(SEL_COMMAND,FXWindow::ID_SETVALUE,FXColorSelector::onCmdSetValue),
   FXMAPFUNC(SEL_COMMAND,FXColorSelector::ID_ACTIVEPANE,FXColorSelector::onCmdActivePane),
+
   };
 
 
 // Object implementation
 FXIMPLEMENT(FXColorSelector,FXPacker,FXColorSelectorMap,ARRAYNUMBER(FXColorSelectorMap))
 
-  
+
 // Well names
 const FXchar* FXColorSelector::wellname[24]={
   "wella","wellb","wellc","welld",
@@ -407,21 +420,22 @@ const unsigned char rgbmode[]={
 
 /*******************************************************************************/
 
-  
+
 // Separator item
 FXColorSelector::FXColorSelector(FXComposite *p,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h):
   FXPacker(p,opts,x,y,w,h){
+  FXLabel *label;
   target=tgt;
   message=sel;
-  
+
   // Buttons
   FXHorizontalFrame *buttons=new FXHorizontalFrame(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH);
   accept=new FXButton(buttons,"&Accept",NULL,NULL,0,BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT,0,0,0,0,20,20);
   cancel=new FXButton(buttons,"&Cancel",NULL,NULL,0,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT,0,0,0,0,20,20);
-  
+
   // Separator
   new FXHorizontalSeparator(this,SEPARATOR_RIDGE|LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X);
-  
+
   // Icons
   rgbmodeicon=new FXGIFIcon(getApp(),rgbmode);
   hsvmodeicon=new FXGIFIcon(getApp(),hsvmode);
@@ -434,38 +448,38 @@ FXColorSelector::FXColorSelector(FXComposite *p,FXObject* tgt,FXSelector sel,FXu
   colorwells[1]=new FXColorWell(colors,FXRGBA(  0,  0,  0,255),this,ID_CUSTOM_FIRST+1,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[2]=new FXColorWell(colors,FXRGBA(255,  0,  0,255),this,ID_CUSTOM_FIRST+2,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[3]=new FXColorWell(colors,FXRGBA(  0,255,  0,255),this,ID_CUSTOM_FIRST+3,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
-  
+
   colorwells[4]=new FXColorWell(colors,FXRGBA(  0,  0,255,255),this,ID_CUSTOM_FIRST+4,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[5]=new FXColorWell(colors,FXRGBA(  0,  0,255,255),this,ID_CUSTOM_FIRST+5,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[6]=new FXColorWell(colors,FXRGBA(255,255,  0,255),this,ID_CUSTOM_FIRST+6,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[7]=new FXColorWell(colors,FXRGBA(  0,255,255,255),this,ID_CUSTOM_FIRST+7,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
-  
+
   colorwells[8]=new FXColorWell(colors,FXRGBA(255,  0,255,255),this,ID_CUSTOM_FIRST+8,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[9]=new FXColorWell(colors,FXRGBA(128,  0,  0,255),this,ID_CUSTOM_FIRST+9,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[10]=new FXColorWell(colors,FXRGBA(  0,128,  0,255),this,ID_CUSTOM_FIRST+10,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[11]=new FXColorWell(colors,FXRGBA(  0,  0,128,255),this,ID_CUSTOM_FIRST+11,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
-  
+
   colorwells[12]=new FXColorWell(colors,FXRGBA(128,128,  0,255),this,ID_CUSTOM_FIRST+12,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[13]=new FXColorWell(colors,FXRGBA(128,  0,128,255),this,ID_CUSTOM_FIRST+13,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[14]=new FXColorWell(colors,FXRGBA(  0,128,128,255),this,ID_CUSTOM_FIRST+14,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[15]=new FXColorWell(colors,FXRGBA(  0,128,128,255),this,ID_CUSTOM_FIRST+15,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
-  
+
   colorwells[16]=new FXColorWell(colors,FXRGBA(255,  0,255,255),this,ID_CUSTOM_FIRST+16,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[17]=new FXColorWell(colors,FXRGBA(128,  0,  0,255),this,ID_CUSTOM_FIRST+17,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[18]=new FXColorWell(colors,FXRGBA(  0,128,  0,255),this,ID_CUSTOM_FIRST+18,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[19]=new FXColorWell(colors,FXRGBA(  0,  0,128,255),this,ID_CUSTOM_FIRST+19,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
-  
+
   colorwells[20]=new FXColorWell(colors,FXRGBA(128,128,  0,255),this,ID_CUSTOM_FIRST+20,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[21]=new FXColorWell(colors,FXRGBA(128,  0,128,255),this,ID_CUSTOM_FIRST+21,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[22]=new FXColorWell(colors,FXRGBA(  0,128,128,255),this,ID_CUSTOM_FIRST+22,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
   colorwells[23]=new FXColorWell(colors,FXRGBA(  0,128,128,255),this,ID_CUSTOM_FIRST+23,COLORWELL_SOURCEONLY|LAYOUT_CENTER_Y|LAYOUT_CENTER_X, 0,0,0,0 ,0,0,0,0);
-    
+
   // Main part
   FXHorizontalFrame *main=new FXHorizontalFrame(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
 
   // Main color well
   well=new FXColorWell(main,FXRGBA(255,255,255,255),this,ID_WELL_CHANGED,COLORWELL_SOURCEONLY|LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FILL_Y|LAYOUT_FIX_WIDTH, 0,0,64,0 ,0,0,0,0);
-  
+
   // Tab book with switchable panels
   panels=new FXTabBook(main,this,ID_ACTIVEPANE,TABBOOK_TOPTABS|LAYOUT_FILL_Y|LAYOUT_FILL_X);
 
@@ -477,30 +491,33 @@ FXColorSelector::FXColorSelector(FXComposite *p,FXObject* tgt,FXSelector sel,FXu
 
     // Red
     new FXLabel(rgbblock,"&Red:",NULL,LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
-    rgbatext[0]=new FXTextField(rgbblock,3,this,FXColorSelector::ID_RGB_RED_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
+    rgbatext[0]=new FXTextField(rgbblock,5,this,FXColorSelector::ID_RGB_RED_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
     rgbaslider[0]=new FXSlider(rgbblock,this,FXColorSelector::ID_RGB_RED_SLIDER,LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR,0,0,0,15);
 
     // Green slider
     new FXLabel(rgbblock,"&Green:",NULL,LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
-    rgbatext[1]=new FXTextField(rgbblock,3,this,FXColorSelector::ID_RGB_GREEN_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
+    rgbatext[1]=new FXTextField(rgbblock,5,this,FXColorSelector::ID_RGB_GREEN_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
     rgbaslider[1]=new FXSlider(rgbblock,this,FXColorSelector::ID_RGB_GREEN_SLIDER,LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR,0,0,0,15);
 
-    // Blue slider  
+    // Blue slider
     new FXLabel(rgbblock,"&Blue:",NULL,LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
-    rgbatext[2]=new FXTextField(rgbblock,3,this,FXColorSelector::ID_RGB_BLUE_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
+    rgbatext[2]=new FXTextField(rgbblock,5,this,FXColorSelector::ID_RGB_BLUE_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
     rgbaslider[2]=new FXSlider(rgbblock,this,FXColorSelector::ID_RGB_BLUE_SLIDER,LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR,0,0,0,15);
 
     // Alpha slider
-    new FXLabel(rgbblock,"&Alpha:",NULL,LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
-    rgbatext[3]=new FXTextField(rgbblock,3,this,FXColorSelector::ID_RGB_ALPHA_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
-    rgbaslider[3]=new FXSlider(rgbblock,this,FXColorSelector::ID_RGB_ALPHA_SLIDER,LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR,0,0,0,15);
+    label=new FXLabel(rgbblock,"&Alpha:",NULL,LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
+    rgbatext[3]=new FXTextField(rgbblock,5,this,FXColorSelector::ID_ALPHA_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
+    rgbaslider[3]=new FXSlider(rgbblock,this,FXColorSelector::ID_ALPHA_SLIDER,LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR,0,0,0,15);
+
+    label->setTarget(this);
+    label->setSelector(ID_ALPHA_LABEL);
 
     // Set ranges and increment
     rgbaslider[0]->setRange(0,255);
     rgbaslider[1]->setRange(0,255);
     rgbaslider[2]->setRange(0,255);
     rgbaslider[3]->setRange(0,255);
-    
+
   // HSV Mode
   new FXTabItem(panels,"\tHue, Saturation, Value",hsvmodeicon,TAB_TOP_NORMAL,0,0,0,0, 6,6,0,0);
 
@@ -509,30 +526,33 @@ FXColorSelector::FXColorSelector(FXComposite *p,FXObject* tgt,FXSelector sel,FXu
 
     // Hue Slider
     new FXLabel(hsvblock,"Hue:",NULL,LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
-    hsvatext[0]=new FXTextField(hsvblock,3,this,FXColorSelector::ID_HSV_HUE_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
+    hsvatext[0]=new FXTextField(hsvblock,5,this,FXColorSelector::ID_HSV_HUE_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
     hsvaslider[0]=new FXSlider(hsvblock,this,FXColorSelector::ID_HSV_HUE_SLIDER,LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR,0,0,0,15);
 
     // Saturation slider
     new FXLabel(hsvblock,"Saturation:",NULL,LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
-    hsvatext[1]=new FXTextField(hsvblock,3,this,FXColorSelector::ID_HSV_SATURATION_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
+    hsvatext[1]=new FXTextField(hsvblock,5,this,FXColorSelector::ID_HSV_SATURATION_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
     hsvaslider[1]=new FXSlider(hsvblock,this,FXColorSelector::ID_HSV_SATURATION_SLIDER,LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR,0,0,0,15);
 
-    // Value slider  
+    // Value slider
     new FXLabel(hsvblock,"Value:",NULL,LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
-    hsvatext[2]=new FXTextField(hsvblock,3,this,FXColorSelector::ID_HSV_VALUE_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
+    hsvatext[2]=new FXTextField(hsvblock,5,this,FXColorSelector::ID_HSV_VALUE_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
     hsvaslider[2]=new FXSlider(hsvblock,this,FXColorSelector::ID_HSV_VALUE_SLIDER,LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR,0,0,0,15);
 
     // Alpha slider
-    new FXLabel(hsvblock,"Alpha:",NULL,LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
-    hsvatext[3]=new FXTextField(hsvblock,3,this,FXColorSelector::ID_HSV_ALPHA_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
-    hsvaslider[3]=new FXSlider(hsvblock,this,FXColorSelector::ID_HSV_ALPHA_SLIDER,LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR,0,0,0,15);
+    label=new FXLabel(hsvblock,"Alpha:",NULL,LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
+    hsvatext[3]=new FXTextField(hsvblock,5,this,FXColorSelector::ID_ALPHA_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
+    hsvaslider[3]=new FXSlider(hsvblock,this,FXColorSelector::ID_ALPHA_SLIDER,LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR,0,0,0,15);
+
+    label->setTarget(this);
+    label->setSelector(ID_ALPHA_LABEL);
 
     // Set ranges and increment
     hsvaslider[0]->setRange(0,360);
     hsvaslider[1]->setRange(0,1000);
     hsvaslider[2]->setRange(0,1000);
     hsvaslider[3]->setRange(0,255);
-    
+
   // CMY Mode
   new FXTabItem(panels,"\tCyan, Magenta, Yellow",cmymodeicon,TAB_TOP_NORMAL,0,0,0,0, 6,6,0,0);
 
@@ -541,64 +561,69 @@ FXColorSelector::FXColorSelector(FXComposite *p,FXObject* tgt,FXSelector sel,FXu
 
     // Cyan Slider
     new FXLabel(cmyblock,"Cyan:",NULL,LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
-    cmytext[0]=new FXTextField(cmyblock,3,this,FXColorSelector::ID_CMY_CYAN_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
+    cmytext[0]=new FXTextField(cmyblock,5,this,FXColorSelector::ID_CMY_CYAN_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
     cmyslider[0]=new FXSlider(cmyblock,this,FXColorSelector::ID_CMY_CYAN_SLIDER,LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR,0,0,0,15);
 
     // Magenta slider
     new FXLabel(cmyblock,"Magenta:",NULL,LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
-    cmytext[1]=new FXTextField(cmyblock,3,this,FXColorSelector::ID_CMY_MAGENTA_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
+    cmytext[1]=new FXTextField(cmyblock,5,this,FXColorSelector::ID_CMY_MAGENTA_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
     cmyslider[1]=new FXSlider(cmyblock,this,FXColorSelector::ID_CMY_MAGENTA_SLIDER,LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR,0,0,0,15);
 
-    // Yellow slider  
+    // Yellow slider
     new FXLabel(cmyblock,"Yellow:",NULL,LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
-    cmytext[2]=new FXTextField(cmyblock,3,this,FXColorSelector::ID_CMY_YELLOW_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
+    cmytext[2]=new FXTextField(cmyblock,5,this,FXColorSelector::ID_CMY_YELLOW_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
     cmyslider[2]=new FXSlider(cmyblock,this,FXColorSelector::ID_CMY_YELLOW_SLIDER,LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR,0,0,0,15);
 
     // Alpha slider
-    new FXLabel(cmyblock,"Alpha:",NULL,LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
-    cmytext[3]=new FXTextField(cmyblock,3,this,FXColorSelector::ID_CMY_ALPHA_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
-    cmyslider[3]=new FXSlider(cmyblock,this,FXColorSelector::ID_CMY_ALPHA_SLIDER,LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR,0,0,0,15);
+    label=new FXLabel(cmyblock,"Alpha:",NULL,LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
+    cmytext[3]=new FXTextField(cmyblock,5,this,FXColorSelector::ID_ALPHA_TEXT,JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, DEFAULT_PAD,DEFAULT_PAD,0,0);
+    cmyslider[3]=new FXSlider(cmyblock,this,FXColorSelector::ID_ALPHA_SLIDER,LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR,0,0,0,15);
+
+    label->setTarget(this);
+    label->setSelector(ID_ALPHA_LABEL);
 
     // Set ranges and increment
     cmyslider[0]->setRange(0,255);
     cmyslider[1]->setRange(0,255);
     cmyslider[2]->setRange(0,255);
     cmyslider[3]->setRange(0,255);
-    
+
   // Named Color Mode
   new FXTabItem(panels,"\tBy Name",txtmodeicon,TAB_TOP_NORMAL,0,0,0,0, 6,6,0,0);
-  
+
     // Name list
     FXHorizontalFrame *outer=new FXHorizontalFrame(panels,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_Y|LAYOUT_FILL_X);
     FXHorizontalFrame *frame=new FXHorizontalFrame(outer,LAYOUT_FILL_Y|LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, 0,0,0,0);
     list=new FXList(frame,6,this,ID_COLOR_LIST,LAYOUT_FILL_Y|LAYOUT_FILL_X|LIST_BROWSESELECT);
-    
+
     // Add color names
-    for(FXuint i=0; i<fxnumcolornames; i++){ 
+    for(FXuint i=0; i<fxnumcolornames; i++){
       list->appendItem(fxcolornames[i].name);
       }
-    
+
   // Init RGBA
   rgba[0]=0.0;
   rgba[1]=0.0;
   rgba[2]=0.0;
   rgba[3]=1.0;
-  
+
   // Init HSVA
   hsva[0]=360.0;
   hsva[1]=0.0;
   hsva[2]=0.0;
   hsva[3]=1.0;
-  
+
   // Reflect color in well
   updateWell();
+
+ // setOpaqueOnly(TRUE);
   }
 
 
 // Init the panel
 void FXColorSelector::create(){
   FXPacker::create();
-  
+
   // Get custom well colors from defaults database
   colorwells[0]->setRGBA(getApp()->reg().readColorEntry("SETTINGS",wellname[0],FXRGBA(255,255,255,255)));
   colorwells[1]->setRGBA(getApp()->reg().readColorEntry("SETTINGS",wellname[1],FXRGBA(  0,  0,  0,255)));
@@ -624,10 +649,76 @@ void FXColorSelector::create(){
   colorwells[21]->setRGBA(getApp()->reg().readColorEntry("SETTINGS",wellname[21],FXRGBA(128,  0,128,255)));
   colorwells[22]->setRGBA(getApp()->reg().readColorEntry("SETTINGS",wellname[22],FXRGBA(  0,128,128,255)));
   colorwells[23]->setRGBA(getApp()->reg().readColorEntry("SETTINGS",wellname[23],FXRGBA(  0,128,128,255)));
-  
+
   // Switch to correct pane
   panels->setCurrent(getApp()->reg().readIntEntry("SETTINGS","activecolorpane",0));
-  
+
+  }
+
+
+/*******************************************************************************/
+
+// ALPHA
+
+// Update well from Alpha slider
+long FXColorSelector::onCmdAlphaSlider(FXObject* sender,FXSelector sel,void*){
+  FXint value;
+  sender->handle(this,MKUINT(ID_GETINTVALUE,SEL_COMMAND),(void*)&value);
+  hsva[3]=rgba[3]=0.003921568627f*value;
+  updateWell();
+  handle(this,MKUINT(ID_COLOR,SELTYPE(sel)),(void*)well->getRGBA());
+  return 1;
+  }
+
+
+// Update well from Alpha text fields
+long FXColorSelector::onCmdAlphaText(FXObject* sender,FXSelector,void*){
+  FXdouble value;
+  sender->handle(this,MKUINT(ID_GETREALVALUE,SEL_COMMAND),(void*)&value);
+  hsva[3]=rgba[3]=0.003921568627f*value;
+  updateWell();
+  handle(this,MKUINT(ID_COLOR,SEL_COMMAND),(void*)well->getRGBA());
+  return 1;
+  }
+
+
+// Update Alpha text fields
+long FXColorSelector::onUpdAlphaText(FXObject* sender,FXSelector,void*){
+  if(isOpaqueOnly()){
+    sender->handle(this,MKUINT(ID_HIDE,SEL_COMMAND),NULL);
+    }
+  else{
+    FXString value=FXStringVal(255.0*rgba[3],1,FALSE);
+    sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&value);
+    sender->handle(this,MKUINT(ID_SHOW,SEL_COMMAND),NULL);
+    }
+  return 1;
+  }
+
+
+// Update Alpha sliders
+long FXColorSelector::onUpdAlphaSlider(FXObject* sender,FXSelector,void*){
+  if(isOpaqueOnly()){
+    sender->handle(this,MKUINT(ID_HIDE,SEL_COMMAND),NULL);
+    }
+  else{
+    FXint value=(FXint)(255.0*rgba[3]);
+    sender->handle(this,MKUINT(ID_SETINTVALUE,SEL_COMMAND),(void*)&value);
+    sender->handle(this,MKUINT(ID_SHOW,SEL_COMMAND),NULL);
+    }
+  return 1;
+  }
+
+
+// Update Alpha Labels
+long FXColorSelector::onUpdAlphaLabel(FXObject* sender,FXSelector,void*){
+  if(isOpaqueOnly()){
+    sender->handle(this,MKUINT(ID_HIDE,SEL_COMMAND),NULL);
+    }
+  else{
+    sender->handle(this,MKUINT(ID_SHOW,SEL_COMMAND),NULL);
+    }
+  return 1;
   }
 
 
@@ -639,9 +730,8 @@ void FXColorSelector::create(){
 // Update well from RGB slider
 long FXColorSelector::onCmdRGBSlider(FXObject*,FXSelector sel,void*){
   FXint which=SELID(sel)-ID_RGB_RED_SLIDER;
-  rgba[which]=0.003921568627f*rgbaslider[which]->getPosition();
-  fxrgb_to_hsv(hsva[0],hsva[1],hsva[2],rgba[0],rgba[1],rgba[2]); 
-  hsva[3]=rgba[3];
+  rgba[which]=0.003921568627f*rgbaslider[which]->getValue();
+  fxrgb_to_hsv(hsva[0],hsva[1],hsva[2],rgba[0],rgba[1],rgba[2]);
   updateWell();
   handle(this,MKUINT(ID_COLOR,SELTYPE(sel)),(void*)well->getRGBA());
   return 1;
@@ -652,8 +742,7 @@ long FXColorSelector::onCmdRGBSlider(FXObject*,FXSelector sel,void*){
 long FXColorSelector::onCmdRGBText(FXObject*,FXSelector sel,void*){
   FXint which=SELID(sel)-ID_RGB_RED_TEXT;
   rgba[which]=0.003921568627f*FXFloatVal(rgbatext[which]->getText());
-  fxrgb_to_hsv(hsva[0],hsva[1],hsva[2],rgba[0],rgba[1],rgba[2]); 
-  hsva[3]=rgba[3];
+  fxrgb_to_hsv(hsva[0],hsva[1],hsva[2],rgba[0],rgba[1],rgba[2]);
   updateWell();
   handle(this,MKUINT(ID_COLOR,SEL_COMMAND),(void*)well->getRGBA());
   return 1;
@@ -663,15 +752,15 @@ long FXColorSelector::onCmdRGBText(FXObject*,FXSelector sel,void*){
 // Update RGB text fields
 long FXColorSelector::onUpdRGBText(FXObject*,FXSelector sel,void*){
   FXint which=SELID(sel)-ID_RGB_RED_TEXT;
-  rgbatext[which]->setText(FXStringVal(255.0*rgba[which],1,EXP_NEVER));
+  rgbatext[which]->setText(FXStringVal(255.0*rgba[which],1,FALSE));
   return 1;
   }
-  
+
 
 // Update RGB sliders
 long FXColorSelector::onUpdRGBSlider(FXObject*,FXSelector sel,void*){
   FXint which=SELID(sel)-ID_RGB_RED_SLIDER;
-  rgbaslider[which]->setPosition((FXint)(255.0*rgba[which]));
+  rgbaslider[which]->setValue((FXint)(255.0*rgba[which]));
   return 1;
   }
 
@@ -683,11 +772,10 @@ long FXColorSelector::onUpdRGBSlider(FXObject*,FXSelector sel,void*){
 
 // Update well from HSV sliders
 long FXColorSelector::onCmdHSVSlider(FXObject*,FXSelector sel,void*){
-  const FXfloat factor[4]={1.0f,0.001f,0.001f,0.003921568627f};
+  const FXfloat factor[3]={1.0f,0.001f,0.001f};
   FXint which=SELID(sel)-ID_HSV_HUE_SLIDER;
-  hsva[which]=factor[which]*hsvaslider[which]->getPosition();
-  fxhsv_to_rgb(rgba[0],rgba[1],rgba[2],hsva[0],hsva[1],hsva[2]); 
-  rgba[3]=hsva[3];
+  hsva[which]=factor[which]*hsvaslider[which]->getValue();
+  fxhsv_to_rgb(rgba[0],rgba[1],rgba[2],hsva[0],hsva[1],hsva[2]);
   updateWell();
   handle(this,MKUINT(ID_COLOR,SELTYPE(sel)),(void*)well->getRGBA());
   return 1;
@@ -696,11 +784,10 @@ long FXColorSelector::onCmdHSVSlider(FXObject*,FXSelector sel,void*){
 
 // Update well from HSV text fields
 long FXColorSelector::onCmdHSVText(FXObject*,FXSelector sel,void*){
-  const FXfloat factor[4]={1.0f,0.01f,0.01f,0.003921568627f};
+  const FXfloat factor[3]={1.0f,0.01f,0.01f};
   FXint which=SELID(sel)-ID_HSV_HUE_TEXT;
   hsva[which]=factor[which]*FXFloatVal(hsvatext[which]->getText());
-  fxhsv_to_rgb(rgba[0],rgba[1],rgba[2],hsva[0],hsva[1],hsva[2]); 
-  rgba[3]=hsva[3];
+  fxhsv_to_rgb(rgba[0],rgba[1],rgba[2],hsva[0],hsva[1],hsva[2]);
   updateWell();
   handle(this,MKUINT(ID_COLOR,SEL_COMMAND),(void*)well->getRGBA());
   return 1;
@@ -709,18 +796,18 @@ long FXColorSelector::onCmdHSVText(FXObject*,FXSelector sel,void*){
 
 // Update HSV text fields
 long FXColorSelector::onUpdHSVText(FXObject*,FXSelector sel,void*){
-  const FXfloat factor[4]={1.0f,100.0f,100.0f,255.0f};
+  const FXfloat factor[3]={1.0f,100.0f,100.0f};
   FXint which=SELID(sel)-ID_HSV_HUE_TEXT;
-  hsvatext[which]->setText(FXStringVal(hsva[which]*factor[which],1,EXP_NEVER));
+  hsvatext[which]->setText(FXStringVal(hsva[which]*factor[which],1,FALSE));
   return 1;
   }
 
 
 // Update HSV sliders
 long FXColorSelector::onUpdHSVSlider(FXObject*,FXSelector sel,void*){
-  const FXfloat factor[4]={1.0f,1000.0f,1000.0f,255.0f};
+  const FXfloat factor[3]={1.0f,1000.0f,1000.0f};
   FXint which=SELID(sel)-ID_HSV_HUE_SLIDER;
-  hsvaslider[which]->setPosition((FXint)(hsva[which]*factor[which]));
+  hsvaslider[which]->setValue((FXint)(hsva[which]*factor[which]));
   return 1;
   }
 
@@ -731,8 +818,9 @@ long FXColorSelector::onUpdHSVSlider(FXObject*,FXSelector sel,void*){
 
 long FXColorSelector::onCmdCMYSlider(FXObject*,FXSelector sel,void*){
   FXint which=SELID(sel)-ID_CMY_CYAN_SLIDER;
-  rgba[which]=1.0f-0.003921568627f*cmyslider[which]->getPosition();
-  fxrgb_to_hsv(hsva[0],hsva[1],hsva[2],rgba[0],rgba[1],rgba[2]); 
+  FXfloat val=0.003921568627f*cmyslider[which]->getValue();
+  rgba[which]=1.0f-val;
+  fxrgb_to_hsv(hsva[0],hsva[1],hsva[2],rgba[0],rgba[1],rgba[2]);
   hsva[3]=rgba[3];
   updateWell();
   handle(this,MKUINT(ID_COLOR,SELTYPE(sel)),(void*)well->getRGBA());
@@ -742,8 +830,9 @@ long FXColorSelector::onCmdCMYSlider(FXObject*,FXSelector sel,void*){
 
 long FXColorSelector::onCmdCMYText(FXObject*,FXSelector sel,void*){
   FXint which=SELID(sel)-ID_CMY_CYAN_TEXT;
-  rgba[which]=1.0f-0.003921568627f*FXFloatVal(cmytext[which]->getText());
-  fxrgb_to_hsv(hsva[0],hsva[1],hsva[2],rgba[0],rgba[1],rgba[2]); 
+  FXfloat val=0.003921568627f*FXFloatVal(cmytext[which]->getText());
+  rgba[which]=1.0f-val;
+  fxrgb_to_hsv(hsva[0],hsva[1],hsva[2],rgba[0],rgba[1],rgba[2]);
   hsva[3]=rgba[3];
   updateWell();
   handle(this,MKUINT(ID_COLOR,SEL_COMMAND),(void*)well->getRGBA());
@@ -753,14 +842,16 @@ long FXColorSelector::onCmdCMYText(FXObject*,FXSelector sel,void*){
 
 long FXColorSelector::onUpdCMYText(FXObject*,FXSelector sel,void*){
   FXint which=SELID(sel)-ID_CMY_CYAN_TEXT;
-  cmytext[which]->setText(FXStringVal(255.0-255.0*rgba[which],1,EXP_NEVER));
+  FXfloat val=255.0f-255.0f*rgba[which];
+  cmytext[which]->setText(FXStringVal(val,1,FALSE));
   return 1;
   }
 
 
 long FXColorSelector::onUpdCMYSlider(FXObject*,FXSelector sel,void*){
   FXint which=SELID(sel)-ID_CMY_CYAN_SLIDER;
-  cmyslider[which]->setPosition((FXint)(255.0-255.0*rgba[which]));
+  FXint val=(FXint)(255.0-255.0*rgba[which]);
+  cmyslider[which]->setValue(val);
   return 1;
   }
 
@@ -770,13 +861,13 @@ long FXColorSelector::onUpdCMYSlider(FXObject*,FXSelector sel,void*){
 
 // Dropped color in main well
 long FXColorSelector::onChgWell(FXObject*,FXSelector,void* ptr){
-  FXTRACE((100,"%s::onChgWell %08x\n",getClassName(),this));
   FXColor color=(FXColor)(long)ptr;
+  if(isOpaqueOnly()) color|=FXRGBA(0,0,0,255);
   rgba[0]=0.003921568627f*FXREDVAL(color);
   rgba[1]=0.003921568627f*FXGREENVAL(color);
   rgba[2]=0.003921568627f*FXBLUEVAL(color);
   rgba[3]=0.003921568627f*FXALPHAVAL(color);
-  fxrgb_to_hsv(hsva[0],hsva[1],hsva[2],rgba[0],rgba[1],rgba[2]); 
+  fxrgb_to_hsv(hsva[0],hsva[1],hsva[2],rgba[0],rgba[1],rgba[2]);
   hsva[3]=rgba[3];
   return 1;
   }
@@ -784,7 +875,6 @@ long FXColorSelector::onChgWell(FXObject*,FXSelector,void* ptr){
 
 // Command from main well
 long FXColorSelector::onCmdWell(FXObject*,FXSelector,void*){
-  FXTRACE((100,"%s::onCmdWell %08x\n",getClassName(),this));
   handle(this,MKUINT(ID_COLOR,SEL_COMMAND),(void*)well->getRGBA());
   return 1;
   }
@@ -802,12 +892,13 @@ void FXColorSelector::updateWell(){
 // Clicked on color in list
 long FXColorSelector::onCmdList(FXObject*,FXSelector,void* ptr){
   FXint index=(FXint)(long)ptr;
-  FXColor clr=FXRGB(fxcolornames[index].r,fxcolornames[index].g,fxcolornames[index].b);
+  FXColor clr=fxcolornames[index].color;
+  if(isOpaqueOnly()) clr|=FXRGBA(0,0,0,255);
   rgba[0]=0.003921568627f*FXREDVAL(clr);
   rgba[1]=0.003921568627f*FXGREENVAL(clr);
   rgba[2]=0.003921568627f*FXBLUEVAL(clr);
-  rgba[3]=1.0;
-  fxrgb_to_hsv(hsva[0],hsva[1],hsva[2],rgba[0],rgba[1],rgba[2]); 
+  rgba[3]=0.003921568627f*FXALPHAVAL(clr);
+  fxrgb_to_hsv(hsva[0],hsva[1],hsva[2],rgba[0],rgba[1],rgba[2]);
   hsva[3]=rgba[3];
   updateWell();
   handle(this,MKUINT(ID_COLOR,SEL_COMMAND),(void*)well->getRGBA());
@@ -831,6 +922,7 @@ long FXColorSelector::onChgCustomWell(FXObject*,FXSelector sel,void* ptr){
 // Custom well clicked
 long FXColorSelector::onCmdCustomWell(FXObject*,FXSelector,void* ptr){
   FXColor color=(FXColor)(long)ptr;
+  if(isOpaqueOnly()) color|=FXRGBA(0,0,0,255);
   setRGBA(color);
   handle(this,MKUINT(ID_COLOR,SEL_COMMAND),(void*)well->getRGBA());
   return 1;
@@ -839,7 +931,7 @@ long FXColorSelector::onCmdCustomWell(FXObject*,FXSelector,void* ptr){
 
 // Switching active pane
 long FXColorSelector::onCmdActivePane(FXObject*,FXSelector,void* ptr){
-  getApp()->reg().writeIntEntry("SETTINGS","activecolorpane",(FXint)ptr);
+  getApp()->reg().writeIntEntry("SETTINGS","activecolorpane",(FXint)(long)ptr);
   return 1;
   }
 
@@ -852,7 +944,7 @@ long FXColorSelector::onCmdColor(FXObject*,FXSelector,void*){
   if(target) target->handle(this,MKUINT(message,SEL_COMMAND),(void*)well->getRGBA());
   return 1;
   }
-  
+
 
 // Forward to target
 long FXColorSelector::onChgColor(FXObject*,FXSelector,void*){
@@ -868,14 +960,14 @@ long FXColorSelector::onCmdSetValue(FXObject*,FXSelector,void* ptr){
   }
 
 
-// Change RGBA color 
+// Change RGBA color
 void FXColorSelector::setRGBA(FXColor clr){
   if(clr!=well->getRGBA()){
     rgba[0]=0.003921568627f*FXREDVAL(clr);
     rgba[1]=0.003921568627f*FXGREENVAL(clr);
     rgba[2]=0.003921568627f*FXBLUEVAL(clr);
     rgba[3]=0.003921568627f*FXALPHAVAL(clr);
-    fxrgb_to_hsv(hsva[0],hsva[1],hsva[2],rgba[0],rgba[1],rgba[2]); 
+    fxrgb_to_hsv(hsva[0],hsva[1],hsva[2],rgba[0],rgba[1],rgba[2]);
     hsva[3]=rgba[3];
     well->setRGBA(clr);
     }
@@ -888,6 +980,24 @@ FXColor FXColorSelector::getRGBA() const {
   }
 
 
+// Return true if only opaque colors allowed
+FXbool FXColorSelector::isOpaqueOnly() const {
+  return well->isOpaqueOnly();
+  }
+
+
+// Change opaque only mode
+void FXColorSelector::setOpaqueOnly(FXbool opaque){
+  if(opaque){
+    well->setOpaqueOnly(TRUE);
+    setRGBA(well->getRGBA() | FXRGBA(0,0,0,255));
+    }
+  else{
+    well->setOpaqueOnly(FALSE);
+    }
+  }
+
+
 /*******************************************************************************/
 
 
@@ -921,7 +1031,7 @@ void FXColorSelector::save(FXStream& store) const {
 
 
 // Load data
-void FXColorSelector::load(FXStream& store){ 
+void FXColorSelector::load(FXStream& store){
   FXPacker::load(store);
   store >> panels;
   store >> well;
@@ -955,10 +1065,10 @@ FXColorSelector::~FXColorSelector(){
   delete hsvmodeicon;
   delete cmymodeicon;
   delete txtmodeicon;
-  rgbmodeicon=(FXGIFIcon*)-1;
-  hsvmodeicon=(FXGIFIcon*)-1;
-  cmymodeicon=(FXGIFIcon*)-1;
-  txtmodeicon=(FXGIFIcon*)-1;
+  rgbmodeicon=(FXIcon*)-1;
+  hsvmodeicon=(FXIcon*)-1;
+  cmymodeicon=(FXIcon*)-1;
+  txtmodeicon=(FXIcon*)-1;
   panels=(FXTabBook*)-1;
   well=(FXColorWell*)-1;
   accept=(FXButton*)-1;
