@@ -3,80 +3,69 @@
 *                         C o l o r W e l l   C l a s s                         *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXColorWell.cpp,v 1.5 1999/11/12 07:59:17 jeroen Exp $                   *
+* $Id: FXColorWell.cpp,v 1.31 2001/04/27 14:30:31 jeroen Exp $                  *
 ********************************************************************************/
 #include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "fxkeys.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
-#include "FXObjectList.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
-#include "FXAccelTable.h"
 #include "FXApp.h"
 #include "FXId.h"
 #include "FXDC.h"
 #include "FXDCWindow.h"
-#include "FXFont.h"
 #include "FXDrawable.h"
-#include "FXImage.h"
-#include "FXIcon.h"
-#include "FXGIFIcon.h"
 #include "FXWindow.h"
 #include "FXFrame.h"
 #include "FXComposite.h"
 #include "FXLabel.h"
-#include "FXButton.h"
-#include "FXComposite.h"
-#include "FXPacker.h"
-#include "FXTab.h"
-#include "FXCanvas.h"
 #include "FXColorWell.h"
 #include "FXShell.h"
 #include "FXTopWindow.h"
 #include "FXDialogBox.h"
-#include "FXScrollbar.h"
-#include "FXScrollWindow.h"
-#include "FXList.h"
-#include "FXTextField.h"
-#include "FXSlider.h"
-#include "FXColorSelector.h"
 #include "FXColorDialog.h"
 
 /*
   Notes:
-  - Need to popup color dialog on double click (but what to do
-    when there's a custom color panel?)
   - Is there any reason why one wouldn't ^C ^V on the clipboard colors same as text?
   - Single-click should send SEL_COMMAND to target.
   - Perhaps change of color should send SEL_CHANGED.
   - Do not start drag operation unless moving a little bit.
   - Drive from keyboard.
-  - Think some more about active well handling.
+  - Yes, you can now drag a color into a text widget, or drag the name of
+    a color into the well, as well as pasting a color into a text widget or
+    vice versa!
+  - KDE/Qt has color drag and drop wrong:- should multiply by 257, not 255,
+    so get full range of [0 - 65535].
 */
 
 #define WELLSIZE    12              // Minimum well size
 #define FOCUSBORDER 3               // Focus border
 
-
 /*******************************************************************************/
 
 
@@ -84,8 +73,12 @@
 FXDEFMAP(FXColorWell) FXColorWellMap[]={
   FXMAPFUNC(SEL_PAINT,0,FXColorWell::onPaint),
   FXMAPFUNC(SEL_MOTION,0,FXColorWell::onMotion),
-  FXMAPFUNC(SEL_DND_MOTION,0,FXColorWell::onDNDMotion),
   FXMAPFUNC(SEL_DRAGGED,0,FXColorWell::onDragged),
+  FXMAPFUNC(SEL_DND_MOTION,0,FXColorWell::onDNDMotion),
+  FXMAPFUNC(SEL_DND_ENTER,0,FXColorWell::onDNDEnter),
+  FXMAPFUNC(SEL_DND_LEAVE,0,FXColorWell::onDNDLeave),
+  FXMAPFUNC(SEL_DND_DROP,0,FXColorWell::onDNDDrop),
+  FXMAPFUNC(SEL_DND_REQUEST,0,FXColorWell::onDNDRequest),
   FXMAPFUNC(SEL_FOCUSIN,0,FXColorWell::onFocusIn),
   FXMAPFUNC(SEL_FOCUSOUT,0,FXColorWell::onFocusOut),
   FXMAPFUNC(SEL_LEFTBUTTONPRESS,0,FXColorWell::onLeftBtnPress),
@@ -98,8 +91,6 @@ FXDEFMAP(FXColorWell) FXColorWellMap[]={
   FXMAPFUNC(SEL_KEYPRESS,0,FXColorWell::onKeyPress),
   FXMAPFUNC(SEL_KEYRELEASE,0,FXColorWell::onKeyRelease),
   FXMAPFUNC(SEL_UNGRABBED,0,FXColorWell::onUngrabbed),
-  FXMAPFUNC(SEL_DND_DROP,0,FXColorWell::onDNDDrop),
-  FXMAPFUNC(SEL_DND_REQUEST,0,FXColorWell::onDNDRequest),
   FXMAPFUNC(SEL_BEGINDRAG,0,FXColorWell::onBeginDrag),
   FXMAPFUNC(SEL_ENDDRAG,0,FXColorWell::onEndDrag),
   FXMAPFUNC(SEL_CHANGED,0,FXColorWell::onChanged),
@@ -121,7 +112,7 @@ FXDEFMAP(FXColorWell) FXColorWellMap[]={
 FXIMPLEMENT(FXColorWell,FXFrame,FXColorWellMap,ARRAYNUMBER(FXColorWellMap))
 
 
-  
+
 /*******************************************************************************/
 
 
@@ -129,11 +120,12 @@ FXIMPLEMENT(FXColorWell,FXFrame,FXColorWellMap,ARRAYNUMBER(FXColorWellMap))
 FXColorWell::FXColorWell(){
   flags|=FLAG_ENABLED|FLAG_SHOWN;
   rgba=0;
+  oldrgba=0;
   wellColor[0]=0;
   wellColor[1]=0;
   }
 
-  
+
 // Make a color well
 FXColorWell::FXColorWell(FXComposite* p,FXColor clr,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb):
   FXFrame(p,opts,x,y,w,h,pl,pr,pt,pb){
@@ -141,6 +133,7 @@ FXColorWell::FXColorWell(FXComposite* p,FXColor clr,FXObject* tgt,FXSelector sel
   target=tgt;
   message=sel;
   rgba=clr;
+  oldrgba=clr;
   wellColor[0]=rgbaoverwhite(rgba);
   wellColor[1]=rgbaoverblack(rgba);
   }
@@ -150,6 +143,7 @@ FXColorWell::FXColorWell(FXComposite* p,FXColor clr,FXObject* tgt,FXSelector sel
 void FXColorWell::create(){
   FXFrame::create();
   if(!colorType){colorType=getApp()->registerDragType(colorTypeName);}
+  if(!textType){textType=getApp()->registerDragType(textTypeName);}
   dropEnable();
   }
 
@@ -158,6 +152,7 @@ void FXColorWell::create(){
 void FXColorWell::detach(){
   FXFrame::detach();
   colorType=0;
+  textType=0;
   }
 
 
@@ -168,19 +163,19 @@ FXbool FXColorWell::canFocus() const { return 1; }
 // Compute color over black
 FXColor FXColorWell::rgbaoverblack(FXColor clr){
   FXint r,g,b,mul=FXALPHAVAL(clr);
-  r=(FXREDVAL(clr)*mul)/255;
-  g=(FXGREENVAL(clr)*mul)/255;
-  b=(FXBLUEVAL(clr)*mul)/255;
+  r=(FXREDVAL(clr)*mul+127)/255;
+  g=(FXGREENVAL(clr)*mul+127)/255;
+  b=(FXBLUEVAL(clr)*mul+127)/255;
   return FXRGB(r,g,b);
   }
 
 
 // Compute color over white
 FXColor FXColorWell::rgbaoverwhite(FXColor clr){
-  FXint r,g,b,mul=FXALPHAVAL(clr);
-  r=((255-mul)*255+FXREDVAL(clr)*mul)/255;
-  g=((255-mul)*255+FXGREENVAL(clr)*mul)/255;
-  b=((255-mul)*255+FXBLUEVAL(clr)*mul)/255;
+  FXint r,g,b,mul=FXALPHAVAL(clr),lum=(255-mul);
+  r=(lum*255+FXREDVAL(clr)*mul+127)/255;
+  g=(lum*255+FXGREENVAL(clr)*mul+127)/255;
+  b=(lum*255+FXBLUEVAL(clr)*mul+127)/255;
   return FXRGB(r,g,b);
   }
 
@@ -196,12 +191,11 @@ FXint FXColorWell::getDefaultHeight(){
   }
 
 
-// Handle repaint 
+// Handle repaint
 long FXColorWell::onPaint(FXObject*,FXSelector,void* ptr){
   FXEvent *ev=(FXEvent*)ptr;
-  FXPoint points[3];
   FXDCWindow dc(this,ev);
-  dc.setClipRectangle(ev->rect.x,ev->rect.y,ev->rect.w,ev->rect.h);
+  FXPoint points[3];
   dc.setForeground(backColor);
   dc.fillRectangle(0,0,width,padtop+FOCUSBORDER);
   dc.fillRectangle(0,padtop+FOCUSBORDER,padleft+FOCUSBORDER,height-padtop-padbottom-(FOCUSBORDER<<1));
@@ -212,13 +206,13 @@ long FXColorWell::onPaint(FXObject*,FXSelector,void* ptr){
     dc.drawRectangle(padleft+1,padtop+1,width-padright-padleft-3,height-padbottom-padtop-3);
     }
   dc.setForeground(wellColor[0]);
-  points[0].x=points[1].x=padleft+FOCUSBORDER+2; 
+  points[0].x=points[1].x=padleft+FOCUSBORDER+2;
   points[2].x=width-padright-FOCUSBORDER-2;
   points[0].y=points[2].y=padtop+FOCUSBORDER+2;
   points[1].y=height-padbottom-FOCUSBORDER-2;
   dc.fillPolygon(points,3);
   dc.setForeground(wellColor[1]);
-  points[0].x=padleft+FOCUSBORDER+2; 
+  points[0].x=padleft+FOCUSBORDER+2;
   points[1].x=points[2].x=width-padright-FOCUSBORDER-2;
   points[0].y=points[1].y=height-padbottom-FOCUSBORDER-2;
   points[2].y=padtop+FOCUSBORDER+2;
@@ -227,7 +221,6 @@ long FXColorWell::onPaint(FXObject*,FXSelector,void* ptr){
   if(hasFocus()){
     drawFocusRectangle(dc,padleft,padtop,width-padright-padleft,height-padbottom-padtop);
     }
-  dc.clearClipRectangle();
   return 1;
   }
 
@@ -239,7 +232,7 @@ long FXColorWell::onFocusIn(FXObject* sender,FXSelector sel,void* ptr){
   return 1;
   }
 
-  
+
 // Lost focus
 long FXColorWell::onFocusOut(FXObject* sender,FXSelector sel,void* ptr){
   FXFrame::onFocusOut(sender,sel,ptr);
@@ -248,39 +241,77 @@ long FXColorWell::onFocusOut(FXObject* sender,FXSelector sel,void* ptr){
   }
 
 
+// Dragging something over well; save old color and block GUI updates
+long FXColorWell::onDNDEnter(FXObject* sender,FXSelector sel,void* ptr){
+  if(FXFrame::onDNDEnter(sender,sel,ptr)) return 1;
+  flags&=~FLAG_UPDATE;
+  oldrgba=rgba;
+  return 1;
+  }
+
+
+// Dragged out of well, so restore old color and reenable GUI updates
+long FXColorWell::onDNDLeave(FXObject* sender,FXSelector sel,void* ptr){
+  if(FXFrame::onDNDLeave(sender,sel,ptr)) return 1;
+  flags|=FLAG_UPDATE;
+  if(oldrgba!=rgba){
+    handle(this,MKUINT(0,SEL_CHANGED),(void*)oldrgba);
+    }
+  return 1;
+  }
+
+
 // Handle drag-and-drop motion
 long FXColorWell::onDNDMotion(FXObject* sender,FXSelector sel,void* ptr){
-  
+  FXushort *clr; FXchar *str; FXuint len; FXColor color;
+
   // Handle base class first
   if(FXFrame::onDNDMotion(sender,sel,ptr)) return 1;
-  
+
   // No more messages while inside
   setDragRectangle(0,0,width,height,FALSE);
-  
-  // Is it a color being dropped?
-  if(offeredDNDType(FROM_DRAGNDROP,FXColorWell::colorType)){
+
+  // Try and obtain the color first
+  if(getDNDData(FROM_DRAGNDROP,colorType,(FXuchar*&)clr,len)){
+    color=FXRGBA((clr[0]+128)/257,(clr[1]+128)/257,(clr[2]+128)/257,(clr[3]+128)/257);
+    FXFREE(&clr);
+    handle(this,MKUINT(0,SEL_CHANGED),(void*)color);
     acceptDrop(DRAG_COPY);
     return 1;
     }
+
+  // Maybe its the name of a color
+  if(getDNDData(FROM_DRAGNDROP,textType,(FXuchar*&)str,len)){
+    FXRESIZE(&str,FXchar,len+1); str[len]='\0';
+    color=fxcolorfromname((FXchar*)str);
+    FXFREE(&str);
+
+    // Accept the drop only if it was a valid color name
+    if(color!=FXRGBA(0,0,0,0)){
+      handle(this,MKUINT(0,SEL_CHANGED),(void*)color);
+      acceptDrop(DRAG_COPY);
+      return 1;
+      }
+    }
+
   return 0;
   }
 
 
 // Handle drag-and-drop drop
 long FXColorWell::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr){
-  FXuchar *data; FXuint len,r,g,b,a;
-  
+  flags|=FLAG_UPDATE;
+
   // Try handling it in base class first
   if(FXFrame::onDNDDrop(sender,sel,ptr)) return 1;
-  
-  // Try handle here
-  if(getDNDData(FROM_DRAGNDROP,colorType,data,len)){
-    sscanf((char*)data,"#%02x%02x%02x%02x",&r,&g,&b,&a);
-    FXFREE(&data);
-    handle(this,MKUINT(0,SEL_CHANGED),(void*)FXRGBA(r,g,b,a)); 
-    handle(this,MKUINT(0,SEL_COMMAND),(void*)rgba); 
+
+  // Now send the command because the drop was finalized
+  if(oldrgba!=rgba){
+    handle(this,MKUINT(0,SEL_COMMAND),(void*)rgba);
+    oldrgba=rgba;
     return 1;
     }
+
   return 0;
   }
 
@@ -288,17 +319,28 @@ long FXColorWell::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr){
 // Service requested DND data
 long FXColorWell::onDNDRequest(FXObject* sender,FXSelector sel,void* ptr){
   FXEvent *event=(FXEvent*)ptr;
-  FXchar *value;
-  
+
   // Try handling it in base class first
   if(FXFrame::onDNDRequest(sender,sel,ptr)) return 1;
-  
-  // Try handle here
+
+  // Requested as a color
   if(event->target==colorType){
-    FXTRACE((100,"%s: Get color\n",getClassName()));
-    FXMALLOC(&value,FXchar,50);
-    sprintf(value,"#%02x%02x%02x%02x",FXREDVAL(rgba),FXGREENVAL(rgba),FXBLUEVAL(rgba),FXALPHAVAL(rgba));
-    setDNDData(event->origin,colorType,(FXuchar*)value,50);
+    FXushort *color;
+    FXMALLOC(&color,FXushort,4);
+    color[0]=257*FXREDVAL(rgba);
+    color[1]=257*FXGREENVAL(rgba);
+    color[2]=257*FXBLUEVAL(rgba);
+    color[3]=257*FXALPHAVAL(rgba);
+    setDNDData(FROM_DRAGNDROP,colorType,(FXuchar*)color,sizeof(FXushort)*4);
+    return 1;
+    }
+
+  // Requested as a color name
+  if(event->target==textType){
+    FXchar *string;
+    FXMALLOC(&string,FXchar,50);
+    fxnamefromcolor(string,rgba);
+    setDNDData(FROM_DRAGNDROP,textType,(FXuchar*)string,strlen(string));
     return 1;
     }
   return 0;
@@ -307,7 +349,7 @@ long FXColorWell::onDNDRequest(FXObject* sender,FXSelector sel,void* ptr){
 
 // We now really do have the selection; repaint the text field
 long FXColorWell::onSelectionGained(FXObject* sender,FXSelector sel,void* ptr){
-  if(FXFrame::onSelectionGained(sender,sel,ptr)) return 1;
+  FXFrame::onSelectionGained(sender,sel,ptr);
   update();
   return 1;
   }
@@ -315,7 +357,7 @@ long FXColorWell::onSelectionGained(FXObject* sender,FXSelector sel,void* ptr){
 
 // We lost the selection somehow; repaint the text field
 long FXColorWell::onSelectionLost(FXObject* sender,FXSelector sel,void* ptr){
-  if(FXFrame::onSelectionLost(sender,sel,ptr)) return 1;
+  FXFrame::onSelectionLost(sender,sel,ptr);
   update();
   return 1;
   }
@@ -324,36 +366,42 @@ long FXColorWell::onSelectionLost(FXObject* sender,FXSelector sel,void* ptr){
 // Somebody wants our selection
 long FXColorWell::onSelectionRequest(FXObject* sender,FXSelector sel,void* ptr){
   FXEvent *event=(FXEvent*)ptr;
-  FXchar *value;
-  
+
   // Try handling it in base class first
   if(FXFrame::onSelectionRequest(sender,sel,ptr)) return 1;
-  
-  // Requested as standard FOX color type
+
+  // Requested as a color
   if(event->target==colorType){
-    FXMALLOC(&value,FXchar,50);
-    sprintf(value,"#%02x%02x%02x%02x",FXREDVAL(rgba),FXGREENVAL(rgba),FXBLUEVAL(rgba),FXALPHAVAL(rgba));
-    setDNDData(event->origin,colorType,(FXuchar*)value,strlen(value));
+    FXushort *color;
+    FXMALLOC(&color,FXushort,4);
+    color[0]=257*FXREDVAL(rgba);
+    color[1]=257*FXGREENVAL(rgba);
+    color[2]=257*FXBLUEVAL(rgba);
+    color[3]=257*FXALPHAVAL(rgba);
+    setDNDData(FROM_DRAGNDROP,colorType,(FXuchar*)color,sizeof(FXushort)*4);
     return 1;
     }
 
-  // Requested as string of the form #rrggbb
+  // Requested as a color name
   if(event->target==stringType){
-    FXMALLOC(&value,FXchar,50);
-    fxnamefromcolor(value,rgba);    // This is so cool!!
-    setDNDData(event->origin,stringType,(FXuchar*)value,strlen(value)+1);
+    FXchar *string;
+    FXMALLOC(&string,FXchar,50);
+    fxnamefromcolor(string,rgba);
+    setDNDData(FROM_DRAGNDROP,stringType,(FXuchar*)string,strlen(string));
     return 1;
     }
-
   return 0;
   }
 
 
 // Start a drag operation
 long FXColorWell::onBeginDrag(FXObject* sender,FXSelector sel,void* ptr){
+  FXDragType types[2];
   if(FXFrame::onBeginDrag(sender,sel,ptr)) return 1;
-  beginDrag(&FXColorWell::colorType,1);
-  setDragCursor(getApp()->swatchCursor);
+  types[0]=colorType;
+  types[1]=textType;
+  beginDrag(types,2);
+  setDragCursor(getApp()->getDefaultCursor(DEF_SWATCH_CURSOR));
   return 1;
   }
 
@@ -362,43 +410,35 @@ long FXColorWell::onBeginDrag(FXObject* sender,FXSelector sel,void* ptr){
 long FXColorWell::onEndDrag(FXObject* sender,FXSelector sel,void* ptr){
   if(FXFrame::onEndDrag(sender,sel,ptr)) return 1;
   endDrag(didAccept()==DRAG_COPY);
-  setDragCursor(getApp()->arrowCursor);
+  setDragCursor(getApp()->getDefaultCursor(DEF_ARROW_CURSOR));
   return 1;
   }
 
 
 // Dragged stuff around
 long FXColorWell::onDragged(FXObject* sender,FXSelector sel,void* ptr){
-  FXEvent *event=(FXEvent*)ptr;
   if(FXFrame::onDragged(sender,sel,ptr)) return 1;
-  handleDrag(event->root_x,event->root_y,DRAG_COPY);
+  handleDrag(((FXEvent*)ptr)->root_x,((FXEvent*)ptr)->root_y,DRAG_COPY);
   if(didAccept()==DRAG_COPY){
-    setDragCursor(getApp()->swatchCursor);
+    setDragCursor(getApp()->getDefaultCursor(DEF_SWATCH_CURSOR));
     }
   else{
-    setDragCursor(getApp()->dontdropCursor);
+    setDragCursor(getApp()->getDefaultCursor(DEF_DNDSTOP_CURSOR));
     }
   return 1;
   }
 
 
-// Moving 
+// Moving
 long FXColorWell::onMotion(FXObject*,FXSelector,void* ptr){
-  FXEvent *event=(FXEvent*)ptr;
-  if(flags&FLAG_PRESSED){
-    if(flags&FLAG_DODRAG){
-      handle(this,MKUINT(0,SEL_DRAGGED),ptr);
-      return 1;
-      }
-    if(flags&FLAG_TRYDRAG){
-      if(event->moved){
-        flags&=~FLAG_TRYDRAG;
-        if(handle(this,MKUINT(0,SEL_BEGINDRAG),ptr)){
-          flags|=FLAG_DODRAG;
-          }
-        }
-      return 1;
-      }
+  if(flags&FLAG_DODRAG){
+    handle(this,MKUINT(0,SEL_DRAGGED),ptr);
+    return 1;
+    }
+  if((flags&FLAG_TRYDRAG) && ((FXEvent*)ptr)->moved){
+    if(handle(this,MKUINT(0,SEL_BEGINDRAG),ptr)) flags|=FLAG_DODRAG;
+    flags&=~FLAG_TRYDRAG;
+    return 1;
     }
   return 0;
   }
@@ -408,13 +448,12 @@ long FXColorWell::onMotion(FXObject*,FXSelector,void* ptr){
 long FXColorWell::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
   FXEvent* event=(FXEvent*)ptr;
   flags&=~FLAG_TIP;
+  handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
   if(isEnabled()){
-    handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
     grab();
     if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONPRESS),ptr)) return 1;
     if(event->click_count==1){
       flags&=~FLAG_UPDATE;
-      flags|=FLAG_PRESSED;
       flags|=FLAG_TRYDRAG;
       }
     }
@@ -422,23 +461,19 @@ long FXColorWell::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
   }
 
 
-// Drop 
+// Drop
 long FXColorWell::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
   FXEvent* event=(FXEvent*)ptr;
+  FXuint flgs=flags;
   if(isEnabled()){
     ungrab();
-    flags&=~FLAG_PRESSED;
     flags|=FLAG_UPDATE;
+    flags&=~(FLAG_TRYDRAG|FLAG_DODRAG);
     if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONRELEASE),ptr)) return 1;
-    if(flags&FLAG_DODRAG){
-      handle(this,MKUINT(0,SEL_ENDDRAG),ptr);
-      }
-    flags&=~FLAG_DODRAG;
+    if(flgs&FLAG_DODRAG){handle(this,MKUINT(0,SEL_ENDDRAG),ptr);}
     if(event->click_count==1){
       handle(this,MKUINT(0,SEL_CLICKED),(void*)rgba);
-      if(!event->moved){
-        handle(this,MKUINT(0,SEL_COMMAND),(void*)rgba); 
-        }
+      if(!event->moved){handle(this,MKUINT(0,SEL_COMMAND),(void*)rgba);}
       }
     else if(event->click_count==2){
       handle(this,MKUINT(0,SEL_DOUBLECLICKED),(void*)rgba);
@@ -455,8 +490,8 @@ long FXColorWell::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
 // Pressed middle button to paste
 long FXColorWell::onMiddleBtnPress(FXObject*,FXSelector,void* ptr){
   flags&=~FLAG_TIP;
+  handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
   if(isEnabled()){
-    handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
     grab();
     if(target && target->handle(this,MKUINT(message,SEL_MIDDLEBUTTONPRESS),ptr)) return 1;
     return 1;
@@ -467,25 +502,28 @@ long FXColorWell::onMiddleBtnPress(FXObject*,FXSelector,void* ptr){
 
 // Released middle button causes paste of selection
 long FXColorWell::onMiddleBtnRelease(FXObject*,FXSelector,void* ptr){
-  FXuchar *data; FXuint len,r,g,b,a; FXColor color;
+  FXushort *clr; FXchar *str; FXuint len; FXColor color;
   if(isEnabled()){
     ungrab();
     if(target && target->handle(this,MKUINT(message,SEL_MIDDLEBUTTONRELEASE),ptr)) return 1;
-    if(getDNDData(FROM_SELECTION,colorType,data,len)){
-      sscanf((char*)data,"#%02x%02x%02x%02x",&r,&g,&b,&a);
-      FXFREE(&data);
-      handle(this,MKUINT(0,SEL_CHANGED),(void*)FXRGBA(r,g,b,a)); 
-      handle(this,MKUINT(0,SEL_COMMAND),(void*)rgba); 
+
+    // Request as FOX color
+    if(getDNDData(FROM_SELECTION,colorType,(FXuchar*&)clr,len)){
+      color=FXRGBA((clr[0]+128)/257,(clr[1]+128)/257,(clr[2]+128)/257,(clr[3]+128)/257);
+      FXFREE(&clr);
+      handle(this,MKUINT(0,SEL_CHANGED),(void*)color);
+      handle(this,MKUINT(0,SEL_COMMAND),(void*)color);
       return 1;
       }
-    if(getDNDData(FROM_SELECTION,stringType,data,len)){
-      color=fxcolorfromname((FXchar*)data);      // This is so cool!!
-      FXFREE(&data);
-      if(color){
-        handle(this,MKUINT(0,SEL_CHANGED),(void*)color); 
-        handle(this,MKUINT(0,SEL_COMMAND),(void*)color); 
-        return 1;
-        }
+
+    // Request as string
+    if(getDNDData(FROM_SELECTION,stringType,(FXuchar*&)str,len)){
+      FXRESIZE(&str,FXchar,len+1); str[len]='\0';
+      color=fxcolorfromname(str);
+      FXFREE(&str);
+      handle(this,MKUINT(0,SEL_CHANGED),(void*)color);
+      handle(this,MKUINT(0,SEL_COMMAND),(void*)color);
+      return 1;
       }
     }
   return 0;
@@ -513,7 +551,7 @@ long FXColorWell::onKeyPress(FXObject*,FXSelector,void* ptr){
   }
 
 
-// Key Release 
+// Key Release
 long FXColorWell::onKeyRelease(FXObject*,FXSelector,void* ptr){
   FXEvent* event=(FXEvent*)ptr;
   if(isEnabled()){
@@ -555,9 +593,9 @@ long FXColorWell::onCommand(FXObject*,FXSelector,void*){
 // Clicked in the well
 long FXColorWell::onClicked(FXObject*,FXSelector,void*){
   FXDragType types[2];
-  
+
   if(target && target->handle(this,MKUINT(message,SEL_CLICKED),(void*)rgba)) return 1;
-  
+
   // Double click means claim selection
   if(!hasSelection()){
     types[0]=stringType;
@@ -570,24 +608,25 @@ long FXColorWell::onClicked(FXObject*,FXSelector,void*){
 
 // Double clicked in well
 long FXColorWell::onDoubleClicked(FXObject*,FXSelector,void*){
-  
+
   // Double click
   if(target && target->handle(this,MKUINT(message,SEL_DOUBLECLICKED),(void*)rgba)) return 1;
-  
+
   // Can not be edited
   if(options&COLORWELL_SOURCEONLY) return 1;
-  
+
   // OK, edit the color now
   FXColorDialog colordialog(this,"Color Dialog");
   FXColor oldcolor=getRGBA();
   colordialog.setTarget(this);
   colordialog.setSelector(ID_COLORDIALOG);
   colordialog.setRGBA(oldcolor);
-  
+  colordialog.setOpaqueOnly(isOpaqueOnly());
+
   // We canceled, so restore the old color
   if(!colordialog.execute()){
-    handle(this,MKUINT(0,SEL_CHANGED),(void*)oldcolor); 
-    handle(this,MKUINT(0,SEL_COMMAND),(void*)oldcolor); 
+    handle(this,MKUINT(0,SEL_CHANGED),(void*)oldcolor);
+    handle(this,MKUINT(0,SEL_COMMAND),(void*)oldcolor);
     }
   return 1;
   }
@@ -602,7 +641,7 @@ long FXColorWell::onTripleClicked(FXObject*,FXSelector,void*){
 // The widget lost the grab for some reason
 long FXColorWell::onUngrabbed(FXObject* sender,FXSelector sel,void* ptr){
   FXFrame::onUngrabbed(sender,sel,ptr);
-  flags&=~FLAG_PRESSED;
+  flags&=~(FLAG_TRYDRAG|FLAG_DODRAG);
   flags|=FLAG_UPDATE;
   endDrag(FALSE);
   return 1;
@@ -611,7 +650,6 @@ long FXColorWell::onUngrabbed(FXObject* sender,FXSelector sel,void* ptr){
 
 // Change from another Color Well
 long FXColorWell::onChgColorWell(FXObject*,FXSelector,void* ptr){
-  FXTRACE((100,"%s::onChgColorWell %08x\n",getClassName(),this));
   flags&=~FLAG_UPDATE;
   setRGBA((FXColor)(long)ptr);
   if(target) target->handle(this,MKUINT(message,SEL_CHANGED),ptr);
@@ -621,7 +659,6 @@ long FXColorWell::onChgColorWell(FXObject*,FXSelector,void* ptr){
 
 // Command from another Color Well
 long FXColorWell::onCmdColorWell(FXObject*,FXSelector,void* ptr){
-  FXTRACE((100,"%s::onCmdColorWell %08x\n",getClassName(),this));
   setRGBA((FXColor)(long)ptr);
   if(target) target->handle(this,MKUINT(message,SEL_COMMAND),ptr);
   flags|=FLAG_UPDATE;
@@ -661,8 +698,9 @@ void FXColorWell::setTipText(const FXString& text){
   }
 
 
-// Change RGBA color 
+// Change RGBA color
 void FXColorWell::setRGBA(FXColor clr){
+  if(options&COLORWELL_OPAQUEONLY) clr|=FXRGBA(0,0,0,255);
   if(clr!=rgba){
     rgba=clr;
     wellColor[0]=rgbaoverwhite(rgba);
@@ -672,6 +710,12 @@ void FXColorWell::setRGBA(FXColor clr){
   }
 
 
+// Get RGBA color
+FXColor FXColorWell::getRGBA() const {
+  return rgba;
+  }
+
+
 // Set color
 long FXColorWell::onCmdSetValue(FXObject*,FXSelector,void* ptr){
   setRGBA((FXColor)(long)ptr);
@@ -681,7 +725,6 @@ long FXColorWell::onCmdSetValue(FXObject*,FXSelector,void* ptr){
 
 // Update value from a message
 long FXColorWell::onCmdSetIntValue(FXObject*,FXSelector,void* ptr){
-  if(ptr==NULL){ fxerror("%s::onCmdSetIntValue: NULL pointer.\n",getClassName()); }
   setRGBA(*((FXColor*)ptr));
   return 1;
   }
@@ -689,15 +732,26 @@ long FXColorWell::onCmdSetIntValue(FXObject*,FXSelector,void* ptr){
 
 // Obtain value from text field
 long FXColorWell::onCmdGetIntValue(FXObject*,FXSelector,void* ptr){
-  if(ptr==NULL){ fxerror("%s::onCmdGetIntValue: NULL pointer.\n",getClassName()); }
   *((FXColor*)ptr)=getRGBA();
   return 1;
   }
 
 
-// Get RGBA color
-FXColor FXColorWell::getRGBA() const { 
-  return rgba; 
+// Return true if only opaque colors allowed
+FXbool FXColorWell::isOpaqueOnly() const {
+  return (options&COLORWELL_OPAQUEONLY)!=0;
+  }
+
+
+// Change opaque only mode
+void FXColorWell::setOpaqueOnly(FXbool opaque){
+  if(opaque){
+    options|=COLORWELL_OPAQUEONLY;
+    setRGBA(rgba);
+    }
+  else{
+    options&=~COLORWELL_OPAQUEONLY;
+    }
   }
 
 
@@ -712,7 +766,7 @@ void FXColorWell::save(FXStream& store) const {
 
 
 // Load data
-void FXColorWell::load(FXStream& store){ 
+void FXColorWell::load(FXStream& store){
   FXFrame::load(store);
   store >> wellColor[0] >> wellColor[1];
   store >> rgba;
@@ -732,9 +786,9 @@ FXColorWell::~FXColorWell(){
 //   FXint tox,toy;
 //   Window tmp;
 //   Window www=getWindowAt(ev->root_x,ev->root_y);
-//   
+//
 //   XTranslateCoordinates(getDisplay(),XDefaultRootWindow(getDisplay()),www,ev->root_x,ev->root_y,&tox,&toy,&tmp);
-// 
+//
 //   se.xbutton.type=ButtonPress;
 //   se.xbutton.serial=0;
 //   se.xbutton.send_event=1;
@@ -752,7 +806,7 @@ FXColorWell::~FXColorWell(){
 //   se.xbutton.same_screen=TRUE;
 // fprintf(stderr,"SendEvent to window %d\n",se.xbutton.window);
 //   XSendEvent(getDisplay(),se.xbutton.window,True,NoEventMask,&se);
-//   
+//
 //   se.xbutton.type=ButtonRelease;
 //   se.xbutton.serial=0;
 //   se.xbutton.send_event=1;
