@@ -1,35 +1,40 @@
 /********************************************************************************
 *                                                                               *
-*           D e v i c e   C o n t e x t   F o r  P o s t S c r i p t            *
+*           D e v i c e   C o n t e x t   F o r   P r i n t i n g               *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * Contributed by celer@ipro.lug.usf.edu                                         *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXDCPrint.cpp,v 1.2 1999/10/15 14:33:10 gui Exp $                    *
+* $Id: FXDCPrint.cpp,v 1.22 2001/08/23 16:24:52 jeroen Exp $                    *
 ********************************************************************************/
 #include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "fxkeys.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXApp.h"
@@ -45,6 +50,7 @@
 #include "FXComposite.h"
 #include "FXRootWindow.h"
 #include "FXShell.h"
+#include "FXRegion.h"
 #include "FXDC.h"
 #include "FXDCPrint.h"
 
@@ -68,11 +74,6 @@
     cool to drag and drop [E]PS into apps...
 */
 
-/*******************************************************************************/
-
-
-// Object implementation
-FXIMPLEMENT(FXDCPrint,FXDC,NULL,0)
 
 
 /*******************************************************************************/
@@ -80,7 +81,7 @@ FXIMPLEMENT(FXDCPrint,FXDC,NULL,0)
 
 // Construct
 FXDCPrint::FXDCPrint(FXApp* a):FXDC(a){
-  font=getApp()->normalFont;
+  font=getApp()->getNormalFont();
   psout=NULL;
   mediawidth=0.0;
   mediaheight=0.0;
@@ -149,50 +150,50 @@ void FXDCPrint::tfm(FXfloat& xo,FXfloat& yo,FXfloat xi,FXfloat yi){
 // Generate print job prolog
 FXbool FXDCPrint::beginPrint(FXPrinter& job){
   int numpages;
-  
+
   Yr=480;  // This is essentially the height of the page(used so that the upper left hand corner is the origin)
   Xr=0;
 
   // Print to file
   if(job.flags&PRINT_DEST_FILE){
-    psout=fopen(job.name,"w");
+    psout=fopen(job.name.text(),"w");
     if(!psout) return FALSE;
     }
 
   // Print to printer
   else{
     char buffer[1000];
-    const FXchar* printercmd=getApp()->reg().readStringEntry("SETTINGS","print","lpr -P");
-    sprintf(buffer,"%s%s",printercmd,job.name);
+    const FXchar* printercmd=getApp()->reg().readStringEntry("PRINTER","command","lpr -P%s -#%d");
+    sprintf(buffer,printercmd,job.name.text(),job.numcopies);
 #ifdef WIN32
 #ifndef _WINDOWS
 #ifdef __CYGWIN__
     psout=popen(buffer,"w");
 #else
     psout=_popen(buffer,"w"); // _popen() available for console apps only!
-#endif /* __CYGWIN__ */
+#endif
 #else
     psout=0;
-#endif /* !_WINDOWS */
+#endif
 #else
     psout=popen(buffer,"w");
-#endif /* WIN32 */
+#endif
     if(!psout) return FALSE;
     }
 
   // Copy flags
   flags=job.flags;
-  
+
   // This determines transformations
   mediawidth=(FXfloat)job.mediawidth;
   mediaheight=(FXfloat)job.mediaheight;
-  
+
   // Set media bb; this determines transformation
   mediabb.xmin=(FXfloat)job.leftmargin;
   mediabb.xmax=(FXfloat)(job.mediawidth-job.rightmargin);
   mediabb.ymin=(FXfloat)job.bottommargin;
   mediabb.ymax=(FXfloat)(job.mediaheight-job.topmargin);
-  
+
   // Initialize page and document bb from media bb
   pagebb=mediabb;
   docbb=mediabb;
@@ -229,7 +230,7 @@ FXbool FXDCPrint::beginPrint(FXPrinter& job){
   else if(flags&PRINT_PAGES_RANGE){
     numpages=1+job.topage-job.frompage;
     }
-  
+
   // How many pages are coming
   if(numpages==0){
     outf("%%%%Pages: (atend)\n");
@@ -237,13 +238,13 @@ FXbool FXDCPrint::beginPrint(FXPrinter& job){
   else{
     outf("%%%%Pages: %d\n",numpages);
     }
-  
+
   outf("%%%%DocumentFonts:\n");
   outf("%%%%EndComments\n");
-  
+
   // Procedure definitions
   outf("%%%%BeginProlog\n\n\n");
-  
+
   // Various definitions
   outf("%% h w x y drawRect\n");
   outf("/drawRect {\n\tnewpath moveto dup 0 rlineto exch dup 0 exch\n\trlineto exch neg 0 rlineto neg 0 exch rlineto\n\tclosepath stroke\n} def\n");
@@ -256,12 +257,12 @@ FXbool FXDCPrint::beginPrint(FXPrinter& job){
   outf("%% x y a b ..... nsegments drawSegmt\n");
   outf("/drawSegmt {\n\tnewpath {\n\t\tmoveto lineto\n\t} repeat stroke\n} def\n");
   outf("%% x y drawPoint\n");
-  outf("/drawPoint {\n\ttranslate 1 1 scale 8 8 1 [ 8 0 0 8 0 0 ] {<0000>} image\n} def\n");	
+  outf("/drawPoint {\n\ttranslate 1 1 scale 8 8 1 [ 8 0 0 8 0 0 ] {<0000>} image\n} def\n");
   outf("%% centerx centery  startAngle endAngle radiusX radiusY drawArc\n");
-  outf("/drawArc {\n\tgsave dup 3 1 roll div dup 1 scale 6 -1 roll\n\texch div 5 1 roll  3 -2 roll arc stroke grestore\n} def\n");	
-  outf("%% (string) x y height drawText\n"); 
+  outf("/drawArc {\n\tgsave dup 3 1 roll div dup 1 scale 6 -1 roll\n\texch div 5 1 roll  3 -2 roll arc stroke grestore\n} def\n");
+  outf("%% (string) x y height drawText\n");
   outf("/drawText {\n\tgsave findfont exch scalefont setfont moveto\n\tshow grestore\n} def\n");
-  
+
   // Image operator
   outf("/bwproc\n");
   outf(" {  rgbproc\n");
@@ -280,17 +281,17 @@ FXbool FXDCPrint::beginPrint(FXPrinter& job){
   outf("     { bwproc } image\n");
   outf(" } def\n");
   outf("} if\n");
-  
+
   // For 3D
   outf("%% Color -  r g b C\n");
   outf("/C { setrgbcolor } bind def\n");
-  
+
   outf("%% Point -  x y r g b P\n");
   outf("/P { C newpath 0.5 0.0 360.0 arc closepath fill } bind def\n");
 
   outf("%% Flat Shaded Line -  x2 y2 x1 y1 r g b L\n");
   outf("/L { C newpath moveto lineto stroke } bind def\n");
-  
+
   outf("%% Smooth-shaded line -  x2 y2 r2 g2 b2 x1 y1 r1 g1 b1 SL\n");
   outf("/SL {\n");
   outf(" /b1 exch def\n");
@@ -323,7 +324,7 @@ FXbool FXDCPrint::beginPrint(FXPrinter& job){
 
   outf("%% Flat-shaded triangle - x3 y3 x2 y2 x1 y1 r g b T\n");
   outf("/T { C newpath moveto lineto lineto closepath fill } bind def\n");
-  
+
   outf("%% Smooth-shaded triangle - x3 y3 r3 g3 b3 x2 y2 r2 g2 b2 x1 y1 r1 g1 b1 ST\n");
   outf("/ST {\n");
   outf(" /b1 exch def\n");
@@ -383,23 +384,23 @@ FXbool FXDCPrint::beginPrint(FXPrinter& job){
   // End of prologue
   outf("%%%%EndProlog\n");
 
-  
+
   // Document setup
   outf("%%%%BeginSetup\n");
   outf("/#copies %d def\n",job.numcopies);
   outf("%%%%EndSetup\n");
-  
+
   // Keep track of #pages
   pagecount=0;
-  
-  return TRUE; 
+
+  return TRUE;
   }
 
 
 // Generate print job epilog
-FXbool FXDCPrint::endPrint(){ 
+FXbool FXDCPrint::endPrint(){
   outf("%%%%Trailer\n");
-  
+
   // We now know the bounding box
   if(flags&PRINT_NOBOUNDS){
     if(docbb.xmin<docbb.xmax && docbb.ymin<docbb.ymax){
@@ -409,25 +410,25 @@ FXbool FXDCPrint::endPrint(){
       outf("%%%%BoundingBox: 0 0 100 100\n");   // Gotta come up with something...
       }
     }
-  
+
   // We now know the page count
   if(!(flags&(PRINT_PAGES_ODD|PRINT_PAGES_EVEN|PRINT_PAGES_RANGE))){
     outf("%%%%Pages: %d\n",pagecount);
     }
-  
+
   // Done!
   outf("%%%%EOF\n");
   fclose((FILE*)psout);
-  return TRUE; 
+  return TRUE;
   }
 
 
 // Generate begin of page
 FXbool FXDCPrint::beginPage(FXuint page){
-  
+
   // Output page number
   outf("%%%%Page: %d\n",page);
-  
+
   // Reset page bounding box
   if(flags&PRINT_NOBOUNDS){
     pagebb.xmin= 1000000;
@@ -436,7 +437,7 @@ FXbool FXDCPrint::beginPage(FXuint page){
     pagebb.ymax=-1000000;
     outf("%%%%PageBoundingBox: (atend)\n");
     }
-  
+
   // Use the doc bounding box
   else{
     pagebb.xmin=docbb.xmin;
@@ -445,24 +446,24 @@ FXbool FXDCPrint::beginPage(FXuint page){
     pagebb.ymax=docbb.ymax;
     outf("%%%%PageBoundingBox: %d %d %d %d\n",(int)pagebb.xmin,(int)pagebb.ymin,(int)pagebb.xmax,(int)pagebb.ymax);
     }
-  
-  // Page setup 
+
+  // Page setup
   outf("%%%%BeginPageSetup\n");
   outf("%%%%EndPageSetup\n");
   outf("gsave\n");
-  
+
   // Maybe in landscape?
   if(flags&PRINT_LANDSCAPE){
     outf("%g %g translate\n",mediawidth,0.0);
     outf("90 rotate\n");
     }
-  
-  return TRUE; 
+
+  return TRUE;
   }
 
 
 // Generate end of page
-FXbool FXDCPrint::endPage(){ 
+FXbool FXDCPrint::endPage(){
   /*
   outf("0 0 0 setcolor\n");
   outf("newpath\n");
@@ -495,12 +496,12 @@ FXbool FXDCPrint::endPage(){
   outf("showpage\n");
   outf("grestore\n");
   pagecount++;
-  return TRUE; 
+  return TRUE;
   }
 
 
 // Draw a point in the current pen color
-void FXDCPrint::drawPoint(FXint x,FXint y){ 
+void FXDCPrint::drawPoint(FXint x,FXint y){
   FXfloat xx,yy;
   tfm(xx,yy,(FXfloat)x,(FXfloat)y);
   bbox(xx,yy);
@@ -508,9 +509,9 @@ void FXDCPrint::drawPoint(FXint x,FXint y){
   }
 
 
-// Draw points in the current pen color. 
+// Draw points in the current pen color.
 // Each point's position is relative to the drawable's origin (as usual).
-void FXDCPrint::drawPoints(const FXPoint* points,FXuint npoints){ 
+void FXDCPrint::drawPoints(const FXPoint* points,FXuint npoints){
   register FXuint i;
   FXfloat xx,yy;
   for(i=0; i<npoints; i++){
@@ -525,7 +526,7 @@ void FXDCPrint::drawPoints(const FXPoint* points,FXuint npoints){
 // relative to the drawable's origin, but each subsequent point's position
 // is relative to the previous point's position; each FXPoint defines
 // the relative coordinates. Think LOGO.
-void FXDCPrint::drawPointsRel(const FXPoint* points,FXuint npoints){
+void FXDCPrint::drawPointsRel(const FXPoint*,FXuint){
   }
 
 
@@ -542,7 +543,7 @@ void FXDCPrint::drawLine(FXint x1,FXint y1,FXint x2,FXint y2){
 
 // Draw multiple lines. All points are drawn connected.
 // Each point is specified relative to Drawable's origin.
-void FXDCPrint::drawLines(const FXPoint* points,FXuint npoints){ 
+void FXDCPrint::drawLines(const FXPoint* points,FXuint npoints){
   register FXuint i;
   FXfloat xx,yy;
   if(npoints<2) return;
@@ -588,9 +589,9 @@ void FXDCPrint::drawLineSegments(const FXSegment* segments,FXuint nsegments){
     outf(" %d %d %d %d",
 		segments[i].x1,Yr-segments[i].y1,
 		segments[i].x2,Yr-segments[i].y2);
-  outf(" %d drawSegmt\n",nsegments);  	
+  outf(" %d drawSegmt\n",nsegments);
   }
- 
+
 
 // Draw unfilled rectangle
 void FXDCPrint::drawRectangle(FXint x,FXint y,FXint w,FXint h){
@@ -610,23 +611,20 @@ void FXDCPrint::drawRectangles(const FXRectangle* rectangles,FXuint nrectangles)
     }
   }
 
-
-// Draw arc
+// Draw arc (patch from: sancelot@crosswinds.net)
 void FXDCPrint::drawArc(FXint x,FXint y,FXint w,FXint h,FXint ang1,FXint ang2){
-  FXfloat startAngle,endAngle;
-	
+  FXfloat startAngle,endAngle,xx,yy,yr;
   startAngle=((float)ang1)/64.0f;
   endAngle=startAngle+(((float) ang2)/64.0f);
-
-  outf("%1.3f %1.3f %1.3f %1.3f %1.3f %1.3f drawArc\n",
-	    x+(w/2.0),Yr-y-(h/2.0),
-	    startAngle,endAngle,
-	    w/2.0,h/2.0);
+  tfm(xx,yy,(FXfloat)x,(FXfloat)y);
+  tfm(xx,yr,(FXfloat)x,(FXfloat)Yr);
+  outf("%1.3f %1.3f %1.3f %1.3f %1.3f %1.3f drawArc\n",xx+(w/2.0),yy-(h/2.0),startAngle,endAngle,w/2.0,h/2.0);
   }
 
 
+
 // Draw arcs
-void FXDCPrint::drawArcs(const FXArc* arcs,FXuint narcs){
+void FXDCPrint::drawArcs(const FXArc*,FXuint){
   }
 
 
@@ -651,12 +649,12 @@ void FXDCPrint::fillRectangles(const FXRectangle* rectangles,FXuint nrectangles)
 
 
 // Fill arc
-void FXDCPrint::fillArc(FXint x,FXint y,FXint w,FXint h,FXint ang1,FXint ang2){
+void FXDCPrint::fillArc(FXint,FXint,FXint,FXint,FXint,FXint){
   }
 
 
 // Fill arcs
-void FXDCPrint::fillArcs(const FXArc* arcs,FXuint narcs){
+void FXDCPrint::fillArcs(const FXArc*,FXuint){
   }
 
 
@@ -678,77 +676,82 @@ void FXDCPrint::fillPolygon(const FXPoint* points,FXuint npoints){
 
 
 // Fill concave polygon
-void FXDCPrint::fillConcavePolygon(const FXPoint* points,FXuint npoints){
+void FXDCPrint::fillConcavePolygon(const FXPoint*,FXuint){
   }
 
 
 // Fill complex (self-intersecting) polygon
-void FXDCPrint::fillComplexPolygon(const FXPoint* points,FXuint npoints){
+void FXDCPrint::fillComplexPolygon(const FXPoint*,FXuint){
   }
 
 
 // Filled simple polygon with relative points
-void FXDCPrint::fillPolygonRel(const FXPoint* points,FXuint npoints){
+void FXDCPrint::fillPolygonRel(const FXPoint*,FXuint){
   }
 
 
 // Fill concave polygon
-void FXDCPrint::fillConcavePolygonRel(const FXPoint* points,FXuint npoints){
+void FXDCPrint::fillConcavePolygonRel(const FXPoint*,FXuint){
   }
 
 
 // Fill complex (self-intersecting) polygon
-void FXDCPrint::fillComplexPolygonRel(const FXPoint* points,FXuint npoints){
+void FXDCPrint::fillComplexPolygonRel(const FXPoint*,FXuint){
   }
 
 
 // Draw string (only foreground bits)
-void FXDCPrint::drawText(FXint x,FXint y,const FXchar* string,FXuint length){
+void FXDCPrint::drawText(FXint x,FXint y,const FXchar* string,FXuint){
   FXfloat xx,yy;
   tfm(xx,yy,(FXfloat)x,(FXfloat)y);
-  
+
   outf("(%s) %g %g %d /Courier drawText\n",string,xx,yy,15);
   }
 
 
 // Draw string (both foreground and background bits)
-void FXDCPrint::drawImageText(FXint x,FXint y,const FXchar* string,FXuint length){
+void FXDCPrint::drawImageText(FXint,FXint,const FXchar*,FXuint){
   }
 
 
 // Draw area from source
-void FXDCPrint::drawArea(const FXDrawable* source,FXint sx,FXint sy,FXint sw,FXint sh,FXint dx,FXint dy){
+void FXDCPrint::drawArea(const FXDrawable*,FXint,FXint,FXint,FXint,FXint,FXint){
   }
 
 
 // Draw image
-void FXDCPrint::drawImage(const FXImage* image,FXint dx,FXint dy){
+void FXDCPrint::drawImage(const FXImage*,FXint,FXint){
   }
 
 
 // Draw bitmap
-void FXDCPrint::drawBitmap(const FXBitmap* bitmap,FXint dx,FXint dy){ 
+void FXDCPrint::drawBitmap(const FXBitmap* bitmap,FXint dx,FXint dy){
   }
 
 
 // Draw icon
-void FXDCPrint::drawIcon(const FXIcon* icon,FXint dx,FXint dy){
+void FXDCPrint::drawIcon(const FXIcon*,FXint,FXint){
   }
 
 
 // Draw icon shaded
-void FXDCPrint::drawIconShaded(const FXIcon* icon,FXint dx,FXint dy){
+void FXDCPrint::drawIconShaded(const FXIcon*,FXint,FXint){
+  }
+
+
+// Draw icon sunken
+void FXDCPrint::drawIconSunken(const FXIcon*,FXint,FXint){
   }
 
 
 // Draw hashed box
-void FXDCPrint::drawHashBox(FXint x,FXint y,FXint w,FXint h,FXint b){
+void FXDCPrint::drawHashBox(FXint,FXint,FXint,FXint,FXint){
   }
 
 
 // Set foreground drawing color (brush)
 void FXDCPrint::setForeground(FXColor clr){
-  outf("%g %g %g setrgbcolor\n",FXREDVAL(clr)/255.0,FXGREENVAL(clr)/255.0,FXBLUEVAL(clr)/255.0);  
+  outf("%g %g %g setrgbcolor\n",FXREDVAL(clr)/255.0,FXGREENVAL(clr)/255.0,FXBLUEVAL(clr)/255.0);
   fg=clr;
   }
 
@@ -760,25 +763,25 @@ void FXDCPrint::setBackground(FXColor clr){
 
 
 // Set dash pattern
-void FXDCPrint::setDashes(FXuint dashoffset,const FXchar *dashlist,FXuint n){
+void FXDCPrint::setDashes(FXuint,const FXchar *,FXuint){
   }
 
 
-// Set line width  
-void FXDCPrint::setLineWidth(FXuint linewidth){ 
+// Set line width
+void FXDCPrint::setLineWidth(FXuint linewidth){
   outf("%d setlinewidth\n",linewidth);
   width=linewidth;
   }
 
 
 // Set line cap style
-void FXDCPrint::setLineCap(FXCapStyle capstyle){ 
+void FXDCPrint::setLineCap(FXCapStyle capstyle){
   FXint ncap=0;
-		
+
   if(CAP_BUTT==capstyle) ncap=0;
   if(CAP_ROUND==capstyle) ncap=1;
   if(CAP_PROJECTING==capstyle) ncap=3;
-		
+
   outf("%d setlinecap\n",ncap);
   cap=capstyle;
   }
@@ -800,8 +803,8 @@ void FXDCPrint::setLineStyle(FXLineStyle linestyle){
 void FXDCPrint::setFillStyle(FXFillStyle fillstyle){
   fill=fillstyle;
   }
-  
-  
+
+
 // Set fill rule
 void FXDCPrint::setFillRule(FXFillRule fillrule){
   rule=fillrule;
@@ -842,11 +845,25 @@ void FXDCPrint::setStipple(FXStipplePattern pat,FXint dx,FXint dy){
 
 // Set clip rectangle
 void FXDCPrint::setClipRectangle(FXint x,FXint y,FXint w,FXint h){
+  clip.x=x;
+  clip.y=y;
+  clip.w=w;
+  clip.h=h;
   }
- 
+
+
+// Set clip rectangle
+void FXDCPrint::setClipRectangle(const FXRectangle& rectangle){
+  clip=rectangle;
+  }
+
 
 // Clear clipping
 void FXDCPrint::clearClipRectangle(){
+  clip.x=0;
+  clip.y=0;
+  clip.w=32767;
+  clip.h=32767;
   }
 
 
@@ -871,5 +888,97 @@ void FXDCPrint::setTextFont(FXFont *fnt){
 
 
 // Change clip-against-child windows mode
-void FXDCPrint::clipChildren(FXbool){ 
+void FXDCPrint::clipChildren(FXbool){
   }
+
+
+
+
+/*
+
+Contrib by "Vasudeva Upadhya" <kvu@cfd1.cfdrc.com>
+FXint FXSetup::DoNTPrint(FXPrinter& printInfo){
+	PDEVMODE pDevMode = NULL;
+	LONG     lDevModeSize;
+	HANDLE   hDevMode = NULL;
+	HDC      pDC = NULL;
+	DOCINFO  docInfo;
+	FXString msg = FXString("License setup wizard was unable to print,\nto printer:")+FXString(printInfo.name);
+	FXint    xOrigin;
+	FXint    yOrigin;
+	FXint    xSize;
+	FXint    ySize;
+
+	lDevModeSize = DocumentProperties(NULL,NULL,printInfo.name,NULL,NULL,0);
+	if(lDevModeSize > 0){
+		hDevMode = GlobalAlloc(GHND,lDevModeSize);
+		pDevMode = (PDEVMODE)GlobalLock(hDevMode);
+		DocumentProperties(NULL,NULL,printInfo.name,pDevMode,NULL,DM_OUT_BUFFER);
+	  }
+	pDC = CreateDC("WINSPOOL",printInfo.name,NULL,pDevMode);
+	if(pDC){
+		xOrigin = GetDeviceCaps(pDC,PHYSICALOFFSETX);
+		yOrigin = GetDeviceCaps(pDC,PHYSICALOFFSETY);
+		xSize   = GetDeviceCaps(pDC,HORZRES);
+		ySize   = GetDeviceCaps(pDC,VERTRES);
+
+		docInfo.cbSize       = sizeof(DOCINFO);
+		docInfo.lpszDocName  = printInfo.name;
+		docInfo.lpszOutput   = NULL;
+		docInfo.lpszDatatype = NULL;
+		docInfo.fwType       = 0;
+
+		if(StartDoc(pDC,&docInfo)){
+			if(StartPage(pDC)){
+				FXint yPos  = yOrigin+200;
+				FXint x1Pos = xOrigin+50;
+				FXint x2Pos = xOrigin+600;
+				FXint n;
+
+				TextOut(pDC,x1Pos,yPos,"Customer Information:",strlen("Customer Information:"));
+				yPos = yPos+50;
+				for(n=0; n < sizeof(machineInfo)/sizeof(char*);n++){
+					TextOut(pDC,x1Pos,yPos,machineInfo[n],strlen(machineInfo[n]));
+					TextOut(pDC,x2Pos,yPos,lockID[n].text(),lockID[n].length());
+					yPos = yPos+50;
+				  }
+				yPos = yPos+50;
+				for(n=0; n < sizeof(CustomerInfo)/sizeof(char*);n++){
+					TextOut(pDC,x1Pos,yPos,CustomerInfo[n],strlen(CustomerInfo[n]));
+					TextOut(pDC,x2Pos,yPos,fieldTexts[n].text(),fieldTexts[n].length());
+					yPos = yPos+50;
+				  }
+				if(EndPage(pDC)){
+					if(EndDoc(pDC)){
+						GlobalFree((PDEVMODE)pDevMode);
+						GlobalFree((HANDLE)hDevMode);
+						DeleteDC(pDC);
+						return 1;
+					  }
+				  }
+			  }
+			else{
+				GlobalFree((PDEVMODE)pDevMode);
+				GlobalFree((HANDLE)hDevMode);
+				DeleteDC(pDC);
+				FXMessageBox::error(this,MBOX_OK,"Print Error",msg.text());
+				return 0;
+			  }
+		  }
+		else{
+			GlobalFree((PDEVMODE)pDevMode);
+			GlobalFree((HANDLE)hDevMode);
+			FXMessageBox::error(this,MBOX_OK,"Print Error",msg.text());
+			return 0;
+		  }
+	  }
+	else{
+		GlobalFree((PDEVMODE)pDevMode);
+		GlobalFree((HANDLE)hDevMode);
+		FXMessageBox::error(this,MBOX_OK,"Print Error",msg.text());
+		return 0;
+	  }
+	return 1;
+  }
+
+*/
