@@ -3,45 +3,45 @@
 *                                D i a l   W i d g e t                          *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * Contributed by: Guoqing Tian                                                  *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXDial.cpp,v 1.5 1999/11/05 18:06:36 jeroen Exp $                        *
+* $Id: FXDial.cpp,v 1.16 2001/04/27 14:30:31 jeroen Exp $                       *
 ********************************************************************************/
 #include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
-#include "FXAccelTable.h"
 #include "FXApp.h"
 #include "FXId.h"
 #include "FXDC.h"
 #include "FXDCWindow.h"
 #include "FXDrawable.h"
-#include "FXImage.h"
-#include "FXIcon.h"
 #include "FXWindow.h"
 #include "FXFrame.h"
-#include "FXComposite.h"
-#include "FXLabel.h"
 #include "FXDial.h"
 
 
@@ -66,6 +66,7 @@
 FXDEFMAP(FXDial) FXDialMap[]={
   FXMAPFUNC(SEL_PAINT,0,FXDial::onPaint),
   FXMAPFUNC(SEL_MOTION,0,FXDial::onMotion),
+  FXMAPFUNC(SEL_MOUSEWHEEL,0,FXDial::onMouseWheel),
   FXMAPFUNC(SEL_LEFTBUTTONPRESS,0,FXDial::onLeftBtnPress),
   FXMAPFUNC(SEL_LEFTBUTTONRELEASE,0,FXDial::onLeftBtnRelease),
   FXMAPFUNC(SEL_UNGRABBED,0,FXDial::onUngrabbed),
@@ -110,21 +111,21 @@ FXDial::FXDial(FXComposite* p,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,F
 FXint FXDial::getDefaultWidth(){
   FXint w;
   if(options&DIAL_HORIZONTAL) w=DIALDIAMETER; else w=DIALWIDTH;
-  return w+padleft+padright+(border<<1); 
+  return w+padleft+padright+(border<<1);
   }
 
 
 // Get minimum height
-FXint FXDial::getDefaultHeight(){ 
+FXint FXDial::getDefaultHeight(){
   FXint h;
-  if(options&DIAL_HORIZONTAL) h=DIALWIDTH; else h=DIALDIAMETER; 
-  return h+padtop+padbottom+(border<<1); 
+  if(options&DIAL_HORIZONTAL) h=DIALWIDTH; else h=DIALDIAMETER;
+  return h+padtop+padbottom+(border<<1);
   }
 
 
 // We were asked about status text
 long FXDial::onQueryHelp(FXObject* sender,FXSelector,void*){
-  if(!help.empty() && (flags&FLAG_HELP)){ 
+  if(!help.empty() && (flags&FLAG_HELP)){
     sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),&help);
     return 1;
     }
@@ -144,35 +145,35 @@ long FXDial::onQueryTip(FXObject* sender,FXSelector,void*){
 
 // Update value from a message
 long FXDial::onCmdSetValue(FXObject*,FXSelector,void* ptr){
-  setPosition((FXint)(long)ptr);
+  setValue((FXint)(long)ptr);
   return 1;
   }
 
 
 // Update value from a message
 long FXDial::onCmdSetIntValue(FXObject*,FXSelector,void* ptr){
-  setPosition(*((FXint*)ptr));
+  setValue(*((FXint*)ptr));
   return 1;
   }
 
 
 // Update value from a message
 long FXDial::onCmdSetRealValue(FXObject*,FXSelector,void* ptr){
-  setPosition((FXint)*((FXdouble*)ptr));
+  setValue((FXint)*((FXdouble*)ptr));
   return 1;
   }
 
 
 // Obtain value from text field
 long FXDial::onCmdGetIntValue(FXObject*,FXSelector,void* ptr){
-  *((FXint*)ptr)=getPosition();
+  *((FXint*)ptr)=getValue();
   return 1;
   }
 
 
 // Obtain value from text field
 long FXDial::onCmdGetRealValue(FXObject*,FXSelector,void* ptr){
-  *((FXdouble*)ptr) = (FXdouble)getPosition();
+  *((FXdouble*)ptr) = (FXdouble)getValue();
   return 1;
   }
 
@@ -186,8 +187,7 @@ long FXDial::onCmdSetIntRange(FXObject*,FXSelector,void* ptr){
 
 // Get range with a message
 long FXDial::onCmdGetIntRange(FXObject*,FXSelector,void* ptr){
-  ((FXint*)ptr)[0]=range[0];
-  ((FXint*)ptr)[1]=range[1];
+  getRange(((FXint*)ptr)[0],((FXint*)ptr)[1]);
   return 1;
   }
 
@@ -211,13 +211,13 @@ long FXDial::onCmdGetRealRange(FXObject*,FXSelector,void* ptr){
 long FXDial::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
   FXEvent *event=(FXEvent*)ptr;
   flags&=~FLAG_TIP;
+  handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
   if(isEnabled()){
-    handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
     grab();
     if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONPRESS),ptr)) return 1;
-    if(options&DIAL_HORIZONTAL) 
-      dragpoint=event->win_x; 
-    else 
+    if(options&DIAL_HORIZONTAL)
+      dragpoint=event->win_x;
+    else
       dragpoint=event->win_y;
     dragpos=pos;
     flags|=FLAG_PRESSED;
@@ -255,7 +255,7 @@ long FXDial::onUngrabbed(FXObject* sender,FXSelector sel,void* ptr){
   }
 
 
-// Moving 
+// Moving
 long FXDial::onMotion(FXObject*,FXSelector,void* ptr){
   FXEvent *event=(FXEvent*)ptr;
   FXint travel,size,delta,newpos,tmp;
@@ -295,7 +295,46 @@ long FXDial::onMotion(FXObject*,FXSelector,void* ptr){
   }
 
 
-// Handle repaint 
+// Mouse wheel (Thanks to  "Lyle Johnson" <ljohnson@resgen.com>)
+long FXDial::onMouseWheel(FXObject*,FXSelector,void* ptr){
+  FXEvent *event=(FXEvent*)ptr;
+  FXint delta,newpos,tmp;
+
+  // Any motion?
+  if(event->code){
+    /**
+     * Determine the change in dial units; this probably still needs
+     * tweaking. The formula below adjusts the dial position by 1/36
+     * of a revolution for each "hop" of the mousewheel.
+     */
+    delta=event->code/120*incr/36;
+
+    // Determine new dial position
+    if(options&DIAL_CYCLIC){
+      tmp=pos+delta-range[0];
+      while(tmp<0) tmp+=(range[1]-range[0]+1);
+      newpos=range[0]+tmp%(range[1]-range[0]+1);
+      }
+    else{
+      if(pos+delta<range[0]) newpos=range[0];
+      else if(pos+delta>range[1]) newpos=range[1];
+      else newpos=pos+delta;
+      }
+    if(pos!=newpos){
+      pos=newpos;
+      FXASSERT(range[0]<=pos && pos<=range[1]);
+      notchangle=(notchoffset+(3600*(pos-range[0]))/incr)%3600;
+      update(border+padleft+1,border+padtop+1,width-(border<<1)-padleft-padright-2,height-(border<<1)-padtop-padbottom-2);
+      if(target) target->handle(this,MKUINT(message,SEL_CHANGED),(void*)pos);
+      if(target) target->handle(this,MKUINT(message,SEL_COMMAND),(void*)pos);
+      return 1;
+      }
+    }
+  return 0;
+  }
+
+
+// Handle repaint
 long FXDial::onPaint(FXObject*,FXSelector,void* ptr){
   const FXdouble fac=0.5*PI/((FXdouble)(NUMSIDECOLORS-1));
   FXEvent *ev=(FXEvent*)ptr;
@@ -304,22 +343,22 @@ long FXDial::onPaint(FXObject*,FXSelector,void* ptr){
   FXint lt,rt,tp,bm;
   FXdouble mid,tmp;
   FXDCWindow dc(this,ev);
-  
+
   // Paint background
   dc.setForeground(backColor);
   dc.fillRectangle(ev->rect.x,ev->rect.y,ev->rect.w,ev->rect.h);
-  
+
   FXASSERT(0<=notchangle && notchangle<3600);
   off=(notchangle+3600)%notchspacing;
   fm=off/notchspacing;
   to=(off+1800-notchspacing+1)/notchspacing;
-  
+
   // Rectangle of dial
   lt=border+padleft+1;
   rt=width-border-padright-2;
   tp=border+padtop+1;
   bm=height-border-padbottom-2;
-  
+
   // Colors for sides
   rmax=(126*FXREDVAL(backColor))/100;
   gmax=(126*FXGREENVAL(backColor))/100;
@@ -409,7 +448,7 @@ long FXDial::onPaint(FXObject*,FXSelector,void* ptr){
     dc.drawLine(lu,tp,ld,tp);
     dc.drawLine(lu,bm,ld,bm);
     }
-  
+
   // Vertical dial
   else{
     size=bm-tp;
@@ -491,7 +530,7 @@ long FXDial::onPaint(FXObject*,FXSelector,void* ptr){
     dc.drawLine(lt,lu,lt,ld);
     dc.drawLine(rt,lu,rt,ld);
     }
-  
+
   // Border
   drawFrame(dc,0,0,width,height);
 
@@ -502,7 +541,7 @@ long FXDial::onPaint(FXObject*,FXSelector,void* ptr){
   }
 
 
-// Set dial range  
+// Set dial range
 void FXDial::setRange(FXint lo,FXint hi){
   if(lo>hi){ fxerror("%s::setRange: trying to set negative range.\n",getClassName()); }
   if(range[0]!=lo || range[1]!=hi){
@@ -516,8 +555,8 @@ void FXDial::setRange(FXint lo,FXint hi){
   }
 
 
-// Set dial position
-void FXDial::setPosition(FXint p){
+// Set dial value
+void FXDial::setValue(FXint p){
   if(p<range[0]) p=range[0];
   if(p>range[1]) p=range[1];
   if(p!=pos){
@@ -554,6 +593,7 @@ void FXDial::setNotchOffset(FXint offset){
   if(offset<-3600) offset=-3600;
   offset=(offset+3600)%3600;
   if(offset!=notchoffset){
+    notchoffset=offset;
     notchangle=(notchoffset+(3600*(pos-range[0]))/incr)%3600;
     update();
     }
@@ -562,7 +602,7 @@ void FXDial::setNotchOffset(FXint offset){
 
 // Get dial options
 FXuint FXDial::getDialStyle() const {
-  return (options&DIAL_MASK); 
+  return (options&DIAL_MASK);
   }
 
 
@@ -603,7 +643,16 @@ void FXDial::load(FXStream& store){
   store >> pos;
   store >> help;
   store >> tip;
-  }  
+  }
+
+
+// Change the Center Notch color
+void FXDial::setNotchColor(FXColor clr){
+  if(clr!=notchColor){
+    notchColor=clr;
+    update();
+    }
+  }
 
 
 // Change help text
