@@ -3,31 +3,37 @@
 *                     D i r e c t o r y   B o x   O b j e c t                   *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1999 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1999,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXDirBox.cpp,v 1.4 1999/11/05 18:06:36 jeroen Exp $                      *
+* $Id: FXDirBox.cpp,v 1.17 2001/08/22 04:01:55 jeroen Exp $                     *
 ********************************************************************************/
 #include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "fxkeys.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXFile.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXObjectList.h"
@@ -51,7 +57,7 @@
 #include "FXShell.h"
 #include "FXPopup.h"
 #include "FXScrollbar.h"
-#include "FXScrollWindow.h"
+#include "FXScrollArea.h"
 #include "FXTreeList.h"
 #include "FXTreeListBox.h"
 #include "FXDirBox.h"
@@ -233,7 +239,7 @@ const unsigned char folder[]={
   0xc7,0x74,0x11,0xef,0xb3,0x5a,0xdf,0x9e,0x1c,0x6f,0x97,0x03,0xba,0x7c,0xa1,0x64,
   0x48,0x05,0x20,0x38,0x9f,0x50,0xe8,0x66,0x4a,0x75,0x24,0x00,0x00,0x3b
   };
-  
+
 
 
 /*******************************************************************************/
@@ -265,16 +271,21 @@ FXDirBox::FXDirBox(FXComposite *p,FXint nvis,FXObject* tgt,FXSelector sel,FXuint
   computericon=new FXBMPIcon(getApp(),mycomputer,0,IMAGE_ALPHAGUESS);
   nethoodicon=new FXBMPIcon(getApp(),nethood,0,IMAGE_ALPHAGUESS);
   zipdiskicon=new FXBMPIcon(getApp(),zipdisk,0,IMAGE_ALPHAGUESS);
-#ifndef FX_NATIVE_WIN32
+#ifndef WIN32
   addItemLast(NULL,"/",foldericon,foldericon);
   setCurrentItem(getFirstItem());
   directory="/";
 #else
+  /*
   FXchar result[1024];
   fxgetcurrentdir(result);
   directory=FXString(result).left(2)+PATHSEPSTRING;
   addItemLast(NULL,directory,harddiskicon,harddiskicon);
   setCurrentItem(getFirstItem());
+  */
+  FXchar result[1024];
+  fxgetcurrentdir(result);
+  setDirectory(FXString(result).left(2)+PATHSEPSTRING);
 #endif
   }
 
@@ -379,21 +390,51 @@ long FXDirBox::onCommand(FXObject*,FXSelector,void* ptr){
   return 1;
   }
 
-                
+
 // Set directory
-void FXDirBox::setDirectory(const FXString& path){
+void FXDirBox::setDirectory(const FXString& pathname){
+  /*
+  FXTreeItem *item;
+  FXString    string;
+  FXint       part;
+  if(!pathname.empty()){
+    FXTRACE((100,"%s::setDirectory(%s)\n",getClassName(),pathname.text()));
+    FXString path=FXFile::absolute(pathname);
+    while(!FXFile::isTopDirectory(path) && !FXFile::isDirectory(path)){
+      path=FXFile::upLevel(path);
+      }
+    if(directory!=path){
+      directory=path;
+      clearItems();
+#ifndef WIN32
+      item=addItemFirst(NULL,PATHSEPSTRING,foldericon,foldericon);
+      part=1;
+      while(1){
+        string=directory.extract(part,PATHSEP);
+        if(string.empty()) break;
+        item=addItemLast(item,string,foldericon,foldericon);
+        part++;
+        }
+      setCurrentItem(item);
+#else
+#endif
+      recalc();
+      }
+    }
+  */
+
   FXchar abspath[MAXPATHLEN+1];
   FXString string;
   FXTreeItem *item;
   FXuint part=1;
-  
+
   // Empty path:- ignore
-  if(path.empty()) return;
-  
+  if(pathname.empty()) return;
+
   // Exactly equal already
-  if(directory!=path){
-    fxabspath(abspath,NULL,path.text());
-    
+  if(directory!=pathname){
+    fxabspath(abspath,NULL,pathname.text());
+
     // Equal
     if(directory!=abspath){
 
@@ -403,10 +444,10 @@ void FXDirBox::setDirectory(const FXString& path){
         }
 
       // Switch
-#ifndef FX_NATIVE_WIN32
+#ifndef WIN32
       if(directory!=abspath){
         directory=abspath;
-        removeAllItems();
+        clearItems();
         item=addItemFirst(NULL,PATHSEPSTRING,foldericon,foldericon);    // Add root
         while(1){
           string=directory.extract(part,PATHSEP);
@@ -420,7 +461,7 @@ void FXDirBox::setDirectory(const FXString& path){
 #else
       if(directory!=abspath){
         directory=abspath;
-        removeAllItems();
+        clearItems();
 
         //FXTreeItem *desktopitem=addItemLast(NULL,"Desktop",desktopicon,desktopicon);
 	//FXTreeItem *computeritem=addItemLast(desktopitem,"My Computer",computericon,computericon);
@@ -475,8 +516,11 @@ void FXDirBox::setDirectory(const FXString& path){
 		    FXASSERT(FALSE);
 		  }
 
-                // Fix submitted by sancelot@crosswinds.net
-                if(islower((int) directory[0])) directory[0] = (char) toupper((int) directory[0]);
+                // Fix from John Burnell <j.burnell@irl.cri.nz> lpParse is NOT always uppercase.
+                if(islower((FXuchar)lpParse[0]))
+                  directory[0]=tolower((FXuchar)directory[0]);
+                else
+                  directory[0]=toupper((FXuchar)directory[0]);
 
 		// Is the selected directory on this drive?
 		if(directory[0]==lpParse[0]){
@@ -536,7 +580,7 @@ void FXDirBox::load(FXStream& store){
   store >> computericon;
   store >> nethoodicon;
   store >> zipdiskicon;
-  }  
+  }
 
 
 // Delete it
