@@ -3,25 +3,26 @@
 *                           O p e n G L   O b j e c t                           *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXGLObject.cpp,v 1.1 1999/09/14 13:41:23 jeroen Exp $                   *
+* $Id: FXGLObject.cpp,v 1.15 2001/07/20 22:18:02 jeroen Exp $                   *
 ********************************************************************************/
 #include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
 #include "FXVec.h"
@@ -30,8 +31,12 @@
 #include "FXHMat.h"
 #include "FXRange.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXObjectList.h"
@@ -40,6 +45,7 @@
 #include "FXDrawable.h"
 #include "FXWindow.h"
 #include "FXCursor.h"
+#include "FXCanvas.h"
 #include "FXGLCanvas.h"
 #include "FXDC.h"
 #include "FXDCPrint.h"
@@ -59,47 +65,47 @@
   Notes:
   - Leaf objects don't push any names!
   - Group objects should do focus traversal.
-  
-  - Suggestion for transform nodes... "Mike Fletcher" <mcfletch@vrtelecom.com>
-  class FXGLTransform( FXGLGroup ):
-  
- def draw( self, viewer):
-         # The transform node sets up a transformation matrix
-         # then renders all children
-         # then removes that transformation matrix
-         # from the stack.
-         GL.PushMatrix () # store previous
-         if self.translation: # three tupple
-                 apply (GL.glTranslatef, self.translation )
-         if self.center: #three tupple
-                 apply (GL.glTranslatef, self.center )
-         # rotation of angle ra around axis rx,ry,rz
-         if self.rotation: # 4 tupple
-                 rx,ry,rz,ra = self.rotation
-                 apply (GL.glRotatef, (saa*self.radtodeg, sax,say,saz) )
-         if self.scale: # three tupple
-                 if self.scaleOrientation: # 4 tupple
-                         sax,say,saz,saa = self.scaleOrientation
-                         apply (GL.glRotatef, (saa*self.radtodeg, sax,say,saz) )
-                 apply (GL.glScalef, tuple( self.scale ) )
-                 # reverse the scaleOrientation
-                 if self.scaleOrientation:
-                         apply (GL.glRotatef, (-saa*self.radtodeg, sax,say,saz) )
-         # now reverse center offset
-         if self.center:
-                 cx,cy,cz = self.center
-                 apply (GL.glTranslatef, ( -cx,-cy,-cz) )
-         # draw children here...
-         FXGLGroup.draw( self, viewer)
-         # now remove the transformation matrix
-         PopMatrix()
 
-Needs to do the same calculation for the pick pass as well...  Note that it
-is fairly easy to calculate the entire matrix at one go whenever it changes,
-thereby saving the system most of the work of calculation per-frame (a
-single matrix multiply with the current "surrounding" matrix
- GL.glGetFloatv(GL_MODELVIEW_MATRIX, ModelView) ).  I have not profiled the
-difference between them, but I would assume a cached matrix would be faster.
+  - Suggestion for transform nodes... "Mike Fletcher" <mcfletch@vrtelecom.com>
+    class FXGLTransform( FXGLGroup ):
+
+    def draw( self, viewer):
+            # The transform node sets up a transformation matrix
+            # then renders all children
+            # then removes that transformation matrix
+            # from the stack.
+            GL.PushMatrix () # store previous
+            if self.translation: # three tupple
+                    apply (GL.glTranslatef, self.translation )
+            if self.center: #three tupple
+                    apply (GL.glTranslatef, self.center )
+            # rotation of angle ra around axis rx,ry,rz
+            if self.rotation: # 4 tupple
+                    rx,ry,rz,ra = self.rotation
+                    apply (GL.glRotatef, (saa*self.radtodeg, sax,say,saz) )
+            if self.scale: # three tupple
+                    if self.scaleOrientation: # 4 tupple
+                            sax,say,saz,saa = self.scaleOrientation
+                            apply (GL.glRotatef, (saa*self.radtodeg, sax,say,saz) )
+                    apply (GL.glScalef, tuple( self.scale ) )
+                    # reverse the scaleOrientation
+                    if self.scaleOrientation:
+                            apply (GL.glRotatef, (-saa*self.radtodeg, sax,say,saz) )
+            # now reverse center offset
+            if self.center:
+                    cx,cy,cz = self.center
+                    apply (GL.glTranslatef, ( -cx,-cy,-cz) )
+            # draw children here...
+            FXGLGroup.draw( self, viewer)
+            # now remove the transformation matrix
+            PopMatrix()
+
+    Needs to do the same calculation for the pick pass as well...  Note that it
+    is fairly easy to calculate the entire matrix at one go whenever it changes,
+    thereby saving the system most of the work of calculation per-frame (a
+    single matrix multiply with the current "surrounding" matrix
+    GL.glGetFloatv(GL_MODELVIEW_MATRIX, ModelView) ).  I have not profiled the
+    difference between them, but I would assume a cached matrix would be faster.
 
 
 */
@@ -110,10 +116,7 @@ difference between them, but I would assume a cached matrix would be faster.
 // Object implementation
 FXIMPLEMENT(FXGLObject,FXObject,NULL,0)
 
-  
-// Construct
-FXGLObject::FXGLObject(){
-  }
+
 
 
 // Get bounding box
@@ -130,6 +133,10 @@ void FXGLObject::draw(FXGLViewer*){ }
 void FXGLObject::hit(FXGLViewer* viewer){ draw(viewer); }
 
 
+// Copy
+FXGLObject* FXGLObject::copy(){ return new FXGLObject(*this); }
+
+
 // Identify object by its path
 FXGLObject* FXGLObject::identify(FXuint*){ return this; }
 
@@ -137,35 +144,22 @@ FXGLObject* FXGLObject::identify(FXuint*){ return this; }
 // Return true if it can be dragged
 FXbool FXGLObject::canDrag() const { return FALSE; }
 
+
 // Return true if OK to delete object
 FXbool FXGLObject::canDelete() const { return FALSE; }
 
+
 // Drag the object
 FXbool FXGLObject::drag(FXGLViewer*,FXint,FXint,FXint,FXint){ return FALSE; }
 
 
-// Destruct
-FXGLObject::~FXGLObject(){ }
-
 
 /*******************************************************************************/
 
-// Drop
-FXDEFMAP(FXGLGroup) FXGLGroupMap[]={
-  FXMAPFUNC(SEL_FOCUSIN,0,FXGLGroup::onFocusIn),
-  FXMAPFUNC(SEL_FOCUSOUT,0,FXGLGroup::onFocusOut),
-  };
 
-  
+
 // Object implementation
-FXIMPLEMENT(FXGLGroup,FXGLObject,FXGLGroupMap,ARRAYNUMBER(FXGLGroupMap))
-
-
-
-// Contruct
-FXGLGroup::FXGLGroup(){
-  current=-1;
-  }
+FXIMPLEMENT(FXGLGroup,FXGLObject,NULL,0)
 
 
 // Get bounding box
@@ -184,20 +178,6 @@ void FXGLGroup::bounds(FXRange& box){
   }
 
 
-// Gained focus
-long FXGLGroup::onFocusIn(FXObject*,FXSelector,void* ptr){
-  FXTRACE((250,"%s::onFocusIn %08x\n",getClassName(),this));
-  return (0<=current)&&(list[current]->handle(list[current],MKUINT(0,SEL_FOCUSIN),ptr));
-  }
-
-
-// Lost focus
-long FXGLGroup::onFocusOut(FXObject*,FXSelector,void* ptr){
-  FXTRACE((250,"%s::onFocusOut %08x\n",getClassName(),this));
-  return (0<=current)&&(list[current]->handle(list[current],MKUINT(0,SEL_FOCUSOUT),ptr));
-  }
-
-
 // Draw
 void FXGLGroup::draw(FXGLViewer* viewer){
   for(FXint i=0; i<list.no(); i++) list[i]->draw(viewer);
@@ -217,8 +197,15 @@ void FXGLGroup::hit(FXGLViewer* viewer){
   }
 
 
+// Copy
+FXGLObject* FXGLGroup::copy(){
+  return new FXGLGroup(*this);
+  }
+
+
+
 // Identify object by its path
-FXGLObject* FXGLGroup::identify(FXuint* path){ 
+FXGLObject* FXGLGroup::identify(FXuint* path){
   FXASSERT(path);
   FXASSERT((FXint)path[0]<list.no());
   return list[path[0]]->identify(path+1);
@@ -242,15 +229,13 @@ FXbool FXGLGroup::drag(FXGLViewer* viewer,FXint fx,FXint fy,FXint tx,FXint ty){
 void FXGLGroup::save(FXStream& store) const {
   FXGLObject::save(store);
   list.save(store);
-  store << current;
   }
 
-      
+
 // Load object from stream
 void FXGLGroup::load(FXStream& store){
   FXGLObject::load(store);
   list.load(store);
-  store >> current;
   }
 
 
@@ -270,22 +255,23 @@ FXIMPLEMENT(FXGLPoint,FXGLObject,NULL,0)
 
 
 // Create point
-FXGLPoint::FXGLPoint():
-  pos(0.0,0.0,0.0){
+FXGLPoint::FXGLPoint():pos(0.0f,0.0f,0.0f){
   }
 
+// Copy constructor
+FXGLPoint::FXGLPoint(const FXGLPoint& orig):FXGLObject(orig),pos(orig.pos){
+  }
 
 // Create initialized point
-FXGLPoint::FXGLPoint(FXfloat x,FXfloat y,FXfloat z):
-  pos(x,y,z){
+FXGLPoint::FXGLPoint(FXfloat x,FXfloat y,FXfloat z):pos(x,y,z){
   }
 
 
 // Get bounding box
 void FXGLPoint::bounds(FXRange& box){
   box[0][0]=box[0][1]=pos[0];
-  box[1][0]=box[1][1]=pos[0];
-  box[2][0]=box[2][1]=pos[0];
+  box[1][0]=box[1][1]=pos[1];
+  box[2][0]=box[2][1]=pos[2];
   }
 
 
@@ -311,13 +297,18 @@ void FXGLPoint::hit(FXGLViewer* ){
   }
 
 
+// Copy
+FXGLObject* FXGLPoint::copy(){
+  return new FXGLPoint(*this);
+  }
+
 // Save object to stream
 void FXGLPoint::save(FXStream& store) const {
   FXGLObject::save(store);
   store << pos;
   }
 
-      
+
 // Load object from stream
 void FXGLPoint::load(FXStream& store){
   FXGLObject::load(store);
@@ -332,13 +323,17 @@ FXIMPLEMENT(FXGLLine,FXGLObject,NULL,0)
 
 
 // Create line
-FXGLLine::FXGLLine():fm(-0.5,0.0,0.0),to(0.5,0.0,0.0){
+FXGLLine::FXGLLine():fm(-0.5f,0.0f,0.0f),to(0.5f,0.0f,0.0f){
+  }
+
+
+// Copy constructor
+FXGLLine::FXGLLine(const FXGLLine& orig):FXGLObject(orig),fm(orig.fm),to(orig.to){
   }
 
 
 // Create inittialized line
-FXGLLine::FXGLLine(FXfloat fx,FXfloat fy,FXfloat fz,FXfloat tx,FXfloat ty,FXfloat tz):
-  fm(fx,fy,fz),to(tx,ty,tz){
+FXGLLine::FXGLLine(FXfloat fx,FXfloat fy,FXfloat fz,FXfloat tx,FXfloat ty,FXfloat tz):fm(fx,fy,fz),to(tx,ty,tz){
   }
 
 
@@ -374,6 +369,12 @@ void FXGLLine::hit(FXGLViewer* ){
   }
 
 
+// Copy
+FXGLObject* FXGLLine::copy(){
+  return new FXGLLine(*this);
+  }
+
+
 // Save object to stream
 void FXGLLine::save(FXStream& store) const {
   FXGLObject::save(store);
@@ -381,7 +382,7 @@ void FXGLLine::save(FXStream& store) const {
   to.save(store);
   }
 
-      
+
 // Load object from stream
 void FXGLLine::load(FXStream& store){
   FXGLObject::load(store);
