@@ -3,27 +3,28 @@
 *                      O p e n G L   S h a p e   O b j e c t                    *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1999 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1999,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * Contributed by: Angel-Ventura Mendo Gomez <ventura@labri.u-bordeaux.fr>       *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXGLShape.cpp,v 1.3 1999/09/28 20:35:44 gui Exp $                     *
+* $Id: FXGLShape.cpp,v 1.17 2001/04/27 22:23:25 jeroen Exp $                    *
 ********************************************************************************/
 #include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
 #include "FXVec.h"
@@ -32,8 +33,12 @@
 #include "FXHMat.h"
 #include "FXRange.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXObjectList.h"
@@ -42,6 +47,7 @@
 #include "FXDrawable.h"
 #include "FXWindow.h"
 #include "FXCursor.h"
+#include "FXCanvas.h"
 #include "FXGLCanvas.h"
 #include "FXDC.h"
 #include "FXDCPrint.h"
@@ -78,31 +84,31 @@ FXDEFMAP(FXGLShape) FXGLShapeMap[]={
   FXMAPFUNCS(SEL_UPDATE,FXGLShape::ID_STYLE_POINTS,FXGLShape::ID_STYLE_BOUNDINGBOX,FXGLShape::onUpdDrawingStyle)
   };
 
-  
-// Object implementation
-FXIMPLEMENT(FXGLShape,FXGLObject,FXGLShapeMap,ARRAYNUMBER(FXGLShapeMap))
 
- 
+// Object implementation
+FXIMPLEMENT_ABSTRACT(FXGLShape,FXGLObject,FXGLShapeMap,ARRAYNUMBER(FXGLShapeMap))
+
+
 
 // Serialization
 FXGLShape::FXGLShape(){
-  
+
   position=FXVec(0.0,0.0,0.0);
-  
+
   material[0].ambient=FXHVec(0.2f,0.2f,0.2f,1.0);      // Front facing material
   material[0].diffuse=FXHVec(0.8f,0.8f,0.8f,1.0);
   material[0].specular=FXHVec(1.0,1.0,1.0,1.0);
   material[0].emission=FXHVec(0.0,0.0,0.0,1.0);
   material[0].shininess=30.0;
-  
+
   material[1].ambient=FXHVec(0.2f,0.2f,0.2f,1.0);      // Back facing material
   material[1].diffuse=FXHVec(0.8f,0.8f,0.8f,1.0);
   material[1].specular=FXHVec(1.0,1.0,1.0,1.0);
   material[1].emission=FXHVec(0.0,0.0,0.0,1.0);
   material[1].shininess=30.0;
-  
+
   range=FXRange(-1.0,1.0,-1.0,1.0,-1.0,1.0);       // Ample bounding box
-  
+
   options=SHADING_SMOOTH|STYLE_SURFACE;
   }
 
@@ -139,6 +145,23 @@ FXGLShape::FXGLShape(FXfloat x,FXfloat y,FXfloat z,FXuint opts,const FXMaterial&
   }
 
 
+// Copy constructor
+FXGLShape::FXGLShape(const FXGLShape& orig):FXGLObject(orig){
+  position=orig.position;
+  material[0]=orig.material[0];
+  material[1]=orig.material[2];
+  range=orig.range;
+  options=orig.options;
+  tip=orig.tip;
+  }
+
+
+// Copy this object
+FXGLObject* FXGLShape::copy(){
+  return new FXGLShape(*this);
+  }
+
+
 // Return true if it can be dragged
 FXbool FXGLShape::canDrag() const { return TRUE; }
 
@@ -149,22 +172,18 @@ FXbool FXGLShape::canDelete() const { return TRUE; }
 
 // Handle drag-and-drop drop
 long FXGLShape::onDNDDrop(FXObject* sender,FXSelector,void*){
-  FXuchar *data; FXuint len,r,g,b,a;
-  FXHVec color;
-  if(((FXWindow*)sender)->offeredDNDType(FROM_DRAGNDROP,FXWindow::colorType)){
-    if(((FXWindow*)sender)->getDNDData(FROM_DRAGNDROP,FXWindow::colorType,data,len)){
-      sscanf((char*)data,"#%02x%02x%02x%02x",&r,&g,&b,&a);
-      color[0]=r*0.0039215686f;
-      color[1]=g*0.0039215686f;
-      color[2]=b*0.0039215686f;
-      color[3]=a*0.0039215686f;
-      material[0].ambient=color;
-      material[0].diffuse=color;
-      material[1].ambient=color;
-      material[1].diffuse=color;
-      FXFREE(&data);
-      return 1;
-      }
+  FXushort *clr; FXuint len; FXHVec color;
+  if(((FXWindow*)sender)->getDNDData(FROM_DRAGNDROP,FXWindow::colorType,(FXuchar*&)clr,len)){
+    color[0]=clr[0]/65535.0f;
+    color[1]=clr[1]/65535.0f;
+    color[2]=clr[2]/65535.0f;
+    color[3]=clr[3]/65535.0f;
+    FXFREE(&clr);
+    material[0].ambient=color;
+    material[0].diffuse=color;
+    material[1].ambient=color;
+    material[1].diffuse=color;
+    return 1;
     }
   return 0;
   }
@@ -243,7 +262,6 @@ FXbool FXGLShape::drag(FXGLViewer* viewer,FXint fx,FXint fy,FXint tx,FXint ty){
 // Draw
 void FXGLShape::draw(FXGLViewer* viewer){
 #ifdef HAVE_OPENGL
-  if(viewer->doesTurbo()) return;
 
   // Save attributes and matrix
   glPushAttrib(GL_CURRENT_BIT|GL_LIGHTING_BIT|GL_POINT_BIT|GL_LINE_BIT);
@@ -252,123 +270,131 @@ void FXGLShape::draw(FXGLViewer* viewer){
   // Object position
   glTranslatef(position[0],position[1],position[2]);
 
-  // Shading
-  if(options&(SHADING_SMOOTH|SHADING_FLAT)){
-    glEnable(GL_LIGHTING);
-    if(options&SHADING_SMOOTH){
-      glEnable(GL_AUTO_NORMAL);
-      glShadeModel(GL_SMOOTH); 
+  // Draw full object
+  if(!viewer->doesTurbo()){
+
+    // Shading
+    if(options&(SHADING_SMOOTH|SHADING_FLAT)){
+      glEnable(GL_LIGHTING);
+      if(options&SHADING_SMOOTH){
+        glEnable(GL_AUTO_NORMAL);
+        glShadeModel(GL_SMOOTH);
+        }
+      else{
+        glDisable(GL_AUTO_NORMAL);
+        glShadeModel(GL_FLAT);
+        }
       }
     else{
-      glDisable(GL_AUTO_NORMAL);
-      glShadeModel(GL_FLAT); 
+      glDisable(GL_LIGHTING);
+      }
+
+    // Material
+    if(options&SURFACE_DUALSIDED){
+      glMaterialfv(GL_FRONT,GL_AMBIENT,material[0].ambient);
+      glMaterialfv(GL_FRONT,GL_DIFFUSE,material[0].diffuse);
+      glMaterialfv(GL_FRONT,GL_SPECULAR,material[0].specular);
+      glMaterialfv(GL_FRONT,GL_EMISSION,material[0].emission);
+      glMaterialf(GL_FRONT,GL_SHININESS,material[0].shininess);
+      glMaterialfv(GL_BACK,GL_AMBIENT,material[1].ambient);
+      glMaterialfv(GL_BACK,GL_DIFFUSE,material[1].diffuse);
+      glMaterialfv(GL_BACK,GL_SPECULAR,material[1].specular);
+      glMaterialfv(GL_BACK,GL_EMISSION,material[1].emission);
+      glMaterialf(GL_BACK,GL_SHININESS,material[1].shininess);
+      }
+    else{
+      glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,material[0].ambient);
+      glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,material[0].diffuse);
+      glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,material[0].specular);
+      glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,material[0].emission);
+      glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,material[0].shininess);
+      }
+
+    // Surface
+    if(options&STYLE_SURFACE){
+      glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
+      if(options&FACECULLING_ON)
+        glEnable(GL_CULL_FACE);
+      else
+        glDisable(GL_CULL_FACE);
+      drawshape(viewer);
+      }
+
+    // Wire frame
+    if(options&STYLE_WIREFRAME){
+      glDisable(GL_LIGHTING);
+      glShadeModel(GL_FLAT);
+      glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
+      if(options&STYLE_SURFACE){                  // Wires over surface
+#if GL_EXT_polygon_offset
+        glEnable(GL_POLYGON_OFFSET_EXT);
+#ifdef GL_VERSION_1_2
+        glPolygonOffset(FACTOR, BIAS);
+#else
+        glPolygonOffsetEXT(FACTOR, BIAS);
+#endif
+        drawshape(viewer);
+        glDisable(GL_POLYGON_OFFSET_EXT);
+#elif GLU_VERSION_1_1
+        glEnable(GL_POLYGON_OFFSET_LINE);
+        glPolygonOffset(FACTOR, BIAS);
+        drawshape(viewer);
+        glDisable(GL_POLYGON_OFFSET_LINE);
+#endif
+        }
+      else{                                       // Wires by themselves
+        glDisable(GL_CULL_FACE);
+        drawshape(viewer);
+        }
+      }
+
+    // Points
+    if(options&STYLE_POINTS){
+      glDisable(GL_LIGHTING);
+      glShadeModel(GL_FLAT);
+      glPolygonMode(GL_FRONT_AND_BACK,GL_POINT);
+      if(options&STYLE_SURFACE){                  // Points over surface
+#if GL_EXT_polygon_offset
+        glEnable(GL_POLYGON_OFFSET_EXT);
+#ifdef GL_VERSION_1_2
+        glPolygonOffset(FACTOR, BIAS);
+#else
+        glPolygonOffsetEXT(FACTOR, BIAS);
+#endif
+        drawshape(viewer);
+        glDisable(GL_POLYGON_OFFSET_EXT);
+#elif GLU_VERSION_1_1
+        glEnable(GL_POLYGON_OFFSET_POINT);
+        glPolygonOffset(FACTOR, BIAS);
+        drawshape(viewer);
+        glDisable(GL_POLYGON_OFFSET_POINT);
+#endif
+        }
+      else{
+        glDisable(GL_CULL_FACE);
+        drawshape(viewer);
+        }
       }
-    }
-  else{
-    glDisable(GL_LIGHTING);
-    }
-  
-  // Material
-  if(options&SURFACE_DUALSIDED){
-    glMaterialfv(GL_FRONT,GL_AMBIENT,material[0].ambient);
-    glMaterialfv(GL_FRONT,GL_DIFFUSE,material[0].diffuse);
-    glMaterialfv(GL_FRONT,GL_SPECULAR,material[0].specular);
-    glMaterialfv(GL_FRONT,GL_EMISSION,material[0].emission);
-    glMaterialf(GL_FRONT,GL_SHININESS,material[0].shininess);
-    glMaterialfv(GL_BACK,GL_AMBIENT,material[1].ambient);
-    glMaterialfv(GL_BACK,GL_DIFFUSE,material[1].diffuse);
-    glMaterialfv(GL_BACK,GL_SPECULAR,material[1].specular);
-    glMaterialfv(GL_BACK,GL_EMISSION,material[1].emission);
-    glMaterialf(GL_BACK,GL_SHININESS,material[1].shininess);
-    }
-  else{
-    glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,material[0].ambient);
-    glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,material[0].diffuse);
-    glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,material[0].specular);
-    glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,material[0].emission);
-    glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,material[0].shininess);
     }
 
-  // Surface
-  if(options&STYLE_SURFACE){
-    glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
-    if(options&FACECULLING_ON)
-      glEnable(GL_CULL_FACE);
-    else
-      glDisable(GL_CULL_FACE);
-    drawshape(viewer);
-    }
-  
-  // Wire frame
-  if(options&STYLE_WIREFRAME){
-    glDisable(GL_LIGHTING);
-    glShadeModel(GL_FLAT); 
-    glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
-    if(options&STYLE_SURFACE){                  // Wires over surface
-#if GL_EXT_polygon_offset
-      glEnable(GL_POLYGON_OFFSET_EXT);
-      glPolygonOffsetEXT(FACTOR, BIAS);
-      drawshape(viewer);
-      glDisable(GL_POLYGON_OFFSET_EXT);
-#elif GLU_VERSION_1_1
-      glEnable(GL_POLYGON_OFFSET_LINE);
-#if !defined(__CYGWIN__) && !defined(__MINGW32__) /* glPolygonOffset is missing from their import library */
-      glPolygonOffset(FACTOR, BIAS);
-#endif
-      drawshape(viewer);
-      glDisable(GL_POLYGON_OFFSET_LINE);
-#endif
-      }
-    else{                                       // Wires by themselves
-      glDisable(GL_CULL_FACE);
-      drawshape(viewer);
-      }
-    }
-  
-  // Points
-  if(options&STYLE_POINTS){
-    glDisable(GL_LIGHTING);
-    glShadeModel(GL_FLAT); 
-    glPolygonMode(GL_FRONT_AND_BACK,GL_POINT);
-    if(options&STYLE_SURFACE){                  // Points over surface
-#if GL_EXT_polygon_offset
-      glEnable(GL_POLYGON_OFFSET_EXT);
-      glPolygonOffsetEXT(FACTOR, BIAS);
-      drawshape(viewer);
-      glDisable(GL_POLYGON_OFFSET_EXT);
-#elif GLU_VERSION_1_1
-      glEnable(GL_POLYGON_OFFSET_POINT);
-#if !defined(__CYGWIN__) && !defined(__MINGW32__) /* glPolygonOffset is missing from their import library */
-      glPolygonOffset(FACTOR, BIAS);
-#endif
-      drawshape(viewer);
-      glDisable(GL_POLYGON_OFFSET_POINT);
-#endif
-      }
-    else{
-      glDisable(GL_CULL_FACE);
-      drawshape(viewer);
-      }
-    }
-  
   // Box
-  if((options&STYLE_BOUNDBOX) || (viewer->getSelection()==this)){
-    glDisable(GL_LIGHTING);
-    glShadeModel(GL_FLAT);
-    if(viewer->getSelection()==this){
-      glColor3f(0.0,1.0,0.0);
-      drawbox();
-      glPointSize(4.0);
-      drawhandles();
-      }
-    else{
-      glColor3f(0.7f,0.7f,0.7f);
-      drawbox();
-      }
-    }
+//  if((options&STYLE_BOUNDBOX) || selected){
+//    glDisable(GL_LIGHTING);
+//    glShadeModel(GL_FLAT);
+//    if(selected){
+//      glColor3f(0.0,1.0,0.0);
+//      drawbox();
+//      glPointSize(4.0);
+//      drawhandles();
+//      }
+//    else{
+//      glColor3f(0.7f,0.7f,0.7f);
+//      drawbox();
+//      }
+//    }
 
   // Restore attributes and matrix
-  glPopMatrix();  
+  glPopMatrix();
   glPopAttrib();
 #endif
   }
