@@ -3,27 +3,28 @@
 *               O p e n G L   T r i a n g l e   M e s h   O b j e c t           *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1999 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1999,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * Contributed by: Angel-Ventura Mendo Gomez <ventura@labri.u-bordeaux.fr>       *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXGLTriangleMesh.cpp,v 1.1 1999/09/14 13:41:23 jeroen Exp $              *
+* $Id: FXGLTriangleMesh.cpp,v 1.9 2001/04/27 14:30:32 jeroen Exp $              *
 ********************************************************************************/
 #include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
 #include "FXVec.h"
@@ -32,8 +33,12 @@
 #include "FXHMat.h"
 #include "FXRange.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXObjectList.h"
@@ -42,6 +47,7 @@
 #include "FXDrawable.h"
 #include "FXWindow.h"
 #include "FXCursor.h"
+#include "FXCanvas.h"
 #include "FXGLCanvas.h"
 #include "FXDC.h"
 #include "FXDCPrint.h"
@@ -57,14 +63,14 @@
 
 /*
   Notes:
-  
+
   - This is actually not a mesh at all but an just a bunch
     of individual triangles.
-    
+
   - To do: make this into a true mesh with shared vertices,
     so we can actually see some speedup from using the vertex-
     array primitives.
-    
+
   - Once we have a true mesh, revisit generatenormals() and make
     it spit out smooth normals based on mesh connectivity.
 */
@@ -72,7 +78,7 @@
 
 /*******************************************************************************/
 
-  
+
 // Object implementation
 FXIMPLEMENT(FXGLTriangleMesh,FXGLShape,NULL,0)
 
@@ -80,6 +86,7 @@ FXIMPLEMENT(FXGLTriangleMesh,FXGLShape,NULL,0)
 // Create cube
 FXGLTriangleMesh::FXGLTriangleMesh():
   vertexBuffer(NULL),colorBuffer(NULL),normalBuffer(NULL),textureBuffer(NULL),vertexNumber(0){
+  FXTRACE((100,"FXGLTriangleMesh::FXGLTriangleMesh\n"));
   range[0][0]=0.0; range[0][1]=0.0;
   range[1][0]=0.0; range[1][1]=0.0;
   range[2][0]=0.0; range[2][1]=0.0;
@@ -90,6 +97,7 @@ FXGLTriangleMesh::FXGLTriangleMesh():
 FXGLTriangleMesh::FXGLTriangleMesh(FXfloat x,FXfloat y,FXfloat z,FXint nv,FXfloat *v,FXfloat *n,FXfloat *c,FXfloat *t):
   FXGLShape(x,y,z,SHADING_SMOOTH|STYLE_SURFACE),
   vertexBuffer(v),colorBuffer(c),normalBuffer(n),textureBuffer(t),vertexNumber(nv){
+  FXTRACE((100,"FXGLTriangleMesh::FXGLTriangleMesh\n"));
   if(!normalBuffer){ generatenormals(); }
   recomputerange();
   }
@@ -99,11 +107,23 @@ FXGLTriangleMesh::FXGLTriangleMesh(FXfloat x,FXfloat y,FXfloat z,FXint nv,FXfloa
 FXGLTriangleMesh::FXGLTriangleMesh(FXfloat x,FXfloat y,FXfloat z,FXint nv,FXfloat *v,FXfloat *n,FXfloat *c,FXfloat *t,const FXMaterial& mtl):
   FXGLShape(x,y,z,SHADING_SMOOTH|STYLE_SURFACE,mtl,mtl),
   vertexBuffer(v),colorBuffer(c),normalBuffer(n),textureBuffer(t),vertexNumber(nv){
+  FXTRACE((100,"FXGLTriangleMesh::FXGLTriangleMesh\n"));
   if(!normalBuffer){ generatenormals(); }
   recomputerange();
   }
 
 
+// Copy constructor
+FXGLTriangleMesh::FXGLTriangleMesh(const FXGLTriangleMesh& orig):FXGLShape(orig){
+  FXTRACE((100,"FXGLTriangleMesh::FXGLTriangleMesh\n"));
+  FXMEMDUP(&vertexBuffer,FXfloat,orig.vertexBuffer,3*orig.vertexNumber);
+  FXMEMDUP(&colorBuffer,FXfloat,orig.colorBuffer,4*orig.vertexNumber);
+  FXMEMDUP(&normalBuffer,FXfloat,orig.normalBuffer,3*orig.vertexNumber);
+  FXMEMDUP(&textureBuffer,FXfloat,orig.textureBuffer,2*orig.vertexNumber);
+  vertexNumber=orig.vertexNumber;
+  }
+
+
 // Change vertex buffer means recompute range
 void FXGLTriangleMesh::setVertexBuffer(FXfloat *vertices){
   vertexBuffer=vertices;
@@ -122,7 +142,7 @@ void FXGLTriangleMesh::recomputerange(){
     range[0][0]=range[0][1]=vertexBuffer[0];
     range[1][0]=range[1][1]=vertexBuffer[1];
     range[2][0]=range[2][1]=vertexBuffer[2];
-    for(i=0,n=3*vertexNumber; n; n--){
+    for(i=0,n=0; n<vertexNumber; n++){
       t=vertexBuffer[i++];
       if(t<range[0][0]) range[0][0]=t;
       if(t>range[0][1]) range[0][1]=t;
@@ -141,11 +161,11 @@ void FXGLTriangleMesh::recomputerange(){
 void FXGLTriangleMesh::drawshape(FXGLViewer*){
   if(!vertexBuffer || vertexNumber<=0) return;
 #ifdef HAVE_OPENGL
-  
+
 #if !defined(GL_VERSION_1_1) && !defined(GL_VERSION_1_2)
-  
+
   register FXint i;
-  
+
   // Yes we know this is not efficient but hell, OpenGL 1.0 is old...
   glBegin(GL_TRIANGLES);
   for(i=0; i<vertexNumber; i++){
@@ -157,7 +177,7 @@ void FXGLTriangleMesh::drawshape(FXGLViewer*){
   glEnd();
 
 #else
-  
+
   // Setting buffers
   glVertexPointer(3,GL_FLOAT,0,vertexBuffer);
   glEnableClientState(GL_VERTEX_ARRAY);
@@ -177,7 +197,7 @@ void FXGLTriangleMesh::drawshape(FXGLViewer*){
   else{
     glDisableClientState(GL_COLOR_ARRAY);
     }
-  
+
   if(textureBuffer){
     glTexCoordPointer(2,GL_FLOAT,0,textureBuffer);
     glDisableClientState(GL_TEXTURE_COORD_ARRAY);
@@ -188,7 +208,7 @@ void FXGLTriangleMesh::drawshape(FXGLViewer*){
 
   // drawing
   glDrawArrays(GL_TRIANGLES,0,vertexNumber);
-  
+
 #endif
 #endif
   }
@@ -218,21 +238,28 @@ void FXGLTriangleMesh::generatenormals(){
 
     normal=normalize(normal);
 
-    normalBuffer[i+0]=normal[0]; 
-    normalBuffer[i+1]=normal[1]; 
-    normalBuffer[i+2]=normal[2]; 
+    normalBuffer[i+0]=normal[0];
+    normalBuffer[i+1]=normal[1];
+    normalBuffer[i+2]=normal[2];
 
-    normalBuffer[i+3]=normal[0]; 
-    normalBuffer[i+4]=normal[1]; 
-    normalBuffer[i+5]=normal[2]; 
+    normalBuffer[i+3]=normal[0];
+    normalBuffer[i+4]=normal[1];
+    normalBuffer[i+5]=normal[2];
 
-    normalBuffer[i+6]=normal[0]; 
-    normalBuffer[i+7]=normal[1]; 
-    normalBuffer[i+8]=normal[2]; 
+    normalBuffer[i+6]=normal[0];
+    normalBuffer[i+7]=normal[1];
+    normalBuffer[i+8]=normal[2];
     }
   }
 
 
+
+// Copy this object
+FXGLObject* FXGLTriangleMesh::copy(){
+  return new FXGLTriangleMesh(*this);
+  }
+
+
 // Save object to stream
 void FXGLTriangleMesh::save(FXStream& store) const {
   FXuchar hadvertices,hadcolors,hadnormals,hadtextures;
@@ -249,7 +276,7 @@ void FXGLTriangleMesh::save(FXStream& store) const {
   if(hadtextures){ store.save(textureBuffer,vertexNumber); }
   }
 
-      
+
 // Load object from stream
 void FXGLTriangleMesh::load(FXStream& store){
   FXuchar hadvertices,hadcolors,hadnormals,hadtextures;
@@ -265,6 +292,7 @@ void FXGLTriangleMesh::load(FXStream& store){
 
 // Zap object
 FXGLTriangleMesh::~FXGLTriangleMesh(){
+  FXTRACE((100,"FXGLTriangleMesh::~FXGLTriangleMesh\n"));
   FXFREE(&vertexBuffer);
   FXFREE(&colorBuffer);
   FXFREE(&normalBuffer);
