@@ -3,29 +3,35 @@
 *          M u l t i p l e   D o c u m e n t   C h i l d   W i n d o w          *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXMDIChild.cpp,v 1.8 1999/10/26 15:35:53 jeroen Exp $                    *
+* $Id: FXMDIChild.cpp,v 1.33 2001/07/17 07:25:02 jeroen Exp $                   *
 ********************************************************************************/
+#include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXApp.h"
@@ -47,23 +53,23 @@
 #include "FXPopup.h"
 #include "FXMenuPane.h"
 #include "FXScrollbar.h"
-#include "FXScrollWindow.h"
+#include "FXScrollArea.h"
 #include "FXMDIButton.h"
 #include "FXMDIChild.h"
 #include "FXMDIClient.h"
 
-  
+
 /*
   Notes:
   - Stacking order changes should be performed by MDIClient!
-  - Need options for MDI child decorations (close btn, window menu, min btn, max btn, 
-    title etc). 
-  - Iconified version should be fixed size, showing as much of title as feasible 
-    (tail with ...'s) 
+  - Need options for MDI child decorations (close btn, window menu, min btn, max btn,
+    title etc).
+  - Iconified version should be fixed size, showing as much of title as feasible
+    (tail with ...'s)
   - Initial icon placement on the bottom of the MDIClient somehow...
 */
 
-#define BORDERWIDTH      6                          // MDI Child border width
+#define BORDERWIDTH      5                          // MDI Child border width
 #define HANDLESIZE       20                         // Resize handle length
 #define MINWIDTH         80                         // Minimum width
 #define MINHEIGHT        30                         // Minimum height
@@ -82,18 +88,6 @@
 #define DRAGTITLE        (DRAGTOP|DRAGBOTTOM|DRAGLEFT|DRAGRIGHT)    // Drag title
 #define DRAGINVERTED     16                         // Drag rectangle is inverted
 
-#define ANIM_MIN_2_RES   0                          // Growing
-#define ANIM_MIN_2_MAX   1
-#define ANIM_RES_2_MAX   2
-
-#define ANIM_MAX_2_MIN   3                          // Shrinking
-#define ANIM_MAX_2_RES   4
-#define ANIM_RES_2_MIN   5
-
-#define ANIM_MAX_2_NIL   6                          // Closing
-#define ANIM_MIN_2_NIL   7  
-#define ANIM_RES_2_NIL   8
-
 #define MDI_MASK         (MDI_MAXIMIZED|MDI_MINIMIZED)
 
 
@@ -103,7 +97,6 @@
 FXDEFMAP(FXMDIChild) FXMDIChildMap[]={
   FXMAPFUNC(SEL_PAINT,0,FXMDIChild::onPaint),
   FXMAPFUNC(SEL_MOTION,0,FXMDIChild::onMotion),
-  FXMAPFUNC(SEL_TIMEOUT,FXMDIChild::ID_TIMER_ANIM,FXMDIChild::onAnimTimer),
   FXMAPFUNC(SEL_FOCUSIN,0,FXMDIChild::onFocusIn),
   FXMAPFUNC(SEL_FOCUSOUT,0,FXMDIChild::onFocusOut),
   FXMAPFUNC(SEL_LEFTBUTTONPRESS,0,FXMDIChild::onLeftBtnPress),
@@ -113,6 +106,7 @@ FXDEFMAP(FXMDIChild) FXMDIChildMap[]={
   FXMAPFUNC(SEL_RIGHTBUTTONPRESS,0,FXMDIChild::onRightBtnPress),
   FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,0,FXMDIChild::onRightBtnRelease),
   FXMAPFUNC(SEL_CLOSE,0,FXMDIChild::onClose),
+  FXMAPFUNC(SEL_CLOSEALL,0,FXMDIChild::onCloseAll),
   FXMAPFUNC(SEL_DELETE,0,FXMDIChild::onDelete),
   FXMAPFUNC(SEL_SELECTED,0,FXMDIChild::onSelected),
   FXMAPFUNC(SEL_DESELECTED,0,FXMDIChild::onDeselected),
@@ -144,10 +138,10 @@ FXDEFMAP(FXMDIChild) FXMDIChildMap[]={
 // Object implementation
 FXIMPLEMENT(FXMDIChild,FXComposite,FXMDIChildMap,ARRAYNUMBER(FXMDIChildMap))
 
-  
-/*******************************************************************************/  
-  
-  
+
+/*******************************************************************************/
+
+
 // Serialization
 FXMDIChild::FXMDIChild(){
   flags|=FLAG_ENABLED|FLAG_SHOWN;
@@ -159,7 +153,6 @@ FXMDIChild::FXMDIChild(){
   maximizebtn=(FXButton*)-1;
   deletebtn=(FXButton*)-1;
   font=(FXFont*)-1;
-  timer=NULL;
   baseColor=0;
   hiliteColor=0;
   shadowColor=0;
@@ -168,6 +161,8 @@ FXMDIChild::FXMDIChild(){
   titleBackColor=0;
   iconPosX=0;
   iconPosY=0;
+  iconWidth=0;
+  iconHeight=0;
   normalPosX=0;
   normalPosY=0;
   normalWidth=0;
@@ -178,9 +173,7 @@ FXMDIChild::FXMDIChild(){
   oldy=0;
   oldw=0;
   oldh=0;
-  tween=0;
   action=DRAGNONE;
-  anim=0;
   }
 
 
@@ -199,16 +192,17 @@ FXMDIChild::FXMDIChild(FXMDIClient* p,const FXString& name,FXIcon* ic,FXMenuPane
   deletebtn=new FXMDIDeleteButton(this,this,FXWindow::ID_MDI_CLOSE,FRAME_RAISED);
   windowbtn->setMenu(mn);
   windowbtn->setIcon(ic);
-  baseColor=getApp()->baseColor;
-  hiliteColor=getApp()->hiliteColor;
-  shadowColor=getApp()->shadowColor;
-  borderColor=getApp()->borderColor;
-  titleColor=getApp()->selforeColor;
-  titleBackColor=getApp()->selbackColor;
-  font=getApp()->normalFont;
-  timer=NULL;
+  baseColor=getApp()->getBaseColor();
+  hiliteColor=getApp()->getHiliteColor();
+  shadowColor=getApp()->getShadowColor();
+  borderColor=getApp()->getBorderColor();
+  titleColor=getApp()->getSelforeColor();
+  titleBackColor=getApp()->getSelbackColor();
+  font=getApp()->getNormalFont();
   iconPosX=xpos;
   iconPosY=ypos;
+  iconWidth=width;
+  iconHeight=height;
   normalPosX=xpos;
   normalPosY=ypos;
   normalWidth=width;
@@ -225,9 +219,7 @@ FXMDIChild::FXMDIChild(FXMDIClient* p,const FXString& name,FXIcon* ic,FXMenuPane
   oldy=0;
   oldw=0;
   oldh=0;
-  tween=0;
   action=DRAGNONE;
-  anim=0;
   }
 
 
@@ -257,7 +249,7 @@ FXint FXMDIChild::getDefaultWidth(){
   FXint mw,bw;
   mw=windowbtn->getDefaultWidth();
   bw=deletebtn->getDefaultWidth();
-  return TITLESPACE+mw+3*bw+(BORDERWIDTH<<1)+1+4+4+6+1;
+  return TITLESPACE+mw+3*bw+(BORDERWIDTH<<1)+2+4+4+6+2;
   }
 
 
@@ -269,7 +261,7 @@ FXint FXMDIChild::getDefaultHeight(){
   bh=deletebtn->getDefaultHeight();
   return FXMAX3(fh,mh,bh)+(BORDERWIDTH<<1)+4;
   }
-  
+
 
 // Just tell server where the windows are!
 void FXMDIChild::layout(){
@@ -281,9 +273,9 @@ void FXMDIChild::layout(){
   bw=deletebtn->getDefaultWidth();
   bh=deletebtn->getDefaultHeight();
   th=FXMAX3(fh,mh,bh)+4;
-  bx=width-BORDERWIDTH-bw-1;
+  bx=width-BORDERWIDTH-bw-2;
   by=BORDERWIDTH+(th-bh)/2;
-  windowbtn->position(BORDERWIDTH+1,BORDERWIDTH+(th-mh)/2,mw,mh);
+  windowbtn->position(BORDERWIDTH+2,BORDERWIDTH+(th-mh)/2,mw,mh);
   if(options&MDI_MAXIMIZED){
     deletebtn->hide();
     maximizebtn->hide();
@@ -337,11 +329,13 @@ FXbool FXMDIChild::isMinimized() const {
 
 
 // Maximize window
-void FXMDIChild::maximize(){
+void FXMDIChild::maximize(FXbool notify){
   if(!(options&MDI_MAXIMIZED)){
     if(options&MDI_MINIMIZED){
       iconPosX=xpos;
       iconPosY=ypos;
+      iconWidth=width;
+      iconHeight=height;
       }
     else{
       normalPosX=xpos;
@@ -355,13 +349,14 @@ void FXMDIChild::maximize(){
     height=getParent()->getHeight();
     options|=MDI_MAXIMIZED;
     options&=~MDI_MINIMIZED;
+    if(notify){handle(this,MKUINT(0,SEL_MAXIMIZE),NULL);}
     recalc();
     }
   }
 
 
 // Minimize window
-void FXMDIChild::minimize(){
+void FXMDIChild::minimize(FXbool notify){
   if(!(options&MDI_MINIMIZED)){
     if(!(options&MDI_MAXIMIZED)){
       normalPosX=xpos;
@@ -375,31 +370,81 @@ void FXMDIChild::minimize(){
     height=getDefaultHeight();
     options|=MDI_MINIMIZED;
     options&=~MDI_MAXIMIZED;
+    if(notify){handle(this,MKUINT(0,SEL_MINIMIZE),NULL);}
     recalc();
     }
   }
 
 
 // Restore window
-void FXMDIChild::restore(){
+void FXMDIChild::restore(FXbool notify){
   if(options&(MDI_MINIMIZED|MDI_MAXIMIZED)){
     if(options&MDI_MINIMIZED){
       iconPosX=xpos;
       iconPosY=ypos;
+      iconWidth=width;
+      iconHeight=height;
       }
     xpos=normalPosX;
     ypos=normalPosY;
     width=normalWidth;
     height=normalHeight;
     options&=~(MDI_MINIMIZED|MDI_MAXIMIZED);
+    if(notify){handle(this,MKUINT(0,SEL_RESTORE),NULL);}
     recalc();
     }
   }
 
 
+// Move this window to the specified position in the parent's coordinates
+void FXMDIChild::move(FXint x,FXint y){
+  FXComposite::move(x,y);
+  if(!(options&(MDI_MAXIMIZED|MDI_MINIMIZED))){
+    normalPosX=x;
+    normalPosY=y;
+    }
+  else if(options&MDI_MINIMIZED){
+    iconPosX=x;
+    iconPosY=y;
+    }
+  }
+
+
+// Resize this window to the specified width and height
+void FXMDIChild::resize(FXint w,FXint h){
+  FXComposite::resize(w,h);
+  if(!(options&(MDI_MAXIMIZED|MDI_MINIMIZED))){
+    normalWidth=w;
+    normalHeight=h;
+    }
+  else if(options&MDI_MINIMIZED){
+    iconWidth=w;
+    iconHeight=h;
+    }
+  }
+
+
+// Move and resize this window in the parent's coordinates
+void FXMDIChild::position(FXint x,FXint y,FXint w,FXint h){
+  FXComposite::position(x,y,w,h);
+  if(!(options&(MDI_MAXIMIZED|MDI_MINIMIZED))){
+    normalPosX=x;
+    normalPosY=y;
+    normalWidth=w;
+    normalHeight=h;
+    }
+  else if(options&MDI_MINIMIZED){
+    iconPosX=x;
+    iconPosY=y;
+    iconWidth=w;
+    iconHeight=h;
+    }
+  }
+
+
 // If window can have focus
-FXbool FXMDIChild::canFocus() const { 
-  return TRUE; 
+FXbool FXMDIChild::canFocus() const {
+  return TRUE;
   }
 
 
@@ -408,27 +453,27 @@ void FXMDIChild::changeCursor(FXint x,FXint y){
   switch(where(x,y)){
     case DRAGTOP:
     case DRAGBOTTOM:
-      setDefaultCursor(getApp()->dragHCursor);
-      setDragCursor(getApp()->dragHCursor);
+      setDefaultCursor(getApp()->getDefaultCursor(DEF_DRAGH_CURSOR));
+      setDragCursor(getApp()->getDefaultCursor(DEF_DRAGH_CURSOR));
       break;
     case DRAGLEFT:
     case DRAGRIGHT:
-      setDefaultCursor(getApp()->dragVCursor);
-      setDragCursor(getApp()->dragVCursor);
+      setDefaultCursor(getApp()->getDefaultCursor(DEF_DRAGV_CURSOR));
+      setDragCursor(getApp()->getDefaultCursor(DEF_DRAGV_CURSOR));
       break;
     case DRAGTOPLEFT:
     case DRAGBOTTOMRIGHT:
-      setDefaultCursor(getApp()->dragTLCursor);
-      setDragCursor(getApp()->dragTLCursor);
+      setDefaultCursor(getApp()->getDefaultCursor(DEF_DRAGTL_CURSOR));
+      setDragCursor(getApp()->getDefaultCursor(DEF_DRAGTL_CURSOR));
       break;
     case DRAGTOPRIGHT:
     case DRAGBOTTOMLEFT:
-      setDefaultCursor(getApp()->dragTRCursor);
-      setDragCursor(getApp()->dragTRCursor);
+      setDefaultCursor(getApp()->getDefaultCursor(DEF_DRAGTR_CURSOR));
+      setDragCursor(getApp()->getDefaultCursor(DEF_DRAGTR_CURSOR));
       break;
     default:
-      setDefaultCursor(getApp()->arrowCursor);
-      setDragCursor(getApp()->arrowCursor);
+      setDefaultCursor(getApp()->getDefaultCursor(DEF_ARROW_CURSOR));
+      setDragCursor(getApp()->getDefaultCursor(DEF_ARROW_CURSOR));
       break;
     }
   }
@@ -436,8 +481,8 @@ void FXMDIChild::changeCursor(FXint x,FXint y){
 
 // Revert cursor to normal one
 void FXMDIChild::revertCursor(){
-  setDefaultCursor(getApp()->arrowCursor);
-  setDragCursor(getApp()->arrowCursor);
+  setDefaultCursor(getApp()->getDefaultCursor(DEF_ARROW_CURSOR));
+  setDragCursor(getApp()->getDefaultCursor(DEF_ARROW_CURSOR));
   }
 
 
@@ -450,185 +495,105 @@ void FXMDIChild::drawRubberBox(FXint x,FXint y,FXint w,FXint h){
     dc.setFunction(BLT_SRC_XOR_DST);
     dc.setForeground(getParent()->getBackColor());
     translateCoordinatesTo(xx,yy,getParent(),x,y);
-    dc.drawHashBox(xx,yy,w,h,BORDERWIDTH);
-    
-    // If your X-Server melts down under IRIX 5.3, here's the reason
-    //dc.setLineWidth(BORDERWIDTH);
-    //dc.drawRectangle(xx+BORDERWIDTH/2,yy+BORDERWIDTH/2,w-BORDERWIDTH,h-BORDERWIDTH);
+    //dc.drawHashBox(xx,yy,w,h,BORDERWIDTH);
+    dc.setLineWidth(BORDERWIDTH);
+    dc.drawRectangle(xx+BORDERWIDTH/2,yy+BORDERWIDTH/2,w-BORDERWIDTH,h-BORDERWIDTH);
     }
   }
 
 
-// Draw in-between box
-void FXMDIChild::drawTweenBox(FXint tw){
-  FXint bx,by,bw,bh,s,t;
-  t=tw; 
-  s=100-tw;
-  
-  // In-between
-  switch(anim){
-    case ANIM_MIN_2_RES:
-      bx=(getX()*s+normalPosX*t)/100;
-      by=(getY()*s+normalPosY*t)/100;
-      bw=(getDefaultWidth()*s+normalWidth*t)/100;
-      bh=(getDefaultHeight()*s+normalHeight*t)/100;
-      break;
-    case ANIM_RES_2_MIN:
-      bx=(iconPosX*s+getX()*t)/100;
-      by=(iconPosY*s+getY()*t)/100;
-      bw=(getDefaultWidth()*s+getWidth()*t)/100;
-      bh=(getDefaultHeight()*s+getHeight()*t)/100;
-      break;
-    case ANIM_MIN_2_MAX:
-      bx=(getX()*s)/100;
-      by=(getY()*s)/100;
-      bw=(getDefaultWidth()*s+getParent()->getWidth()*t)/100;
-      bh=(getDefaultHeight()*s+getParent()->getHeight()*t)/100;
-      break;
-    case ANIM_MAX_2_MIN:
-      bx=(iconPosX*s)/100;
-      by=(iconPosY*s)/100;
-      bw=(getDefaultWidth()*s+getParent()->getWidth()*t)/100;
-      bh=(getDefaultHeight()*s+getParent()->getHeight()*t)/100;
-      break;
-    case ANIM_RES_2_MAX:
-      bx=(getX()*s)/100;
-      by=(getY()*s)/100;
-      bw=(getWidth()*s+getParent()->getWidth()*t)/100;
-      bh=(getHeight()*s+getParent()->getHeight()*t)/100;
-      break;
-    case ANIM_MAX_2_RES:
-      bx=(normalPosX*s)/100;
-      by=(normalPosY*s)/100;
-      bw=(normalWidth*s+getParent()->getWidth()*t)/100;
-      bh=(normalHeight*s+getParent()->getHeight()*t)/100;
-      break;
-    case ANIM_MAX_2_NIL:
-      bx=((getParent()->getWidth()/2)*s)/100;
-      by=((getParent()->getHeight()/2)*s)/100;
-      bw=(getParent()->getWidth()*t)/100;
-      bh=(getParent()->getHeight()*t)/100;
-      break;
-    case ANIM_MIN_2_NIL:
-    case ANIM_RES_2_NIL:
-      bx=((getX()+getWidth()/2)*s+getX()*t)/100;
-      by=((getY()+getHeight()/2)*s+getY()*t)/100;
-      bw=(getWidth()*t)/100;
-      bh=(getHeight()*t)/100;
-      break;
-    }
-  
-  // Draw box
-  if(BORDERWIDTH*2<bw && BORDERWIDTH*2<bh){
+// Draw animation morphing from old to new rectangle
+void FXMDIChild::animateRectangles(FXint ox,FXint oy,FXint ow,FXint oh,FXint nx,FXint ny,FXint nw,FXint nh){
+  if(xid && getApp()->getAnimSpeed()){
     FXDCWindow dc(getParent());
+    FXint bx,by,bw,bh,s,t;
     dc.clipChildren(FALSE);
     dc.setFunction(BLT_SRC_XOR_DST);
     dc.setForeground(getParent()->getBackColor());
-    //dc.setLineWidth(BORDERWIDTH);
-    //dc.drawRectangle(bx+BORDERWIDTH/2,by+BORDERWIDTH/2,bw-BORDERWIDTH,bh-BORDERWIDTH);
-    dc.drawHashBox(bx,by,bw,bh,BORDERWIDTH);
-    }
-  }
-
-
-// Animate timer
-long FXMDIChild::onAnimTimer(FXObject*,FXSelector,void*){
-  timer=NULL;
-  if(0<tween && tween<100) drawTweenBox(tween);
-  tween += ((anim<=ANIM_RES_2_MAX) ? 5 : -5);
-  if(tween<=0 || 100<=tween){
-    switch(anim){
-      case ANIM_MIN_2_RES:
-      case ANIM_MAX_2_RES:
-        handle(this,MKUINT(0,SEL_RESTORE),NULL);
-        break;
-      case ANIM_RES_2_MIN:
-      case ANIM_MAX_2_MIN:
-        handle(this,MKUINT(0,SEL_MINIMIZE),NULL);
-        break;
-      case ANIM_MIN_2_MAX:
-      case ANIM_RES_2_MAX:
-        handle(this,MKUINT(0,SEL_MAXIMIZE),NULL);
-        break;
-      case ANIM_MAX_2_NIL:
-      case ANIM_RES_2_NIL:
-      case ANIM_MIN_2_NIL:
-        handle(this,MKUINT(0,SEL_DELETE),NULL);
-        break;
+    for(s=0,t=100; s<=100; s+=5,t-=5){
+      bx=(nx*s+ox*t)/100;
+      by=(ny*s+oy*t)/100;
+      bw=(nw*s+ow*t)/100;
+      bh=(nh*s+oh*t)/100;
+      if(BORDERWIDTH*2<bw && BORDERWIDTH*2<bh){
+        dc.drawHashBox(bx,by,bw,bh,BORDERWIDTH);
+        getApp()->flush(TRUE);
+        fxsleep(getApp()->getAnimSpeed()*1000);
+        dc.drawHashBox(bx,by,bw,bh,BORDERWIDTH);
+        getApp()->flush(TRUE);    
+        }
       }
-    return 1;
-    }
-  timer=getApp()->addTimeout(getApp()->animSpeed,this,ID_TIMER_ANIM);
-  drawTweenBox(tween);
-  return 0;
+    } 
   }
 
 
-// Handle repaint 
+// Handle repaint
 long FXMDIChild::onPaint(FXObject*,FXSelector,void* ptr){
   FXEvent *ev=(FXEvent*)ptr;
   FXint xx,yy,tw,th,titlespace,letters,dots,dotspace;
   FXint fh,mh,bh,bw,mw;
   FXDCWindow dc(this,ev);
-  
-  fh=font->getFontHeight();
-  mw=windowbtn->getDefaultWidth();
-  mh=windowbtn->getDefaultHeight();
-  bw=deletebtn->getDefaultWidth();
-  bh=deletebtn->getDefaultHeight();
-  th=FXMAX3(fh,mh,bh)+4;
 
-  dc.setClipRectangle(ev->rect.x,ev->rect.y,ev->rect.w,ev->rect.h);
-  dc.setForeground(baseColor);
-  
   // Draw MDIChild background
-  dc.fillRectangle(ev->rect.x,ev->rect.y,ev->rect.w,ev->rect.h);
-  
-  // Draw border
   dc.setForeground(baseColor);
-  dc.drawLine(0,0,width-2,0);
-  dc.drawLine(0,0,0,height-2);
-  dc.setForeground(hiliteColor);
-  dc.drawLine(1,0+1,width-3,1);
-  dc.drawLine(1,0+1,1,height-3);
-  dc.setForeground(shadowColor);
-  dc.drawLine(1,height-2,width-2,height-2);
-  dc.drawLine(width-2,height-2,width-2,01);
-  dc.setForeground(borderColor);
-  dc.drawLine(0,height-1,width-1,height-1);
-  dc.drawLine(width-1,0,width-1,height-1);
-  
-  // Draw title background
-  dc.setForeground(isActive() ? (hasFocus() ? titleBackColor : shadowColor) : backColor);
-  dc.fillRectangle(BORDERWIDTH,BORDERWIDTH,width-BORDERWIDTH*2,th);
-  
-  // Draw title
-  if(!title.empty()){
-    xx=BORDERWIDTH+mw+1+4;
-    yy=BORDERWIDTH+font->getFontAscent()+(th-fh)/2;
-    
-    // Compute space for title
-    titlespace=width-mw-3*bw-(BORDERWIDTH<<1)-1-4-4-6-1;
-    
-    dots=0;
-    letters=title.length();
-    
-    // Title too large for space
-    if(font->getTextWidth(title.text(),letters)>titlespace){
-      dotspace=titlespace-font->getTextWidth("...",3);
-      while(letters>0 && (tw=font->getTextWidth(title.text(),letters))>dotspace) letters--;
-      dots=3;
-      if(letters==0){
-        letters=1;
-        dots=0;
+  dc.fillRectangle(ev->rect.x,ev->rect.y,ev->rect.w,ev->rect.h);
+
+  // Only draw stuff when not maximized
+  if(!(options&MDI_MAXIMIZED)){
+    fh=font->getFontHeight();
+    mw=windowbtn->getDefaultWidth();
+    mh=windowbtn->getDefaultHeight();
+    bw=deletebtn->getDefaultWidth();
+    bh=deletebtn->getDefaultHeight();
+    th=FXMAX3(fh,mh,bh)+4;
+
+
+    // Draw border
+    dc.setForeground(baseColor);
+    dc.drawLine(0,0,width-2,0);
+    dc.drawLine(0,0,0,height-2);
+    dc.setForeground(hiliteColor);
+    dc.drawLine(1,0+1,width-3,1);
+    dc.drawLine(1,0+1,1,height-3);
+    dc.setForeground(shadowColor);
+    dc.drawLine(1,height-2,width-2,height-2);
+    dc.drawLine(width-2,height-2,width-2,01);
+    dc.setForeground(borderColor);
+    dc.drawLine(0,height-1,width-1,height-1);
+    dc.drawLine(width-1,0,width-1,height-1);
+
+    // Draw title background
+    dc.setForeground(isActive() ? (hasFocus() ? titleBackColor : shadowColor) : backColor);
+    dc.fillRectangle(BORDERWIDTH,BORDERWIDTH,width-BORDERWIDTH*2,th);
+
+    // Draw title
+    if(!title.empty()){
+      xx=BORDERWIDTH+mw+2+4;
+      yy=BORDERWIDTH+font->getFontAscent()+(th-fh)/2;
+
+      // Compute space for title
+      titlespace=width-mw-3*bw-(BORDERWIDTH<<1)-2-4-4-6-2;
+
+      dots=0;
+      letters=title.length();
+
+      // Title too large for space
+      if(font->getTextWidth(title.text(),letters)>titlespace){
+        dotspace=titlespace-font->getTextWidth("...",3);
+        while(letters>0 && (tw=font->getTextWidth(title.text(),letters))>dotspace) letters--;
+        dots=3;
+        if(letters==0){
+          letters=1;
+          dots=0;
+          }
         }
+
+      // Draw as much of the title as possible
+      dc.setForeground(isActive() ? titleColor : borderColor);
+      dc.setTextFont(font);
+      dc.drawText(xx,yy,title.text(),letters);
+      dc.drawText(xx+font->getTextWidth(title.text(),letters),yy,"...",dots);
       }
-    
-    // Draw as much of the title as possible
-    dc.setForeground(isActive() ? titleColor : borderColor);
-    dc.setTextFont(font);
-    dc.drawText(xx,yy,title.text(),letters);
-    dc.drawText(xx+font->getTextWidth(title.text(),letters),yy,"...",dots);
     }
   return 1;
   }
@@ -642,17 +607,17 @@ FXuchar FXMDIChild::where(FXint x,FXint y){
   mh=windowbtn->getDefaultHeight();
   bh=deletebtn->getDefaultHeight();
   th=FXMAX3(fh,mh,bh)+4;
-  if(!isMinimized() && x<HANDLESIZE) code|=DRAGLEFT; 
-  if(!isMinimized() && width-HANDLESIZE<=x) code|=DRAGRIGHT; 
-  if(!isMinimized() && y<HANDLESIZE) code|=DRAGTOP; 
-  if(!isMinimized() && height-HANDLESIZE<=y) code|=DRAGBOTTOM; 
+  if(!isMinimized() && x<HANDLESIZE) code|=DRAGLEFT;
+  if(!isMinimized() && width-HANDLESIZE<=x) code|=DRAGRIGHT;
+  if(!isMinimized() && y<HANDLESIZE) code|=DRAGTOP;
+  if(!isMinimized() && height-HANDLESIZE<=y) code|=DRAGBOTTOM;
   if(BORDERWIDTH<=x && x<=width-BORDERWIDTH && BORDERWIDTH<=y && y<BORDERWIDTH+th) code=DRAGTITLE;
   return code;
   }
 
 
 // Focus on widget itself
-long FXMDIChild::onFocusSelf(FXObject* sender,FXSelector sel,void* ptr){
+long FXMDIChild::onFocusSelf(FXObject* sender,FXSelector sel,void* ptr){    // FIXME look at this carefully
   FXWindow *child=contentWindow();
   if(child){
     if(child->isEnabled() && child->canFocus()){
@@ -661,8 +626,7 @@ long FXMDIChild::onFocusSelf(FXObject* sender,FXSelector sel,void* ptr){
       }
     if(child->isComposite() && child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr)) return 1;
     }
-  FXComposite::onFocusSelf(sender,sel,ptr);
-  return 1;
+  return FXComposite::onFocusSelf(sender,sel,ptr);
   }
 
 
@@ -680,7 +644,7 @@ long FXMDIChild::onFocusIn(FXObject* sender,FXSelector sel,void* ptr){
   return 1;
   }
 
-  
+
 // Lost focus
 long FXMDIChild::onFocusOut(FXObject* sender,FXSelector sel,void* ptr){
   FXint fh,mh,bh,th;
@@ -695,7 +659,7 @@ long FXMDIChild::onFocusOut(FXObject* sender,FXSelector sel,void* ptr){
   }
 
 
-// Pressed LEFT button 
+// Pressed LEFT button
 long FXMDIChild::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
   FXEvent *event=(FXEvent*)ptr;
   flags&=~FLAG_TIP;
@@ -769,10 +733,10 @@ long FXMDIChild::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
   }
 
 
-// Released LEFT button 
+// Released LEFT button
 long FXMDIChild::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
   FXEvent *event=(FXEvent*)ptr;
-  if(isEnabled()){ 
+  if(isEnabled()){
     ungrab();
     flags&=~FLAG_PRESSED;
     if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONRELEASE),ptr)) return 1;
@@ -787,14 +751,17 @@ long FXMDIChild::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
         }
       }
     else if(event->click_count==2){
-      if(isMinimized()){
-        handle(this,MKUINT(0,SEL_RESTORE),NULL);
+      if(options&MDI_MINIMIZED){
+        animateRectangles(xpos,ypos,width,height,normalPosX,normalPosY,normalWidth,normalHeight);
+        restore(TRUE);
         }
-      else if(!isMaximized()){
-        handle(this,MKUINT(0,SEL_MAXIMIZE),NULL);
+      else if(options&MDI_MAXIMIZED){
+        animateRectangles(xpos,ypos,width,height,normalPosX,normalPosY,normalWidth,normalHeight);
+        restore(TRUE);
         }
-      else if(isMaximized()){
-        handle(this,MKUINT(0,SEL_RESTORE),NULL);
+      else{
+        animateRectangles(xpos,ypos,width,height,0,0,getParent()->getWidth(),getParent()->getHeight());
+        maximize(TRUE);
         }
       }
     return 1;
@@ -855,7 +822,7 @@ long FXMDIChild::onMotion(FXObject*,FXSelector,void* ptr){
         if(!event->moved) return 1;       // Have not yet moved enough to qualify as a drag
         oldx=event->win_x-xoff;
         oldy=event->win_y-yoff;
-        setDragCursor(getApp()->moveCursor);
+        setDragCursor(getApp()->getDefaultCursor(DEF_MOVE_CURSOR));
         break;
       }
     drawRubberBox(oldx,oldy,oldw,oldh);
@@ -869,7 +836,7 @@ long FXMDIChild::onMotion(FXObject*,FXSelector,void* ptr){
   }
 
 
-// Pressed MIDDLE button 
+// Pressed MIDDLE button
 long FXMDIChild::onMiddleBtnPress(FXObject*,FXSelector,void* ptr){
   flags&=~FLAG_TIP;
   if(isEnabled()){
@@ -882,7 +849,7 @@ long FXMDIChild::onMiddleBtnPress(FXObject*,FXSelector,void* ptr){
   }
 
 
-// Released MIDDLE button 
+// Released MIDDLE button
 long FXMDIChild::onMiddleBtnRelease(FXObject*,FXSelector,void* ptr){
   if(isEnabled()){
     ungrab();
@@ -893,21 +860,20 @@ long FXMDIChild::onMiddleBtnRelease(FXObject*,FXSelector,void* ptr){
   }
 
 
-// Pressed RIGHT button 
+// Pressed RIGHT button
 long FXMDIChild::onRightBtnPress(FXObject*,FXSelector,void* ptr){
   flags&=~FLAG_TIP;
   if(isEnabled()){
     grab();
     if(target && target->handle(this,MKUINT(message,SEL_RIGHTBUTTONPRESS),ptr)) return 1;
-    lower();    // If active, if will pop back up after layout(); we should actually switch active windows (make topmost one active)
-    //getApp()->flush();
+    lower();
     return 1;
     }
   return 0;
   }
 
 
-// Released RIGHT button 
+// Released RIGHT button
 long FXMDIChild::onRightBtnRelease(FXObject*,FXSelector,void* ptr){
   if(isEnabled()){
     ungrab();
@@ -927,7 +893,7 @@ long FXMDIChild::onCmdSetStringValue(FXObject*,FXSelector,void* ptr){
 
 
 // Window was selected
-long FXMDIChild::onSelected(FXObject*,FXSelector,void* ptr){
+long FXMDIChild::onSelected(FXObject*,FXSelector,void* ptr){    // FIXME
   if(!(flags&FLAG_ACTIVE)){
     if(target) target->handle(this,MKUINT(message,SEL_SELECTED),ptr);
     windowbtn->setBackColor(hasFocus() ? titleBackColor : shadowColor);
@@ -940,7 +906,7 @@ long FXMDIChild::onSelected(FXObject*,FXSelector,void* ptr){
 
 
 // Window was deselected
-long FXMDIChild::onDeselected(FXObject*,FXSelector,void* ptr){
+long FXMDIChild::onDeselected(FXObject*,FXSelector,void* ptr){    // FIXME
   if(flags&FLAG_ACTIVE){
     if(target) target->handle(this,MKUINT(message,SEL_DESELECTED),ptr);
     windowbtn->setBackColor(backColor);
@@ -954,56 +920,50 @@ long FXMDIChild::onDeselected(FXObject*,FXSelector,void* ptr){
 
 // Window was minimized
 long FXMDIChild::onMinimize(FXObject*,FXSelector,void*){
-  if(!(options&MDI_MINIMIZED)){
-    minimize();
-    if(target) target->handle(this,MKUINT(message,SEL_MINIMIZE),NULL);
-    }
-  return 1;
+  return target && target->handle(this,MKUINT(message,SEL_MINIMIZE),NULL);
   }
 
 
 // Window was restored
 long FXMDIChild::onRestore(FXObject*,FXSelector,void*){
-  if(options&(MDI_MINIMIZED|MDI_MAXIMIZED)){
-    restore();
-    if(target) target->handle(this,MKUINT(message,SEL_RESTORE),NULL);
-    }
-  return 1;
+  return target && target->handle(this,MKUINT(message,SEL_RESTORE),NULL);
   }
 
 
 // Window was maximized
 long FXMDIChild::onMaximize(FXObject*,FXSelector,void*){
-  if(!(options&MDI_MAXIMIZED)){
-    maximize();
-    if(target) target->handle(this,MKUINT(message,SEL_MAXIMIZE),NULL);
-    }
-  return 1;
+  return target && target->handle(this,MKUINT(message,SEL_MAXIMIZE),NULL);
   }
 
 
-// Ask target if it's OK to close this window
+// Returns 1 if its OK to close the window
 long FXMDIChild::onClose(FXObject*,FXSelector,void*){
   return !target || target->handle(this,MKUINT(message,SEL_CLOSE),NULL);
   }
 
 
+// Returns 1 if its OK to close all windows
+long FXMDIChild::onCloseAll(FXObject*,FXSelector,void*){
+  return !target || target->handle(this,MKUINT(message,SEL_CLOSEALL),NULL);
+  }
+
+
 // Delete window
 long FXMDIChild::onDelete(FXObject*,FXSelector,void*){
   FXMDIChild *alternative;
-  
+
   // Try find another window to activate
   alternative=mdinext?mdinext:mdiprev;
-  
+
   // First make sure we're inactive
   getParent()->handle(this,MKUINT(0,SEL_CHANGED),alternative);
-  
+
   // Tell target we're history
   if(target) target->handle(this,MKUINT(message,SEL_DELETE),NULL);
-  
+
   // Self destruct
   delete this;
-  
+
   return 1;
   }
 
@@ -1013,97 +973,42 @@ long FXMDIChild::onDelete(FXObject*,FXSelector,void*){
 
 // Restore window command
 long FXMDIChild::onCmdRestore(FXObject*,FXSelector,void*){
-  if(timer) return 1;
-  if(!isMaximized() && !isMinimized()) return 1;
-  if(getApp()->animSpeed>0){
-    if(isMinimized()){
-      tween=0;
-      anim=ANIM_MIN_2_RES;
-      }
-    else{
-      tween=100;
-      anim=ANIM_MAX_2_RES;
-      }
-    timer=getApp()->addTimeout(getApp()->animSpeed,this,ID_TIMER_ANIM);
-    }
-  else{
-    handle(this,MKUINT(0,SEL_RESTORE),NULL);
-    }
+  animateRectangles(xpos,ypos,width,height,normalPosX,normalPosY,normalWidth,normalHeight);
+  restore(TRUE);
   return 1;
   }
 
 
 // Maximize window command
 long FXMDIChild::onCmdMaximize(FXObject*,FXSelector,void*){
-  if(timer) return 1;
-  if(isMaximized()) return 1;
-  if(getApp()->animSpeed>0){
-    tween=0;
-    if(isMinimized()){
-      anim=ANIM_MIN_2_MAX;
-      }
-    else{
-      anim=ANIM_RES_2_MAX;
-      }
-    timer=getApp()->addTimeout(getApp()->animSpeed,this,ID_TIMER_ANIM);
-    }
-  else{
-    handle(this,MKUINT(0,SEL_MAXIMIZE),NULL);
-    }
+  animateRectangles(xpos,ypos,width,height,0,0,getParent()->getWidth(),getParent()->getHeight());
+  maximize(TRUE);
   return 1;
   }
 
 
 // Minimize window command
 long FXMDIChild::onCmdMinimize(FXObject*,FXSelector,void*){
-  if(timer) return 1;
-  if(isMinimized()) return 1;
-  if(getApp()->animSpeed>0){
-    tween=100;
-    if(isMaximized()){
-      anim=ANIM_MAX_2_MIN;
-      }
-    else{
-      anim=ANIM_RES_2_MIN;
-      }
-    timer=getApp()->addTimeout(getApp()->animSpeed,this,ID_TIMER_ANIM);
-    }
-  else{
-    handle(this,MKUINT(0,SEL_MINIMIZE),NULL);
-    }
+  animateRectangles(xpos,ypos,width,height,iconPosX,iconPosY,getDefaultWidth(),getDefaultHeight());
+  minimize(TRUE);
   return 1;
   }
 
 
 // Close window after asking target if its allowed
 long FXMDIChild::onCmdClose(FXObject*,FXSelector,void*){
-  if(timer) return 1;
   if(handle(this,MKUINT(0,SEL_CLOSE),NULL)){
-    if(getApp()->animSpeed>0){
-      tween=100;
-      if(isMaximized()){
-        anim=ANIM_MAX_2_NIL;
-        }
-      else if(isMinimized()){
-        anim=ANIM_MIN_2_NIL;
-        }
-      else{
-        anim=ANIM_RES_2_NIL;
-        }
-      timer=getApp()->addTimeout(getApp()->animSpeed,this,ID_TIMER_ANIM);
-      }
-    else{
-      handle(this,MKUINT(0,SEL_DELETE),NULL);
-      }
+    handle(this,MKUINT(ID_DELETE,SEL_COMMAND),NULL);
     return 1;
     }
   return 0;
   }
 
 
-// Delete window
+// Delete window command
 long FXMDIChild::onCmdDelete(FXObject*,FXSelector,void*){
-  handle(this,MKUINT(0,SEL_DELETE),NULL);
+  animateRectangles(xpos,ypos,width,height,xpos+width/2,ypos+height/2,0,0);
+  handle(this,MKUINT(0,SEL_DELETE),NULL);   // FIXME
   return 1;
   }
 
@@ -1164,7 +1069,7 @@ long FXMDIChild::onUpdWindow(FXObject* sender,FXSelector,void*){
 /*******************************************************************************/
 
 
-// Update menu's close button
+// Update MDI close button on menu bar
 long FXMDIChild::onUpdMenuClose(FXObject* sender,FXSelector,void*){
   if(isMaximized()){
     sender->handle(this,MKUINT(ID_ENABLE,SEL_COMMAND),NULL);
@@ -1177,7 +1082,7 @@ long FXMDIChild::onUpdMenuClose(FXObject* sender,FXSelector,void*){
   }
 
 
-// Update menu's restore button
+// Update MDI restore button on menu bar
 long FXMDIChild::onUpdMenuRestore(FXObject* sender,FXSelector,void*){
   if(isMaximized()){
     sender->handle(this,MKUINT(ID_SHOW,SEL_COMMAND),NULL);
@@ -1190,7 +1095,7 @@ long FXMDIChild::onUpdMenuRestore(FXObject* sender,FXSelector,void*){
   }
 
 
-// Update menu's minimized button
+// Update MDI minimized button on menu bar
 long FXMDIChild::onUpdMenuMinimize(FXObject* sender,FXSelector,void*){
   if(isMaximized()){
     sender->handle(this,MKUINT(ID_SHOW,SEL_COMMAND),NULL);
@@ -1203,7 +1108,7 @@ long FXMDIChild::onUpdMenuMinimize(FXObject* sender,FXSelector,void*){
   }
 
 
-// Update menu's window menu button
+// Update MDI window menu button on menu bar
 long FXMDIChild::onUpdMenuWindow(FXObject* sender,FXSelector,void*){
   if(isMaximized()){
     sender->handle(this,MKUINT(ID_ENABLE,SEL_COMMAND),NULL);
@@ -1249,6 +1154,20 @@ void FXMDIChild::setBorderColor(FXColor clr){
   }
 
 
+// Set title color
+void FXMDIChild::setTitleColor(FXColor clr){
+  titleColor=clr;
+  update();
+  }
+
+
+// Set title color
+void FXMDIChild::setTitleBackColor(FXColor clr){
+  titleBackColor=clr;
+  update();
+  }
+
+
 // Set new window title
 void FXMDIChild::setTitle(const FXString& name){
   if(title!=name){
@@ -1258,16 +1177,9 @@ void FXMDIChild::setTitle(const FXString& name){
   }
 
 
-// Handle message
-long FXMDIChild::handle(FXObject* sender,FXSelector key,void* data){
-  FXWindow *contents=contentWindow();
-  FXSelFunction func;
-  
-  // Filter out messages for MDIChild, regarless of return value
-  if((func=assoc(key))!=FXObject::null){ return (this->*func)(sender,key,data); }
-  
-  // Unknown messages are forwarded to contents of MDIChild
-  return contents && contents->handle(sender,key,data);
+// Delegate all other messages to child window
+long FXMDIChild::onDefault(FXObject* sender,FXSelector key,void* data){
+  return contentWindow() && contentWindow()->handle(sender,key,data);
   }
 
 
@@ -1324,6 +1236,8 @@ void FXMDIChild::save(FXStream& store) const {
   store << titleBackColor;
   store << iconPosX;
   store << iconPosY;
+  store << iconWidth;
+  store << iconHeight;
   store << normalPosX;
   store << normalPosY;
   store << normalWidth;
@@ -1349,18 +1263,19 @@ void FXMDIChild::load(FXStream& store){
   store >> titleBackColor;
   store >> iconPosX;
   store >> iconPosY;
+  store >> iconWidth;
+  store >> iconHeight;
   store >> normalPosX;
   store >> normalPosY;
   store >> normalWidth;
   store >> normalHeight;
-  }  
+  }
 
 
 // Destruct thrashes the pointers
 FXMDIChild::~FXMDIChild(){
-  if(timer) getApp()->removeTimeout(timer);
   if(mdiprev) mdiprev->mdinext=mdinext; else ((FXMDIClient*)getParent())->mdifirst=mdinext;
-  if(mdinext) mdinext->mdiprev=mdiprev; else ((FXMDIClient*)getParent())->mdilast=mdiprev; 
+  if(mdinext) mdinext->mdiprev=mdiprev; else ((FXMDIClient*)getParent())->mdilast=mdiprev;
   if(((FXMDIClient*)getParent())->active==this) ((FXMDIClient*)getParent())->active=NULL;
   windowbtn=(FXMenuButton*)-1;
   minimizebtn=(FXButton*)-1;
@@ -1368,5 +1283,4 @@ FXMDIChild::~FXMDIChild(){
   maximizebtn=(FXButton*)-1;
   deletebtn=(FXButton*)-1;
   font=(FXFont*)-1;
-  timer=(FXTimer*)-1;
   }
