@@ -3,30 +3,35 @@
 *          M u l t i p l e  D o c u m e n t   C l i e n t   W i n d o w         *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXMDIClient.cpp,v 1.5 1999/11/05 06:57:16 jeroen Exp $                   *
+* $Id: FXMDIClient.cpp,v 1.25 2001/04/27 14:30:34 jeroen Exp $                  *
 ********************************************************************************/
 #include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXApp.h"
@@ -46,22 +51,23 @@
 #include "FXMDIButton.h"
 #include "FXPacker.h"
 #include "FXHorizontalFrame.h"
-#include "FXMenuBar.h"
+#include "FXToolbar.h"
+#include "FXMenubar.h"
 #include "FXMDIChild.h"
 #include "FXScrollbar.h"
-#include "FXScrollWindow.h"
+#include "FXScrollArea.h"
 #include "FXMDIClient.h"
 
 
 /*
   Notes:
-  - This brings up the question of the cascade design.  Do you want the windows 
-    to be cascaded/tiled in the order that they were created, or do you want them 
-    to be cascaded in some sort of focus order, with the one that had the focus 
+  - This brings up the question of the cascade design.  Do you want the windows
+    to be cascaded/tiled in the order that they were created, or do you want them
+    to be cascaded in some sort of focus order, with the one that had the focus
     to be on top,  keeping its focus.
-    In the test app, if you click within any of the child windows, it pops to 
-    the top with the focus, with the exception of "TEST3", which has a button. 
-    If this button is clicked, that window dos not pop to the top when it 
+    In the test app, if you click within any of the child windows, it pops to
+    the top with the focus, with the exception of "TEST3", which has a button.
+    If this button is clicked, that window dos not pop to the top when it
     gets the focus.  Seems like it should...
   - Minor problems with mdi.  When a window is maximized and then
     deleted, the next window that is created is created in "normal" mode.
@@ -103,6 +109,8 @@ FXDEFMAP(FXMDIClient) FXMDIClientMap[]={
   FXMAPFUNC(SEL_UPDATE,FXWindow::ID_MDI_MENUWINDOW,FXMDIClient::onUpdMenuWindow),
   FXMAPFUNC(SEL_UPDATE,FXWindow::ID_CLOSE_DOCUMENT,FXMDIClient::onUpdCloseDocument),
   FXMAPFUNC(SEL_UPDATE,FXWindow::ID_CLOSE_ALL_DOCUMENTS,FXMDIClient::onUpdCloseAllDocuments),
+  FXMAPFUNC(SEL_UPDATE,FXMDIClient::ID_MDI_ANY,FXMDIClient::onUpdAnyWindows),
+  FXMAPFUNCS(SEL_UPDATE,FXMDIClient::ID_MDI_1,FXMDIClient::ID_MDI_10,FXMDIClient::onUpdWindowSelect),
   FXMAPFUNC(SEL_COMMAND,FXWindow::ID_MDI_NEXT,FXMDIClient::onCmdActivateNext),
   FXMAPFUNC(SEL_COMMAND,FXWindow::ID_MDI_PREV,FXMDIClient::onCmdActivatePrev),
   FXMAPFUNC(SEL_COMMAND,FXWindow::ID_MDI_TILEHORIZONTAL,FXMDIClient::onCmdTileHorizontal),
@@ -110,6 +118,7 @@ FXDEFMAP(FXMDIClient) FXMDIClientMap[]={
   FXMAPFUNC(SEL_COMMAND,FXWindow::ID_MDI_CASCADE,FXMDIClient::onCmdCascade),
   FXMAPFUNC(SEL_COMMAND,FXWindow::ID_CLOSE_DOCUMENT,FXMDIClient::onCmdCloseDocument),
   FXMAPFUNC(SEL_COMMAND,FXWindow::ID_CLOSE_ALL_DOCUMENTS,FXMDIClient::onCmdCloseAllDocuments),
+  FXMAPFUNCS(SEL_COMMAND,FXMDIClient::ID_MDI_1,FXMDIClient::ID_MDI_10,FXMDIClient::onCmdWindowSelect),
   };
 
 
@@ -134,7 +143,7 @@ FXMDIClient::FXMDIClient(){
 // Construct and init
 FXMDIClient::FXMDIClient(FXComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXint h):
   FXScrollArea(p,opts,x,y,w,h){
-  backColor=getApp()->shadowColor;
+  backColor=getApp()->getShadowColor();
   mdifirst=NULL;
   mdilast=NULL;
   active=NULL;
@@ -148,12 +157,64 @@ FXMDIClient::FXMDIClient(FXComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXin
 
 
 // Propagate size change
-void FXMDIClient::recalc(){ 
+void FXMDIClient::recalc(){
   FXScrollArea::recalc();
   flags|=FLAG_RECALC;
   }
 
 
+// User clicks on one of the window menus
+long FXMDIClient::onCmdWindowSelect(FXObject*,FXSelector sel,void*){
+  register FXMDIChild *child=getMDIChildFirst();
+  register FXint which=SELID(sel)-ID_MDI_1;
+  while(child && which){
+    child=child->getMDINext();
+    which--;
+    }
+  setActiveChild(child,TRUE);
+  return 1;
+  }
+
+
+// Update handler for window menus
+long FXMDIClient::onUpdWindowSelect(FXObject *sender,FXSelector sel,void*){
+  register FXMDIChild *child=getMDIChildFirst();
+  register FXint n=SELID(sel)-ID_MDI_1;
+  register FXint which=n+1;
+  while(child && n){
+    child=child->getMDINext();
+    n--;
+    }
+  if(child){
+    FXString string;
+    if(which<10)
+      string.format("&%d %s",which,child->getTitle().text());
+    else
+      string.format("1&0 %s",child->getTitle().text());
+    sender->handle(this,MKUINT(FXWindow::ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&string);
+    sender->handle(this,MKUINT(FXWindow::ID_SHOW,SEL_COMMAND),NULL);
+    if(child==active)
+      sender->handle(this,MKUINT(FXWindow::ID_CHECK,SEL_COMMAND),NULL);
+    else
+      sender->handle(this,MKUINT(FXWindow::ID_UNCHECK,SEL_COMMAND),NULL);
+    }
+  else{
+    sender->handle(this,MKUINT(FXWindow::ID_HIDE,SEL_COMMAND),NULL);
+    }
+  return 1;
+  }
+
+
+// Show or hide depending on whether there are any windows
+long FXMDIClient::onUpdAnyWindows(FXObject *sender,FXSelector,void*){
+  if(getMDIChildFirst())
+    sender->handle(this,MKUINT(FXWindow::ID_SHOW,SEL_COMMAND),NULL);
+  else
+    sender->handle(this,MKUINT(FXWindow::ID_HIDE,SEL_COMMAND),NULL);
+  return 1;
+  }
+
+
 // Update restore; gray if no active
 long FXMDIClient::onUpdRestore(FXObject* sender,FXSelector sel,void* ptr){
   if(active) return active->handle(sender,sel,ptr);
@@ -219,32 +280,38 @@ long FXMDIClient::onUpdMenuClose(FXObject* sender,FXSelector sel,void* ptr){
 
 
 // Set the active child
-FXbool FXMDIClient::setActiveChild(FXMDIChild* child){
+FXbool FXMDIClient::setActiveChild(FXMDIChild* child,FXbool notify){
   FXbool wasmax=FALSE;
   if(active!=child){
-    
+
     if(active){
-      
+
       // Was it maximized?
       wasmax=active->isMaximized();
-    
+
       // Deactivate old MDIChild
-      active->handle(this,MKUINT(0,SEL_DESELECTED),(void*)child);
-      
+      active->handle(this,MKUINT(0,SEL_DESELECTED),(void*)child);     // FIXME should call member function
+
       // Restore to normal size if it was maximized
-      if(wasmax) active->handle(this,MKUINT(0,SEL_RESTORE),NULL);
+      //if(wasmax) active->handle(this,MKUINT(0,SEL_RESTORE),NULL);
+      if(wasmax) active->restore(notify);
       }
-    
+
     if(child){
-      
+
       // Activate new MDIChild
-      child->handle(this,MKUINT(0,SEL_SELECTED),(void*)active);
-      
-      // Maximize because the old MDIChild was maximized 
-      if(wasmax) child->handle(this,MKUINT(0,SEL_MAXIMIZE),NULL);
+      child->handle(this,MKUINT(0,SEL_SELECTED),(void*)active);     // FIXME should call member function
+
+      // Maximize because the old MDIChild was maximized
+      //if(wasmax) active->handle(this,MKUINT(0,SEL_MAXIMIZE),NULL);
+      if(wasmax) child->maximize(notify);
       }
-    
+
     active=child;
+
+    // Notify target
+    //if(notify){handle(this,MKUINT(0,SEL_CHANGED),ptr);}   // FIXME
+
     recalc();
     return TRUE;
     }
@@ -254,13 +321,13 @@ FXbool FXMDIClient::setActiveChild(FXMDIChild* child){
 
 // Current child changed
 long FXMDIClient::onChanged(FXObject*,FXSelector,void* ptr){
+  //return target && target->handle(this,MKUINT(message,SEL_CHANGED),ptr);
   setActiveChild((FXMDIChild*)ptr);
-  if(target) target->handle(this,MKUINT(message,SEL_CHANGED),ptr);
+  if(target) target->handle(this,MKUINT(message,SEL_CHANGED),ptr);    // FIXME
   return 1;
   }
 
 
-
 // Tile horizontally (actually, prefer wider windows)
 long FXMDIClient::onCmdTileHorizontal(FXObject*,FXSelector,void*){
   register FXMDIChild* child;
@@ -282,7 +349,7 @@ long FXMDIClient::onCmdTileHorizontal(FXObject*,FXSelector,void*){
     if(child->shown() && !child->isMinimized()){
       r=n/nc;
       c=n%nc;
-      child->handle(this,MKUINT(0,SEL_RESTORE),NULL);
+      child->restore(TRUE);
       child->position(c*hroom,r*vroom,hroom,vroom);
       n++;
       }
@@ -322,7 +389,7 @@ long FXMDIClient::onCmdTileVertical(FXObject*,FXSelector,void*){
     if(child->shown() && !child->isMinimized()){
       r=n/nc;
       c=n%nc;
-      child->handle(this,MKUINT(0,SEL_RESTORE),NULL);
+      child->restore(TRUE);
       child->position(c*hroom,r*vroom,hroom,vroom);
       n++;
       }
@@ -344,7 +411,7 @@ long FXMDIClient::onUpdTileVertical(FXObject* sender,FXSelector,void* ptr){
 // Pass message to all MDI Child windows
 long FXMDIClient::forallWindows(FXObject* sender,FXSelector sel,void* ptr){
   register FXMDIChild *child,*nxtchild;
-  register FXbool handled=FALSE;
+  register FXbool handled=0;
   for(child=mdifirst; child; child=nxtchild){
     nxtchild=child->getMDINext();
     handled|=child->handle(sender,sel,ptr);
@@ -356,7 +423,7 @@ long FXMDIClient::forallWindows(FXObject* sender,FXSelector sel,void* ptr){
 // Pass message to all MDI Child windows whose target is document
 long FXMDIClient::forallDocWindows(FXObject* document,FXObject* sender,FXSelector sel,void* ptr){
   register FXMDIChild *child,*nxtchild;
-  register FXbool handled=FALSE;
+  register FXbool handled=0;
   if(document){
     for(child=mdifirst; child; child=nxtchild){
       nxtchild=child->getMDINext();
@@ -388,7 +455,7 @@ long FXMDIClient::onCmdCascade(FXObject*,FXSelector,void*){
       }
     }
   if(active && active->shown() && !active->isMinimized()){
-    active->restore();
+    active->restore(TRUE);
     active->position(childx,childy,childw,childh);
     active->raise();
     }
@@ -443,90 +510,58 @@ long FXMDIClient::onUpdActivatePrev(FXObject* sender,FXSelector,void*){
   }
 
 
-// Close all windows of current document; return 1 if ALL are closed
+// Close all windows whose document is the same as the active
+// window, or, if the active window does not have a document,
+// close the active window only.
+// Before any windows are closed, a SEL_CLOSEALL message is sent
+// to the active window to ask if it is OK to close the windows.
 long FXMDIClient::onCmdCloseDocument(FXObject* sender,FXSelector,void* ptr){
   register FXMDIChild *child,*nxtchild;
   register FXObject *document;
-  register long allclosed=TRUE;
-  if(active){
+  if(active && active->handle(this,MKUINT(0,SEL_CLOSEALL),NULL)){
     document=active->getTarget();
     if(document){
       for(child=mdifirst; child; child=nxtchild){
         nxtchild=child->getMDINext();
-        if(document==child->getTarget()){ 
-          allclosed &= child->handle(sender,MKUINT(ID_MDI_CLOSE,SEL_COMMAND),ptr); 
+        if(document==child->getTarget()){
+          child->handle(sender,MKUINT(ID_DELETE,SEL_COMMAND),ptr);
           }
         }
       }
     else{
-      allclosed=active->handle(sender,MKUINT(ID_MDI_CLOSE,SEL_COMMAND),ptr);
+      active->handle(sender,MKUINT(ID_DELETE,SEL_COMMAND),ptr);
       }
+    return 1;
     }
-  return allclosed;
-//   register FXMDIChild *child,*nxtchild;
-//   register FXObject *document;
-//   register long closed=0;
-//   if(active){
-//     document=active->getTarget();
-//     if(document){
-//       if(document->handle(active,MKUINT(active->getSelector(),SEL_CLOSEALL),ptr)){
-//         closed=1;
-//         for(child=mdifirst; child; child=nxtchild){
-//           nxtchild=child->getMDINext();
-//           if(document==child->getTarget()){
-//             closed &= child->handle(sender,MKUINT(ID_MDI_CLOSE,SEL_COMMAND),ptr);
-//             }
-//           }
-//         }
-//       }
-//     else{
-//       closed=active->handle(sender,MKUINT(ID_MDI_CLOSE,SEL_COMMAND),ptr);
-//       }
-//     }
-//   return closed;
+  return 0;
   }
 
 
-// Close all windows of all documents; return 1 if ALL are closed
+// Close all windows. Before any windows are closed, a SEL_CLOSEALL
+// message is sent to the window. The message is sent only once for
+// each group of windows having the same document.
+// If any single window does not want to be closed, no windows will
+// be closed at all.
 long FXMDIClient::onCmdCloseAllDocuments(FXObject* sender,FXSelector,void* ptr){
-  register FXMDIChild *child,*nxtchild;
-  register long allclosed=TRUE;
+  register FXMDIChild *child,*nxtchild,*ch;
+  register FXObject *document;
+  for(child=mdifirst; child; child=child->getMDINext()){
+    document=child->getTarget();
+    if(document){
+      for(ch=child->getMDIPrev(); ch; ch=ch->getMDIPrev()){
+        if(ch->getTarget()==document) goto nxt;
+        }
+      }
+    if(!child->handle(this,MKUINT(0,SEL_CLOSEALL),NULL)) return 0;
+nxt:continue;
+    }
   for(child=mdifirst; child; child=nxtchild){
     nxtchild=child->getMDINext();
-    allclosed &= child->handle(sender,MKUINT(ID_MDI_CLOSE,SEL_COMMAND),ptr);
+    child->handle(sender,MKUINT(ID_DELETE,SEL_COMMAND),ptr);
     }
-  return allclosed;
-//   register FXMDIChild *child,*docchild,*nxtchild,*n,*p;
-//   register FXObject *document;
-//   register long closed=0;
-//   if(mdifirst){
-//     closed=1;
-//     for(docchild=mdifirst; docchild; docchild=nxtchild){
-//       document=docchild->getTarget();
-//       nxtchild=docchild->getMDINext();
-//       if(document){
-//         while(nxtchild){
-//           for(p=docchild; p && p->getTarget()!=nxtchild->getTarget(); p=p->getMDIPrev());
-//           if(!p) break;
-//           nxtchild=nxtchild->getMDINext();
-//           }
-//         if(document->handle(docchild,MKUINT(docchild->getSelector(),SEL_CLOSEALL),ptr)){
-//           for(child=mdifirst; child; child=n){
-//             n=child->getMDINext();
-//             if(document==child->getTarget()){
-//               closed &= child->handle(sender,MKUINT(ID_MDI_CLOSE,SEL_COMMAND),ptr);
-//               }
-//             }
-//           }
-//         }
-//       else{
-//         closed &= docchild->handle(sender,MKUINT(ID_MDI_CLOSE,SEL_COMMAND),ptr);
-//         }
-//       }
-//     }
-//   return closed;
+  return 1;
   }
-        
+
 
 // Update close current document
 long FXMDIClient::onUpdCloseDocument(FXObject* sender,FXSelector,void*){
@@ -548,7 +583,7 @@ long FXMDIClient::onUpdCloseAllDocuments(FXObject* sender,FXSelector,void*){
   }
 
 
-// Recompute interior 
+// Recompute interior
 void FXMDIClient::recompute(){
   register FXMDIChild* child;
   register FXint xl,yl,xh,yh;
@@ -589,6 +624,7 @@ void FXMDIClient::recompute(){
 
 // Determine content width of scroll area
 FXint FXMDIClient::getContentWidth(){
+  FXint ww,hh,vw;
 
   // If active child is maximized, client is non-scrollable
   if(active && active->isMaximized()){
@@ -599,21 +635,23 @@ FXint FXMDIClient::getContentWidth(){
     flags&=~FLAG_RECALC;
     return width;
     }
-  
+
   // Recalculate contents
   if(flags&FLAG_RECALC) recompute();
-  
-  // Return the right size
-  if(xmin>xmax) return 1;
-  if(xmin<0 && xmax<=width) return width-xmin;
-  if(0<=xmin && width<xmax) return xmax;
-  return xmax-xmin;
+
+  // Return the content width
+  vw=width;
+  hh=FXMAX(ymax,height)-FXMIN(ymin,0);
+  if(hh>height) vw-=vertical->getDefaultWidth();
+  ww=FXMAX(xmax,vw)-FXMIN(xmin,0);
+  return ww;
   }
 
 
 // Determine content height of scroll area
-FXint FXMDIClient::getContentHeight(){ 
-  
+FXint FXMDIClient::getContentHeight(){
+  FXint ww,hh,vh;
+
   // If active child is maximized, client is non-scrollable
   if(active && active->isMaximized()){
     xmin=0;
@@ -623,15 +661,16 @@ FXint FXMDIClient::getContentHeight(){
     flags&=~FLAG_RECALC;
     return height;
     }
-  
+
   // Recalculate contents
   if(flags&FLAG_RECALC) recompute();
-  
-  // Return the right size
-  if(ymin>ymax) return 1;
-  if(ymin<0 && ymax<=height) return height-ymin;
-  if(0<=ymin && height<ymax) return ymax;
-  return ymax-ymin;
+
+  // Return the content height
+  vh=height;
+  ww=FXMAX(xmax,width)-FXMIN(xmin,0);
+  if(ww>width) vh-=horizontal->getDefaultHeight();
+  hh=FXMAX(ymax,vh)-FXMIN(ymin,0);
+  return hh;
   }
 
 
@@ -678,7 +717,10 @@ void FXMDIClient::layout(){
 
   // Raise active child
   if(active && active->shown()) active->raise();
-  
+
+  FXTRACE((150,"xmin=%4d xmax=%4d\n",xmin,xmax));
+  FXTRACE((150,"ymin=%4d ymax=%4d\n",ymin,ymax));
+
   // Recompute scroll bars, and raise them on top of everything
   FXScrollArea::layout();
 
@@ -703,14 +745,8 @@ void FXMDIClient::layout(){
   }
 
 
-// Handle message
-long FXMDIClient::handle(FXObject* sender,FXSelector key,void* data){
-  FXSelFunction func;
-
-  // Filter out messages for MDIClient, regardless of return value
-  if((func=assoc(key))!=FXObject::null){ return (this->*func)(sender,key,data); }
-
-  // Unknown messages get forwarded to active MDIChild
+// Delegate all other messages to active child
+long FXMDIClient::onDefault(FXObject* sender,FXSelector key,void* data){
   return active && active->handle(sender,key,data);
   }
 
@@ -742,7 +778,7 @@ void FXMDIClient::load(FXStream& store){
   store >> xmax;
   store >> ymin;
   store >> ymax;
-  }  
+  }
 
 
 // Destruct thrashes object
