@@ -3,31 +3,36 @@
 *                   M a t r i x   C o n t a i n e r   O b j e c t               *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXMatrix.cpp,v 1.1 1999/09/14 13:41:23 jeroen Exp $                     *
+* $Id: FXMatrix.cpp,v 1.13 2001/06/06 14:23:09 jeroen Exp $                     *
 ********************************************************************************/
+#include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
-#include "FXAccelTable.h"
 #include "FXApp.h"
 #include "FXId.h"
 #include "FXDrawable.h"
@@ -36,9 +41,8 @@
 #include "FXComposite.h"
 #include "FXPacker.h"
 #include "FXMatrix.h"
-#include "FXShell.h"
 
-  
+
 /*
   Notes:
   - Need to observe FILL vs CENTER options.
@@ -48,18 +52,23 @@
   - Row/Column stretchable iff all elements in row/column have row/column
     stretch hint.
   - Row/Column may be 0 pixels wide
+  - We should probably change layout so hidden children do not
+    affect numbering (but if all children in a row/column are
+    hidden, make the whole row disappear!).
+  - Navigating around 
   - Packing order:
-  
+
     MATRIX_BY_ROWS:
     [1] [4] [7]
     [2] [5] [8]
     [3] [6] [9]
-    
+
     MATRIX_BY_COLUMNS:
     [1] [2] [3]
     [4] [5] [6]
     [7] [8] [9]
     
+
 */
 
 #define MAXNUM    512        // Maximum number of columns/rows
@@ -83,8 +92,8 @@ FXIMPLEMENT(FXMatrix,FXPacker,FXMatrixMap,ARRAYNUMBER(FXMatrixMap))
 
 // Make a vertical one
 FXMatrix::FXMatrix(FXComposite* p,FXint n,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs):
-  FXPacker(p,opts,x,y,w,h,pl,pr,pt,pb,hs,vs),num(n){
-  if(num>MAXNUM) num=MAXNUM; else if(num<1) num=1;
+  FXPacker(p,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){
+  num=FXCLAMP(1,n,MAXNUM);
   }
 
 
@@ -96,22 +105,22 @@ FXWindow* FXMatrix::childAtRowCol(FXint r,FXint c) const {
 
 // Get child's row
 FXint FXMatrix::rowOfChild(FXWindow* child) const {
-  FXint i=indexOfChild(child);
+  register FXint i=indexOfChild(child);
   return (options&MATRIX_BY_COLUMNS) ? i/num : i%num;
   }
 
 
 // Get child's column
 FXint FXMatrix::colOfChild(FXWindow* child) const {
-  FXint i=indexOfChild(child);
+  register FXint i=indexOfChild(child);
   return (options&MATRIX_BY_COLUMNS) ? i%num : i/num;
   }
 
 
 // Focus moved up
 long FXMatrix::onFocusUp(FXObject*,FXSelector sel,void* ptr){
-  FXWindow *child;
-  FXint r,c;
+  register FXWindow *child;
+  register FXint r,c;
   if(getFocus()){
     r=rowOfChild(getFocus());
     c=colOfChild(getFocus());
@@ -143,13 +152,13 @@ long FXMatrix::onFocusUp(FXObject*,FXSelector sel,void* ptr){
 
 // Focus moved down
 long FXMatrix::onFocusDown(FXObject*,FXSelector sel,void* ptr){
-  FXWindow *child;
-  FXint r,c;
+  register FXWindow *child;
+  register FXint r,c;
   if(getFocus()){
     r=rowOfChild(getFocus());
     c=colOfChild(getFocus());
     child=childAtRowCol(r+1,c);
-    FXTRACE((100,"r=%d c=%d child=%08x\n",r,c,child));
+    FXTRACE((100,"r=%d c=%d child=%p\n",r,c,child));
     if(child){
       if(child->isEnabled() && child->canFocus()){
         child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
@@ -177,8 +186,8 @@ long FXMatrix::onFocusDown(FXObject*,FXSelector sel,void* ptr){
 
 // Focus moved to left
 long FXMatrix::onFocusLeft(FXObject*,FXSelector sel,void* ptr){
-  FXWindow *child;
-  FXint r,c;
+  register FXWindow *child;
+  register FXint r,c;
   if(getFocus()){
     r=rowOfChild(getFocus());
     c=colOfChild(getFocus());
@@ -210,13 +219,13 @@ long FXMatrix::onFocusLeft(FXObject*,FXSelector sel,void* ptr){
 
 // Focus moved to right
 long FXMatrix::onFocusRight(FXObject*,FXSelector sel,void* ptr){
-  FXWindow *child;
-  FXint r,c;
+  register FXWindow *child;
+  register FXint r,c;
   if(getFocus()){
     r=rowOfChild(getFocus());
     c=colOfChild(getFocus());
     child=childAtRowCol(r,c+1);
-    FXTRACE((100,"r=%d c=%d child=%08x\n",r,c,child));
+    FXTRACE((100,"r=%d c=%d child=%p\n",r,c,child));
     if(child){
       if(child->isEnabled() && child->canFocus()){
         child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
@@ -247,31 +256,31 @@ void FXMatrix::setNumRows(FXint nr){
   if(nr<1 || nr>=MAXNUM){ fxerror("%s::setNumRows: bad number of rows specified.\n",getClassName()); }
   if(!(options&MATRIX_BY_COLUMNS)) num=nr;
   }
-  
-  
+
+
 // Get number of rows
 FXint FXMatrix::getNumRows() const {
   register FXWindow *child;
   register FXint n;
   if(!(options&MATRIX_BY_COLUMNS)) return num;
-  for(child=getFirst(),n=0; child; child=child->getNext()){if(child->shown()) n++;}
+  for(child=getFirst(),n=0; child; child=child->getNext()){ if(child->shown()) n++; }
   return (n+num-1)/num;
   }
-  
-  
+
+
 // Set number of columns (if possible)
 void FXMatrix::setNumColumns(FXint nc){
   if(nc<1 || nc>=MAXNUM){ fxerror("%s::setNumColumns: bad number of columns specified.\n",getClassName()); }
   if(options&MATRIX_BY_COLUMNS) num=nc;
   }
-  
-  
+
+
 // Get number of columns
 FXint FXMatrix::getNumColumns() const {
   register FXWindow *child;
   register FXint n;
   if(options&MATRIX_BY_COLUMNS) return num;
-  for(child=getFirst(),n=0; child; child=child->getNext()){if(child->shown()) n++;}
+  for(child=getFirst(),n=0; child; child=child->getNext()){ if(child->shown()) n++; }
   return (n+num-1)/num;
   }
 
@@ -287,7 +296,7 @@ FXint FXMatrix::getDefaultWidth(){
   for(child=getFirst(),n=0; child; child=child->getNext(),n++){
     if(child->shown()){
       hints=child->getLayoutHints();
-      if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth(); 
+      if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth();
       else if(options&PACK_UNIFORM_WIDTH) w=mw;
       else w=child->getDefaultWidth();
       c=(options&MATRIX_BY_COLUMNS)?n%num:n/num;
@@ -316,7 +325,7 @@ FXint FXMatrix::getDefaultHeight(){
   for(child=getFirst(),n=0; child; child=child->getNext(),n++){
     if(child->shown()){
       hints=child->getLayoutHints();
-      if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight(); 
+      if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight();
       else if(options&PACK_UNIFORM_HEIGHT) h=mh;
       else h=child->getDefaultHeight();
       r=(options&MATRIX_BY_COLUMNS)?n/num:n%num;
@@ -346,12 +355,7 @@ void FXMatrix::layout(){
   FXint vsumexpand,vnumexpand;
   FXWindow *child;
   FXuint hints;
-  
-  flags&=~FLAG_DIRTY;
-  
-  // Done already?
-  if(getFirst()==NULL) return;
-  
+
   // Placement rectangle; right/bottom non-inclusive
   left=border+padleft;
   right=width-border-padright;
@@ -359,11 +363,11 @@ void FXMatrix::layout(){
   bottom=height-border-padbottom;
   hremain=right-left;
   vremain=bottom-top;
-  
+
   // Non-zero rows/columns
   nzrow=0;
   nzcol=0;
-  
+
   // Clear column/row sizes
   for(n=0; n<MAXNUM; n++){
     colw[n]=rowh[n]=0;          // Columns may be 0 size
@@ -373,17 +377,17 @@ void FXMatrix::layout(){
   // Get maximum child size
   if(options&PACK_UNIFORM_WIDTH) mw=maxChildWidth();
   if(options&PACK_UNIFORM_HEIGHT) mh=maxChildHeight();
-    
+
   // Find expandable columns and rows
   for(child=getFirst(),n=0; child; child=child->getNext(),n++){
     if(child->shown()){
       hints=child->getLayoutHints();
       if(options&MATRIX_BY_COLUMNS){r=n/num;c=n%num;}else{r=n%num;c=n/num;}
       FXASSERT(r<MAXNUM && c<MAXNUM);
-      if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth(); 
+      if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth();
       else if(options&PACK_UNIFORM_WIDTH) w=mw;
       else w=child->getDefaultWidth();
-      if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight(); 
+      if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight();
       else if(options&PACK_UNIFORM_HEIGHT) h=mh;
       else h=child->getDefaultHeight();
       FXASSERT(w>=0);
@@ -394,7 +398,7 @@ void FXMatrix::layout(){
       if(!(hints&LAYOUT_FILL_ROW)) srow[r]=0;
       }
     }
-      
+
   // Get number of rows and columns
   if(options&MATRIX_BY_COLUMNS){
     ncol=num;
@@ -406,27 +410,27 @@ void FXMatrix::layout(){
     }
 
   // Find stretch in columns
-  for(c=hsumexpand=hnumexpand=0; c<ncol; c++){ 
+  for(c=hsumexpand=hnumexpand=0; c<ncol; c++){
     if(colw[c]){
       if(scol[c]){
-        hsumexpand+=colw[c]; 
+        hsumexpand+=colw[c];
         hnumexpand++;
         }
       else{
-        hremain-=colw[c]; 
+        hremain-=colw[c];
         }
       }
     }
 
   // Find stretch in rows
-  for(r=vsumexpand=vnumexpand=0; r<nrow; r++){ 
+  for(r=vsumexpand=vnumexpand=0; r<nrow; r++){
     if(rowh[r]){
       if(srow[r]){
-        vsumexpand+=rowh[r]; 
+        vsumexpand+=rowh[r];
         vnumexpand++;
         }
       else{
-        vremain-=rowh[r]; 
+        vremain-=rowh[r];
         }
       }
     }
@@ -434,20 +438,20 @@ void FXMatrix::layout(){
   // Substract spacing for non-zero rows/columns
   if(nzcol>1) hremain-=(nzcol-1)*hspacing;
   if(nzrow>1) vremain-=(nzrow-1)*vspacing;
-  
+
   // Disburse space horizontally
   for(c=e=0,x=border+padleft; c<ncol; c++){
     w=colw[c];
     colw[c]=x;
     if(w){
       if(scol[c]){
-        if(hsumexpand>0){                         // Divide proportionally 
+        if(hsumexpand>0){                         // Divide proportionally
           t=w*hremain;
           w=t/hsumexpand;
           e+=t%hsumexpand;
           if(e>=hsumexpand){w++;e-=hsumexpand;}
           }
-        else{                                     // Divide equally 
+        else{                                     // Divide equally
           FXASSERT(hnumexpand>0);
           w=hremain/hnumexpand;
           e+=hremain%hnumexpand;
@@ -465,13 +469,13 @@ void FXMatrix::layout(){
     rowh[r]=y;
     if(h){
       if(srow[r]){
-        if(vsumexpand>0){                         // Divide proportionally 
+        if(vsumexpand>0){                         // Divide proportionally
           t=h*vremain;
           h=t/vsumexpand;
           e+=t%vsumexpand;
           if(e>=vsumexpand){h++;e-=vsumexpand;}
           }
-        else{                                     // Divide equally 
+        else{                                     // Divide equally
           FXASSERT(vnumexpand>0);
           h=vremain/vnumexpand;
           e+=vremain%vnumexpand;
@@ -482,7 +486,7 @@ void FXMatrix::layout(){
       }
     }
   rowh[nrow]=y;
-        
+
   // Do the layout
   for(child=getFirst(),n=0; child; child=child->getNext(),n++){
     if(child->shown()){
@@ -490,28 +494,29 @@ void FXMatrix::layout(){
       if(options&MATRIX_BY_COLUMNS){r=n/num;c=n%num;}else{r=n%num;c=n/num;}
       cw=colw[c+1]-colw[c]-hspacing;
       rh=rowh[r+1]-rowh[r]-vspacing;
-      
-      if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth(); 
+
+      if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth();
       else if(hints&LAYOUT_FILL_X) w=cw;
       else if(options&PACK_UNIFORM_WIDTH) w=mw;
       else w=child->getDefaultWidth();
-      
+
       if(hints&LAYOUT_CENTER_X) x=colw[c]+(cw-w)/2;
       else if(hints&LAYOUT_RIGHT) x=colw[c]+cw-w;
       else x=colw[c];
-      
-      if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight(); 
+
+      if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight();
       else if(hints&LAYOUT_FILL_Y) h=rh;
       else if(options&PACK_UNIFORM_HEIGHT) h=mh;
       else h=child->getDefaultHeight();
-    
+
       if(hints&LAYOUT_CENTER_Y) y=rowh[r]+(rh-h)/2;
       else if(hints&LAYOUT_BOTTOM) y=rowh[r]+rh-h;
       else y=rowh[r];
-      
+
       child->position(x,y,w,h);
       }
     }
+  flags&=~FLAG_DIRTY;
   }
 
 
@@ -524,7 +529,7 @@ void FXMatrix::setMatrixStyle(FXuint style){
     update();
     }
   }
-  
+
 
 // Return matrix style
 FXuint FXMatrix::getMatrixStyle() const {
