@@ -3,29 +3,36 @@
 *                       M e n u    B u t t o n    O b j e c t                   *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXMenuButton.cpp,v 1.6 1999/10/20 03:39:09 jeroen Exp $                  *
+* $Id: FXMenuButton.cpp,v 1.22 2001/08/10 18:09:52 jeroen Exp $                 *
 ********************************************************************************/
+#include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
+#include "fxkeys.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXApp.h"
@@ -53,22 +60,22 @@
     the case of popping it down:
 
     MENUBUTTON_ATTACH_LEFT:     MENUBUTTON_ATTACH_RIGHT:
- 
+
      +------+                        +------+
      |Button|                        |Button|
      +------+---+                +---+------+
      |   Pane   |                |   Pane   |
      +----------+                +----------+
-     
-   
+
+
      MENUBUTTON_ATTACH_CENTER:  MENUBUTTON_ATTACH_BOTH:
- 
+
        +------+                  +----------+
        |Button|                  |  Button  |
      +-+------+-+                +----------+
      |   Pane   |                |   Pane   |
      +----------+                +----------+
-     
+
   - The width (height) of the pane is taken into account by getDefaultWidth()
     (getDefaultHeight()), so that the button will stretch to fit the pane.
   - You can specify horizontal or vertical offset to position the pane away
@@ -100,14 +107,10 @@ FXDEFMAP(FXMenuButton) FXMenuButtonMap[]={
   FXMAPFUNC(SEL_LEFTBUTTONRELEASE,0,FXMenuButton::onLeftBtnRelease),
   FXMAPFUNC(SEL_KEYPRESS,0,FXMenuButton::onKeyPress),
   FXMAPFUNC(SEL_KEYRELEASE,0,FXMenuButton::onKeyRelease),
-  FXMAPFUNC(SEL_ACTIVATE,0,FXMenuButton::onActivate),
-  FXMAPFUNC(SEL_DEACTIVATE,0,FXMenuButton::onDeactivate),
   FXMAPFUNC(SEL_KEYPRESS,FXWindow::ID_HOTKEY,FXMenuButton::onHotKeyPress),
   FXMAPFUNC(SEL_KEYRELEASE,FXWindow::ID_HOTKEY,FXMenuButton::onHotKeyRelease),
   FXMAPFUNC(SEL_COMMAND,FXWindow::ID_POST,FXMenuButton::onCmdPost),
   FXMAPFUNC(SEL_COMMAND,FXWindow::ID_UNPOST,FXMenuButton::onCmdUnpost),
-  FXMAPFUNC(SEL_UPDATE,FXWindow::ID_QUERY_TIP,FXMenuButton::onQueryTip),
-  FXMAPFUNC(SEL_UPDATE,FXWindow::ID_QUERY_HELP,FXMenuButton::onQueryHelp),
   };
 
 
@@ -117,7 +120,6 @@ FXIMPLEMENT(FXMenuButton,FXLabel,FXMenuButtonMap,ARRAYNUMBER(FXMenuButtonMap))
 
 // Deserialization
 FXMenuButton::FXMenuButton(){
-  flags|=FLAG_ENABLED;
   pane=(FXPopup*)-1;
   offsetx=0;
   offsety=0;
@@ -128,9 +130,6 @@ FXMenuButton::FXMenuButton(){
 // Make a check button
 FXMenuButton::FXMenuButton(FXComposite* p,const FXString& text,FXIcon* ic,FXPopup* pup,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb):
   FXLabel(p,text,ic,opts,x,y,w,h,pl,pr,pt,pb){
-  tip=text.extract(1,'\t');
-  help=text.extract(2,'\t');
-  flags|=FLAG_ENABLED;
   pane=pup;
   offsetx=0;
   offsety=0;
@@ -152,24 +151,6 @@ void FXMenuButton::detach(){
   }
 
 
-// Enable the window
-void FXMenuButton::enable(){
-  if(!(flags&FLAG_ENABLED)){
-    FXWindow::enable();
-    update();
-    }
-  }
-
-
-// Disable the window
-void FXMenuButton::disable(){
-  if(flags&FLAG_ENABLED){
-    FXWindow::disable();
-    update();
-    }
-  }
-
-
 // If window can have focus
 FXbool FXMenuButton::canFocus() const { return 1; }
 
@@ -225,7 +206,7 @@ long FXMenuButton::onFocusIn(FXObject* sender,FXSelector sel,void* ptr){
   return 1;
   }
 
-  
+
 // Lost focus
 long FXMenuButton::onFocusOut(FXObject* sender,FXSelector sel,void* ptr){
   FXLabel::onFocusOut(sender,sel,ptr);
@@ -234,309 +215,6 @@ long FXMenuButton::onFocusOut(FXObject* sender,FXSelector sel,void* ptr){
   }
 
 
-// We were asked about status text
-long FXMenuButton::onQueryHelp(FXObject* sender,FXSelector,void*){
-  if(!help.empty() && (flags&FLAG_HELP)){ 
-    sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),&help);
-    return 1;
-    }
-  return 0;
-  }
-
-
-// We were asked about tip text
-long FXMenuButton::onQueryTip(FXObject* sender,FXSelector,void*){
-  if(!tip.empty() && (flags&FLAG_TIP)){
-    sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),&tip);
-    return 1;
-    }
-  return 0;
-  }
-
-
-
-// Hot key combination pressed
-long FXMenuButton::onHotKeyPress(FXObject*,FXSelector,void* ptr){
-  FXTRACE((200,"%s::onHotKeyPress %08x\n",getClassName(),this));
-  flags&=~FLAG_TIP;
-  if(isEnabled()){
-    handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
-    handle(this,MKUINT(0,SEL_ACTIVATE),ptr);
-    }
-  return 1;
-  }
-
-
-// Hot key combination released
-long FXMenuButton::onHotKeyRelease(FXObject*,FXSelector,void* ptr){
-  FXTRACE((200,"%s::onHotKeyRelease %08x\n",getClassName(),this));
-  flags&=~FLAG_TIP;
-  if(isEnabled()){ 
-    handle(this,MKUINT(0,SEL_DEACTIVATE),ptr); 
-    }
-  return 1;
-  }
-
- 
-// The widget lost the grab for some reason
-long FXMenuButton::onUngrabbed(FXObject* sender,FXSelector sel,void* ptr){
-  FXLabel::onUngrabbed(sender,sel,ptr);
-  flags&=~FLAG_PRESSED;
-  flags|=FLAG_UPDATE;
-  handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
-  return 1;
-  }
-
-
-// Handle repaint 
-long FXMenuButton::onPaint(FXObject*,FXSelector,void* ptr){
-  FXint tw=0,th=0,iw=0,ih=0,tx,ty,ix,iy;
-  FXEvent *ev=(FXEvent*)ptr;
-  FXPoint points[3];
-  FXDCWindow dc(this,ev);
-  
-  // Got a border at all?
-  if(options&(FRAME_RAISED|FRAME_SUNKEN)){
-
-    // Toolbar style
-    if(options&MENUBUTTON_TOOLBAR){
-      
-      // Enabled and cursor inside, and not popped up
-      if(isEnabled() && underCursor() && !state){
-        dc.setForeground(backColor);
-        dc.fillRectangle(border,border,width-border*2,height-border*2);
-        if(options&FRAME_THICK) drawDoubleRaisedRectangle(dc,0,0,width,height);
-        else drawRaisedRectangle(dc,0,0,width,height);
-        }
-      
-      // Enabled and popped up
-      else if(isEnabled() && state){
-        dc.setForeground(hiliteColor);
-        dc.fillRectangle(border,border,width-border*2,height-border*2);
-        if(options&FRAME_THICK) drawDoubleSunkenRectangle(dc,0,0,width,height);
-        else drawSunkenRectangle(dc,0,0,width,height);
-        }
-        
-      // Disabled or unchecked or not under cursor
-      else{
-        dc.setForeground(backColor);
-        dc.fillRectangle(0,0,width,height);
-        }
-      }
- 
-    // Normal style
-    else{
-      
-      // Draw in up state if disabled or up
-      if(!isEnabled() || !state){
-        dc.setForeground(backColor);
-        dc.fillRectangle(border,border,width-border*2,height-border*2);
-        if(options&FRAME_THICK) drawDoubleRaisedRectangle(dc,0,0,width,height);
-        else drawRaisedRectangle(dc,0,0,width,height);
-        }
-
-      // Draw sunken if enabled and either checked or pressed
-      else{
-        dc.setForeground(hiliteColor);
-        dc.fillRectangle(border,border,width-border*2,height-border*2);
-        if(options&FRAME_THICK) drawDoubleSunkenRectangle(dc,0,0,width,height);
-        else drawSunkenRectangle(dc,0,0,width,height);
-        }
-      }
-    }
-  
-  // No borders
-  else{
-    if(isEnabled() && state){
-      dc.setForeground(hiliteColor);
-      dc.fillRectangle(0,0,width,height);
-      }
-    else{
-      dc.setForeground(backColor);
-      dc.fillRectangle(0,0,width,height);
-      }
-    }
-  
-  // Position text & icon
-  if(!label.empty()){
-    tw=labelWidth(label);
-    th=labelHeight(label);
-    }
-  
-  // Icon?
-  if(icon){
-    iw=icon->getWidth();
-    ih=icon->getHeight();
-    }
-  
-  // Arrows?
-  else if(!(options&MENUBUTTON_NOARROWS)){
-    if(options&MENUBUTTON_LEFT){
-      ih=MENUBUTTONARROW_WIDTH;
-      iw=MENUBUTTONARROW_HEIGHT;
-      }
-    else{
-      iw=MENUBUTTONARROW_WIDTH;
-      ih=MENUBUTTONARROW_HEIGHT;
-      }
-    }
-  
-  // Keep some room for the arrow!
-  just_x(tx,ix,tw,iw);
-  just_y(ty,iy,th,ih);
-
-  // Move a bit when pressed
-  if(state){ ++tx; ++ty; ++ix; ++iy; }
-
-  // Draw icon
-  if(icon){
-    if(isEnabled())
-      dc.drawIcon(icon,ix,iy);
-    else
-      dc.drawIconShaded(icon,ix,iy);
-    }
-  
-  // Draw arrows
-  else if(!(options&MENUBUTTON_NOARROWS)){
-    
-    // Right arrow
-    if((options&MENUBUTTON_RIGHT)==MENUBUTTON_RIGHT){
-      if(isEnabled()) 
-        dc.setForeground(textColor); 
-      else 
-        dc.setForeground(shadowColor);
-      points[0].x=ix;
-      points[0].y=iy;
-      points[1].x=ix;
-      points[1].y=iy+MENUBUTTONARROW_WIDTH-1;
-      points[2].x=ix+MENUBUTTONARROW_HEIGHT;
-      points[2].y=(FXshort)(iy+(MENUBUTTONARROW_WIDTH>>1));
-      dc.fillPolygon(points,3);
-      }
-
-    // Left arrow
-    else if(options&MENUBUTTON_LEFT){
-      if(isEnabled()) 
-        dc.setForeground(textColor); 
-      else 
-        dc.setForeground(shadowColor);
-      points[0].x=ix+MENUBUTTONARROW_HEIGHT;
-      points[0].y=iy;
-      points[1].x=ix+MENUBUTTONARROW_HEIGHT;
-      points[1].y=iy+MENUBUTTONARROW_WIDTH-1;
-      points[2].x=ix;
-      points[2].y=(FXshort)(iy+(MENUBUTTONARROW_WIDTH>>1));
-      dc.fillPolygon(points,3);
-      }
-
-    // Up arrow
-    else if(options&MENUBUTTON_UP){
-      if(isEnabled()) 
-        dc.setForeground(textColor); 
-      else 
-        dc.setForeground(shadowColor);
-      points[0].x=(FXshort)(ix+(MENUBUTTONARROW_WIDTH>>1));
-      points[0].y=iy-1;
-      points[1].x=ix;
-      points[1].y=iy+MENUBUTTONARROW_HEIGHT;
-      points[2].x=ix+MENUBUTTONARROW_WIDTH;
-      points[2].y=iy+MENUBUTTONARROW_HEIGHT;
-      dc.fillPolygon(points,3);
-      }
-
-    // Down arrow
-    else{
-      if(isEnabled()) 
-        dc.setForeground(textColor); 
-      else 
-        dc.setForeground(shadowColor);
-      points[0].x=ix+1;
-      points[0].y=iy;
-      points[2].x=ix+MENUBUTTONARROW_WIDTH-1;
-      points[2].y=iy;
-      points[1].x=(FXshort)(ix+(MENUBUTTONARROW_WIDTH>>1));
-      points[1].y=iy+MENUBUTTONARROW_HEIGHT;
-      dc.fillPolygon(points,3);
-      }
-    }
-
-  // Draw text
-  if(!label.empty()){
-    dc.setTextFont(font);
-    if(isEnabled()){
-      dc.setForeground(textColor);
-      drawLabel(dc,label,hotoff,tx,ty,tw,th);
-      if(hasFocus()){
-        drawFocusRectangle(dc,border+2,border+2,width-2*border-4,height-2*border-4); 
-        }
-      }
-    else{
-      dc.setForeground(hiliteColor);
-      drawLabel(dc,label,hotoff,tx+1,ty+1,tw,th);
-      dc.setForeground(shadowColor);
-      drawLabel(dc,label,hotoff,tx,ty,tw,th);
-      }
-    }
-  return 1;
-  }
-
-
-// Pressed left button
-long FXMenuButton::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
-  flags&=~FLAG_TIP;
-  if(isEnabled()){
-    handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
-    if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONPRESS),ptr)) return 1;
-    return handle(this,MKUINT(0,SEL_ACTIVATE),ptr);
-    }
-  return 0;
-  }
-
-
-// Released left button
-long FXMenuButton::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
-  if(isEnabled()){
-    if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONRELEASE),ptr)) return 1;
-    return handle(this,MKUINT(0,SEL_DEACTIVATE),ptr);
-    }
-  return 0;
-  }
-
-
-// Keyboard press; forward to menu pane, or handle it here
-long FXMenuButton::onKeyPress(FXObject* sender,FXSelector sel,void* ptr){
-  if(pane && pane->shown() && pane->handle(pane,sel,ptr)) return 1;
-  return FXLabel::onKeyPress(sender,sel,ptr);
-  }
-    
-
-// Keyboard release; forward to menu pane, or handle here
-long FXMenuButton::onKeyRelease(FXObject* sender,FXSelector sel,void* ptr){
-  if(pane && pane->shown() && pane->handle(pane,sel,ptr)) return 1;
-  return FXLabel::onKeyRelease(sender,sel,ptr);
-  }
-
-
-// Button being pressed
-long FXMenuButton::onActivate(FXObject*,FXSelector,void*){
-  if(state){
-    handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
-    }
-  else{
-    handle(this,MKUINT(ID_POST,SEL_COMMAND),NULL);
-    }
-  return 1;
-  }
-  
-
-// Button being released
-long FXMenuButton::onDeactivate(FXObject*,FXSelector,void* ptr){
-  FXEvent* ev=(FXEvent*)ptr;
-  if(ev->moved){ handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL); }
-  return 1;
-  }
-
-
 // Inside the button
 long FXMenuButton::onEnter(FXObject* sender,FXSelector sel,void* ptr){
   FXLabel::onEnter(sender,sel,ptr);
@@ -557,6 +235,38 @@ long FXMenuButton::onLeave(FXObject* sender,FXSelector sel,void* ptr){
   }
 
 
+// Pressed left button
+long FXMenuButton::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
+  flags&=~FLAG_TIP;
+  handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+  if(isEnabled()){
+    if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONPRESS),ptr)) return 1;
+    if(state)
+      handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
+    else
+      handle(this,MKUINT(ID_POST,SEL_COMMAND),NULL);
+    flags|=FLAG_PRESSED;
+    flags&=~FLAG_UPDATE;
+    return 1;
+    }
+  return 0;
+  }
+
+
+// Released left button
+long FXMenuButton::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
+  FXEvent* ev=(FXEvent*)ptr;
+  if(isEnabled()){
+    flags|=FLAG_UPDATE;
+    flags&=~FLAG_PRESSED;
+    if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONRELEASE),ptr)) return 1;
+    if(ev->moved){ handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL); }
+    return 1;
+    }
+  return 0;
+  }
+
+
 // If we moved over the pane, we'll ungrab again, or re-grab
 // when outside of the plane
 long FXMenuButton::onMotion(FXObject*,FXSelector,void* ptr){
@@ -576,14 +286,79 @@ long FXMenuButton::onMotion(FXObject*,FXSelector,void* ptr){
   }
 
 
+// The widget lost the grab for some reason
+long FXMenuButton::onUngrabbed(FXObject* sender,FXSelector sel,void* ptr){
+  FXLabel::onUngrabbed(sender,sel,ptr);
+  handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
+  flags&=~FLAG_PRESSED;
+  flags|=FLAG_UPDATE;
+  return 1;
+  }
+
+
+// Keyboard press; forward to menu pane, or handle it here
+long FXMenuButton::onKeyPress(FXObject*,FXSelector sel,void* ptr){
+  FXEvent* event=(FXEvent*)ptr;
+  flags&=~FLAG_TIP;
+  if(pane && pane->shown() && pane->handle(pane,sel,ptr)) return 1;
+  if(isEnabled()){
+    if(target && target->handle(this,MKUINT(message,SEL_KEYPRESS),ptr)) return 1;
+    if(event->code==KEY_space || event->code==KEY_KP_Space){
+      if(state)
+        handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
+      else
+        handle(this,MKUINT(ID_POST,SEL_COMMAND),NULL);
+      return 1;
+      }
+    }
+  return 0;
+  }
+
+
+// Keyboard release; forward to menu pane, or handle here
+long FXMenuButton::onKeyRelease(FXObject*,FXSelector sel,void* ptr){
+  FXEvent* event=(FXEvent*)ptr;
+  if(pane && pane->shown() && pane->handle(pane,sel,ptr)) return 1;
+  if(isEnabled()){
+    if(target && target->handle(this,MKUINT(message,SEL_KEYRELEASE),ptr)) return 1;
+    if(event->code==KEY_space || event->code==KEY_KP_Space){
+      return 1;
+      }
+    }
+  return 0;
+  }
+
+
+// Hot key combination pressed
+long FXMenuButton::onHotKeyPress(FXObject*,FXSelector,void* ptr){
+  FXTRACE((200,"%s::onHotKeyPress %p\n",getClassName(),this));
+  flags&=~FLAG_TIP;
+  handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+  if(isEnabled()){
+    if(state)
+      handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
+    else
+      handle(this,MKUINT(ID_POST,SEL_COMMAND),NULL);
+    }
+  return 1;
+  }
+
+
+// Hot key combination released
+long FXMenuButton::onHotKeyRelease(FXObject*,FXSelector,void*){
+  FXTRACE((200,"%s::onHotKeyRelease %p\n",getClassName(),this));
+  return 1;
+  }
+
+
 // Post the menu
 long FXMenuButton::onCmdPost(FXObject*,FXSelector,void*){
   if(!state){
     if(pane){
       FXint x,y,w,h;
       translateCoordinatesTo(x,y,getRoot(),0,0);
-      w=pane->getWidth();
-      h=pane->getHeight();
+      w=pane->getShrinkWrap() ? pane->getDefaultWidth() : pane->getWidth();
+      h=pane->getShrinkWrap() ? pane->getDefaultHeight() : pane->getHeight();
       if((options&MENUBUTTON_LEFT)&&(options&MENUBUTTON_UP)){   // Right
         if((options&MENUBUTTON_ATTACH_BOTTOM)&&(options&MENUBUTTON_ATTACH_CENTER)){
           h=height;
@@ -660,6 +435,199 @@ long FXMenuButton::onCmdUnpost(FXObject*,FXSelector,void*){
   }
 
 
+// Handle repaint
+long FXMenuButton::onPaint(FXObject*,FXSelector,void* ptr){
+  FXint tw=0,th=0,iw=0,ih=0,tx,ty,ix,iy;
+  FXEvent *ev=(FXEvent*)ptr;
+  FXPoint points[3];
+  FXDCWindow dc(this,ev);
+
+  // Got a border at all?
+  if(options&(FRAME_RAISED|FRAME_SUNKEN)){
+
+    // Toolbar style
+    if(options&MENUBUTTON_TOOLBAR){
+
+      // Enabled and cursor inside, and not popped up
+      if(isEnabled() && underCursor() && !state){
+        dc.setForeground(backColor);
+        dc.fillRectangle(border,border,width-border*2,height-border*2);
+        if(options&FRAME_THICK) drawDoubleRaisedRectangle(dc,0,0,width,height);
+        else drawRaisedRectangle(dc,0,0,width,height);
+        }
+
+      // Enabled and popped up
+      else if(isEnabled() && state){
+        dc.setForeground(hiliteColor);
+        dc.fillRectangle(border,border,width-border*2,height-border*2);
+        if(options&FRAME_THICK) drawDoubleSunkenRectangle(dc,0,0,width,height);
+        else drawSunkenRectangle(dc,0,0,width,height);
+        }
+
+      // Disabled or unchecked or not under cursor
+      else{
+        dc.setForeground(backColor);
+        dc.fillRectangle(0,0,width,height);
+        }
+      }
+
+    // Normal style
+    else{
+
+      // Draw in up state if disabled or up
+      if(!isEnabled() || !state){
+        dc.setForeground(backColor);
+        dc.fillRectangle(border,border,width-border*2,height-border*2);
+        if(options&FRAME_THICK) drawDoubleRaisedRectangle(dc,0,0,width,height);
+        else drawRaisedRectangle(dc,0,0,width,height);
+        }
+
+      // Draw sunken if enabled and either checked or pressed
+      else{
+        dc.setForeground(hiliteColor);
+        dc.fillRectangle(border,border,width-border*2,height-border*2);
+        if(options&FRAME_THICK) drawDoubleSunkenRectangle(dc,0,0,width,height);
+        else drawSunkenRectangle(dc,0,0,width,height);
+        }
+      }
+    }
+
+  // No borders
+  else{
+    if(isEnabled() && state){
+      dc.setForeground(hiliteColor);
+      dc.fillRectangle(0,0,width,height);
+      }
+    else{
+      dc.setForeground(backColor);
+      dc.fillRectangle(0,0,width,height);
+      }
+    }
+
+  // Position text & icon
+  if(!label.empty()){
+    tw=labelWidth(label);
+    th=labelHeight(label);
+    }
+
+  // Icon?
+  if(icon){
+    iw=icon->getWidth();
+    ih=icon->getHeight();
+    }
+
+  // Arrows?
+  else if(!(options&MENUBUTTON_NOARROWS)){
+    if(options&MENUBUTTON_LEFT){
+      ih=MENUBUTTONARROW_WIDTH;
+      iw=MENUBUTTONARROW_HEIGHT;
+      }
+    else{
+      iw=MENUBUTTONARROW_WIDTH;
+      ih=MENUBUTTONARROW_HEIGHT;
+      }
+    }
+
+  // Keep some room for the arrow!
+  just_x(tx,ix,tw,iw);
+  just_y(ty,iy,th,ih);
+
+  // Move a bit when pressed
+  if(state){ ++tx; ++ty; ++ix; ++iy; }
+
+  // Draw icon
+  if(icon){
+    if(isEnabled())
+      dc.drawIcon(icon,ix,iy);
+    else
+      dc.drawIconSunken(icon,ix,iy);
+    }
+
+  // Draw arrows
+  else if(!(options&MENUBUTTON_NOARROWS)){
+
+    // Right arrow
+    if((options&MENUBUTTON_RIGHT)==MENUBUTTON_RIGHT){
+      if(isEnabled())
+        dc.setForeground(textColor);
+      else
+        dc.setForeground(shadowColor);
+      points[0].x=ix;
+      points[0].y=iy;
+      points[1].x=ix;
+      points[1].y=iy+MENUBUTTONARROW_WIDTH-1;
+      points[2].x=ix+MENUBUTTONARROW_HEIGHT;
+      points[2].y=(FXshort)(iy+(MENUBUTTONARROW_WIDTH>>1));
+      dc.fillPolygon(points,3);
+      }
+
+    // Left arrow
+    else if(options&MENUBUTTON_LEFT){
+      if(isEnabled())
+        dc.setForeground(textColor);
+      else
+        dc.setForeground(shadowColor);
+      points[0].x=ix+MENUBUTTONARROW_HEIGHT;
+      points[0].y=iy;
+      points[1].x=ix+MENUBUTTONARROW_HEIGHT;
+      points[1].y=iy+MENUBUTTONARROW_WIDTH-1;
+      points[2].x=ix;
+      points[2].y=(FXshort)(iy+(MENUBUTTONARROW_WIDTH>>1));
+      dc.fillPolygon(points,3);
+      }
+
+    // Up arrow
+    else if(options&MENUBUTTON_UP){
+      if(isEnabled())
+        dc.setForeground(textColor);
+      else
+        dc.setForeground(shadowColor);
+      points[0].x=(FXshort)(ix+(MENUBUTTONARROW_WIDTH>>1));
+      points[0].y=iy-1;
+      points[1].x=ix;
+      points[1].y=iy+MENUBUTTONARROW_HEIGHT;
+      points[2].x=ix+MENUBUTTONARROW_WIDTH;
+      points[2].y=iy+MENUBUTTONARROW_HEIGHT;
+      dc.fillPolygon(points,3);
+      }
+
+    // Down arrow
+    else{
+      if(isEnabled())
+        dc.setForeground(textColor);
+      else
+        dc.setForeground(shadowColor);
+      points[0].x=ix+1;
+      points[0].y=iy;
+      points[2].x=ix+MENUBUTTONARROW_WIDTH-1;
+      points[2].y=iy;
+      points[1].x=(FXshort)(ix+(MENUBUTTONARROW_WIDTH>>1));
+      points[1].y=iy+MENUBUTTONARROW_HEIGHT;
+      dc.fillPolygon(points,3);
+      }
+    }
+
+  // Draw text
+  if(!label.empty()){
+    dc.setTextFont(font);
+    if(isEnabled()){
+      dc.setForeground(textColor);
+      drawLabel(dc,label,hotoff,tx,ty,tw,th);
+      if(hasFocus()){
+        drawFocusRectangle(dc,border+2,border+2,width-2*border-4,height-2*border-4);
+        }
+      }
+    else{
+      dc.setForeground(hiliteColor);
+      drawLabel(dc,label,hotoff,tx+1,ty+1,tw,th);
+      dc.setForeground(shadowColor);
+      drawLabel(dc,label,hotoff,tx,ty,tw,th);
+      }
+    }
+  return 1;
+  }
+
+
 // Out of focus chain
 void FXMenuButton::killFocus(){
   FXLabel::killFocus();
@@ -685,8 +653,8 @@ void FXMenuButton::setButtonStyle(FXuint style){
 
 
 // Get icon positioning
-FXuint FXMenuButton::getButtonStyle() const { 
-  return (options&MENUBUTTON_MASK); 
+FXuint FXMenuButton::getButtonStyle() const {
+  return (options&MENUBUTTON_MASK);
   }
 
 
@@ -722,26 +690,12 @@ FXuint FXMenuButton::getAttachment() const {
   }
 
 
-// Change help text
-void FXMenuButton::setHelpText(const FXString& text){
-  help=text;
-  }
-
-
-// Change tip text
-void FXMenuButton::setTipText(const FXString& text){
-  tip=text;
-  }
-
-
 // Save object to stream
 void FXMenuButton::save(FXStream& store) const {
   FXLabel::save(store);
   store << pane;
   store << offsetx;
   store << offsety;
-  store << tip;
-  store << help;
   }
 
 
@@ -751,9 +705,7 @@ void FXMenuButton::load(FXStream& store){
   store >> pane;
   store >> offsetx;
   store >> offsety;
-  store >> tip;
-  store >> help;
-  }  
+  }
 
 
 // Delete it
