@@ -3,30 +3,35 @@
 *                         M e s s a g e   B o x e s                             *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXMessageBox.cpp,v 1.3 1999/11/12 07:59:17 jeroen Exp $                  *
+* $Id: FXMessageBox.cpp,v 1.12 2001/08/10 18:09:52 jeroen Exp $                 *
 ********************************************************************************/
 #include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXApp.h"
@@ -49,10 +54,9 @@
 #include "FXDialogBox.h"
 #include "FXMessageBox.h"
 
-
 /*
   Notes:
-  - Want justify modes for the message box so the label can be aligned 
+  - Want justify modes for the message box so the label can be aligned
     different ways.
 */
 
@@ -66,6 +70,7 @@
 
 // Map
 FXDEFMAP(FXMessageBox) FXMessageBoxMap[]={
+  FXMAPFUNC(SEL_COMMAND,FXMessageBox::ID_CANCEL,FXMessageBox::onCmdCancel),
   FXMAPFUNCS(SEL_COMMAND,FXMessageBox::ID_CLICKED_YES,FXMessageBox::ID_CLICKED_SAVE,FXMessageBox::onCmdClicked),
   };
 
@@ -74,11 +79,11 @@ FXDEFMAP(FXMessageBox) FXMessageBoxMap[]={
 // Object implementation
 FXIMPLEMENT(FXMessageBox,FXDialogBox,FXMessageBoxMap,ARRAYNUMBER(FXMessageBoxMap))
 
-  
-  
+
+
 // Create message box
 FXMessageBox::FXMessageBox(FXWindow* owner,const FXString& caption,const FXString& text,FXIcon* ic,FXuint opts,FXint x,FXint y):
-  FXDialogBox(owner,caption,opts|DECOR_TITLE|DECOR_BORDER,x,y,0,0){
+  FXDialogBox(owner,caption,opts|DECOR_TITLE|DECOR_BORDER,x,y,0,0, 0,0,0,0, 4,4){
   FXuint whichbuttons=opts&MBOX_BUTTON_MASK;
   FXVerticalFrame* content=new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y);
   FXHorizontalFrame* info=new FXHorizontalFrame(content,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,10,10,10,10);
@@ -122,6 +127,12 @@ long FXMessageBox::onCmdClicked(FXObject*,FXSelector sel,void*){
   }
 
 
+// Close dialog with a cancel
+long FXMessageBox::onCmdCancel(FXObject* sender,FXSelector,void* ptr){
+  return FXMessageBox::onCmdClicked(sender,MKUINT(ID_CLICKED_CANCEL,SEL_COMMAND),ptr);
+  }
+
+
 /*******************************************************************************/
 
 // Show a modal error message
@@ -142,15 +153,9 @@ FXuint FXMessageBox::error(FXWindow* owner,FXuint opts,const char* caption,const
     };
   FXGIFIcon icon(owner->getApp(),errorIcon);
   va_list arguments;
-  FXchar buf[1000];
   va_start(arguments,message);
-#ifdef _GNU_SOURCE
-  vsnprintf(buf,sizeof(buf),message,arguments);
-#else
-  vsprintf(buf,message,arguments);
-#endif
+  FXMessageBox box(owner,caption,FXStringVFormat(message,arguments),&icon,opts|DECOR_TITLE|DECOR_BORDER);
   va_end(arguments);
-  FXMessageBox box(owner,caption,buf,&icon,opts|DECOR_TITLE|DECOR_BORDER);
   return box.execute();
   }
 
@@ -174,15 +179,9 @@ FXuint FXMessageBox::warning(FXWindow* owner,FXuint opts,const char* caption,con
     };
   FXGIFIcon icon(owner->getApp(),warningIcon);
   va_list arguments;
-  FXchar buf[1000];
   va_start(arguments,message);
-#ifdef _GNU_SOURCE
-  vsnprintf(buf,sizeof(buf),message,arguments);
-#else
-  vsprintf(buf,message,arguments);
-#endif
+  FXMessageBox box(owner,caption,FXStringVFormat(message,arguments),&icon,opts|DECOR_TITLE|DECOR_BORDER);
   va_end(arguments);
-  FXMessageBox box(owner,caption,buf,&icon,opts|DECOR_TITLE|DECOR_BORDER);
   return box.execute();
   }
 
@@ -206,15 +205,9 @@ FXuint FXMessageBox::question(FXWindow* owner,FXuint opts,const char* caption,co
     };
   FXGIFIcon icon(owner->getApp(),questionIcon);
   va_list arguments;
-  FXchar buf[1000];
   va_start(arguments,message);
-#ifdef _GNU_SOURCE
-  vsnprintf(buf,sizeof(buf),message,arguments);
-#else
-  vsprintf(buf,message,arguments);
-#endif
+  FXMessageBox box(owner,caption,FXStringVFormat(message,arguments),&icon,opts|DECOR_TITLE|DECOR_BORDER);
   va_end(arguments);
-  FXMessageBox box(owner,caption,buf,&icon,opts|DECOR_TITLE|DECOR_BORDER);
   return box.execute();
   }
 
@@ -237,15 +230,9 @@ FXuint FXMessageBox::information(FXWindow* owner,FXuint opts,const char* caption
     };
   FXGIFIcon icon(owner->getApp(),infoIcon);
   va_list arguments;
-  FXchar buf[1000];
   va_start(arguments,message);
-#ifdef _GNU_SOURCE
-  vsnprintf(buf,sizeof(buf),message,arguments);
-#else
-  vsprintf(buf,message,arguments);
-#endif
+  FXMessageBox box(owner,caption,FXStringVFormat(message,arguments),&icon,opts|DECOR_TITLE|DECOR_BORDER);
   va_end(arguments);
-  FXMessageBox box(owner,caption,buf,&icon,opts|DECOR_TITLE|DECOR_BORDER);
   return box.execute();
   }
 
