@@ -3,30 +3,36 @@
 *                             O p t i o n   M e n u                             *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXOptionMenu.cpp,v 1.6 1999/11/03 21:56:04 jeroen Exp $                  *
+* $Id: FXOptionMenu.cpp,v 1.29 2001/08/10 18:09:52 jeroen Exp $                 *
 ********************************************************************************/
+#include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "fxkeys.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXApp.h"
@@ -56,6 +62,16 @@
     SEL_COMMAND at the end, and SEL_UPDATE during gui update.
   - FXOptionMenu should understand ID_SETINTVALUE and ID_GETINTVALUE
     so you may substitute FXOptionMenu for a slider or dial.
+  - Perhaps support arrow keys to select by means of keyboard.
+  - Right-click to cycle through options (as suggested by
+    Stephane Ancelot <sancelot@crosswinds.net>) would be nice.
+  - Hotkey's don't work in FXOption.
+  - Need to have "notify" in setCurrent.
+  - Need to send callback message from FXOptionMeny, and not from
+    FXOption.
+  - Zecchini Mauro <mauro.zecchini@realtimesrl.com> donated code
+    to add FXDataTarget get/set connectivity here also.
+
 */
 
 
@@ -75,23 +91,19 @@ FXDEFMAP(FXOption) FXOptionMap[]={
   FXMAPFUNC(SEL_KEYRELEASE,0,FXOption::onKeyRelease),
   FXMAPFUNC(SEL_KEYPRESS,FXWindow::ID_HOTKEY,FXOption::onHotKeyPress),
   FXMAPFUNC(SEL_KEYRELEASE,FXWindow::ID_HOTKEY,FXOption::onHotKeyRelease),
-  FXMAPFUNC(SEL_UPDATE,FXWindow::ID_QUERY_HELP,FXOption::onQueryHelp),
   };
 
 
 // Object implementation
 FXIMPLEMENT(FXOption,FXLabel,FXOptionMap,ARRAYNUMBER(FXOptionMap))
 
-  
+
 // Make option menu entry
 FXOption::FXOption(FXComposite* p,const FXString& text,FXIcon* ic,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb):
   FXLabel(p,text,ic,opts,x,y,w,h,pl,pr,pt,pb){
-  tip=text.extract(1,'\t');
-  help=text.extract(2,'\t');
   target=tgt;
   message=sel;
-  flags|=FLAG_ENABLED;
-  defaultCursor=getApp()->rarrowCursor;
+  defaultCursor=getApp()->getDefaultCursor(DEF_RARROW_CURSOR);
   }
 
 
@@ -102,11 +114,11 @@ FXbool FXOption::canFocus() const { return 1; }
 // Get default width
 FXint FXOption::getDefaultWidth(){
   FXint tw=0,iw=MENUGLYPH_WIDTH,s=0,w;
-  if(!label.empty()){ 
-    tw=labelWidth(label); 
+  if(!label.empty()){
+    tw=labelWidth(label);
     }
-  if(icon){ 
-    iw=icon->getWidth(); 
+  if(icon){
+    iw=icon->getWidth();
     }
   if(iw && tw) s=4;
   if(!(options&(ICON_AFTER_TEXT|ICON_BEFORE_TEXT))) w=FXMAX(tw,iw); else w=tw+iw+s;
@@ -117,11 +129,11 @@ FXint FXOption::getDefaultWidth(){
 // Get default height
 FXint FXOption::getDefaultHeight(){
   FXint th=0,ih=MENUGLYPH_HEIGHT,h;
-  if(!label.empty()){ 
-    th=labelHeight(label); 
+  if(!label.empty()){
+    th=labelHeight(label);
     }
-  if(icon){ 
-    ih=icon->getHeight(); 
+  if(icon){
+    ih=icon->getHeight();
     }
   if(!(options&(ICON_ABOVE_TEXT|ICON_BELOW_TEXT))) h=FXMAX(th,ih); else h=th+ih;
   return padtop+padbottom+(border<<1)+h;
@@ -129,12 +141,11 @@ FXint FXOption::getDefaultHeight(){
 
 
 
-// Handle repaint 
+// Handle repaint
 long FXOption::onPaint(FXObject*,FXSelector,void* ptr){
   FXint tw=0,th=0,iw=MENUGLYPH_WIDTH,ih=MENUGLYPH_HEIGHT,tx,ty,ix,iy;
   FXEvent *ev=(FXEvent*)ptr;
   FXDCWindow dc(this,ev);
-  drawFrame(dc,0,0,width,height);
   if(!label.empty()){
     tw=labelWidth(label);
     th=labelHeight(label);
@@ -173,6 +184,7 @@ long FXOption::onPaint(FXObject*,FXSelector,void* ptr){
       drawLabel(dc,label,hotoff,tx,ty,tw,th);
       }
     }
+  drawFrame(dc,0,0,width,height);
   return 1;
   }
 
@@ -181,7 +193,6 @@ long FXOption::onPaint(FXObject*,FXSelector,void* ptr){
 long FXOption::onEnter(FXObject* sender,FXSelector sel,void* ptr){
   FXLabel::onEnter(sender,sel,ptr);
   if(isEnabled() && canFocus()) setFocus();
-  //////////// Should this generate an SEL_FOCUS_SELF ?
   return 1;
   }
 
@@ -208,10 +219,9 @@ long FXOption::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
 
 // Released left button; unpost menu if cursor has moved
 long FXOption::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
-  FXEvent* ev=(FXEvent*)ptr;
   if(isEnabled()){
     if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONRELEASE),ptr)) return 1;
-    if(ev->moved){ 
+    if(((FXEvent*)ptr)->moved){
       getParent()->handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),this);
       if(target) target->handle(this,MKUINT(message,SEL_COMMAND),ptr);
       }
@@ -226,12 +236,8 @@ long FXOption::onKeyPress(FXObject*,FXSelector,void* ptr){
   FXEvent* event=(FXEvent*)ptr;
   if(isEnabled()){
     if(target && target->handle(this,MKUINT(message,SEL_KEYPRESS),ptr)) return 1;
-    switch(event->code){
-      case KEY_KP_Enter:
-      case KEY_Return:
-      case KEY_space:
-      case KEY_KP_Space:
-        return 1;
+    if(event->code==KEY_space || event->code==KEY_KP_Space){
+      return 1;
       }
     }
   return 0;
@@ -243,14 +249,10 @@ long FXOption::onKeyRelease(FXObject*,FXSelector,void* ptr){
   FXEvent* event=(FXEvent*)ptr;
   if(isEnabled()){
     if(target && target->handle(this,MKUINT(message,SEL_KEYRELEASE),ptr)) return 1;
-    switch(event->code){
-      case KEY_KP_Enter:
-      case KEY_Return:
-      case KEY_space:
-      case KEY_KP_Space:
-        getParent()->handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),this);
-        if(target) target->handle(this,MKUINT(message,SEL_COMMAND),ptr);
-        return 1;
+    if(event->code==KEY_space || event->code==KEY_KP_Space){
+      getParent()->handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),this);
+      if(target) target->handle(this,MKUINT(message,SEL_COMMAND),ptr);
+      return 1;
       }
     }
   return 0;
@@ -259,38 +261,25 @@ long FXOption::onKeyRelease(FXObject*,FXSelector,void* ptr){
 
 // Hot key combination pressed
 long FXOption::onHotKeyPress(FXObject*,FXSelector,void* ptr){
-  FXTRACE((200,"%s::onHotKeyPress %08x\n",getClassName(),this));
   flags&=~FLAG_TIP;
-  if(isEnabled()){
-    handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
-    handle(this,MKUINT(0,SEL_ACTIVATE),ptr);
-    }
+  handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+  FXTRACE((100,"FXOption::onHotKeyPress\n"));
   return 1;
   }
 
 
 // Hot key combination released
 long FXOption::onHotKeyRelease(FXObject*,FXSelector,void* ptr){
-  FXTRACE((200,"%s::onHotKeyRelease %08x\n",getClassName(),this));
   flags&=~FLAG_TIP;
-  if(isEnabled()){ 
-    handle(this,MKUINT(0,SEL_DEACTIVATE),ptr); 
+  FXTRACE((100,"FXOption::onHotKeyRelease\n"));
+  if(isEnabled()){
+    getParent()->handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),this);
+    if(target) target->handle(this,MKUINT(message,SEL_COMMAND),ptr);
     }
   return 1;
   }
 
 
-// We were asked about status text
-long FXOption::onQueryHelp(FXObject* sender,FXSelector,void*){
-  FXTRACE((150,"%s::onQueryHelp %08x\n",getClassName(),this));
-  if(!help.empty() && (flags&FLAG_HELP)){
-    sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&help);
-    return 1;
-    }
-  return 0;
-  }
-
-
 // Into focus chain
 void FXOption::setFocus(){
   FXLabel::setFocus();
@@ -310,34 +299,6 @@ void FXOption::killFocus(){
   }
 
 
-// Save object to stream
-void FXOption::save(FXStream& store) const {
-  FXLabel::save(store);
-  store << tip;
-  store << help;
-  }
-
-
-// Load object from stream
-void FXOption::load(FXStream& store){
-  FXLabel::load(store);
-  store >> tip;
-  store >> help;
-  }  
-
-
-// Change help text
-void FXOption::setHelpText(const FXString& text){
-  help=text;
-  }
-
-
-// Change tip text
-void FXOption::setTipText(const FXString& text){
-  tip=text;
-  }
-
-
 // Delete
 FXOption::~FXOption(){
   }
@@ -360,6 +321,9 @@ FXDEFMAP(FXOptionMenu) FXOptionMenuMap[]={
   FXMAPFUNC(SEL_COMMAND,FXWindow::ID_UNPOST,FXOptionMenu::onCmdUnpost),
   FXMAPFUNC(SEL_UPDATE,FXWindow::ID_QUERY_TIP,FXOptionMenu::onQueryTip),
   FXMAPFUNC(SEL_UPDATE,FXWindow::ID_QUERY_HELP,FXOptionMenu::onQueryHelp),
+  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_SETVALUE,FXOptionMenu::onCmdSetValue),
+  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_SETINTVALUE,FXOptionMenu::onCmdSetIntValue),
+  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_GETINTVALUE,FXOptionMenu::onCmdGetIntValue),
   };
 
 
@@ -367,12 +331,10 @@ FXDEFMAP(FXOptionMenu) FXOptionMenuMap[]={
 FXIMPLEMENT(FXOptionMenu,FXLabel,FXOptionMenuMap,ARRAYNUMBER(FXOptionMenuMap))
 
 
-
 // Make a option menu button
 FXOptionMenu::FXOptionMenu(FXComposite* p,FXPopup* pup,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb):
   FXLabel(p,NULL,NULL,opts,x,y,w,h,pl,pr,pt,pb){
-  flags|=FLAG_ENABLED;
-  dragCursor=getApp()->rarrowCursor;
+  dragCursor=getApp()->getDefaultCursor(DEF_RARROW_CURSOR);
   pane=pup;
   current=NULL;
   if(pane){
@@ -430,7 +392,7 @@ long FXOptionMenu::onFocusIn(FXObject* sender,FXSelector sel,void* ptr){
   return 1;
   }
 
-  
+
 // Lost focus
 long FXOptionMenu::onFocusOut(FXObject* sender,FXSelector sel,void* ptr){
   FXLabel::onFocusOut(sender,sel,ptr);
@@ -439,18 +401,18 @@ long FXOptionMenu::onFocusOut(FXObject* sender,FXSelector sel,void* ptr){
   }
 
 
-// Handle repaint 
+// Handle repaint
 long FXOptionMenu::onPaint(FXObject*,FXSelector,void* ptr){
   FXint tw=0,th=0,iw=MENUGLYPH_WIDTH,ih=MENUGLYPH_HEIGHT,tx,ty,ix,iy;
   FXEvent *ev=(FXEvent*)ptr;
   FXDCWindow dc(this,ev);
-  
+
   drawFrame(dc,0,0,width,height);
-  
+
   // Draw background
   dc.setForeground(backColor);
   dc.fillRectangle(border,border,width-border*2,height-border*2);
-  
+
   // Position text & icon
   if(!label.empty()){
     tw=labelWidth(label);
@@ -460,7 +422,7 @@ long FXOptionMenu::onPaint(FXObject*,FXSelector,void* ptr){
     iw=icon->getWidth();
     ih=icon->getHeight();
     }
-  
+
   // Keep some room for the arrow!
   just_x(tx,ix,tw,iw);
   just_y(ty,iy,th,ih);
@@ -469,7 +431,7 @@ long FXOptionMenu::onPaint(FXObject*,FXSelector,void* ptr){
   if(icon){
     dc.drawIcon(icon,ix,iy);
     }
-  
+
   // Or draw rectangle
   else{
     drawDoubleRaisedRectangle(dc,ix,iy,MENUGLYPH_WIDTH,MENUGLYPH_HEIGHT);
@@ -481,8 +443,8 @@ long FXOptionMenu::onPaint(FXObject*,FXSelector,void* ptr){
     if(isEnabled()){
       dc.setForeground(textColor);
       drawLabel(dc,label,hotoff,tx,ty,tw,th);
-      if(hasFocus()){ 
-        drawFocusRectangle(dc,border+2,border+2,width-2*border-4,height-2*border-4); 
+      if(hasFocus()){
+        drawFocusRectangle(dc,border+2,border+2,width-2*border-4,height-2*border-4);
         }
       }
     else{
@@ -541,8 +503,8 @@ long FXOptionMenu::onKeyRelease(FXObject*,FXSelector sel,void* ptr){
 // Pressed left button
 long FXOptionMenu::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
   flags&=~FLAG_TIP;
+  handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
   if(isEnabled()){
-    handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
     if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONPRESS),ptr)) return 1;
     if(pane){
       if(pane->shown()){
@@ -552,6 +514,8 @@ long FXOptionMenu::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
         handle(this,MKUINT(ID_POST,SEL_COMMAND),NULL);
         }
       }
+    flags|=FLAG_PRESSED;
+    flags&=~FLAG_UPDATE;
     return 1;
     }
   return 0;
@@ -563,10 +527,10 @@ long FXOptionMenu::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
   FXEvent* ev=(FXEvent*)ptr;
   flags&=~FLAG_TIP;
   if(isEnabled()){
+    flags|=FLAG_UPDATE;
+    flags&=~FLAG_PRESSED;
     if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONRELEASE),ptr)) return 1;
-    if(pane){
-      if(ev->moved){ handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL); }
-      }
+    if(ev->moved && pane){ handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL); }
     return 1;
     }
   return 0;
@@ -578,9 +542,8 @@ long FXOptionMenu::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
 long FXOptionMenu::onMotion(FXObject*,FXSelector,void* ptr){
   FXEvent* ev=(FXEvent*)ptr;
   if(pane && pane->shown()){
-    flags&=~FLAG_TIP;
     if(pane->contains(ev->root_x,ev->root_y)){
-      if(grabbed()) ungrab(); 
+      if(grabbed()) ungrab();
       }
     else{
       if(!grabbed()) grab();
@@ -602,7 +565,6 @@ long FXOptionMenu::onCmdPost(FXObject*,FXSelector,void*){
     y+=2-current->getY();
     pane->popup(this,x,y,width,pane->getDefaultHeight());
     current->setFocus();
-    flags&=~FLAG_UPDATE;
     if(!grabbed()) grab();
     }
   return 1;
@@ -617,7 +579,27 @@ long FXOptionMenu::onCmdUnpost(FXObject*,FXSelector,void* ptr){
     if(grabbed()) ungrab();
     if(ptr) setCurrent((FXOption*)ptr);
     }
-  flags|=FLAG_UPDATE;
+  return 1;
+  }
+
+
+// Update value from a message
+long FXOptionMenu::onCmdSetValue(FXObject*,FXSelector,void* ptr){
+  setCurrentNo((FXint)(long)ptr);
+  return 1;
+  }
+
+
+// Update value from a message
+long FXOptionMenu::onCmdSetIntValue(FXObject*,FXSelector,void* ptr){
+  setCurrentNo(*((FXint*)ptr));
+  return 1;
+  }
+
+
+// Obtain value from text field
+long FXOptionMenu::onCmdGetIntValue(FXObject*,FXSelector,void* ptr){
+  *((FXint*)ptr)=getCurrentNo();
   return 1;
   }
 
@@ -667,6 +649,7 @@ void FXOptionMenu::setCurrent(FXOption *win){
 void FXOptionMenu::setCurrentNo(FXint no){
   register FXint i=0;
   if(pane){
+    // FXOption *win=(FXOption*)pane->childAtIndex(no);
     FXOption *win=(FXOption*)pane->getFirst();
     while(win && i!=no){
       win=(FXOption*)win->getNext();
@@ -681,6 +664,7 @@ void FXOptionMenu::setCurrentNo(FXint no){
 FXint FXOptionMenu::getCurrentNo() const {
   register FXint i=0;
   if(pane){
+    // return pane->indexOfChild(current);
     FXOption *win=(FXOption*)pane->getFirst();
     while(win && win!=current){
       win=(FXOption*)win->getNext();
@@ -706,10 +690,17 @@ void FXOptionMenu::setPopup(FXPopup *pup){
   }
 
 
-// We were asked about status text
+// The current option's help is returned, unless there is no help,
+// in which case the option menu's help is returned
 long FXOptionMenu::onQueryHelp(FXObject* sender,FXSelector,void*){
-  if((flags&FLAG_HELP) && current){
-    FXString help=current->getHelpText();
+  if(flags&FLAG_HELP){
+    if(current){
+      FXString optionhelp=current->getHelpText();
+      if(!optionhelp.empty()){
+        sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&optionhelp);
+        return 1;
+        }
+      }
     if(!help.empty()){
       sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&help);
       return 1;
@@ -719,10 +710,17 @@ long FXOptionMenu::onQueryHelp(FXObject* sender,FXSelector,void*){
   }
 
 
-// We were asked about tip text
+// The current option's tip is returned, unless there is no tip,
+// in which case the option menu's tip is returned
 long FXOptionMenu::onQueryTip(FXObject* sender,FXSelector,void*){
-  if((flags&FLAG_TIP) && current){
-    FXString tip=current->getTipText();
+  if(flags&FLAG_TIP){
+    if(current){
+      FXString optiontip=current->getTipText();
+      if(!optiontip.empty()){
+        sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&optiontip);
+        return 1;
+        }
+      }
     if(!tip.empty()){
       sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&tip);
       return 1;
@@ -751,7 +749,7 @@ void FXOptionMenu::load(FXStream& store){
   FXLabel::load(store);
   store >> pane;
   store >> current;
-  }  
+  }
 
 
 // Delete it
