@@ -3,31 +3,36 @@
 *                P a c k e r   C o n t a i n e r   O b j e c t                  *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXPacker.cpp,v 1.3 1999/11/09 22:33:01 jeroen Exp $                      *
+* $Id: FXPacker.cpp,v 1.18 2001/08/28 22:52:27 jeroen Exp $                     *
 ********************************************************************************/
+#include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
-#include "FXAccelTable.h"
 #include "FXApp.h"
 #include "FXId.h"
 #include "FXDC.h"
@@ -38,7 +43,7 @@
 #include "FXComposite.h"
 #include "FXPacker.h"
 
-  
+
 /*
   To do:
   - Now observes LAYOUT_FIX_X and LAYOUT_FIX_Y hints.
@@ -75,7 +80,7 @@ FXDEFMAP(FXPacker) FXPackerMap[]={
 // Object implementation
 FXIMPLEMENT(FXPacker,FXComposite,FXPackerMap,ARRAYNUMBER(FXPackerMap))
 
-  
+
 // Deserialization
 FXPacker::FXPacker(){
   flags|=FLAG_SHOWN;
@@ -86,10 +91,10 @@ FXPacker::FXPacker(){
 FXPacker::FXPacker(FXComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs):
   FXComposite(p,opts,x,y,w,h){
   flags|=FLAG_SHOWN;
-  baseColor=getApp()->baseColor;
-  hiliteColor=getApp()->hiliteColor;
-  shadowColor=getApp()->shadowColor;
-  borderColor=getApp()->borderColor;
+  baseColor=getApp()->getBaseColor();
+  hiliteColor=getApp()->getHiliteColor();
+  shadowColor=getApp()->getShadowColor();
+  borderColor=getApp()->getBorderColor();
   padtop=pt;
   padbottom=pb;
   padleft=pl;
@@ -100,99 +105,92 @@ FXPacker::FXPacker(FXComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXint h,FX
   }
 
 
-// Draw simple line border
 void FXPacker::drawBorderRectangle(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
   dc.setForeground(borderColor);
-  dc.drawRectangle(x,y,w,h);
+  dc.drawRectangle(x,y,w-1,h-1);
   }
 
 
-// Draw 1 pixel raised border
 void FXPacker::drawRaisedRectangle(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
-  dc.setForeground(hiliteColor);
-  dc.drawLine(x,y,x+w-2,y);
-  dc.drawLine(x,y,x,y+h-2);
   dc.setForeground(shadowColor);
-  dc.drawLine(x,y+h-1,x+w-1,y+h-1);
-  dc.drawLine(x+w-1,y,x+w-1,y+h-1);
+  dc.fillRectangle(x,y+h-1,w,1);
+  dc.fillRectangle(x+w-1,y,1,h);
+  dc.setForeground(hiliteColor);
+  dc.fillRectangle(x,y,w,1);
+  dc.fillRectangle(x,y,1,h);
   }
 
 
-// Draw 1 pixel sunken border
 void FXPacker::drawSunkenRectangle(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
   dc.setForeground(shadowColor);
-  dc.drawLine(x,y,x+w-2,y);
-  dc.drawLine(x,y,x,y+h-2);
+  dc.fillRectangle(x,y,w,1);
+  dc.fillRectangle(x,y,1,h);
   dc.setForeground(hiliteColor);
-  dc.drawLine(x,y+h-1,x+w-1,y+h-1);
-  dc.drawLine(x+w-1,y,x+w-1,y+h-1);
+  dc.fillRectangle(x,y+h-1,w,1);
+  dc.fillRectangle(x+w-1,y,1,h);
   }
 
 
-// Draw 2 pixel ridge border
 void FXPacker::drawRidgeRectangle(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
   dc.setForeground(hiliteColor);
-  dc.drawLine(x,y,x+w-1,y);
-  dc.drawLine(x,y,x,y+h-1);
-  dc.drawLine(x+1,y+h-2,x+w-2,y+h-2);
-  dc.drawLine(x+w-2,y+1,x+w-2,y+h-2);
+  dc.fillRectangle(x,y,w,1);
+  dc.fillRectangle(x,y,1,h);
+  dc.fillRectangle(x+1,y+h-2,w-2,1);
+  dc.fillRectangle(x+w-2,y+1,1,h-2);
   dc.setForeground(shadowColor);
-  dc.drawLine(x+1,y+1,x+w-3,y+1);
-  dc.drawLine(x+1,y+1,x+1,y+h-3);
-  dc.drawLine(x,y+h-1,x+w-1,y+h-1);
-  dc.drawLine(x+w-1,y,x+w-1,y+h-1);
+  dc.fillRectangle(x+1,y+1,w-3,1);
+  dc.fillRectangle(x+1,y+1,1,h-3);
+  dc.fillRectangle(x,y+h-1,w,1);
+  dc.fillRectangle(x+w-1,y,1,h);
   }
 
 
-// Draw 2 pixel groove border
 void FXPacker::drawGrooveRectangle(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
   dc.setForeground(shadowColor);
-  dc.drawLine(x,y,x+w-1,y);
-  dc.drawLine(x,y,x,y+h-1);
-  dc.drawLine(x+1,y+h-2,x+w-2,y+h-2);
-  dc.drawLine(x+w-2,y+1,x+w-2,y+h-2);
+  dc.fillRectangle(x,y,w,1);
+  dc.fillRectangle(x,y,1,h);
+  dc.fillRectangle(x+1,y+h-2,w-2,1);
+  dc.fillRectangle(x+w-2,y+1,1,h-2);
   dc.setForeground(hiliteColor);
-  dc.drawLine(x+1,y+1,x+w-2,y+1);
-  dc.drawLine(x+1,y+1,x+1,y+h-2);
-  dc.drawLine(x+1,y+h-1,x+w-1,y+h-1);
-  dc.drawLine(x+w-1,y+1,x+w-1,y+h-1);
+  dc.fillRectangle(x+1,y+1,w-2,1);
+  dc.fillRectangle(x+1,y+1,1,h-2);
+  dc.fillRectangle(x+1,y+h-1,w,1);
+  dc.fillRectangle(x+w-1,y+1,1,h);
   }
 
 
-// Draw 2 pixel double raised border
 void FXPacker::drawDoubleRaisedRectangle(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
   dc.setForeground(hiliteColor);
-  dc.drawLine(x,y,x+w-2,y);
-  dc.drawLine(x,y,x,y+h-2);
+  dc.fillRectangle(x,y,w-1,1);
+  dc.fillRectangle(x,y,1,h-1);
   dc.setForeground(baseColor);
-  dc.drawLine(x+1,y+1,x+w-3,y+1);
-  dc.drawLine(x+1,y+1,x+1,y+h-3);
+  dc.fillRectangle(x+1,y+1,w-2,1);
+  dc.fillRectangle(x+1,y+1,1,h-2);
   dc.setForeground(shadowColor);
-  dc.drawLine(x+1,y+h-2,x+w-2,y+h-2);
-  dc.drawLine(x+w-2,y+h-2,x+w-2,y+1);
+  dc.fillRectangle(x+1,y+h-2,w-2,1);
+  dc.fillRectangle(x+w-2,y+1,1,h-1);
   dc.setForeground(borderColor);
-  dc.drawLine(x,y+h-1,x+w-1,y+h-1);
-  dc.drawLine(x+w-1,y,x+w-1,y+h-1);
+  dc.fillRectangle(x,y+h-1,w,1);
+  dc.fillRectangle(x+w-1,y,1,h);
   }
 
-
-// Draw 2 pixel double sunken border
 void FXPacker::drawDoubleSunkenRectangle(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
   dc.setForeground(shadowColor);
-  dc.drawLine(x,y,x+w-1,y);
-  dc.drawLine(x,y,x,y+h-1);
+  dc.fillRectangle(x,y,w-1,1);
+  dc.fillRectangle(x,y,1,h-1);
   dc.setForeground(borderColor);
-  dc.drawLine(x+1,y+1,x+w-2,y+1);
-  dc.drawLine(x+1,y+1,x+1,y+h-2);
+  dc.fillRectangle(x+1,y+1,w-3,1);
+  dc.fillRectangle(x+1,y+1,1,h-3);
   dc.setForeground(hiliteColor);
-  dc.drawLine(x+1,y+h-1,x+w-1,y+h-1);
-  dc.drawLine(x+w-1,y+h-1,x+w-1,y+1);
+  dc.fillRectangle(x,y+h-1,w,1);
+  dc.fillRectangle(x+w-1,y,1,h);
   dc.setForeground(baseColor);
-  dc.drawLine(x+2,y+h-2,x+w-2,y+h-2);
-  dc.drawLine(x+w-2,y+2,x+w-2,y+h-2);
+  dc.fillRectangle(x+1,y+h-2,w-2,1);
+  dc.fillRectangle(x+w-2,y+1,1,h-2);
   }
 
 
+
 // Draw border
 void FXPacker::drawFrame(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
   switch(options&FRAME_MASK) {
@@ -207,7 +205,7 @@ void FXPacker::drawFrame(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
   }
 
 
-// Handle repaint 
+// Handle repaint
 long FXPacker::onPaint(FXObject*,FXSelector,void* ptr){
   FXEvent *ev=(FXEvent*)ptr;
   FXDCWindow dc(this,ev);
@@ -228,8 +226,8 @@ void FXPacker::setFrameStyle(FXuint style){
 
 
 // Get frame style
-FXuint FXPacker::getFrameStyle() const { 
-  return (options&FRAME_MASK); 
+FXuint FXPacker::getFrameStyle() const {
+  return (options&FRAME_MASK);
   }
 
 
@@ -245,7 +243,7 @@ void FXPacker::setPackingHints(FXuint ph){
 
 
 // Get packing hints
-FXuint FXPacker::getPackingHints() const { 
+FXuint FXPacker::getPackingHints() const {
   return (options&(PACK_UNIFORM_HEIGHT|PACK_UNIFORM_WIDTH));
   }
 
@@ -342,7 +340,7 @@ void FXPacker::setVSpacing(FXint vs){
 long FXPacker::onFocusUp(FXObject*,FXSelector sel,void* ptr){
   FXWindow *child,*c;
   FXint cury,childy;
-  FXTRACE((100,"%s::onFocusUp %08x\n",getClassName(),this));
+  FXTRACE((100,"%s::onFocusUp %p\n",getClassName(),this));
   if(getFocus()){
     cury=getFocus()->getY();
     while(1){
@@ -381,7 +379,7 @@ long FXPacker::onFocusUp(FXObject*,FXSelector sel,void* ptr){
 long FXPacker::onFocusDown(FXObject*,FXSelector sel,void* ptr){
   FXWindow *child,*c;
   FXint cury,childy;
-  FXTRACE((100,"%s::onFocusDown %08x\n",getClassName(),this));
+  FXTRACE((100,"%s::onFocusDown %p\n",getClassName(),this));
   if(getFocus()){
     cury=getFocus()->getY();
     while(1){
@@ -420,7 +418,7 @@ long FXPacker::onFocusDown(FXObject*,FXSelector sel,void* ptr){
 long FXPacker::onFocusLeft(FXObject*,FXSelector sel,void* ptr){
   FXWindow *child,*c;
   FXint curx,childx;
-  FXTRACE((100,"%s::onFocusLeft %08x\n",getClassName(),this));
+  FXTRACE((100,"%s::onFocusLeft %p\n",getClassName(),this));
   if(getFocus()){
     curx=getFocus()->getX();
     while(1){
@@ -459,7 +457,7 @@ long FXPacker::onFocusLeft(FXObject*,FXSelector sel,void* ptr){
 long FXPacker::onFocusRight(FXObject*,FXSelector sel,void* ptr){
   FXWindow *child,*c;
   FXint curx,childx;
-  FXTRACE((100,"%s::onFocusRight %08x\n",getClassName(),this));
+  FXTRACE((100,"%s::onFocusRight %p\n",getClassName(),this));
   if(getFocus()){
     curx=getFocus()->getX();
     while(1){
@@ -496,7 +494,7 @@ long FXPacker::onFocusRight(FXObject*,FXSelector sel,void* ptr){
 
 // Compute minimum width based on child layout hints
 int FXPacker::getDefaultWidth(){
-  register FXint t,wcum,wmax,mw=0;
+  register FXint w,wcum,wmax,mw=0;
   register FXWindow* child;
   register FXuint hints,side;
   wmax=wcum=0;
@@ -505,19 +503,19 @@ int FXPacker::getDefaultWidth(){
     if(child->shown()){
       hints=child->getLayoutHints();
       side=hints&LAYOUT_SIDE_MASK;
-      if(hints&LAYOUT_FIX_WIDTH) t=child->getWidth(); 
-      else if(options&PACK_UNIFORM_WIDTH) t=mw;
-      else t=child->getDefaultWidth();
-      if(hints&LAYOUT_FIX_X){ 
-        t=child->getX()+t;
-        if(t>wmax) wmax=t;
+      if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth();
+      else if(options&PACK_UNIFORM_WIDTH) w=mw;
+      else w=child->getDefaultWidth();
+      if((hints&LAYOUT_RIGHT)&&(hints&LAYOUT_CENTER_X)){
+        w=child->getX()+w;
+        if(w>wmax) wmax=w;
         }
       else if(side==LAYOUT_SIDE_LEFT || side==LAYOUT_SIDE_RIGHT){
-        if(child->getPrev()) wcum+=hspacing;
-        wcum+=t;
+        if(child->getNext()) wcum+=hspacing;
+        wcum+=w;
         }
       else{
-        if(t>wcum) wcum=t;
+        if(w>wcum) wcum=w;
         }
       }
     }
@@ -527,7 +525,7 @@ int FXPacker::getDefaultWidth(){
 
 // Compute minimum height based on child layout hints
 int FXPacker::getDefaultHeight(){
-  register FXint t,hcum,hmax,mh=0;
+  register FXint h,hcum,hmax,mh=0;
   register FXWindow* child;
   register FXuint hints,side;
   hmax=hcum=0;
@@ -536,19 +534,19 @@ int FXPacker::getDefaultHeight(){
     if(child->shown()){
       hints=child->getLayoutHints();
       side=hints&LAYOUT_SIDE_MASK;
-      if(hints&LAYOUT_FIX_HEIGHT) t=child->getHeight(); 
-      else if(options&PACK_UNIFORM_HEIGHT) t=mh;
-      else t=child->getDefaultHeight();
-      if(hints&LAYOUT_FIX_Y){
-        t=child->getY()+t;
-        if(t>hmax) hmax=t;
+      if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight();
+      else if(options&PACK_UNIFORM_HEIGHT) h=mh;
+      else h=child->getDefaultHeight();
+      if((hints&LAYOUT_BOTTOM)&&(hints&LAYOUT_CENTER_Y)){
+        h=child->getY()+h;
+        if(h>hmax) hmax=h;
         }
       else if(side==LAYOUT_SIDE_TOP || side==LAYOUT_SIDE_BOTTOM){
-        if(child->getPrev()) hcum+=vspacing;
-        hcum+=t;
+        if(child->getNext()) hcum+=vspacing;
+        hcum+=h;
         }
       else{
-        if(t>hcum) hcum=t;
+        if(h>hcum) hcum=h;
         }
       }
     }
@@ -558,178 +556,90 @@ int FXPacker::getDefaultHeight(){
 
 // Recalculate layout
 void FXPacker::layout(){
-  FXint left,right,top,bottom;
-  FXint mw=0,mh=0;
-  FXWindow* child;
-  FXint x,y,w,h;
-  FXint extra_space,total_space;
-  FXuint hints,side;
-  
+  register FXint left,right,top,bottom,x,y,w,h;
+  register FXint mw=0,mh=0;
+  register FXWindow* child;
+  register FXuint hints;
+
   // Placement rectangle; right/bottom non-inclusive
   left=border+padleft;
   right=width-border-padright;
   top=border+padtop;
   bottom=height-border-padbottom;
-  
+
   // Get maximum child size
   if(options&PACK_UNIFORM_WIDTH) mw=maxChildWidth();
   if(options&PACK_UNIFORM_HEIGHT) mh=maxChildHeight();
-  
+
   // Pack them in the cavity
   for(child=getFirst(); child; child=child->getNext()){
     if(child->shown()){
       hints=child->getLayoutHints();
-      side=hints&LAYOUT_SIDE_MASK;
       x=child->getX();
       y=child->getY();
-      
-      if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth(); 
-      else if(options&PACK_UNIFORM_WIDTH) w=mw;
-      else w=child->getDefaultWidth();
-      
-      if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight(); 
+
+      // Height
+      if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight();
       else if(options&PACK_UNIFORM_HEIGHT) h=mh;
+      else if(hints&LAYOUT_FILL_Y) h=bottom-top;
       else h=child->getDefaultHeight();
-      
-      if(side==LAYOUT_SIDE_LEFT){           // Left side
-        if(!(hints&LAYOUT_FIX_Y)){
-          extra_space=0;
-          if((hints&LAYOUT_FILL_Y) && !(hints&LAYOUT_FIX_HEIGHT)){
-            h=bottom-top;
-            if(h<0) h=0;
-            }
-          else if(hints&LAYOUT_CENTER_Y){
-            if(h<(bottom-top)) extra_space=(bottom-top-h)/2;
-            }
-          if(hints&LAYOUT_BOTTOM)
-            y=bottom-extra_space-h;
-          else /*hints&LAYOUT_TOP*/
-            y=top+extra_space;
+
+      // Width
+      if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth();
+      else if(options&PACK_UNIFORM_WIDTH) w=mw;
+      else if(hints&LAYOUT_FILL_X) w=right-left;
+      else w=child->getDefaultWidth();
+
+      // Vertical
+      if(hints&LAYOUT_SIDE_LEFT){
+
+        // Y
+        if(!((hints&LAYOUT_BOTTOM)&&(hints&LAYOUT_CENTER_Y))){
+          if(hints&LAYOUT_CENTER_Y) y=top+(bottom-top-h)/2;
+          else if(hints&LAYOUT_BOTTOM) y=bottom-h;
+          else y=top;
           }
-        if(!(hints&LAYOUT_FIX_X)){
-          extra_space=0;
-          total_space=0;
-          if((hints&LAYOUT_FILL_X) && !(hints&LAYOUT_FIX_WIDTH)){
-            w=right-left;
-            if(w<0) w=0;
+
+        // X
+        if(!((hints&LAYOUT_RIGHT)&&(hints&LAYOUT_CENTER_X))){
+          if(hints&LAYOUT_CENTER_X) x=left+(right-left-w)/2;
+          else if(hints&LAYOUT_SIDE_BOTTOM){
+            x=right-w;
+            right-=(w+hspacing);
             }
-          else if(hints&LAYOUT_CENTER_X){
-            if(w<(right-left)){
-              total_space=right-left-w;
-              extra_space=total_space/2;
-              }
+          else{
+            x=left;
+            left+=(w+hspacing);
             }
-          x=left+extra_space;
-          left+=(w+hspacing+total_space);
           }
         }
-      
-      else if(side==LAYOUT_SIDE_RIGHT){     // Right side
-        if(!(hints&LAYOUT_FIX_Y)){
-          extra_space=0;
-          if((hints&LAYOUT_FILL_Y) && !(hints&LAYOUT_FIX_HEIGHT)){
-            h=bottom-top;
-            if(h<0) h=0;
-            }
-          else if(hints&LAYOUT_CENTER_Y){
-            if(h<(bottom-top)) extra_space=(bottom-top-h)/2;
-            }
-          if(hints&LAYOUT_BOTTOM)
-            y=bottom-extra_space-h;
-          else /*hints&LAYOUT_TOP*/
-            y=top+extra_space;
+
+      // Horizontal
+      else{
+
+        // X
+        if(!((hints&LAYOUT_RIGHT)&&(hints&LAYOUT_CENTER_X))){
+          if(hints&LAYOUT_CENTER_X) x=left+(right-left-w)/2;
+          else if(hints&LAYOUT_RIGHT) x=right-w;
+          else x=left;
           }
-        if(!(hints&LAYOUT_FIX_X)){
-          extra_space=0;
-          total_space=0;
-          if((hints&LAYOUT_FILL_X) && !(hints&LAYOUT_FIX_WIDTH)){
-            w=right-left;
-            if(w<0) w=0;
+
+        // Y
+        if(!((hints&LAYOUT_BOTTOM)&&(hints&LAYOUT_CENTER_Y))){
+          if(hints&LAYOUT_CENTER_Y) y=top+(bottom-top-h)/2;
+          else if(hints&LAYOUT_SIDE_BOTTOM){
+            y=bottom-h;
+            bottom-=(h+vspacing);
             }
-          else if(hints&LAYOUT_CENTER_X){
-            if(w<(right-left)){
-              total_space=right-left-w;
-              extra_space=total_space/2;
-              }
+          else{
+            y=top;
+            top+=(h+vspacing);
             }
-          x=right-w-extra_space;
-          right-=(w+hspacing+total_space);
           }
         }
-      
-      else if(side==LAYOUT_SIDE_BOTTOM){    // Bottom side
-        if(!(hints&LAYOUT_FIX_X)){
-          extra_space=0;
-          if((hints&LAYOUT_FILL_X) && !(hints&LAYOUT_FIX_WIDTH)){
-            w=right-left;
-            if(w<0) w=0;
-            }
-          else if(hints&LAYOUT_CENTER_X){
-            if(w<(right-left)) extra_space=(right-left-w)/2;
-            }
-          if(hints&LAYOUT_RIGHT)
-            x=right-extra_space-w;
-          else /*hints&LAYOUT_LEFT*/
-            x=left+extra_space;
-          }
-        if(!(hints&LAYOUT_FIX_Y)){
-          extra_space=0;
-          total_space=0;
-          if((hints&LAYOUT_FILL_Y) && !(hints&LAYOUT_FIX_HEIGHT)){
-            h=bottom-top;
-            if(h<0) h=0;
-            }
-          else if(hints&LAYOUT_CENTER_Y){
-            if(h<(bottom-top)){
-              total_space=bottom-top-h;
-              extra_space=total_space/2;
-              }
-            }
-          y=bottom-h-extra_space;
-          bottom-=(h+vspacing+total_space);
-          }
-        }
-      
-      else if(side==LAYOUT_SIDE_TOP){       // Top side
-        if(!(hints&LAYOUT_FIX_X)){
-          extra_space=0;
-          if((hints&LAYOUT_FILL_X) && !(hints&LAYOUT_FIX_WIDTH)){
-            w=right-left;
-            if(w<0) w=0;
-            }
-          else if(hints&LAYOUT_CENTER_X){
-            if(w<(right-left)) extra_space=(right-left-w)/2;
-            }
-          if(hints&LAYOUT_RIGHT)
-            x=right-extra_space-w;
-          else /*hints&LAYOUT_LEFT*/
-            x=left+extra_space;
-          }
-        if(!(hints&LAYOUT_FIX_Y)){
-          extra_space=0;
-          total_space=0;
-          if((hints&LAYOUT_FILL_Y) && !(hints&LAYOUT_FIX_HEIGHT)){
-            //if(h<(bottom-top)) h=bottom-top;
-            h=bottom-top;
-            if(h<0) h=0;
-            }
-          else if(hints&LAYOUT_CENTER_Y){
-            if(h<(bottom-top)){
-              total_space=bottom-top-h;
-              extra_space=total_space/2;
-              }
-            }
-          y=top+extra_space;
-          top+=(h+vspacing+total_space);
-          }
-        }
-      
-      // Place it
       child->position(x,y,w,h);
       }
     }
-  
-  // No more dirty
   flags&=~FLAG_DIRTY;
   }
 
@@ -757,4 +667,4 @@ void FXPacker::load(FXStream& store){
   store >> padtop >> padbottom >> padleft >> padright;
   store >> hspacing >> vspacing;
   store >> border;
-  }  
+  }
