@@ -3,42 +3,45 @@
 *                     P o p u p   W i n d o w   O b j e c t                     *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXPopup.cpp,v 1.6 1999/11/16 21:24:45 jeroen Exp $                       *
+* $Id: FXPopup.cpp,v 1.44 2001/08/29 04:12:48 jeroen Exp $                      *
 ********************************************************************************/
+#include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "fxkeys.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
-#include "FXAccelTable.h"
 #include "FXApp.h"
 #include "FXId.h"
 #include "FXDC.h"
 #include "FXDCWindow.h"
-#include "FXCursor.h"
 #include "FXDrawable.h"
 #include "FXWindow.h"
 #include "FXFrame.h"
 #include "FXComposite.h"
-#include "FXRootWindow.h"
 #include "FXShell.h"
 #include "FXPopup.h"
 
@@ -48,12 +51,13 @@
     are displayed & forced to other size than default, e.g. for a ComboBox!
   - LAYOUT_FIX_xxx takes precedence over PACK_UNIFORM_xxx!!
   - Perhaps the grab owner could be equal to the owner?
+  - Perhaps popup should resize when recalc() has been called!
+  - I believe that popup() should in fact enter a modal loop runModalWhileShown().
 */
 
 
 // Frame styles
 #define FRAME_MASK        (FRAME_SUNKEN|FRAME_RAISED|FRAME_THICK)
-#define POPUP_MASK        POPUP_HORIZONTAL
 
 /*******************************************************************************/
 
@@ -65,12 +69,12 @@ FXDEFMAP(FXPopup) FXPopupMap[]={
   FXMAPFUNC(SEL_LEAVE,0,FXPopup::onLeave),
   FXMAPFUNC(SEL_MOTION,0,FXPopup::onMotion),
   FXMAPFUNC(SEL_MAP,0,FXPopup::onMap),
-  FXMAPFUNC(SEL_FOCUS_UP,0,FXPopup::onFocusUp),
-  FXMAPFUNC(SEL_FOCUS_DOWN,0,FXPopup::onFocusDown),
   FXMAPFUNC(SEL_FOCUS_NEXT,0,FXPopup::onDefault),
   FXMAPFUNC(SEL_FOCUS_PREV,0,FXPopup::onDefault),
-  FXMAPFUNC(SEL_FOCUS_RIGHT,0,FXPopup::onDefault),
-  FXMAPFUNC(SEL_FOCUS_LEFT,0,FXPopup::onDefault),
+  FXMAPFUNC(SEL_FOCUS_UP,0,FXPopup::onFocusUp),
+  FXMAPFUNC(SEL_FOCUS_DOWN,0,FXPopup::onFocusDown),
+  FXMAPFUNC(SEL_FOCUS_LEFT,0,FXPopup::onFocusLeft),
+  FXMAPFUNC(SEL_FOCUS_RIGHT,0,FXPopup::onFocusRight),
   FXMAPFUNC(SEL_LEFTBUTTONPRESS,0,FXPopup::onButtonPress),
   FXMAPFUNC(SEL_LEFTBUTTONRELEASE,0,FXPopup::onButtonRelease),
   FXMAPFUNC(SEL_MIDDLEBUTTONPRESS,0,FXPopup::onButtonPress),
@@ -81,6 +85,7 @@ FXDEFMAP(FXPopup) FXPopupMap[]={
   FXMAPFUNC(SEL_KEYRELEASE,0,FXPopup::onKeyRelease),
   FXMAPFUNC(SEL_UNGRABBED,0,FXPopup::onUngrabbed),
   FXMAPFUNC(SEL_COMMAND,FXWindow::ID_UNPOST,FXPopup::onCmdUnpost),
+  FXMAPFUNCS(SEL_COMMAND,FXPopup::ID_CHOICE,FXPopup::ID_CHOICE+999,FXPopup::onCmdChoice),
   };
 
 
@@ -91,63 +96,74 @@ FXIMPLEMENT(FXPopup,FXShell,FXPopupMap,ARRAYNUMBER(FXPopupMap))
 // Transient window used for popups
 FXPopup::FXPopup(FXWindow* owner,FXuint opts,FXint x,FXint y,FXint w,FXint h):
   FXShell(owner,opts,x,y,w,h){
-  defaultCursor=getApp()->rarrowCursor;
-  dragCursor=getApp()->rarrowCursor;
+  defaultCursor=getApp()->getDefaultCursor(DEF_RARROW_CURSOR);
+  dragCursor=getApp()->getDefaultCursor(DEF_RARROW_CURSOR);
   flags|=FLAG_ENABLED;
   grabowner=NULL;
-  baseColor=getApp()->baseColor;
-  hiliteColor=getApp()->hiliteColor;
-  shadowColor=getApp()->shadowColor;
-  borderColor=getApp()->borderColor;
+  baseColor=getApp()->getBaseColor();
+  hiliteColor=getApp()->getHiliteColor();
+  shadowColor=getApp()->getShadowColor();
+  borderColor=getApp()->getBorderColor();
   border=(options&FRAME_THICK)?2:(options&(FRAME_SUNKEN|FRAME_RAISED))?1:0;
   }
 
 
 // Popups do override-redirect
-FXbool FXPopup::doesOverrideRedirect() const { 
-  return 1; 
+FXbool FXPopup::doesOverrideRedirect() const {
+  return 1;
   }
 
 
 // Popups do save-unders
-FXbool FXPopup::doesSaveUnder() const { 
-  return 1; 
+FXbool FXPopup::doesSaveUnder() const {
+  return 1;
+  }
+
+
+#ifdef WIN32
+
+const char* FXPopup::GetClass() const { return "FXPopup"; }
+
+#endif
+
+
+// Popup can not get focus
+void FXPopup::setFocus(){ 
+  FXShell::setFocus();
+  //grabKeyboard();
+  }
+
+
+// Popup can not get focus
+void FXPopup::killFocus(){ 
+  FXShell::killFocus();
+  //ungrabKeyboard();
   }
 
 
 // Get owner; if it has none, it's owned by itself
-FXWindow* FXPopup::getGrabOwner() const { 
-  return grabowner ? grabowner : (FXWindow*)this; 
-  }
-
-
-// Will not receive focus
-void FXPopup::setFocus(){ 
-  }
-
-
-// Won't loose it either
-void FXPopup::killFocus(){ 
+FXWindow* FXPopup::getGrabOwner() const {
+  return grabowner ? grabowner : (FXWindow*)this;
   }
 
 
 // Get width
 FXint FXPopup::getDefaultWidth(){
   register FXWindow* child;
-  register FXint w,wmax,wcum,mw=0;
+  register FXint w,wmax,wcum,n;
   register FXuint hints;
-  wmax=wcum=0;
-  if(options&PACK_UNIFORM_WIDTH) mw=maxChildWidth();
+  wmax=wcum=n=0;
   for(child=getFirst(); child; child=child->getNext()){
     if(child->shown()){
       hints=child->getLayoutHints();
-      if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth(); 
-      else if(options&PACK_UNIFORM_WIDTH) w=mw;
+      if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth();
       else w=child->getDefaultWidth();
       if(wmax<w) wmax=w;
       wcum+=w;
+      n++;
       }
     }
+  if(options&PACK_UNIFORM_WIDTH) wcum=n*wmax;
   if(options&POPUP_HORIZONTAL) wmax=wcum;
   return wmax+(border<<1);
   }
@@ -156,20 +172,20 @@ FXint FXPopup::getDefaultWidth(){
 // Get height
 FXint FXPopup::getDefaultHeight(){
   register FXWindow* child;
-  register FXint h,hmax,hcum,mh=0;
+  register FXint h,hmax,hcum,n;
   register FXuint hints;
-  hmax=hcum=0;
-  if(options&PACK_UNIFORM_HEIGHT) mh=maxChildHeight();
+  hmax=hcum=n=0;
   for(child=getFirst(); child; child=child->getNext()){
     if(child->shown()){
       hints=child->getLayoutHints();
-      if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight(); 
-      else if(options&PACK_UNIFORM_HEIGHT) h=mh;
+      if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight();
       else h=child->getDefaultHeight();
       if(hmax<h) hmax=h;
       hcum+=h;
+      n++;
       }
     }
+  if(options&PACK_UNIFORM_HEIGHT) hcum=n*hmax;
   if(!(options&POPUP_HORIZONTAL)) hmax=hcum;
   return hmax+(border<<1);
   }
@@ -184,34 +200,34 @@ void FXPopup::layout(){
   register FXint sumexpand=0;
   register FXint numexpand=0;
   register FXint e=0;
-  
+
   // Horizontal
   if(options&POPUP_HORIZONTAL){
-    
+
     // Get maximum size if uniform packed
     if(options&PACK_UNIFORM_WIDTH) mh=maxChildWidth();
-    
+
     // Space available
     remain=width-(border<<1);
-    
+
     // Find number of paddable children and total space remaining
     for(child=getFirst(); child; child=child->getNext()){
       if(child->shown()){
         hints=child->getLayoutHints();
-        if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth(); 
+        if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth();
         else if(options&PACK_UNIFORM_WIDTH) w=mw;
         else w=child->getDefaultWidth();
         FXASSERT(w>=0);
         if((hints&LAYOUT_FILL_X) && !(hints&LAYOUT_FIX_WIDTH)){
-          sumexpand+=w; 
-          numexpand++; 
+          sumexpand+=w;
+          numexpand++;
           }
         else{
           remain-=w;
           }
         }
       }
-    
+
     // Do the layout
     for(x=border,child=getFirst(); child; child=child->getNext()){
       if(child->shown()){
@@ -239,27 +255,27 @@ void FXPopup::layout(){
         }
       }
     }
-  
+
   // Vertical
   else{
-    
+
     // Get maximum size if uniform packed
     if(options&PACK_UNIFORM_HEIGHT) mh=maxChildHeight();
-    
+
     // Space available
     remain=height-(border<<1);
-    
+
     // Find number of paddable children and total space remaining
     for(child=getFirst(); child; child=child->getNext()){
       if(child->shown()){
         hints=child->getLayoutHints();
-        if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight(); 
+        if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight();
         else if(options&PACK_UNIFORM_HEIGHT) h=mh;
         else h=child->getDefaultHeight();
         FXASSERT(h>=0);
         if((hints&LAYOUT_FILL_Y) && !(hints&LAYOUT_FIX_HEIGHT)){
-          sumexpand+=h; 
-          numexpand++; 
+          sumexpand+=h;
+          numexpand++;
           }
         else{
           remain-=h;
@@ -305,78 +321,82 @@ void FXPopup::drawBorderRectangle(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h
 
 
 void FXPopup::drawRaisedRectangle(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
-  dc.setForeground(hiliteColor);
-  dc.drawLine(x,y,x+w-2,y);
-  dc.drawLine(x,y,x,y+h-2);
   dc.setForeground(shadowColor);
-  dc.drawLine(x,y+h-1,x+w-1,y+h-1);
-  dc.drawLine(x+w-1,y,x+w-1,y+h-1);
+  dc.fillRectangle(x,y+h-1,w,1);
+  dc.fillRectangle(x+w-1,y,1,h);
+  dc.setForeground(hiliteColor);
+  dc.fillRectangle(x,y,w,1);
+  dc.fillRectangle(x,y,1,h);
   }
 
+
 void FXPopup::drawSunkenRectangle(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
   dc.setForeground(shadowColor);
-  dc.drawLine(x,y,x+w-2,y);
-  dc.drawLine(x,y,x,y+h-2);
+  dc.fillRectangle(x,y,w,1);
+  dc.fillRectangle(x,y,1,h);
   dc.setForeground(hiliteColor);
-  dc.drawLine(x,y+h-1,x+w-1,y+h-1);
-  dc.drawLine(x+w-1,y,x+w-1,y+h-1);
+  dc.fillRectangle(x,y+h-1,w,1);
+  dc.fillRectangle(x+w-1,y,1,h);
   }
 
+
 void FXPopup::drawRidgeRectangle(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
   dc.setForeground(hiliteColor);
-  dc.drawLine(x,y,x+w-1,y);
-  dc.drawLine(x,y,x,y+h-1);
-  dc.drawLine(x+1,y+h-2,x+w-2,y+h-2);
-  dc.drawLine(x+w-2,y+1,x+w-2,y+h-2);
+  dc.fillRectangle(x,y,w,1);
+  dc.fillRectangle(x,y,1,h);
+  dc.fillRectangle(x+1,y+h-2,w-2,1);
+  dc.fillRectangle(x+w-2,y+1,1,h-2);
   dc.setForeground(shadowColor);
-  dc.drawLine(x+1,y+1,x+w-3,y+1);
-  dc.drawLine(x+1,y+1,x+1,y+h-3);
-  dc.drawLine(x,y+h-1,x+w-1,y+h-1);
-  dc.drawLine(x+w-1,y,x+w-1,y+h-1);
+  dc.fillRectangle(x+1,y+1,w-3,1);
+  dc.fillRectangle(x+1,y+1,1,h-3);
+  dc.fillRectangle(x,y+h-1,w,1);
+  dc.fillRectangle(x+w-1,y,1,h);
   }
 
+
 void FXPopup::drawGrooveRectangle(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
   dc.setForeground(shadowColor);
-  dc.drawLine(x,y,x+w-1,y);
-  dc.drawLine(x,y,x,y+h-1);
-  dc.drawLine(x+1,y+h-2,x+w-2,y+h-2);
-  dc.drawLine(x+w-2,y+1,x+w-2,y+h-2);
+  dc.fillRectangle(x,y,w,1);
+  dc.fillRectangle(x,y,1,h);
+  dc.fillRectangle(x+1,y+h-2,w-2,1);
+  dc.fillRectangle(x+w-2,y+1,1,h-2);
   dc.setForeground(hiliteColor);
-  dc.drawLine(x+1,y+1,x+w-2,y+1);
-  dc.drawLine(x+1,y+1,x+1,y+h-2);
-  dc.drawLine(x+1,y+h-1,x+w-1,y+h-1);
-  dc.drawLine(x+w-1,y+1,x+w-1,y+h-1);
+  dc.fillRectangle(x+1,y+1,w-2,1);
+  dc.fillRectangle(x+1,y+1,1,h-2);
+  dc.fillRectangle(x+1,y+h-1,w,1);
+  dc.fillRectangle(x+w-1,y+1,1,h);
   }
 
 
 void FXPopup::drawDoubleRaisedRectangle(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
-  dc.setForeground(hiliteColor);
-  dc.drawLine(x,y,x+w-2,y);
-  dc.drawLine(x,y,x,y+h-2);
   dc.setForeground(baseColor);
-  dc.drawLine(x+1,y+1,x+w-3,y+1);
-  dc.drawLine(x+1,y+1,x+1,y+h-3);
+  dc.fillRectangle(x,y,w-1,1);
+  dc.fillRectangle(x,y,1,h-1);
+  dc.setForeground(hiliteColor);
+  dc.fillRectangle(x+1,y+1,w-2,1);
+  dc.fillRectangle(x+1,y+1,1,h-2);
   dc.setForeground(shadowColor);
-  dc.drawLine(x+1,y+h-2,x+w-2,y+h-2);
-  dc.drawLine(x+w-2,y+h-2,x+w-2,y+1);
+  dc.fillRectangle(x+1,y+h-2,w-2,1);
+  dc.fillRectangle(x+w-2,y+1,1,h-1);
   dc.setForeground(borderColor);
-  dc.drawLine(x,y+h-1,x+w-1,y+h-1);
-  dc.drawLine(x+w-1,y,x+w-1,y+h-1);
+  dc.fillRectangle(x,y+h-1,w,1);
+  dc.fillRectangle(x+w-1,y,1,h);
   }
 
+
 void FXPopup::drawDoubleSunkenRectangle(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
   dc.setForeground(shadowColor);
-  dc.drawLine(x,y,x+w-1,y);
-  dc.drawLine(x,y,x,y+h-1);
+  dc.fillRectangle(x,y,w-1,1);
+  dc.fillRectangle(x,y,1,h-1);
   dc.setForeground(borderColor);
-  dc.drawLine(x+1,y+1,x+w-2,y+1);
-  dc.drawLine(x+1,y+1,x+1,y+h-2);
+  dc.fillRectangle(x+1,y+1,w-3,1);
+  dc.fillRectangle(x+1,y+1,1,h-3);
   dc.setForeground(hiliteColor);
-  dc.drawLine(x+1,y+h-1,x+w-1,y+h-1);
-  dc.drawLine(x+w-1,y+h-1,x+w-1,y+1);
+  dc.fillRectangle(x,y+h-1,w,1);
+  dc.fillRectangle(x+w-1,y,1,h);
   dc.setForeground(baseColor);
-  dc.drawLine(x+2,y+h-2,x+w-2,y+h-2);
-  dc.drawLine(x+w-2,y+2,x+w-2,y+h-2);
+  dc.fillRectangle(x+1,y+h-2,w-2,1);
+  dc.fillRectangle(x+w-2,y+1,1,h-2);
   }
 
 
@@ -394,17 +414,43 @@ void FXPopup::drawFrame(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
   }
 
 
-// Handle repaint 
+// Handle repaint
 long FXPopup::onPaint(FXObject*,FXSelector,void* ptr){
   FXEvent *ev=(FXEvent*)ptr;
   FXDCWindow dc(this,ev);
+  dc.setForeground(backColor);
+  dc.fillRectangle(border,border,width-(border<<1),height-(border<<1));
   drawFrame(dc,0,0,width,height);
   return 1;
   }
 
 
+// Moving focus up
+long FXPopup::onFocusUp(FXObject* sender,FXSelector sel,void* ptr){
+  if(!(options&POPUP_HORIZONTAL)) return FXPopup::onFocusPrev(sender,sel,ptr);
+  return 0;
+  }
+
+// Moving focus down
+long FXPopup::onFocusDown(FXObject* sender,FXSelector sel,void* ptr){
+  if(!(options&POPUP_HORIZONTAL)) return FXPopup::onFocusNext(sender,sel,ptr);
+  return 0;
+  }
+
+// Moving focus left
+long FXPopup::onFocusLeft(FXObject* sender,FXSelector sel,void* ptr){
+  if(options&POPUP_HORIZONTAL) return FXPopup::onFocusPrev(sender,sel,ptr);
+  return 0;
+  }
+
+// Moving focus right
+long FXPopup::onFocusRight(FXObject* sender,FXSelector sel,void* ptr){
+  if(options&POPUP_HORIZONTAL) return FXPopup::onFocusNext(sender,sel,ptr);
+  return 0;
+  }
+
 // Focus moved down; wrap back to begin if at end
-long FXPopup::onFocusDown(FXObject*,FXSelector,void* ptr){
+long FXPopup::onFocusNext(FXObject*,FXSelector,void* ptr){
   FXWindow *child;
   if(getFocus()){
     child=getFocus()->getNext();
@@ -429,7 +475,7 @@ long FXPopup::onFocusDown(FXObject*,FXSelector,void* ptr){
 
 
 // Focus moved up; wrap back to end if at begin
-long FXPopup::onFocusUp(FXObject*,FXSelector,void* ptr){
+long FXPopup::onFocusPrev(FXObject*,FXSelector,void* ptr){
   FXWindow *child;
   if(getFocus()){
     child=getFocus()->getPrev();
@@ -455,9 +501,9 @@ long FXPopup::onFocusUp(FXObject*,FXSelector,void* ptr){
 
 // Moved into the popup:- tell the target
 long FXPopup::onEnter(FXObject* sender,FXSelector sel,void* ptr){
-  FXEvent* ev=(FXEvent*)ptr;
   FXShell::onEnter(sender,sel,ptr);
-  if(ev->code==CROSSINGNORMAL){ 
+  if(((FXEvent*)ptr)->code==CROSSINGNORMAL){
+  //if(((FXEvent*)ptr)->code!=CROSSINGGRAB){
     if(getGrabOwner()->grabbed()) getGrabOwner()->ungrab();
     }
   return 1;
@@ -466,15 +512,10 @@ long FXPopup::onEnter(FXObject* sender,FXSelector sel,void* ptr){
 
 // Moved outside the popup:- tell the target
 long FXPopup::onLeave(FXObject* sender,FXSelector sel,void* ptr){
-  FXEvent* ev=(FXEvent*)ptr;
-  FXint xx,yy;
   FXShell::onLeave(sender,sel,ptr);
-  if(ev->code==CROSSINGNORMAL && shown()){ 
-    getGrabOwner()->getParent()->translateCoordinatesFrom(xx,yy,getRoot(),ev->root_x,ev->root_y);
-    if(!getGrabOwner()->contains(xx,yy)){
-      //if(!getGrabOwner()->grabbed()) getGrabOwner()->grab();
-      if(!getGrabOwner()->grabbed() && getGrabOwner()->shown()) getGrabOwner()->grab();
-      }
+  if(((FXEvent*)ptr)->code==CROSSINGNORMAL){
+  //if(((FXEvent*)ptr)->code!=CROSSINGGRAB){
+    if(shown() && !getGrabOwner()->grabbed() && getGrabOwner()->shown()) getGrabOwner()->grab();
     }
   return 1;
   }
@@ -484,14 +525,12 @@ long FXPopup::onLeave(FXObject* sender,FXSelector sel,void* ptr){
 long FXPopup::onMotion(FXObject*,FXSelector,void* ptr){
   FXEvent* ev=(FXEvent*)ptr;
   FXint xx,yy;
-  FXTRACE((250,"%s::onMotion %08x\n",getClassName(),this));
   if(contains(ev->root_x,ev->root_y)){
     if(getGrabOwner()->grabbed()) getGrabOwner()->ungrab();
     }
   else{
     getGrabOwner()->getParent()->translateCoordinatesFrom(xx,yy,getRoot(),ev->root_x,ev->root_y);
     if(!getGrabOwner()->contains(xx,yy)){
-      //if(!getGrabOwner()->grabbed()) getGrabOwner()->grab();
       if(!getGrabOwner()->grabbed() && getGrabOwner()->shown()) getGrabOwner()->grab();
       }
     }
@@ -502,10 +541,10 @@ long FXPopup::onMotion(FXObject*,FXSelector,void* ptr){
 // Window may have appeared under the cursor, so ungrab if it was grabbed
 long FXPopup::onMap(FXObject* sender,FXSelector sel,void* ptr){
   FXint x,y; FXuint buttons;
-  FXTRACE((200,"%s::onMap %08x\n",getClassName(),this));
   FXShell::onMap(sender,sel,ptr);
   getCursorPosition(x,y,buttons);
   if(0<=x && 0<=y && x<width && y<height){
+    FXTRACE((200,"under cursor\n"));
     if(getGrabOwner()->grabbed()) getGrabOwner()->ungrab();
     }
   return 1;
@@ -514,8 +553,9 @@ long FXPopup::onMap(FXObject* sender,FXSelector sel,void* ptr){
 
 // Pressed button outside popup
 long FXPopup::onButtonPress(FXObject*,FXSelector,void*){
-  FXTRACE((200,"%s::onButtonPress %08x\n",getClassName(),this));
+  FXTRACE((200,"%s::onButtonPress %p\n",getClassName(),this));
   handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
+  //popdown(0);
   return 1;
   }
 
@@ -523,8 +563,9 @@ long FXPopup::onButtonPress(FXObject*,FXSelector,void*){
 // Released button outside popup
 long FXPopup::onButtonRelease(FXObject*,FXSelector,void* ptr){
   FXEvent* event=(FXEvent*)ptr;
-  FXTRACE((200,"%s::onButtonRelease %08x\n",getClassName(),this));
+  FXTRACE((200,"%s::onButtonRelease %p\n",getClassName(),this));
   if(event->moved){handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);}
+  //popdown(0);
   return 1;
   }
 
@@ -562,7 +603,7 @@ long FXPopup::onKeyRelease(FXObject* sender,FXSelector sel,void* ptr){
 // Unpost menu in case it was its own owner; otherwise
 // tell the owner to do so.
 long FXPopup::onCmdUnpost(FXObject*,FXSelector,void* ptr){
-  FXTRACE((200,"%s::onCmdUnpost %08x\n",getClassName(),this));
+  FXTRACE((200,"%s::onCmdUnpost %p\n",getClassName(),this));
   if(grabowner){
     grabowner->handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),ptr);
     }
@@ -574,20 +615,22 @@ long FXPopup::onCmdUnpost(FXObject*,FXSelector,void* ptr){
   }
 
 
+
 // Popup the menu at some location
 void FXPopup::popup(FXWindow* grabto,FXint x,FXint y,FXint w,FXint h){
   FXint rw=getRoot()->getWidth();
   FXint rh=getRoot()->getHeight();
   grabowner=grabto;
-  if(w<=1) w=getDefaultWidth();
-  if(h<=1) h=getDefaultHeight();
+  if((options&POPUP_SHRINKWRAP) || w<=1) w=getDefaultWidth();
+  if((options&POPUP_SHRINKWRAP) || h<=1) h=getDefaultHeight();
   if(x+w>rw) x=rw-w;
   if(y+h>rh) y=rh-h;
-  if(x<0) x=0; 
-  if(y<0) y=0; 
+  if(x<0) x=0;
+  if(y<0) y=0;
   position(x,y,w,h);
   show();
   raise();
+  setFocus();
   if(!grabowner) grab();    // Perhaps should NOT grab here!
   }
 
@@ -595,12 +638,47 @@ void FXPopup::popup(FXWindow* grabto,FXint x,FXint y,FXint w,FXint h){
 // Pops down menu and its submenus
 void FXPopup::popdown(){
   grabowner=NULL;
-  if(getFocus()) getFocus()->killFocus();
-//  killFocus();
+  //if(getFocus()) getFocus()->killFocus();
+  killFocus();
   hide();
   }
 
 
+// // Popup the menu and grab to the given owner
+// FXint FXPopup::popup(FXint x,FXint y,FXint w,FXint h){
+//   FXint rw,rh;
+//   create();
+//   if((options&POPUP_SHRINKWRAP) || w<=1) w=getDefaultWidth();
+//   if((options&POPUP_SHRINKWRAP) || h<=1) h=getDefaultHeight();
+//   rw=getRoot()->getWidth();
+//   rh=getRoot()->getHeight();
+//   if(x+w>rw) x=rw-w;
+//   if(y+h>rh) y=rh-h;
+//   if(x<0) x=0;
+//   if(y<0) y=0;
+//   position(x,y,w,h);
+//   show();
+//   raise();
+//   //setFocus();
+//   return getApp()->runPopup(this);
+//   }
+// 
+// 
+// // Pop down the menu
+// void FXPopup::popdown(FXint value){
+//   getApp()->stopModal(this,value);
+//   //killFocus();
+//   hide();
+//   }
+
+
+// Close popup
+long FXPopup::onCmdChoice(FXObject*,FXSelector sel,void*){
+  //popdown((SELID(sel)-ID_CHOICE));
+  return 1;
+  }
+
+
 // Set base color
 void FXPopup::setBaseColor(FXColor clr){
   baseColor=clr;
@@ -631,13 +709,13 @@ void FXPopup::setBorderColor(FXColor clr){
 
 // Get popup orientation
 FXuint FXPopup::getOrientation() const {
-  return (options&POPUP_MASK);
+  return (options&POPUP_HORIZONTAL);
   }
 
 
 // Set popup orientation
 void FXPopup::setOrientation(FXuint orient){
-  FXuint opts=(options&~POPUP_MASK) | (orient&POPUP_MASK);
+  FXuint opts=(options&~POPUP_HORIZONTAL) | (orient&POPUP_HORIZONTAL);
   if(options!=opts){
     options=opts;
     recalc();
@@ -645,6 +723,18 @@ void FXPopup::setOrientation(FXuint orient){
   }
 
 
+// Return shrinkwrap mode
+FXbool FXPopup::getShrinkWrap() const {
+  return (options&POPUP_SHRINKWRAP)!=0;
+  }
+
+
+// Change shrinkwrap mode
+void FXPopup::setShrinkWrap(FXbool sw){
+  options=sw ? (options|POPUP_SHRINKWRAP) : (options&~POPUP_SHRINKWRAP);
+  }
+
+
 // Change frame border style
 void FXPopup::setFrameStyle(FXuint style){
   FXuint opts=(options&~FRAME_MASK) | (style&FRAME_MASK);
@@ -661,8 +751,8 @@ void FXPopup::setFrameStyle(FXuint style){
 
 
 // Get frame style
-FXuint FXPopup::getFrameStyle() const { 
-  return (options&FRAME_MASK); 
+FXuint FXPopup::getFrameStyle() const {
+  return (options&FRAME_MASK);
   }
 
 
