@@ -3,46 +3,46 @@
 *                      P r o g r e s s B a r   W i d g e t                      *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
-* Contributed by: Jonathan Bush                                                 *
+* Contributed by: Jonathan Bush and Sander Jansen <sxj@cfdrc.com>               *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXProgressBar.cpp,v 1.2 1999/11/04 23:37:19 jeroen Exp $                 *
+* $Id: FXProgressBar.cpp,v 1.20 2001/08/29 18:28:57 jeroen Exp $                *
 ********************************************************************************/
 #include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
-#include "FXAccelTable.h"
 #include "FXApp.h"
 #include "FXId.h"
 #include "FXDC.h"
 #include "FXDCWindow.h"
 #include "FXFont.h"
 #include "FXDrawable.h"
-#include "FXImage.h"
-#include "FXIcon.h"
 #include "FXWindow.h"
 #include "FXFrame.h"
-#include "FXComposite.h"
-#include "FXLabel.h"
 #include "FXProgressBar.h"
 
 
@@ -53,10 +53,12 @@
   - Progress bar has a target, as it can send update messages.
 */
 
+#define PROGRESSBAR_MASK (PROGRESSBAR_PERCENTAGE|PROGRESSBAR_VERTICAL|PROGRESSBAR_DIAL)
+
 
 /*******************************************************************************/
 
-    
+
 // Map
 FXDEFMAP(FXProgressBar) FXProgressBarMap[]={
   FXMAPFUNC(SEL_PAINT,0,FXProgressBar::onPaint),
@@ -67,9 +69,9 @@ FXDEFMAP(FXProgressBar) FXProgressBarMap[]={
 
 
 // Object implementation
-FXIMPLEMENT(FXProgressBar,FXFrame,FXProgressBarMap,ARRAYNUMBER(FXProgressBarMap))    
+FXIMPLEMENT(FXProgressBar,FXFrame,FXProgressBarMap,ARRAYNUMBER(FXProgressBarMap))
+
 
-  
 
 // Make progress bar
 FXProgressBar::FXProgressBar(FXComposite* p,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb):
@@ -78,11 +80,20 @@ FXProgressBar::FXProgressBar(FXComposite* p,FXObject* tgt,FXSelector sel,FXuint
   message=sel;
   progress=0;
   total=100;
-  barsize=5;
-  font=getApp()->normalFont;
-  barBGColor=getApp()->backColor;
+  if(opts&PROGRESSBAR_DIAL){
+    textNumColor=FXRGB(255,255,255);
+    barBGColor=FXRGB(0,0,0);
+    barBGColor=getApp()->getBackColor();
+    textNumColor=FXRGB(0,0,255);
+    barsize=60;
+    }
+  else{
+    barBGColor=getApp()->getBackColor();
+    textNumColor=FXRGB(0,0,255);
+    barsize=5;
+    }
+  font=getApp()->getNormalFont();
   barColor=FXRGB(0,0,255);
-  textNumColor=FXRGB(0,0,255);
   textAltColor=FXRGB(255,255,255);
   backColor=barBGColor;
   }
@@ -90,35 +101,41 @@ FXProgressBar::FXProgressBar(FXComposite* p,FXObject* tgt,FXSelector sel,FXuint
 
 // Get minimum width
 FXint FXProgressBar::getDefaultWidth(){
-  FXint w=barsize;
-  if(options&PROGRESSBAR_PERCENTAGE){
-    w=font->getTextWidth("100%",4);
-    if(w<barsize) w=barsize;
+  FXint w=1,t;
+  if((options&PROGRESSBAR_VERTICAL) || (options&PROGRESSBAR_DIAL)){
+    w=barsize;
+    if(options&PROGRESSBAR_PERCENTAGE){
+      t=font->getTextWidth("100%",4);
+      if(w<t) w=t;
+      }
     }
-  return w+padleft+padright+border;  
+  return w+padleft+padright+border;
   }
 
 
 // Get minimum height
 FXint FXProgressBar::getDefaultHeight(){
-  FXint h=barsize;
-  if(options&PROGRESSBAR_PERCENTAGE){
-    h=font->getFontHeight();
-    if(h<barsize) h=barsize;
+  FXint h=1,t;
+  if(!(options&PROGRESSBAR_VERTICAL ) ||  (options&PROGRESSBAR_DIAL)){
+    h=barsize;
+    if(options&PROGRESSBAR_PERCENTAGE){
+      t=font->getFontHeight();
+      if(h<t) h=t;
+      }
     }
-  return h+padtop+padbottom+border;  
+  return h+padtop+padbottom+border;
   }
 
 
 // Create window
-void FXProgressBar::create(){ 
+void FXProgressBar::create(){
   FXFrame::create();
   font->create();
   }
 
 
 // Detach window
-void FXProgressBar::detach(){ 
+void FXProgressBar::detach(){
   FXFrame::detach();
   font->detach();
   }
@@ -133,15 +150,13 @@ long FXProgressBar::onCmdSetValue(FXObject*,FXSelector,void* ptr){
 
 // Set value
 long FXProgressBar::onCmdSetIntValue(FXObject*,FXSelector,void* ptr){
-  if(ptr==NULL){ fxerror("%s::onCmdSetIntValue: NULL pointer.\n",getClassName()); }
   setProgress(*((FXint*)ptr));
   return 1;
   }
 
 
-// Get value 
+// Get value
 long FXProgressBar::onCmdGetIntValue(FXObject*,FXSelector,void* ptr){
-  if(ptr==NULL){ fxerror("%s::onCmdGetIntValue: NULL pointer.\n",getClassName()); }
   *((FXint*)ptr)=getProgress();
   return 1;
   }
@@ -151,15 +166,77 @@ long FXProgressBar::onCmdGetIntValue(FXObject*,FXSelector,void* ptr){
 long FXProgressBar::onPaint(FXObject*,FXSelector,void *ptr){
   FXEvent *event=(FXEvent*)ptr;
   FXDCWindow dc(this,event);
-  FXint percent,barlength,barfilled,tx,ty,tw,th,n;
+  FXint percent,barlength,barfilled,tx,ty,tw,th,n,d;
   FXchar numtext[5];
-  
-  // Just draw the frame, not the background
+
+  // Draw borders if any
   drawFrame(dc,0,0,width,height);
-    
-  // Vertical
-  if(options&PROGRESSBAR_VERTICAL){
-    
+
+  if(options&PROGRESSBAR_DIAL){
+
+    // If total is 0, it's 100%
+    barfilled=23040;
+    percent=100;
+    if(total!=0){
+      barfilled=(FXuint) (((double)progress * (double)23040) / (double)total);
+      percent=(FXuint) (((double)progress * 100.0) / (double)total);
+      }
+
+    tw=width-(border<<1)-padleft-padright;
+    th=height-(border<<1)-padtop-padbottom;
+    d=FXMIN(tw,th);
+
+    tx=border+padleft+((tw-d)/2);
+    ty=border+padtop+((th-d)/2);
+
+    dc.setForeground(getBaseColor());
+    dc.fillRectangle(border,border,width-(border<<1),height-(border<<1));
+
+    if(barfilled!=23040){
+      dc.setForeground(barBGColor);
+      dc.fillArc(tx+2,ty+2,d-4,d-4,5760,23040-barfilled);
+      }
+    if(barfilled!=0){
+      dc.setForeground(barColor);
+      dc.fillArc(tx+2,ty+2,d-4,d-4,5760,-barfilled);
+      }
+
+//    int ang2 = (-ar*64) + (90*64);
+//    int xStart=int(tx+0.5*d+(d/2)*cos((90*64)*PI/(180.0*64.0)));
+//    int yStart=int(ty+0.5*d-(d/2)*sin((90*64)*PI/(180.0*64.0)));
+//    int xEnd=int(tx+0.5*d+(d/2)*cos(ang2*PI/(180.0*64.0)));
+//    int yEnd=int(ty+0.5*d-(d/2)*sin(ang2*PI/(180.0*64.0)));
+
+
+//    dc.setForeground(barBGColor);
+//    dc.drawLine(xStart,yStart,tx+d/2,ty+d/2);
+//    dc.drawLine(xEnd,yEnd,tx+d/2,ty+d/2);
+
+    // Draw outside circle
+//    dc.drawArc(tx,ty,d,d,0,64*360);
+
+    // Draw text
+    if(options&PROGRESSBAR_PERCENTAGE){
+      tw=font->getTextWidth("100%",4);
+      if(tw>(11*d)/16) return 1;
+      th=font->getFontHeight();
+      if(th>d/2) return 1;
+      sprintf(numtext,"%d%%",percent);
+      n=strlen(numtext);
+      tw=font->getTextWidth(numtext,n-1);       // Not including the % looks better
+      th=font->getFontHeight();
+      tx=tx+d/2-tw/2;
+      ty=ty+d/2+font->getFontAscent()+5;
+      //dc.setForeground(textNumColor);
+      dc.setForeground(FXRGB(255,255,255));
+      dc.setFunction(BLT_SRC_XOR_DST);      // FIXME
+      dc.drawText(tx,ty,numtext,n);
+      }
+    }
+
+  // Vertical bar
+  else if(options&PROGRESSBAR_VERTICAL){
+
     // If total is 0, it's 100%
     barlength=height-border-border;
     barfilled=barlength;
@@ -168,20 +245,20 @@ long FXProgressBar::onPaint(FXObject*,FXSelector,void *ptr){
       barfilled=(FXuint) (((double)progress * (double)barlength) / (double)total);
       percent=(FXuint) (((double)progress * 100.0) / (double)total);
       }
-    
+
     // Draw completed bar
     if(0<barfilled){
       dc.setForeground(barColor);
       dc.fillRectangle(border,height-border-barfilled,width-(border<<1),barfilled);
       }
-    
+
     // Draw uncompleted bar
     if(barfilled<barlength){
       dc.setForeground(barBGColor);
       dc.fillRectangle(border,border,width-(border<<1),barlength-barfilled);
       }
-    
-    // Draw text 
+
+    // Draw text
     if(options&PROGRESSBAR_PERCENTAGE){
       sprintf(numtext,"%d%%",percent);
       n=strlen(numtext);
@@ -208,10 +285,10 @@ long FXProgressBar::onPaint(FXObject*,FXSelector,void *ptr){
         }
       }
     }
-  
-  // Horizontal
+
+  // Horizontal bar
   else{
-    
+
     // If total is 0, it's 100%
     barlength=width-border-border;
     barfilled=barlength;
@@ -220,24 +297,24 @@ long FXProgressBar::onPaint(FXObject*,FXSelector,void *ptr){
       barfilled=(FXuint) (((double)progress * (double)barlength) / (double)total);
       percent=(FXuint) (((double)progress * 100.0) / (double)total);
       }
-    
+
     // Draw completed bar
     if(0<barfilled){
       dc.setForeground(barColor);
       dc.fillRectangle(border,border,barfilled,height-(border<<1));
       }
-    
+
     // Draw uncompleted bar
     if(barfilled<barlength){
       dc.setForeground(barBGColor);
       dc.fillRectangle(border+barfilled,border,barlength-barfilled,height-(border<<1));
       }
-    
-    // Draw text 
+
+    // Draw text
     if(options&PROGRESSBAR_PERCENTAGE){
       sprintf(numtext,"%d%%",percent);
       n=strlen(numtext);
-      tw=font->getTextWidth(numtext,n);
+      tw=font->getTextWidth(numtext,n-1); // Not including the % looks better
       th=font->getFontHeight();
       ty=(height-th)/2+font->getFontAscent();
       tx=(width-tw)/2;
@@ -262,25 +339,25 @@ long FXProgressBar::onPaint(FXObject*,FXSelector,void *ptr){
     }
   return 1;
   }
-    
+
 
 // Set amount of progress made
-void FXProgressBar::setProgress(FXuint value){ 
+void FXProgressBar::setProgress(FXuint value){
+  if(value>total) value=total;
   if(value!=progress){
-    progress=value; 
-    if(progress>total) progress=total; 
-    update(border,border,width-(border<<1),height-(border<<1)); 
-    getApp()->flush();
+    progress=value;
+    update(border,border,width-(border<<1),height-(border<<1));
+    repaint(border,border,width-(border<<1),height-(border<<1));
     }
   }
 
 
 // Increment amount of progress
-void FXProgressBar::increment(FXuint value){ 
+void FXProgressBar::increment(FXuint value){
   if(value!=0){
-    progress+=value; 
+    progress+=value;
     if(progress>total) progress=total;
-    update(border,border,width-(border<<1),height-(border<<1)); 
+    update(border,border,width-(border<<1),height-(border<<1));
     getApp()->flush();
     }
   }
@@ -297,35 +374,35 @@ void FXProgressBar::setTotal(FXuint value){
 
 
 // Change bar color
-void FXProgressBar::setBarColor(FXColor clr){  
-  barColor=clr;  
-  update(border,border,width-(border<<1),height-(border<<1)); 
+void FXProgressBar::setBarColor(FXColor clr){
+  barColor=clr;
+  update(border,border,width-(border<<1),height-(border<<1));
   }
 
 
 // Change bar background color
-void FXProgressBar::setBarBGColor(FXColor clr){  
-  barBGColor=clr;  
-  update(border,border,width-(border<<1),height-(border<<1)); 
+void FXProgressBar::setBarBGColor(FXColor clr){
+  barBGColor=clr;
+  update(border,border,width-(border<<1),height-(border<<1));
   }
 
 
 // Change text foreground color
-void FXProgressBar::setTextColor(FXColor clr) {   
-  textNumColor=clr;   
+void FXProgressBar::setTextColor(FXColor clr) {
+  textNumColor=clr;
   update();
   }
 
 
 // Change alternate text color
-void FXProgressBar::setTextAltColor(FXColor clr) {  
-  textAltColor=clr;   
+void FXProgressBar::setTextAltColor(FXColor clr) {
+  textAltColor=clr;
   update();
   }
 
 
 // Hide percentage display
-void FXProgressBar::hideNumber(){ 
+void FXProgressBar::hideNumber(){
   if(options&PROGRESSBAR_PERCENTAGE){
     options&=~PROGRESSBAR_PERCENTAGE;
     recalc();
@@ -335,7 +412,7 @@ void FXProgressBar::hideNumber(){
 
 
 // Show percentage display
-void FXProgressBar::showNumber(){ 
+void FXProgressBar::showNumber(){
   if(!(options&PROGRESSBAR_PERCENTAGE)){
     options|=PROGRESSBAR_PERCENTAGE;
     recalc();
@@ -365,9 +442,55 @@ void FXProgressBar::setFont(FXFont *fnt){
   }
 
 
+// Change spinner style
+void FXProgressBar::setBarStyle(FXuint style){
+  FXuint opts=(options&~PROGRESSBAR_MASK) | (style&PROGRESSBAR_MASK);
+  if(options!=opts){
+    options=opts;
+    recalc();
+    }
+  }
+
+
+// Get spinner style
+FXuint FXProgressBar::getBarStyle() const {
+  return (options&PROGRESSBAR_MASK);
+  }
+
+
+// Save object to stream
+void FXProgressBar::save(FXStream& store) const {
+  FXFrame::save(store);
+  store << progress;
+  store << total;
+  store << barsize;
+  store << font;
+  store << barBGColor;
+  store << barColor;
+  store << textNumColor;
+  store << textAltColor;
+  }
+
+
+// Load object from stream
+void FXProgressBar::load(FXStream& store){
+  FXFrame::load(store);
+  store >> progress;
+  store >> total;
+  store >> barsize;
+  store >> font;
+  store >> barBGColor;
+  store >> barColor;
+  store >> textNumColor;
+  store >> textAltColor;
+  }
+
+
+
 // Destroy
 FXProgressBar::~FXProgressBar(){
   font=(FXFont*)-1;
   }
 
-  
+
+
