@@ -3,48 +3,49 @@
 *                             S l i d e r   O b j e c t s                       *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXSlider.cpp,v 1.5 1999/11/05 18:06:36 jeroen Exp $                      *
+* $Id: FXSlider.cpp,v 1.28 2001/08/28 22:52:27 jeroen Exp $                     *
 ********************************************************************************/
+#include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
-#include "FXAccelTable.h"
 #include "FXApp.h"
 #include "FXId.h"
 #include "FXDC.h"
 #include "FXDCWindow.h"
 #include "FXDrawable.h"
-#include "FXImage.h"
-#include "FXIcon.h"
 #include "FXWindow.h"
 #include "FXFrame.h"
-#include "FXComposite.h"
-#include "FXLabel.h"
 #include "FXSlider.h"
 
 
 
 
-/* 
+/*
   Notes:
   - Add other options for tickmarks and so on.
   - Add API's to change sizes and appearance.
@@ -53,18 +54,19 @@
   - Sliderhead starts too small for SLIDER_INSIDE_BAR mode.
   - What messages should be sent?
   - Slider should work in double, not int.
-  - setPosition should not move head unless current headpos does not
+  - setValue should not move head unless current headpos does not
     reflect pos (due to resolution issues, several headpos's might
     map to the same pos).
   - If changing pos, you may or may not have to move the head;
     if changing the head, you may or may not get a new pos.
-  - In case of a course range, we have rounding also
+  - In case of a coarse range, we have rounding also.
 */
 
-#define OVERHANG        4
-#define MINOVERHANG     3
+#define OVERHANG        4           // Default amount of overhang
+#define MINOVERHANG     3           // Minimal amount of overhang
 #define HEADINSIDEBAR   20          // Default for inside bar head size
 #define HEADOVERHANGING 9           // Default for overhanging head size
+
 #define SLIDER_MASK (SLIDER_VERTICAL|SLIDER_ARROW_UP|SLIDER_ARROW_DOWN|SLIDER_INSIDE_BAR)
 
 /*******************************************************************************/
@@ -98,11 +100,12 @@ FXDEFMAP(FXSlider) FXSliderMap[]={
 // Object implementation
 FXIMPLEMENT(FXSlider,FXFrame,FXSliderMap,ARRAYNUMBER(FXSliderMap))
 
-  
+
 // Make a slider
 FXSlider::FXSlider(){
   flags|=FLAG_ENABLED|FLAG_SHOWN;
   timer=NULL;
+  headpos=0;
   dragpoint=0;
   }
 
@@ -116,17 +119,17 @@ FXSlider::FXSlider(FXComposite* p,FXObject* tgt,FXSelector sel,FXuint opts,FXint
   timer=NULL;
   range[0]=0;
   range[1]=100;
-  slotColor=getApp()->backColor;
-  baseColor=getApp()->baseColor;
-  hiliteColor=getApp()->hiliteColor;
-  shadowColor=getApp()->shadowColor;
-  borderColor=getApp()->borderColor;
+  pos=50;
+  incr=1;
+  slotColor=getApp()->getBackColor();
+  baseColor=getApp()->getBaseColor();
+  hiliteColor=getApp()->getHiliteColor();
+  shadowColor=getApp()->getShadowColor();
+  borderColor=getApp()->getBorderColor();
   headpos=0;
   dragpoint=0;
   headsize=(options&SLIDER_INSIDE_BAR)?HEADINSIDEBAR:HEADOVERHANGING;
   slotsize=5;
-  incr=1;
-  pos=50;
   }
 
 
@@ -159,7 +162,7 @@ FXint FXSlider::getDefaultWidth(){
   else{
     w=headsize+4;
     }
-  return w+padleft+padright+(border<<1); 
+  return w+padleft+padright+(border<<1);
   }
 
 
@@ -173,21 +176,21 @@ FXint FXSlider::getDefaultHeight(){
     else if(options&(SLIDER_ARROW_UP|SLIDER_ARROW_DOWN)) h=slotsize+2*MINOVERHANG+headsize/2;
     else h=slotsize+MINOVERHANG*2;
     }
-  return h+padtop+padbottom+(border<<1); 
+  return h+padtop+padbottom+(border<<1);
   }
 
 
 // Layout changed; even though the position is still
 // the same, the head may have to be moved.
 void FXSlider::layout(){
-  setPosition(pos);
+  setValue(pos);
   flags&=~FLAG_DIRTY;
   }
 
 
 // We were asked about status text
 long FXSlider::onQueryHelp(FXObject* sender,FXSelector,void*){
-  if(!help.empty() && (flags&FLAG_HELP)){ 
+  if(!help.empty() && (flags&FLAG_HELP)){
     sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),&help);
     return 1;
     }
@@ -207,35 +210,35 @@ long FXSlider::onQueryTip(FXObject* sender,FXSelector,void*){
 
 // Update value from a message
 long FXSlider::onCmdSetValue(FXObject*,FXSelector,void* ptr){
-  setPosition((FXint)(long)ptr);
+  setValue((FXint)(long)ptr);
   return 1;
   }
 
 
 // Update value from a message
 long FXSlider::onCmdSetIntValue(FXObject*,FXSelector,void* ptr){
-  setPosition(*((FXint*)ptr));
+  setValue(*((FXint*)ptr));
   return 1;
   }
 
 
 // Update value from a message
 long FXSlider::onCmdSetRealValue(FXObject*,FXSelector,void* ptr){
-  setPosition((FXint)*((FXdouble*)ptr));
+  setValue((FXint)*((FXdouble*)ptr));
   return 1;
   }
 
 
 // Obtain value from text field
 long FXSlider::onCmdGetIntValue(FXObject*,FXSelector,void* ptr){
-  *((FXint*)ptr)=getPosition();
+  *((FXint*)ptr)=getValue();
   return 1;
   }
 
 
-// Obtain value from text field
+// Obtain value with a message
 long FXSlider::onCmdGetRealValue(FXObject*,FXSelector,void* ptr){
-  *((FXdouble*)ptr)=(FXdouble)getPosition();
+  *((FXdouble*)ptr)=(FXdouble)getValue();
   return 1;
   }
 
@@ -249,8 +252,7 @@ long FXSlider::onCmdSetIntRange(FXObject*,FXSelector,void* ptr){
 
 // Get range with a message
 long FXSlider::onCmdGetIntRange(FXObject*,FXSelector,void* ptr){
-  ((FXint*)ptr)[0]=range[0];
-  ((FXint*)ptr)[1]=range[1];
+  getRange(((FXint*)ptr)[0],((FXint*)ptr)[1]);
   return 1;
   }
 
@@ -273,47 +275,45 @@ long FXSlider::onCmdGetRealRange(FXObject*,FXSelector,void* ptr){
 // Pressed LEFT button
 long FXSlider::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
   register FXEvent *event=(FXEvent*)ptr;
-  register FXint p;
+  register FXint p=pos;
   flags&=~FLAG_TIP;
   if(isEnabled()){
-    handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
     grab();
+    if(timer) timer=getApp()->removeTimeout(timer);
     if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONPRESS),ptr)) return 1;
     if(options&SLIDER_VERTICAL){
       if(event->win_y<headpos){
-        autoSlideDec();
-        p=pos-incr;
+        timer=getApp()->addTimeout(getApp()->getScrollDelay(),this,ID_AUTOINC);
+        p=pos+incr;
         }
       else if(event->win_y>(headpos+headsize)){
-        autoSlideInc();
-        p=pos+incr;
+        timer=getApp()->addTimeout(getApp()->getScrollDelay(),this,ID_AUTODEC);
+        p=pos-incr;
         }
       else{
         dragpoint=event->win_y-headpos;
-        p=pos;
         flags|=FLAG_PRESSED;
         }
       }
     else{
       if(event->win_x<headpos){
-        autoSlideDec();
+        timer=getApp()->addTimeout(getApp()->getScrollDelay(),this,ID_AUTODEC);
         p=pos-incr;
         }
       else if(event->win_x>(headpos+headsize)){
-        autoSlideInc();
+        timer=getApp()->addTimeout(getApp()->getScrollDelay(),this,ID_AUTOINC);
         p=pos+incr;
         }
       else{
         dragpoint=event->win_x-headpos;
-        p=pos;
         flags|=FLAG_PRESSED;
         }
       }
     if(p<range[0]) p=range[0];
     if(p>range[1]) p=range[1];
     if(p!=pos){
-      setPosition(p);
-      if(target) target->handle(this,MKUINT(message,SEL_CHANGED),(void*)pos); 
+      setValue(p);
+      if(target) target->handle(this,MKUINT(message,SEL_CHANGED),(void*)pos);
       flags|=FLAG_CHANGED;
       }
     flags&=~FLAG_UPDATE;
@@ -328,8 +328,8 @@ long FXSlider::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
   FXuint flgs=flags;
   if(isEnabled()){
     ungrab();
-    stopAutoSlide();
-    setPosition(pos);                                                 // Hop to exact position
+    if(timer) timer=getApp()->removeTimeout(timer);
+    setValue(pos);                                                 // Hop to exact position
     flags&=~FLAG_PRESSED;
     flags&=~FLAG_CHANGED;
     flags|=FLAG_UPDATE;
@@ -343,7 +343,7 @@ long FXSlider::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
   }
 
 
-// Moving 
+// Moving
 long FXSlider::onMotion(FXObject*,FXSelector,void* ptr){
   register FXEvent *event=(FXEvent*)ptr;
   register FXint xx,yy,ww,hh,lo,hi,p,h,travel;
@@ -364,7 +364,7 @@ long FXSlider::onMotion(FXObject*,FXSelector,void* ptr){
         update(border,lo-1,width-(border<<1),hi+headsize+2-lo);
         }
       if(travel>0)
-        p=range[0]+((range[1]-range[0])*(h-yy-2)+travel/2)/travel;    // Use rounding!!
+        p=range[0]+((range[1]-range[0])*(yy+2+travel-h)+travel/2)/travel;    // Use rounding!!
       else
         p=range[0];
       }
@@ -402,7 +402,6 @@ long FXSlider::onMiddleBtnPress(FXObject*,FXSelector,void* ptr){
   register FXint xx,yy,ww,hh,lo,hi,p,h,travel;
   flags&=~FLAG_TIP;
   if(isEnabled()){
-    handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
     grab();
     if(target && target->handle(this,MKUINT(message,SEL_MIDDLEBUTTONRELEASE),ptr)) return 1;
     dragpoint=headsize/2;
@@ -421,7 +420,7 @@ long FXSlider::onMiddleBtnPress(FXObject*,FXSelector,void* ptr){
         update(border,lo-1,width-(border<<1),hi+headsize+2-lo);
         }
       if(travel>0)
-        p=range[0]+((range[1]-range[0])*(h-yy-2)+travel/2)/travel;    // Use rounding!!
+        p=range[0]+((range[1]-range[0])*(yy+2+travel-h)+travel/2)/travel;    // Use rounding!!
       else
         p=range[0];
       }
@@ -460,11 +459,11 @@ long FXSlider::onMiddleBtnRelease(FXObject*,FXSelector,void* ptr){
   FXuint flgs=flags;
   if(isEnabled()){
     ungrab();
-    stopAutoSlide();
+    if(timer) timer=getApp()->removeTimeout(timer);
     flags&=~FLAG_PRESSED;
     flags&=~FLAG_CHANGED;
     flags|=FLAG_UPDATE;
-    setPosition(pos);                                                 // Hop to exact position
+    setValue(pos);                                                 // Hop to exact position
     if(target && target->handle(this,MKUINT(message,SEL_MIDDLEBUTTONRELEASE),ptr)) return 1;
     if(flgs&FLAG_CHANGED){
       if(target) target->handle(this,MKUINT(message,SEL_COMMAND),(void*)pos);
@@ -478,7 +477,7 @@ long FXSlider::onMiddleBtnRelease(FXObject*,FXSelector,void* ptr){
 // The widget lost the grab for some reason
 long FXSlider::onUngrabbed(FXObject* sender,FXSelector sel,void* ptr){
   FXFrame::onUngrabbed(sender,sel,ptr);
-  stopAutoSlide();
+  if(timer) timer=getApp()->removeTimeout(timer);
   flags&=~FLAG_PRESSED;
   flags&=~FLAG_CHANGED;
   flags|=FLAG_UPDATE;
@@ -489,16 +488,16 @@ long FXSlider::onUngrabbed(FXObject* sender,FXSelector sel,void* ptr){
 // Increment line timeout
 long FXSlider::onTimeInc(FXObject*,FXSelector,void*){
   FXint p=pos+incr;
-  if(p>range[1]){
+  if(p>=range[1]){
     p=range[1];
     timer=NULL;
     }
   else{
-    timer=getApp()->addTimeout(getApp()->scrollSpeed,this,ID_AUTOINC);
+    timer=getApp()->addTimeout(getApp()->getScrollSpeed(),this,ID_AUTOINC);
     }
   if(p!=pos){
-    setPosition(p);
-    if(target) target->handle(this,MKUINT(message,SEL_CHANGED),(void*)pos); 
+    setValue(p);
+    if(target) target->handle(this,MKUINT(message,SEL_CHANGED),(void*)pos);
     flags|=FLAG_CHANGED;
     return 1;
     }
@@ -509,16 +508,16 @@ long FXSlider::onTimeInc(FXObject*,FXSelector,void*){
 // Decrement line timeout
 long FXSlider::onTimeDec(FXObject*,FXSelector,void*){
   FXint p=pos-incr;
-  if(p<range[0]){
+  if(p<=range[0]){
     p=range[0];
     timer=NULL;
     }
   else{
-    timer=getApp()->addTimeout(getApp()->scrollSpeed,this,ID_AUTODEC);
+    timer=getApp()->addTimeout(getApp()->getScrollSpeed(),this,ID_AUTODEC);
     }
   if(p!=pos){
-    setPosition(p);
-    if(target) target->handle(this,MKUINT(message,SEL_CHANGED),(void*)pos); 
+    setValue(p);
+    if(target) target->handle(this,MKUINT(message,SEL_CHANGED),(void*)pos);
     flags|=FLAG_CHANGED;
     return 1;
     }
@@ -539,24 +538,33 @@ void FXSlider::drawSliderHead(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
       dc.drawLine(x,y+m,x+m,y);
       dc.setForeground(shadowColor);
       dc.drawLine(x+1,y+h-m-1,x+m+1,y+h-1);
-      dc.drawLine(x+m,y+h-2,x+w-2,y+h-2);
-      dc.drawLine(x+w-2,y+1,x+w-2,y+h-2);
+      dc.drawLine(x+m,y+h-2,x+w-1,y+h-2);
+      dc.drawLine(x+w-2,y+1,x+w-2,y+h-1);
       dc.setForeground(borderColor);
       dc.drawLine(x,y+h-m-1,x+m,y+h-1);
-      dc.drawLine(x+w-1,y,x+w-1,y+h-1);
-      dc.drawLine(x+m,y+h-1,x+w-1,y+h-1);
+      dc.drawLine(x+w-1,y+h-1,x+w-1,y);
+      dc.fillRectangle(x+m,y+h-1,w-m,1);
       }
     else if(options&SLIDER_ARROW_RIGHT){
       dc.setForeground(hiliteColor);
-      dc.drawLine(x,y,x+w-m-2,y);
+      dc.drawLine(x,y,x+w-m-1,y);
       dc.drawLine(x,y+1,x,y+h-1);
-      dc.drawLine(x+w-m-1,y,x+w-1,y+m);
+      dc.drawLine(x+w-1,y+m,x+w-m-1,y);
+#ifndef WIN32
       dc.setForeground(shadowColor);
-      dc.drawLine(x+w-m-2,y+h-1,x+w-2,y+h-m-1);
-      dc.drawLine(x+1,y+h-2,x+w-m-2,y+h-2);
+      dc.drawLine(x+w-2,y+h-m-1,x+w-m-2,y+h-1);
+      dc.drawLine(x+1,y+h-2,x+w-m-1,y+h-2);
       dc.setForeground(borderColor);
-      dc.drawLine(x+w-m-1,y+h-1,x+w-1,y+h-m-1);
-      dc.drawLine(x,y+h-1,x+w-m-2,y+h-1);
+      dc.drawLine(x+w-1,y+h-m-1,x+w-m-1,y+h-1);
+      dc.drawLine(x,y+h-1,x+w-m-1,y+h-1);
+#else
+      dc.setForeground(shadowColor);
+      dc.drawLine(x+w-1,y+h-m-2,x+w-m-2,y+h-1);
+      dc.drawLine(x+1,y+h-2,x+w-m-1,y+h-2);
+      dc.setForeground(borderColor);
+      dc.drawLine(x+w,y+h-m-2,x+w-m-1,y+h-1);
+      dc.drawLine(x,y+h-1,x+w-m-1,y+h-1);
+#endif
       }
     else if(options&SLIDER_INSIDE_BAR){
       drawDoubleRaisedRectangle(dc,x,y,w,h);
@@ -577,12 +585,12 @@ void FXSlider::drawSliderHead(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
       dc.drawLine(x,y+m,x,y+h-1);
       dc.setForeground(shadowColor);
       dc.drawLine(x+w-1,y+m+1,x+w-m-1,y+1);
-      dc.drawLine(x+w-2,y+m+1,x+w-2,y+h-2);
+      dc.drawLine(x+w-2,y+m+1,x+w-2,y+h-1);
       dc.drawLine(x+1,y+h-2,x+w-2,y+h-2);
       dc.setForeground(borderColor);
       dc.drawLine(x+w-1,y+m,x+w-m-1,y);
       dc.drawLine(x+w-1,y+m,x+w-1,y+h-1);
-      dc.drawLine(x,y+h-1,x+w-1,y+h-1);
+      dc.fillRectangle(x,y+h-1,w,1);
       }
     else if(options&SLIDER_ARROW_DOWN){
       dc.setForeground(hiliteColor);
@@ -590,11 +598,11 @@ void FXSlider::drawSliderHead(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
       dc.drawLine(x,y+1,x,y+h-m-1);
       dc.drawLine(x,y+h-m-1,x+m,y+h-1);
       dc.setForeground(shadowColor);
-      dc.drawLine(x+w-2,y+1,x+w-2,y+h-m-2);
+      dc.drawLine(x+w-2,y+1,x+w-2,y+h-m-1);
       dc.drawLine(x+w-1,y+h-m-2,x+w-m-1,y+h-2);
       dc.setForeground(borderColor);
-      dc.drawLine(x+w-1,y,x+w-1,y+h-m-1);
       dc.drawLine(x+w-1,y+h-m-1,x+w-m-1,y+h-1);
+      dc.fillRectangle(x+w-1,y,1,h-m);
       }
     else if(options&SLIDER_INSIDE_BAR){
       drawDoubleRaisedRectangle(dc,x,y,w,h);
@@ -610,24 +618,24 @@ void FXSlider::drawSliderHead(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
   }
 
 
-// Handle repaint 
+// Handle repaint
 long FXSlider::onPaint(FXObject*,FXSelector,void* ptr){
   FXEvent *ev=(FXEvent*)ptr;
   FXint tx,ty,hhs=headsize/2;
   FXint xx,yy,ww,hh;
   FXDCWindow dc(this,ev);
-  
+
   // Repaint border
   drawFrame(dc,0,0,width,height);
-  
+
   // Slot placement
   xx=border+padleft;
   yy=border+padtop;
   ww=width-(border<<1)-padleft-padright;
   hh=height-(border<<1)-padtop-padbottom;
-  
+
   FXASSERT(range[0]<=pos && pos<=range[1]);
-  
+
   // Draw the slot
   if(options&SLIDER_VERTICAL){
     if(options&SLIDER_INSIDE_BAR){
@@ -704,7 +712,7 @@ void FXSlider::setRange(FXint lo,FXint hi){
   if(range[0]!=lo || range[1]!=hi){
     range[0]=lo;
     range[1]=hi;
-    setPosition(pos);
+    setValue(pos);
     }
   }
 
@@ -714,14 +722,14 @@ void FXSlider::setRange(FXint lo,FXint hi){
 // head positions may represent the same position!
 // Also, the minimal amount is repainted, as one sometimes as very
 // large/wide sliders.
-void FXSlider::setPosition(FXint p){
+void FXSlider::setValue(FXint p){
   FXint travel,lo,hi,h;
   if(p<range[0]) p=range[0];
   if(p>range[1]) p=range[1];
   if(options&SLIDER_VERTICAL){
     travel=height-(border<<1)-padtop-padbottom-headsize-4;
     if(range[1]>range[0])
-      h=border+padtop+2+(travel*(p-range[0]))/(range[1]-range[0]);
+      h=border+padtop+2+(travel*(range[1]-p))/(range[1]-range[0]);
     else
       h=border+padtop+2;
     if(h!=headpos){
@@ -732,7 +740,7 @@ void FXSlider::setPosition(FXint p){
     }
   else{
     travel=width-(border<<1)-padleft-padright-headsize-4;
-    if(range[1]>range[0]) 
+    if(range[1]>range[0])
       h=border+padleft+2+(travel*(p-range[0]))/(range[1]-range[0]);
     else
       h=border+padleft+2;
@@ -748,7 +756,7 @@ void FXSlider::setPosition(FXint p){
 
 // Get slider options
 FXuint FXSlider::getSliderStyle() const {
-  return (options&SLIDER_MASK); 
+  return (options&SLIDER_MASK);
   }
 
 
@@ -773,7 +781,7 @@ void FXSlider::setHeadSize(FXint hs){
     }
   }
 
-  
+
 // Set slot size
 void FXSlider::setSlotSize(FXint bs){
   if(slotsize!=bs){
@@ -792,64 +800,10 @@ void FXSlider::setIncrement(FXint inc){
 
 // Set slot color
 void FXSlider::setSlotColor(FXColor clr){
-  slotColor=clr;
-  update();
-  }
-
-
-// Set base color
-void FXSlider::setBaseColor(FXColor clr){
-  baseColor=clr;
-  update();
-  }
-
-
-// Set highlight color
-void FXSlider::setHiliteColor(FXColor clr){
-  hiliteColor=clr;
-  update();
-  }
-
-
-// Set shadow color
-void FXSlider::setShadowColor(FXColor clr){
-  shadowColor=clr;
-  update();
-  }
-
-
-// Set border color
-void FXSlider::setBorderColor(FXColor clr){
-  borderColor=clr;
-  update();
-  }
-
-
-// Start automatic increment
-void FXSlider::autoSlideInc(){
-  if(timer) getApp()->removeTimeout(timer);
-  timer=getApp()->addTimeout(getApp()->scrollSpeed,this,ID_AUTOINC);
-  }
-
-
-// Start automatic decrement
-void FXSlider::autoSlideDec(){
-  if(timer) getApp()->removeTimeout(timer);
-  timer=getApp()->addTimeout(getApp()->scrollSpeed,this,ID_AUTODEC);
-  }
-
-
-
-// Stop it
-void FXSlider::stopAutoSlide(){
-  if(timer) getApp()->removeTimeout(timer);
-  timer=NULL;
-  }
-
-
-// True if automatic scrolling
-FXbool FXSlider::isAutoSliding(){
-  return timer!=NULL;
+  if(slotColor!=clr){
+    slotColor=clr;
+    update();
+    }
   }
 
 
@@ -869,12 +823,11 @@ void FXSlider::setTipText(const FXString& text){
 void FXSlider::save(FXStream& store) const {
   FXFrame::save(store);
   store << range[0] << range[1];
+  store << pos;
+  store << incr;
   store << slotColor;
-  store << headpos;
   store << headsize;
   store << slotsize;
-  store << incr;
-  store << pos;
   store << help;
   store << tip;
   }
@@ -884,12 +837,11 @@ void FXSlider::save(FXStream& store) const {
 void FXSlider::load(FXStream& store){
   FXFrame::load(store);
   store >> range[0] >> range[1];
+  store >> pos;
+  store >> incr;
   store >> slotColor;
-  store >> headpos;
   store >> headsize;
   store >> slotsize;
-  store >> incr;
-  store >> pos;
   store >> help;
   store >> tip;
   }
@@ -897,7 +849,7 @@ void FXSlider::load(FXStream& store){
 
 // Delete
 FXSlider::~FXSlider(){
-  stopAutoSlide();
+  if(timer){getApp()->removeTimeout(timer);}
   timer=(FXTimer*)-1;
   }
 
