@@ -3,31 +3,36 @@
 *                S p l i t t e r   W i n d o w   O b j e c t                    *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXSplitter.cpp,v 1.8 1999/11/13 06:15:15 jeroen Exp $                    *
+* $Id: FXSplitter.cpp,v 1.24 2001/08/14 21:56:16 jeroen Exp $                   *
 ********************************************************************************/
+#include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
-#include "FXAccelTable.h"
 #include "FXApp.h"
 #include "FXId.h"
 #include "FXDC.h"
@@ -43,9 +48,22 @@
   Notes:
   - Reversed split option starts parting out from right [bottom].
   - Minimal partition of a split is 0 pixels.
-  - Minimum width/height determined by number of visible children & barsize.
+  - Minimum width of horizontal splitter is determined by sum of
+    all visible children's default widths, and height by maximum
+    of all visible children's default heights; analogous for vertical
+    splitter of course.
+    [This can be done because the default width (height) does not depend
+    on the widget's current user-specified actual width (height)].
   - For convenience, width/height of <=1 replaced by minimum width/height,
     but only if both width and height of child is <=1 at the same time.
+  - Should we send SEL_CHANGED and SEL_COMMAND also when splitter arrangement
+    was changed programmatically?  When bar size changed?
+  - FXSplitter interprets layout hints on some of the children so that certain 
+    children have a minimum width (height) constraint on them.
+    Note that this rule must have one exception: the last (or first if the option
+    SPLITTER_REVERSED has been passed) must necessarily be allowed to be
+    any size.
+
 */
 
 // Splitter styles
@@ -71,7 +89,7 @@ FXDEFMAP(FXSplitter) FXSplitterMap[]={
 // Object implementation
 FXIMPLEMENT(FXSplitter,FXComposite,FXSplitterMap,ARRAYNUMBER(FXSplitterMap))
 
-  
+
 // Make a splitter
 FXSplitter::FXSplitter(){
   flags|=FLAG_ENABLED|FLAG_SHOWN;
@@ -86,7 +104,7 @@ FXSplitter::FXSplitter(){
 FXSplitter::FXSplitter(FXComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXint h):
   FXComposite(p,opts,x,y,w,h){
   flags|=FLAG_ENABLED|FLAG_SHOWN;
-  defaultCursor=(options&SPLITTER_VERTICAL) ? getApp()->vsplitCursor : getApp()->hsplitCursor;
+  defaultCursor=(options&SPLITTER_VERTICAL) ? getApp()->getDefaultCursor(DEF_VSPLIT_CURSOR) : getApp()->getDefaultCursor(DEF_HSPLIT_CURSOR);
   dragCursor=defaultCursor;
   window=NULL;
   split=0;
@@ -95,56 +113,91 @@ FXSplitter::FXSplitter(FXComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXint
   }
 
 
+// Make a splitter; it has no interior padding, and no borders
+FXSplitter::FXSplitter(FXComposite* p,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h):
+  FXComposite(p,opts,x,y,w,h){
+  flags|=FLAG_ENABLED|FLAG_SHOWN;
+  defaultCursor=(options&SPLITTER_VERTICAL) ? getApp()->getDefaultCursor(DEF_VSPLIT_CURSOR) : getApp()->getDefaultCursor(DEF_HSPLIT_CURSOR);
+  dragCursor=defaultCursor;
+  target=tgt;
+  message=sel;
+  window=NULL;
+  split=0;
+  offset=0;
+  barsize=4;
+  }
+
+
 // Get default width
 FXint FXSplitter::getDefaultWidth(){
-  FXWindow* child;
-  FXint numc,w;
+  register FXWindow* child;
+  register FXint wmax,w,numc;
+  wmax=numc=0;
   if(options&SPLITTER_VERTICAL){
-    w=1;
+    for(child=getFirst(); child; child=child->getNext()){
+      if(child->shown()){ 
+        w=child->getDefaultWidth();
+        if(wmax<w) wmax=w; 
+        }
+      }
     }
   else{
-    numc=w=0;
     for(child=getFirst(); child; child=child->getNext()){
-      if(child->shown()){ w+=1; numc++; }
+      if(child->shown()){ 
+        wmax+=child->getDefaultWidth(); 
+        numc++; 
+        }
       }
-    if(numc>1) w+=(numc-1)*barsize;
+    if(numc>1) wmax+=(numc-1)*barsize;
     }
-  return w;
+  return wmax;
   }
-  
+
 
 // Get default height
 FXint FXSplitter::getDefaultHeight(){
-  FXWindow* child;
-  FXint numc,h;
+  register FXWindow* child;
+  register FXint hmax,h,numc;
+  hmax=numc=0;
   if(options&SPLITTER_VERTICAL){
-    numc=h=0;
     for(child=getFirst(); child; child=child->getNext()){
-      if(child->shown()){ h+=1; numc++; }
+      if(child->shown()){ 
+        hmax+=child->getDefaultHeight(); 
+        numc++; 
+        }
       }
-    if(numc>1) h+=(numc-1)*barsize;
+    if(numc>1) hmax+=(numc-1)*barsize;
     }
   else{
-    h=1;
+    for(child=getFirst(); child; child=child->getNext()){
+      if(child->shown()){ 
+        h=child->getDefaultHeight();
+        if(hmax<h) hmax=h; 
+        }
+      }
     }
-  return h;
+  return hmax;
   }
 
 
 // Recompute layout
 void FXSplitter::layout(){
   FXint pos,w,h;
-  FXWindow* child;
+  FXWindow *child,*stretcher;
   if(options&SPLITTER_VERTICAL){          // Vertical
     if(options&SPLITTER_REVERSED){
       pos=height;
       child=getLast();
+      stretcher=getFirst();
+      while(stretcher && !stretcher->shown()){
+        stretcher=stretcher->getNext();
+        }
       while(child){
         if(child->shown()){
           w=child->getWidth();
           h=child->getHeight();
           if(w<=1 && h<=1) h=child->getDefaultHeight();
-          if(child==getFirst()){ h=pos; if(h<0) h=0; }
+          if(child==stretcher){ h=pos; if(h<0) h=0; }
           child->position(0,pos-h,width,h);
           pos=pos-h-barsize;
           }
@@ -154,12 +207,16 @@ void FXSplitter::layout(){
     else{
       pos=0;
       child=getFirst();
+      stretcher=getLast();
+      while(stretcher && !stretcher->shown()){
+        stretcher=stretcher->getPrev();
+        }
       while(child){
         if(child->shown()){
           w=child->getWidth();
           h=child->getHeight();
           if(w<=1 && h<=1) h=child->getDefaultHeight();
-          if(child==getLast()){ h=height-pos; if(h<0) h=0; }
+          if(child==stretcher){ h=height-pos; if(h<0) h=0; }
           child->position(0,pos,width,h);
           pos=pos+h+barsize;
           }
@@ -171,12 +228,16 @@ void FXSplitter::layout(){
     if(options&SPLITTER_REVERSED){
       pos=width;
       child=getLast();
+      stretcher=getFirst();
+      while(stretcher && !stretcher->shown()){
+        stretcher=stretcher->getNext();
+        }
       while(child){
         if(child->shown()){
           w=child->getWidth();
           h=child->getHeight();
           if(w<=1 && h<=1) w=child->getDefaultWidth();
-          if(child==getFirst()){ w=pos; if(w<0) w=0; }
+          if(child==stretcher){ w=pos; if(w<0) w=0; }
           child->position(pos-w,0,w,height);
           pos=pos-w-barsize;
           }
@@ -186,12 +247,16 @@ void FXSplitter::layout(){
     else{
       pos=0;
       child=getFirst();
+      stretcher=getLast();
+      while(stretcher && !stretcher->shown()){
+        stretcher=stretcher->getPrev();
+        }
       while(child){
         if(child->shown()){
           w=child->getWidth();
           h=child->getHeight();
           if(w<=1 && h<=1) w=child->getDefaultWidth();
-          if(child==getLast()){ w=width-pos; if(w<0) w=0; }
+          if(child==stretcher){ w=width-pos; if(w<0) w=0; }
           child->position(pos,0,w,height);
           pos=pos+w+barsize;
           }
@@ -205,23 +270,21 @@ void FXSplitter::layout(){
 
 // Adjust horizontal layout
 void FXSplitter::adjustHLayout(){
-  FXWindow *child;
+  FXWindow *child,*stretcher;
   FXint w,h,pos;
+  FXASSERT(window);
   if(options&SPLITTER_REVERSED){
     pos=window->getX()+window->getWidth();
     window->position(split,0,pos-split,height);
-//     window->setX(split);
-//     window->setWidth(pos-split);
     pos=split-barsize;
+    for(stretcher=getFirst(); stretcher && !stretcher->shown(); stretcher=stretcher->getNext());
     for(child=window->getPrev(); child; child=child->getPrev()){
       if(child->shown()){
         w=child->getWidth();
         h=child->getHeight();
         if(w<=1 && h<=1) w=child->getDefaultWidth();
-        if(child==getFirst()){ w=pos; if(w<0) w=0; }
+        if(child==stretcher){ w=pos; if(w<0) w=0; }
         child->position(pos-w,0,w,height);
-//         child->setX(pos-w);
-//         child->setWidth(w);
         pos=pos-w-barsize;
         }
       }
@@ -229,18 +292,15 @@ void FXSplitter::adjustHLayout(){
   else{
     pos=window->getX();
     window->position(pos,0,split-pos,height);
-//     window->setX(pos);
-//     window->setWidth(split-pos);
     pos=split+barsize;
+    for(stretcher=getLast(); stretcher && !stretcher->shown(); stretcher=stretcher->getPrev());
     for(child=window->getNext(); child; child=child->getNext()){
       if(child->shown()){
         w=child->getWidth();
         h=child->getHeight();
         if(w<=1 && h<=1) w=child->getDefaultWidth();
-        if(child==getLast()){ w=width-pos; if(w<0) w=0; }
+        if(child==stretcher){ w=width-pos; if(w<0) w=0; }
         child->position(pos,0,w,height);
-//         child->setX(pos);
-//         child->setWidth(w);
         pos=pos+w+barsize;
         }
       }
@@ -250,23 +310,21 @@ void FXSplitter::adjustHLayout(){
 
 // Adjust vertical layout
 void FXSplitter::adjustVLayout(){
-  FXWindow *child;
+  FXWindow *child,*stretcher;
   FXint w,h,pos;
+  FXASSERT(window);
   if(options&SPLITTER_REVERSED){
     pos=window->getY()+window->getHeight();
     window->position(0,split,width,pos-split);
-//     window->setY(split);
-//     window->setHeight(pos-split);
     pos=split-barsize;
+    for(stretcher=getFirst(); stretcher && !stretcher->shown(); stretcher=stretcher->getNext());
     for(child=window->getPrev(); child; child=child->getPrev()){
       if(child->shown()){
         w=child->getWidth();
         h=child->getHeight();
         if(w<=1 && h<=1) h=child->getDefaultHeight();
-        if(child==getFirst()){ h=pos; if(h<0) h=0; }
+        if(child==stretcher){ h=pos; if(h<0) h=0; }
         child->position(0,pos-h,width,h);
-//         child->setY(pos-h);
-//         child->setHeight(h);
         pos=pos-h-barsize;
         }
       }
@@ -274,18 +332,15 @@ void FXSplitter::adjustVLayout(){
   else{
     pos=window->getY();
     window->position(0,pos,width,split-pos);
-//     window->setY(pos);
-//     window->setHeight(split-pos);
     pos=split+barsize;
+    for(stretcher=getLast(); stretcher && !stretcher->shown(); stretcher=stretcher->getPrev());
     for(child=window->getNext(); child; child=child->getNext()){
       if(child->shown()){
         w=child->getWidth();
         h=child->getHeight();
         if(w<=1 && h<=1) h=child->getDefaultHeight();
-        if(child==getLast()){ h=height-pos; if(h<0) h=0; }
+        if(child==stretcher){ h=height-pos; if(h<0) h=0; }
         child->position(0,pos,width,h);
-//         child->setY(pos);
-//         child->setHeight(h);
         pos=pos+h+barsize;
         }
       }
@@ -295,9 +350,8 @@ void FXSplitter::adjustVLayout(){
 
 // Find child just before split
 FXWindow* FXSplitter::findHSplit(FXint pos){
-  FXWindow* child;
+  register FXWindow* child=getFirst();
   if(options&SPLITTER_REVERSED){
-    child=getFirst();
     while(child){
       if(child->shown()){
         if(child->getX()-barsize<=pos && pos<child->getX()) return child;
@@ -306,7 +360,6 @@ FXWindow* FXSplitter::findHSplit(FXint pos){
       }
     }
   else{
-    child=getFirst();
     while(child){
       if(child->shown()){
         if(child->getX()+child->getWidth()<=pos && pos<child->getX()+child->getWidth()+barsize) return child;
@@ -320,9 +373,8 @@ FXWindow* FXSplitter::findHSplit(FXint pos){
 
 // Find child just before split
 FXWindow* FXSplitter::findVSplit(FXint pos){
-  FXWindow* child;
+  register FXWindow* child=getFirst();
   if(options&SPLITTER_REVERSED){
-    child=getFirst();
     while(child){
       if(child->shown()){
         if(child->getY()-barsize<=pos && pos<child->getY()) return child;
@@ -331,7 +383,6 @@ FXWindow* FXSplitter::findVSplit(FXint pos){
       }
     }
   else{
-    child=getFirst();
     while(child){
       if(child->shown()){
         if(child->getY()+child->getHeight()<=pos && pos<child->getY()+child->getHeight()+barsize) return child;
@@ -345,15 +396,19 @@ FXWindow* FXSplitter::findVSplit(FXint pos){
 
 // Move the horizontal split intelligently
 void FXSplitter::moveHSplit(FXint pos){
-  FXint smin,smax;
+  register FXint smin,smax;
+  register FXuint hints;
   FXASSERT(window);
+  hints=window->getLayoutHints();
   if(options&SPLITTER_REVERSED){
-    smin=0;
+    smin=barsize;
     smax=window->getX()+window->getWidth();
+    if((hints&LAYOUT_FILL_X)&&(hints&LAYOUT_FIX_WIDTH)) smax-=window->getDefaultWidth();
     }
   else{
     smin=window->getX();
     smax=width-barsize;
+    if((hints&LAYOUT_FILL_X)&&(hints&LAYOUT_FIX_WIDTH)) smin+=window->getDefaultWidth();
     }
   split=pos;
   if(split<smin) split=smin;
@@ -363,15 +418,19 @@ void FXSplitter::moveHSplit(FXint pos){
 
 // Move the vertical split intelligently
 void FXSplitter::moveVSplit(FXint pos){
-  FXint smin,smax;
+  register FXint smin,smax;
+  register FXuint hints;
   FXASSERT(window);
+  hints=window->getLayoutHints();
   if(options&SPLITTER_REVERSED){
-    smin=0;
+    smin=barsize;
     smax=window->getY()+window->getHeight();
+    if((hints&LAYOUT_FILL_Y)&&(hints&LAYOUT_FIX_HEIGHT)) smax-=window->getDefaultHeight();
     }
   else{
     smin=window->getY();
-    smax=width-barsize;
+    smax=height-barsize;
+    if((hints&LAYOUT_FILL_Y)&&(hints&LAYOUT_FIX_HEIGHT)) smin+=window->getDefaultHeight();
     }
   split=pos;
   if(split<smin) split=smin;
@@ -419,7 +478,7 @@ long FXSplitter::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
     }
   return 0;
   }
-  
+
 
 // Button being released
 long FXSplitter::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
@@ -627,7 +686,7 @@ void FXSplitter::setSplitterStyle(FXuint style){
           child->setHeight(0);
           }
         }
-      setDefaultCursor((opts&SPLITTER_VERTICAL) ? getApp()->vsplitCursor : getApp()->hsplitCursor);
+      setDefaultCursor((opts&SPLITTER_VERTICAL) ? getApp()->getDefaultCursor(DEF_VSPLIT_CURSOR) : getApp()->getDefaultCursor(DEF_HSPLIT_CURSOR));
       setDragCursor(getDefaultCursor());
       recalc();
       }
@@ -667,7 +726,7 @@ void FXSplitter::save(FXStream& store) const {
 void FXSplitter::load(FXStream& store){
   FXComposite::load(store);
   store >> barsize;
-  }  
+  }
 
 
 // Zap it
