@@ -3,27 +3,29 @@
 *              P e r s i s t e n t   S t o r a g e   S t r e a m                *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXStream.cpp,v 1.3 1999/10/01 05:18:54 jeroen Exp $                      *
+* $Id: FXStream.cpp,v 1.16 2001/07/23 19:54:42 jeroen Exp $                     *
 ********************************************************************************/
 #include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
+#include "FXString.h"
 #include "FXObject.h"
 
 
@@ -33,8 +35,11 @@
   - Need some sort of seek() or position() API's.
   - Programming errors are punished by calling fxerror(); for example, saving
     into a stream which is set for loading, NULL buffer pointers, and so on.
-  - Status codes are set when a correct program encounters errors such as 
+  - Status codes are set when a correct program encounters errors such as
     writing to full disk, running out of memory, and so on.
+  - Single character insert/extract operators are virtual so subclasses can
+    overload these specific cases for greater speed.
+  - Need to redesign a bit some day for greater i/o speed (buffering).
 */
 
 
@@ -85,8 +90,8 @@ void FXStream::loadItems(void*,FXuint){ }
 
 
 // Set status code, unless there was a previous code already
-void FXStream::setError(FXStreamStatus err){ 
-  if(code==FXStreamOK) code=err; 
+void FXStream::setError(FXStreamStatus err){
+  if(code==FXStreamOK) code=err;
   }
 
 
@@ -103,7 +108,7 @@ static inline void swap4(void *p){
   t=((FXuchar*)p)[0]; ((FXuchar*)p)[0]=((FXuchar*)p)[3]; ((FXuchar*)p)[3]=t;
   t=((FXuchar*)p)[1]; ((FXuchar*)p)[1]=((FXuchar*)p)[2]; ((FXuchar*)p)[2]=t;
   }
-  
+
 
 // Swap octuplets
 static inline void swap8(void *p){
@@ -198,42 +203,42 @@ FXStream& FXStream::save(const FXchar* p,FXuint n){
 FXStream& FXStream::save(const FXushort* p,FXuint n){
   FXASSERT(n==0 || (n>0 && p!=NULL));
   saveItems(p,n<<1);
-  pos+=(n<<1);
+  pos+=(unsigned long)(n<<1);
   return *this;
   }
 
 FXStream& FXStream::save(const FXshort* p,FXuint n){
   FXASSERT(n==0 || (n>0 && p!=NULL));
   saveItems(p,n<<1);
-  pos+=(n<<1);
+  pos+=(unsigned long)(n<<1);
   return *this;
   }
 
 FXStream& FXStream::save(const FXuint* p,FXuint n){
   FXASSERT(n==0 || (n>0 && p!=NULL));
   saveItems(p,n<<2);
-  pos+=(n<<2);
+  pos+=(unsigned long)(n<<2);
   return *this;
   }
 
 FXStream& FXStream::save(const FXint* p,FXuint n){
   FXASSERT(n==0 || (n>0 && p!=NULL));
   saveItems(p,n<<2);
-  pos+=(n<<2);
+  pos+=(unsigned long)(n<<2);
   return *this;
   }
 
 FXStream& FXStream::save(const FXfloat* p,FXuint n){
   FXASSERT(n==0 || (n>0 && p!=NULL));
   saveItems(p,n<<2);
-  pos+=(n<<2);
+  pos+=(unsigned long)(n<<2);
   return *this;
   }
 
 FXStream& FXStream::save(const FXdouble* p,FXuint n){
   FXASSERT(n==0 || (n>0 && p!=NULL));
   saveItems(p,n<<3);
-  pos+=(n<<3);
+  pos+=(unsigned long)(n<<3);
   return *this;
   }
 
@@ -241,14 +246,14 @@ FXStream& FXStream::save(const FXdouble* p,FXuint n){
 FXStream& FXStream::save(const FXulong* p,FXuint n){
   FXASSERT(n==0 || (n>0 && p!=NULL));
   saveItems(p,n<<3);
-  pos+=(n<<3);
+  pos+=(unsigned long)(n<<3);
   return *this;
   }
 
 FXStream& FXStream::save(const FXlong* p,FXuint n){
   FXASSERT(n==0 || (n>0 && p!=NULL));
   saveItems(p,n<<3);
-  pos+=(n<<3);
+  pos+=(unsigned long)(n<<3);
   return *this;
   }
 #endif
@@ -355,7 +360,7 @@ FXStream& FXStream::load(FXushort* p,FXuint n){
 FXStream& FXStream::load(FXshort* p,FXuint n){
   FXASSERT(n==0 || (n>0 && p!=NULL));
   loadItems(p,n<<1);
-  pos+=(n<<1);
+  pos+=(unsigned long)(n<<1);
   if(swap&&n){do{swap2(p++);}while(--n);}
   return *this;
   }
@@ -363,7 +368,7 @@ FXStream& FXStream::load(FXshort* p,FXuint n){
 FXStream& FXStream::load(FXuint* p,FXuint n){
   FXASSERT(n==0 || (n>0 && p!=NULL));
   loadItems(p,n<<2);
-  pos+=(n<<2);
+  pos+=(unsigned long)(n<<2);
   if(swap&&n){do{swap4(p++);}while(--n);}
   return *this;
   }
@@ -371,7 +376,7 @@ FXStream& FXStream::load(FXuint* p,FXuint n){
 FXStream& FXStream::load(FXint* p,FXuint n){
   FXASSERT(n==0 || (n>0 && p!=NULL));
   loadItems(p,n<<2);
-  pos+=(n<<2);
+  pos+=(unsigned long)(n<<2);
   if(swap&&n){do{swap4(p++);}while(--n);}
   return *this;
   }
@@ -379,7 +384,7 @@ FXStream& FXStream::load(FXint* p,FXuint n){
 FXStream& FXStream::load(FXfloat* p,FXuint n){
   FXASSERT(n==0 || (n>0 && p!=NULL));
   loadItems(p,n<<2);
-  pos+=(n<<2);
+  pos+=(unsigned long)(n<<2);
   if(swap&&n){do{swap4(p++);}while(--n);}
   return *this;
   }
@@ -387,7 +392,7 @@ FXStream& FXStream::load(FXfloat* p,FXuint n){
 FXStream& FXStream::load(FXdouble* p,FXuint n){
   FXASSERT(n==0 || (n>0 && p!=NULL));
   loadItems(p,n<<3);
-  pos+=(n<<3);
+  pos+=(unsigned long)(n<<3);
   if(swap&&n){do{swap8(p++);}while(--n);}
   return *this;
   }
@@ -396,7 +401,7 @@ FXStream& FXStream::load(FXdouble* p,FXuint n){
 FXStream& FXStream::load(FXulong* p,FXuint n){
   FXASSERT(n==0 || (n>0 && p!=NULL));
   loadItems(p,n<<3);
-  pos+=(n<<3);
+  pos+=(unsigned long)(n<<3);
   if(swap&&n){do{swap8(p++);}while(--n);}
   return *this;
   }
@@ -404,7 +409,7 @@ FXStream& FXStream::load(FXulong* p,FXuint n){
 FXStream& FXStream::load(FXlong* p,FXuint n){
   FXASSERT(n==0 || (n>0 && p!=NULL));
   loadItems(p,n<<3);
-  pos+=(n<<3);
+  pos+=(unsigned long)(n<<3);
   if(swap&&n){do{swap8(p++);}while(--n);}
   return *this;
   }
@@ -455,8 +460,8 @@ FXbool FXStream::close(){
   no=0;
   return code==FXStreamOK;
   }
-  
-  
+
+
 
 // Grow hash table
 void FXStream::grow(){
@@ -466,9 +471,9 @@ void FXStream::grow(){
   FXMALLOC(&h,FXStreamHashEntry,n);
   if(!h){code=FXStreamAlloc;return;}
   switch(dir){
-  
+
     // Rehash table when FXStreamSave
-    case FXStreamSave:	
+    case FXStreamSave:
       for(i=0; i<n; i++) h[i].ref=UNUSEDSLOT;
       for(i=0; i<ntable; i++){
         if(table[i].ref==UNUSEDSLOT) continue;
@@ -481,33 +486,33 @@ void FXStream::grow(){
         h[p].obj=table[i].obj;
         }
       break;
-      
+
     // Simply copy over when FXStreamLoad
-    case FXStreamLoad:	
+    case FXStreamLoad:
       for(i=0; i<ntable; i++){
         h[i].ref=table[i].ref;
         h[i].obj=table[i].obj;
         }
       break;
-      
+
     // Nothing for FXStreamDead
     case FXStreamDead:
       break;
       }
-  
+
   // Ditch old table
   FXFREE(&table);
   table=h;
   ntable=n;
   }
-        
 
-// Save object 
+
+// Save object
 FXStream& FXStream::saveObject(const FXObject* v){
   const FXMetaClass *cls;
   register FXuint p,x;
   FXuint tag,esc=0;
-  if(dir!=FXStreamSave){ fxerror("FXStream::saveObject: wrong stream mode.\n"); }
+  if(dir!=FXStreamSave){ fxerror("FXStream::saveObject: wrong stream direction.\n"); }
   if(code==FXStreamOK){
     if(v==NULL){
       tag=0;
@@ -552,7 +557,7 @@ FXStream& FXStream::loadObject(FXObject*& v){
   const FXMetaClass *cls;
   FXchar obnam[MAXCLASSNAME];
   FXuint tag,esc;
-  if(dir!=FXStreamLoad){ fxerror("FXStream::loadObject: wrong stream mode.\n"); }
+  if(dir!=FXStreamLoad){ fxerror("FXStream::loadObject: wrong stream direction.\n"); }
   if(code==FXStreamOK){
     *this >> tag;
     if(tag==0){
@@ -592,7 +597,8 @@ FXStream& FXStream::loadObject(FXObject*& v){
       }
     FXASSERT(no<ntable);
     table[no].obj=v;                          // Save pointer in table
-    table[no].ref=no++;                       // Save reference number also!
+    table[no].ref=no;                         // Save reference number also!
+    no++;
     if(no>=ntable) grow();
     FXTRACE((100,"loadObject(%s)\n",v->getClassName()));
     v->load(*this);
@@ -625,7 +631,7 @@ FXFileStream::FXFileStream(const FXObject* cont):FXStream(cont){
 void FXFileStream::saveItems(const void *buf,FXuint n){
   if(dir!=FXStreamSave){ fxerror("FXFileStream::saveItems: wrong stream direction.\n"); }
   if(code==FXStreamOK){
-    if(n){if(fwrite(buf,1,n,(FILE*)file)!=n){code=FXStreamFull;}}
+    if(fwrite(buf,1,n,(FILE*)file)!=n){code=FXStreamFull;}
     }
   }
 
@@ -634,36 +640,33 @@ void FXFileStream::saveItems(const void *buf,FXuint n){
 void FXFileStream::loadItems(void *buf,FXuint n){
   if(dir!=FXStreamLoad){ fxerror("FXFileStream::loadItems: wrong stream direction.\n"); }
   if(code==FXStreamOK){
-    if(n){if(fread(buf,1,n,(FILE*)file)!=n){code=FXStreamEnd;}}
+    if(fread(buf,1,n,(FILE*)file)!=n){code=FXStreamEnd;}
     }
   }
 
 
 // Try open file stream
-FXbool FXFileStream::open(const char* filename,FXStreamDirection save_or_load){
-  
-  // Check for NULL filename
-  if(!filename){ fxerror("FXFileStream::open: NULL filename argument.\n"); }
+FXbool FXFileStream::open(const FXString& filename,FXStreamDirection save_or_load){
 
   // Stream should not yet be open
   if(dir!=FXStreamDead){ fxerror("FXFileStream::open: stream is already open.\n"); }
 
   // Open for read
   if(save_or_load==FXStreamLoad){
-    file=fopen(filename,"rb");
+    file=fopen(filename.text(),"rb");
     if(file==NULL){code=FXStreamNoRead;return FALSE;}
     }
- 
+
   // Open for write
   else{
-    file=fopen(filename,"wb");
+    file=fopen(filename.text(),"wb");
     if(file==NULL){code=FXStreamNoWrite;return FALSE;}
     }
 
   // Do the generic book-keeping
   return FXStream::open(save_or_load);
   }
-  
+
 
 // Close file stream
 FXbool FXFileStream::close(){
@@ -682,7 +685,7 @@ FXbool FXFileStream::close(){
 FXbool FXFileStream::position(unsigned long p){
   if(dir==FXStreamDead){ fxerror("FXFileStream::position: stream is not open.\n"); }
   if(code==FXStreamOK){
-    
+
     // Seek file from beginning
     if(dir==FXStreamSave){
       if(fseek((FILE*)file,p,SEEK_SET)!=0){ code=FXStreamFull; return FALSE; }
@@ -690,7 +693,7 @@ FXbool FXFileStream::position(unsigned long p){
     else{
       if(fseek((FILE*)file,p,SEEK_SET)!=0){ code=FXStreamEnd; return FALSE; }
       }
-    
+
     // Read back to make sure we're in sync
     pos=ftell((FILE*)file);
     return TRUE;
@@ -699,7 +702,49 @@ FXbool FXFileStream::position(unsigned long p){
   }
 
 
-// Close file stream 
+FXStream& FXFileStream::operator<<(const FXuchar& v){
+  if(code==FXStreamOK){
+    register FXint c=putc(v,(FILE*)file);
+    if(c==EOF){code=FXStreamFull;}
+    }
+  pos+=1;
+  return *this;
+  }
+
+
+FXStream& FXFileStream::operator<<(const FXchar& v){
+  if(code==FXStreamOK){
+    register FXint c=putc(v,(FILE*)file);
+    if(c==EOF){code=FXStreamFull;}
+    }
+  pos+=1;
+  return *this;
+  }
+
+
+FXStream& FXFileStream::operator>>(FXuchar& v){
+  if(code==FXStreamOK){
+    register FXint c=getc((FILE*)file);
+    if(c==EOF){code=FXStreamEnd;}
+    v=c;
+    }
+  pos+=1;
+  return *this;
+  }
+
+
+FXStream& FXFileStream::operator>>(FXchar& v){
+  if(code==FXStreamOK){
+    register FXint c=getc((FILE*)file);
+    if(c==EOF){code=FXStreamEnd;}
+    v=c;
+    }
+  pos+=1;
+  return *this;
+  }
+
+
+// Close file stream
 FXFileStream::~FXFileStream(){
   if(file){fclose((FILE*)file);}
   file=(void*)-1;
@@ -756,6 +801,53 @@ void FXMemoryStream::loadItems(void *buf,FXuint n){
     }
   }
 
+FXStream& FXMemoryStream::operator<<(const FXuchar& v){
+  if(code==FXStreamOK){
+    if(pos>=space){
+      if(!owns){code=FXStreamFull;return *this;}
+      setSpace(pos+1);
+      if(pos>=space) return *this;
+      }
+    ptr[pos]=v;
+    }
+  pos+=1;
+  return *this;
+  }
+
+
+FXStream& FXMemoryStream::operator<<(const FXchar& v){
+  if(code==FXStreamOK){
+    if(pos>=space){
+      if(!owns){code=FXStreamFull;return *this;}
+      setSpace(pos+1);
+      if(pos>=space) return *this;
+      }
+    ptr[pos]=(FXuchar)v;
+    }
+  pos+=1;
+  return *this;
+  }
+
+
+FXStream& FXMemoryStream::operator>>(FXuchar& v){
+  if(code==FXStreamOK){
+    if(pos>=space){code=FXStreamEnd;return *this;}
+    v=ptr[pos];
+    }
+  pos+=1;
+  return *this;
+  }
+
+
+FXStream& FXMemoryStream::operator>>(FXchar& v){
+  if(code==FXStreamOK){
+    if(pos>=space){code=FXStreamEnd;return *this;}
+    v=(FXchar)ptr[pos];
+    }
+  pos+=1;
+  return *this;
+  }
+
 
 // Open a stream, possibly with an initial data array
 FXbool FXMemoryStream::open(FXuchar* data,FXStreamDirection save_or_load){
@@ -782,6 +874,7 @@ FXbool FXMemoryStream::open(FXuchar* data,FXuint sp,FXStreamDirection save_or_lo
     owns=FALSE;
     }
   else{
+    if(sp==0) sp=1;
     FXCALLOC(&ptr,FXuchar,sp);
     if(!ptr){code=FXStreamAlloc;return FALSE;}
     space=sp;
@@ -809,7 +902,7 @@ void FXMemoryStream::giveBuffer(FXuchar *buffer,FXuint sp){
   space=sp;
   owns=TRUE;
   }
-  
+
 
 // Close the stream
 FXbool FXMemoryStream::close(){
@@ -821,7 +914,7 @@ FXbool FXMemoryStream::close(){
   }
 
 
-// Move to position; if saving and we own the buffer, try to resize 
+// Move to position; if saving and we own the buffer, try to resize
 // and 0-fill the space; if loading and not out of range, move the pointer;
 // otherwise, return error code.
 FXbool FXMemoryStream::position(unsigned long p){
