@@ -3,29 +3,36 @@
 *                               T a b   O b j e c t                             *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXTab.cpp,v 1.5 1999/10/29 07:58:39 jeroen Exp $                         *
+* $Id: FXTab.cpp,v 1.27 2001/05/03 19:17:11 jeroen Exp $                        *
 ********************************************************************************/
+#include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
+#include "fxkeys.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXApp.h"
@@ -48,13 +55,12 @@
   Notes:
   - Should focus go to tab items?
   - Should callbacks come from tab items?
-  - FXTabItems should catch SEL_ACTIVATE etc.
   - Should redesign this stuff a little.
   - Tab items should observe various border styles.
   - TAB/TABTAB should go into content, arrow keys navigate between tabs.
   - FXTabBook: pane's hints make no sense to observe
   - We hide the panes in FXTabBook.  This way, we don't have to change
-    the position of each pane when the FXTabBook itself changes. 
+    the position of each pane when the FXTabBook itself changes.
     Only the active pane needs to be moved.
 */
 
@@ -69,12 +75,13 @@ FXDEFMAP(FXTabItem) FXTabItemMap[]={
   FXMAPFUNC(SEL_PAINT,0,FXTabItem::onPaint),
   FXMAPFUNC(SEL_FOCUSIN,0,FXTabItem::onFocusIn),
   FXMAPFUNC(SEL_FOCUSOUT,0,FXTabItem::onFocusOut),
-  FXMAPFUNC(SEL_ACTIVATE,0,FXTabItem::onActivate),
-  FXMAPFUNC(SEL_DEACTIVATE,0,FXTabItem::onDeactivate),
+  FXMAPFUNC(SEL_UNGRABBED,0,FXTabItem::onUngrabbed),
+  FXMAPFUNC(SEL_LEFTBUTTONPRESS,0,FXTabItem::onLeftBtnPress),
+  FXMAPFUNC(SEL_LEFTBUTTONRELEASE,0,FXTabItem::onLeftBtnRelease),
+  FXMAPFUNC(SEL_KEYPRESS,0,FXTabItem::onKeyPress),
+  FXMAPFUNC(SEL_KEYRELEASE,0,FXTabItem::onKeyRelease),
   FXMAPFUNC(SEL_KEYPRESS,FXWindow::ID_HOTKEY,FXTabItem::onHotKeyPress),
   FXMAPFUNC(SEL_KEYRELEASE,FXWindow::ID_HOTKEY,FXTabItem::onHotKeyRelease),
-  FXMAPFUNC(SEL_UPDATE,FXWindow::ID_QUERY_TIP,FXTabItem::onQueryTip),
-  FXMAPFUNC(SEL_UPDATE,FXWindow::ID_QUERY_HELP,FXTabItem::onQueryHelp),
   };
 
 
@@ -85,31 +92,10 @@ FXIMPLEMENT(FXTabItem,FXLabel,FXTabItemMap,ARRAYNUMBER(FXTabItemMap))
 // Tab item
 FXTabItem::FXTabItem(FXTabBar* p,const FXString& text,FXIcon* ic,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb):
   FXLabel(p,text,ic,opts,x,y,w,h,pl,pr,pt,pb){
-  tip=text.extract(1,'\t');
-  help=text.extract(2,'\t');
-  flags|=FLAG_ENABLED;
   border=2;
   }
 
 
-// Enable the window
-void FXTabItem::enable(){
-  if(!(flags&FLAG_ENABLED)){
-    FXWindow::enable();
-    update();
-    }
-  }
-
-
-// Disable the window
-void FXTabItem::disable(){
-  if(flags&FLAG_ENABLED){
-    FXWindow::disable();
-    update();
-    }
-  }
-
-
 // If window can have focus
 FXbool FXTabItem::canFocus() const { return 1; }
 
@@ -121,7 +107,7 @@ long FXTabItem::onFocusIn(FXObject* sender,FXSelector sel,void* ptr){
   return 1;
   }
 
-  
+
 // Lost focus
 long FXTabItem::onFocusOut(FXObject* sender,FXSelector sel,void* ptr){
   FXLabel::onFocusOut(sender,sel,ptr);
@@ -130,13 +116,76 @@ long FXTabItem::onFocusOut(FXObject* sender,FXSelector sel,void* ptr){
   }
 
 
-// Hot key combination pressed
-long FXTabItem::onHotKeyPress(FXObject*,FXSelector,void* ptr){
-  FXTRACE((200,"%s::onHotKeyPress %08x\n",getClassName(),this));
+// Pressed mouse button
+long FXTabItem::onLeftBtnPress(FXObject* sender,FXSelector sel,void* ptr){
+  if(!FXLabel::onLeftBtnPress(sender,sel,ptr)){
+    if(isEnabled()){
+      getParent()->handle(this,MKUINT(FXTabBar::ID_OPEN_ITEM,SEL_COMMAND),ptr);
+      flags|=FLAG_PRESSED;
+      flags&=~FLAG_UPDATE;
+      return 1;
+      }
+    }
+  return 0;
+  }
+
+
+// Released mouse button
+long FXTabItem::onLeftBtnRelease(FXObject* sender,FXSelector sel,void* ptr){
+  if(!FXLabel::onLeftBtnRelease(sender,sel,ptr)){
+    if(isEnabled()){
+      flags|=FLAG_UPDATE;
+      flags&=~FLAG_PRESSED;
+      return 1;
+      }
+    }
+  return 0;
+  }
+
+
+// Lost the grab for some reason
+long FXTabItem::onUngrabbed(FXObject* sender,FXSelector sel,void* ptr){
+  FXLabel::onUngrabbed(sender,sel,ptr);
+  flags&=~FLAG_PRESSED;
+  flags|=FLAG_UPDATE;
+  return 1;
+  }
+
+
+// Key Press
+long FXTabItem::onKeyPress(FXObject*,FXSelector,void* ptr){
+  FXEvent* event=(FXEvent*)ptr;
   flags&=~FLAG_TIP;
   if(isEnabled()){
-    handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
-    handle(this,MKUINT(0,SEL_ACTIVATE),ptr);
+    if(target && target->handle(this,MKUINT(message,SEL_KEYPRESS),ptr)) return 1;
+    if(event->code==KEY_space || event->code==KEY_KP_Space){
+      getParent()->handle(this,MKUINT(FXTabBar::ID_OPEN_ITEM,SEL_COMMAND),ptr);
+      return 1;
+      }
+    }
+  return 0;
+  }
+
+
+// Key Release
+long FXTabItem::onKeyRelease(FXObject*,FXSelector,void* ptr){
+  FXEvent* event=(FXEvent*)ptr;
+  if(isEnabled()){
+    if(target && target->handle(this,MKUINT(message,SEL_KEYRELEASE),ptr)) return 1;
+    if(event->code==KEY_space || event->code==KEY_KP_Space){
+      return 1;
+      }
+    }
+  return 0;
+  }
+
+
+// Hot key combination pressed
+long FXTabItem::onHotKeyPress(FXObject*,FXSelector,void* ptr){
+  handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+  flags&=~FLAG_TIP;
+  if(isEnabled()){
+    getParent()->handle(this,MKUINT(FXTabBar::ID_OPEN_ITEM,SEL_COMMAND),ptr);
     }
   return 1;
   }
@@ -144,16 +193,11 @@ long FXTabItem::onHotKeyPress(FXObject*,FXSelector,void* ptr){
 
 // Hot key combination released
 long FXTabItem::onHotKeyRelease(FXObject*,FXSelector,void* ptr){
-  FXTRACE((200,"%s::onHotKeyRelease %08x\n",getClassName(),this));
-  flags&=~FLAG_TIP;
-  if(isEnabled()){ 
-    handle(this,MKUINT(0,SEL_DEACTIVATE),ptr); 
-    }
   return 1;
   }
 
 
-// Handle repaint 
+// Handle repaint
 long FXTabItem::onPaint(FXObject*,FXSelector,void* ptr){
   FXEvent *ev=(FXEvent*)ptr;
   FXDCWindow dc(this,ev);
@@ -188,19 +232,19 @@ long FXTabItem::onPaint(FXObject*,FXSelector,void* ptr){
       dc.drawLine(0,0,0,height-4);
       dc.drawLine(0,height-4,1,height-2);
       dc.setForeground(shadowColor);
-      dc.drawLine(2,height-2,width-3,height-2);
+      dc.fillRectangle(2,height-2,width-4,1);
       dc.drawLine(width-2,0,width-2,height-3);
-      dc.drawLine(width-2,0,width-1,0);
+      dc.fillRectangle(width-2,0,2,1);
       dc.setForeground(borderColor);
       dc.drawLine(3,height-1,width-4,height-1);
       dc.drawLine(width-4,height-1,width-1,height-4);
-      dc.drawLine(width-1,1,width-1,height-4);
+      dc.fillRectangle(width-1,1,1,height-4);
       break;
     case TAB_TOP:
       dc.setForeground(hiliteColor);
-      dc.drawLine(0,height-1,0,2);
+      dc.fillRectangle(0,2,1,height-2);
       dc.drawLine(0,2,2,0);
-      dc.drawLine(2,0,width-3,0);
+      dc.fillRectangle(2,0,width-4,1);
       dc.setForeground(shadowColor);
       dc.drawLine(width-2,1,width-2,height-1);
       dc.setForeground(borderColor);
@@ -224,7 +268,7 @@ long FXTabItem::onPaint(FXObject*,FXSelector,void* ptr){
     if(isEnabled())
       dc.drawIcon(icon,ix,iy);
     else
-      dc.drawIconShaded(icon,ix,iy);
+      dc.drawIconSunken(icon,ix,iy);
     }
   if(!label.empty()){
     dc.setTextFont(font);
@@ -246,51 +290,6 @@ long FXTabItem::onPaint(FXObject*,FXSelector,void* ptr){
   }
 
 
-// We were asked about status text
-long FXTabItem::onQueryHelp(FXObject* sender,FXSelector,void*){
-  if(!help.empty() && (flags&FLAG_HELP)){
-    sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&help);
-    return 1;
-    }
-  return 0;
-  }
-
-
-// We were asked about tip text
-long FXTabItem::onQueryTip(FXObject* sender,FXSelector,void*){
-  if(!tip.empty() && (flags&FLAG_TIP)){
-    sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&tip);
-    return 1;
-    }
-  return 0;
-  }
-
-
-// Pressed left button
-long FXTabItem::onActivate(FXObject*,FXSelector,void* ptr){
-  getParent()->handle(this,MKUINT(FXTabBar::ID_OPEN_ITEM,SEL_COMMAND),ptr);
-  return 1;
-  }
-
-
-// Released left button
-long FXTabItem::onDeactivate(FXObject*,FXSelector,void*){
-  return 1;
-  }
-
-
-// Change help text
-void FXTabItem::setHelpText(const FXString& text){
-  help=text;
-  }
-
-
-// Change tip text
-void FXTabItem::setTipText(const FXString& text){
-  tip=text;
-  }
-
-
 // Get tab style
 FXuint FXTabItem::getTabOrientation() const {
   return (options&TAB_ORIENT_MASK);
@@ -308,22 +307,6 @@ void FXTabItem::setTabOrientation(FXuint style){
   }
 
 
-// Save object to stream
-void FXTabItem::save(FXStream& store) const {
-  FXLabel::save(store);
-  store << tip;
-  store << help;
-  }
-
-
-// Load object from stream
-void FXTabItem::load(FXStream& store){
-  FXLabel::load(store);
-  store >> tip;
-  store >> help;
-  }  
-
-
 /*******************************************************************************/
 
 // Map
@@ -335,7 +318,7 @@ FXDEFMAP(FXTabBar) FXTabBarMap[]={
   FXMAPFUNC(SEL_FOCUS_DOWN,0,FXTabBar::onFocusDown),
   FXMAPFUNC(SEL_FOCUS_LEFT,0,FXTabBar::onFocusLeft),
   FXMAPFUNC(SEL_FOCUS_RIGHT,0,FXTabBar::onFocusRight),
-  FXMAPFUNC(SEL_COMMAND,FXTabBar::ID_OPEN_ITEM,FXTabBar::onOpenItem),
+  FXMAPFUNC(SEL_COMMAND,FXTabBar::ID_OPEN_ITEM,FXTabBar::onCmdOpenItem),
   FXMAPFUNC(SEL_COMMAND,FXWindow::ID_SETVALUE,FXTabBar::onCmdSetValue),
   FXMAPFUNC(SEL_COMMAND,FXWindow::ID_SETINTVALUE,FXTabBar::onCmdSetIntValue),
   FXMAPFUNC(SEL_COMMAND,FXWindow::ID_GETINTVALUE,FXTabBar::onCmdGetIntValue),
@@ -348,7 +331,7 @@ FXDEFMAP(FXTabBar) FXTabBarMap[]={
 FXIMPLEMENT(FXTabBar,FXPacker,FXTabBarMap,ARRAYNUMBER(FXTabBarMap))
 
 
-// Make a tab bar 
+// Make a tab bar
 FXTabBar::FXTabBar(FXComposite* p,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb):
   FXPacker(p,opts,x,y,w,h,pl,pr,pt,pb,0,0){
   flags|=FLAG_ENABLED;
@@ -432,7 +415,7 @@ void FXTabBar::layout(){
   register FXWindow *raisetab=NULL;
   register FXWindow *tab;
   register FXuint hints;
-  
+
   newcurrent=-1;
 
   // Measure tabs again
@@ -447,13 +430,13 @@ void FXTabBar::layout(){
       if(newcurrent<0 || i<=current) newcurrent=i;
       }
     }
-  
+
   // Changes current only if old current no longer visible
   current=newcurrent;
-  
+
   // Tabs on left or right
   if(options&TABBOOK_SIDEWAYS){
-    
+
     // Placements for tab items and tab panels
     y=border+padtop;
     if(options&TABBOOK_BOTTOMTABS){         // Right tabs
@@ -462,7 +445,7 @@ void FXTabBar::layout(){
     else{
       x=border+padleft;
       }
-    
+
     // Place all of the children
     for(i=0,tab=getFirst(); tab; tab=tab->getNext(),i++){
       if(tab->shown()){
@@ -489,10 +472,10 @@ void FXTabBar::layout(){
         }
       }
     }
-  
+
   // Tabs on top or bottom
   else{
-    
+
     // Placements for tab items and tab panels
     x=border+padleft;
     if(options&TABBOOK_BOTTOMTABS){         // Bottom tabs
@@ -501,7 +484,7 @@ void FXTabBar::layout(){
     else{
       y=border+padtop;
       }
-    
+
     // Place all of the children
     for(i=0,tab=getFirst(); tab; tab=tab->getNext(),i++){
       if(tab->shown()){
@@ -534,15 +517,16 @@ void FXTabBar::layout(){
 
 
 // Set current subwindow
-void FXTabBar::setCurrent(FXint panel){
+void FXTabBar::setCurrent(FXint panel,FXbool notify){
   if(0<=panel && panel!=current){
     current=panel;
+    if(notify && target){ target->handle(this,MKUINT(message,SEL_COMMAND),(void*)current); }
     recalc();
     }
   }
 
 
-// Handle repaint 
+// Handle repaint
 long FXTabBar::onPaint(FXObject*,FXSelector,void* ptr){
   FXEvent *ev=(FXEvent*)ptr;
   FXDCWindow dc(this,ev);
@@ -559,7 +543,7 @@ long FXTabBar::onFocusNext(FXObject*,FXSelector,void* ptr){
   if(child) child=child->getNext(); else child=getFirst();
   while(child && !child->shown()) child=child->getNext();
   if(child){
-    setCurrent(indexOfChild(child));
+    setCurrent(indexOfChild(child),TRUE);
     child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
     return 1;
     }
@@ -573,7 +557,7 @@ long FXTabBar::onFocusPrev(FXObject*,FXSelector,void* ptr){
   if(child) child=child->getPrev(); else child=getLast();
   while(child && !child->shown()) child=child->getPrev();
   if(child){
-    setCurrent(indexOfChild(child));
+    setCurrent(indexOfChild(child),TRUE);
     child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
     return 1;
     }
@@ -626,41 +610,36 @@ long FXTabBar::onCmdSetValue(FXObject*,FXSelector,void* ptr){
 
 // Update value from a message
 long FXTabBar::onCmdSetIntValue(FXObject*,FXSelector,void* ptr){
-  if(ptr==NULL){ fxerror("%s::onCmdSetIntValue: NULL pointer.\n",getClassName()); }
   setCurrent(*((FXint*)ptr));
   return 1;
   }
 
+
 // Obtain value from text field
 long FXTabBar::onCmdGetIntValue(FXObject*,FXSelector,void* ptr){
-  if(ptr==NULL){ fxerror("%s::onCmdGetIntValue: NULL pointer.\n",getClassName()); }
-  *((FXint*)ptr)=current;
+  *((FXint*)ptr)=getCurrent();
   return 1;
   }
 
 
 // Open item
 long FXTabBar::onCmdOpen(FXObject*,FXSelector sel,void*){
-  setCurrent(SELID(sel)-ID_OPEN_FIRST);
+  setCurrent(SELID(sel)-ID_OPEN_FIRST,TRUE);
   return 1;
   }
 
 
 // Update the nth button
 long FXTabBar::onUpdOpen(FXObject* sender,FXSelector sel,void* ptr){
-  FXuint msg=((SELID(sel)-ID_OPEN_FIRST)==(FXuint)current) ? ID_CHECK : ID_UNCHECK;
+  FXuint msg=((SELID(sel)-ID_OPEN_FIRST)==current) ? ID_CHECK : ID_UNCHECK;
   sender->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
   return 1;
   }
 
 
 // The sender of the message is the item to open up
-long FXTabBar::onOpenItem(FXObject* sender,FXSelector,void*){
-  FXint which=indexOfChild((FXWindow*)sender);
-  if(0<=which && which!=current){
-    setCurrent(which);
-    if(target) target->handle(this,MKUINT(message,SEL_COMMAND),(void*)which);
-    }
+long FXTabBar::onCmdOpenItem(FXObject* sender,FXSelector,void*){
+  setCurrent(indexOfChild((FXWindow*)sender),TRUE);
   return 1;
   }
 
@@ -693,7 +672,7 @@ void FXTabBar::save(FXStream& store) const {
 void FXTabBar::load(FXStream& store){
   FXPacker::load(store);
   store >> current;
-  }  
+  }
 
 
 /*******************************************************************************/
@@ -706,7 +685,7 @@ FXDEFMAP(FXTabBook) FXTabBookMap[]={
   FXMAPFUNC(SEL_FOCUS_DOWN,0,FXTabBook::onFocusDown),
   FXMAPFUNC(SEL_FOCUS_LEFT,0,FXTabBook::onFocusLeft),
   FXMAPFUNC(SEL_FOCUS_RIGHT,0,FXTabBook::onFocusRight),
-  FXMAPFUNC(SEL_COMMAND,FXTabBar::ID_OPEN_ITEM,FXTabBook::onOpenItem),
+  FXMAPFUNC(SEL_COMMAND,FXTabBar::ID_OPEN_ITEM,FXTabBook::onCmdOpenItem),
   };
 
 
@@ -725,7 +704,7 @@ FXint FXTabBook::getDefaultWidth(){
   register FXint w,wtabs,wmaxtab,wpnls,t,ntabs;
   register FXuint hints;
   register FXWindow *tab,*pane;
-  
+
   // Left or right tabs
   if(options&TABBOOK_SIDEWAYS){
     wtabs=wpnls=0;
@@ -741,7 +720,7 @@ FXint FXTabBook::getDefaultWidth(){
       }
     w=wtabs+wpnls;
     }
-  
+
   // Top or bottom tabs
   else{
     wtabs=wpnls=wmaxtab=ntabs=0;
@@ -769,7 +748,7 @@ FXint FXTabBook::getDefaultHeight(){
   register FXint h,htabs,hmaxtab,hpnls,t,ntabs;
   register FXuint hints;
   register FXWindow *tab,*pane;
-  
+
   // Left or right tabs
   if(options&TABBOOK_SIDEWAYS){
     htabs=hpnls=hmaxtab=ntabs=0;
@@ -788,7 +767,7 @@ FXint FXTabBook::getDefaultHeight(){
     if(options&PACK_UNIFORM_HEIGHT) htabs=ntabs*hmaxtab;
     h=FXMAX(htabs,hpnls)+5;
     }
-  
+
   // Top or bottom tabs
   else{
     htabs=hpnls=0;
@@ -815,7 +794,7 @@ void FXTabBook::layout(){
   register FXWindow *raisetab=NULL;
   register FXWindow *pane,*tab;
   register FXuint hints;
-  
+
   newcurrent=-1;
 
   // Measure tabs again
@@ -831,13 +810,13 @@ void FXTabBook::layout(){
       if(newcurrent<0 || i<=current) newcurrent=i;
       }
     }
-  
+
   // This will change only if current now invisible
   current=newcurrent;
-  
+
   // Left or right tabs
   if(options&TABBOOK_SIDEWAYS){
-    
+
     // Placements for tab items and tab panels
     y=border+padtop;
     py=y;
@@ -851,7 +830,7 @@ void FXTabBook::layout(){
       x=border+padleft;
       px=x+wmaxtab;
       }
-    
+
     // Place all of the children
     for(tab=getFirst(),i=0; tab && tab->getNext(); tab=tab->getNext()->getNext(),i++){
       pane=tab->getNext();
@@ -860,13 +839,13 @@ void FXTabBook::layout(){
         if(hints&LAYOUT_FIX_HEIGHT) h=tab->getHeight();
         else if(options&PACK_UNIFORM_HEIGHT) h=hmaxtab;
         else h=tab->getDefaultHeight();
+        pane->position(px,py,pw,ph);
         if(current==i){
           if(options&TABBOOK_BOTTOMTABS)      // Right tabs
             tab->position(x-2,y,wmaxtab+2,h+3);
           else
             tab->position(x,y,wmaxtab+2,h+3);
           tab->update(0,0,wmaxtab+2,h+3);
-          pane->position(px,py,pw,ph);
           pane->show();
           raisetab=tab;
           raisepane=pane;
@@ -885,14 +864,14 @@ void FXTabBook::layout(){
         pane->hide();
         }
       }
-    
+
     // Hide spurious last tab
     if(tab) tab->resize(0,0);
     }
-  
+
   // Top or bottom tabs
   else{
-    
+
     // Placements for tab items and tab panels
     x=border+padleft;
     px=x;
@@ -906,7 +885,7 @@ void FXTabBook::layout(){
       y=border+padtop;
       py=y+hmaxtab;
       }
-    
+
     // Place all of the children
     for(tab=getFirst(),i=0; tab && tab->getNext(); tab=tab->getNext()->getNext(),i++){
       pane=tab->getNext();
@@ -915,13 +894,13 @@ void FXTabBook::layout(){
         if(hints&LAYOUT_FIX_WIDTH) w=tab->getWidth();
         else if(options&PACK_UNIFORM_WIDTH) w=wmaxtab;
         else w=tab->getDefaultWidth();
+        pane->position(px,py,pw,ph);
         if(current==i){
           if(options&TABBOOK_BOTTOMTABS)      // Bottom tabs
             tab->position(x,y-2,w+3,hmaxtab+2);
           else
             tab->position(x,y,w+3,hmaxtab+2);
           tab->update(0,0,w+3,hmaxtab+2);
-          pane->position(px,py,pw,ph);
           pane->show();
           raisepane=pane;
           raisetab=tab;
@@ -944,27 +923,23 @@ void FXTabBook::layout(){
     // Hide spurious last tab
     if(tab) tab->resize(0,0);
     }
-    
+
   // Raise tab over panel and panel over all other tabs
   if(raisepane) raisepane->raise();
   if(raisetab) raisetab->raise();
-  
+
   flags&=~FLAG_DIRTY;
   }
 
 
 // The sender of the message is the item to open up
-long FXTabBook::onOpenItem(FXObject* sender,FXSelector,void*){
-  FXint which=indexOfChild((FXWindow*)sender)/2;
-  if(0<=which && which!=current){
-    setCurrent(which);
-    if(target) target->handle(this,MKUINT(message,SEL_COMMAND),(void*)which);
-    }
+long FXTabBook::onCmdOpenItem(FXObject* sender,FXSelector,void*){
+  setCurrent(indexOfChild((FXWindow*)sender)/2,TRUE);
   return 1;
   }
 
 
-// Handle repaint 
+// Handle repaint
 long FXTabBook::onPaint(FXObject*,FXSelector,void* ptr){
   FXEvent *ev=(FXEvent*)ptr;
   FXDCWindow dc(this,ev);
@@ -994,7 +969,7 @@ long FXTabBook::onFocusNext(FXObject*,FXSelector,void* ptr){
     }
   while(child && !child->shown()){ child=child->getNext(); which++; }
   if(child){
-    setCurrent(which>>1);
+    setCurrent(which>>1,TRUE);
     child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
     return 1;
     }
@@ -1029,7 +1004,7 @@ long FXTabBook::onFocusPrev(FXObject*,FXSelector,void* ptr){
     while(child && !child->shown()){ child=child->getPrev(); which--; }
     }
   if(child){
-    setCurrent(which>>1);
+    setCurrent(which>>1,TRUE);
     child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
     return 1;
     }
