@@ -3,30 +3,36 @@
 *                       T r e e  L i s t  B o x  O b j e c t                    *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1999 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1999,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXTreeListBox.cpp,v 1.2 1999/09/21 13:54:26 jeroen Exp $                *
+* $Id: FXTreeListBox.cpp,v 1.18 2001/06/06 16:49:56 jeroen Exp $                *
 ********************************************************************************/
+#include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "fxkeys.h"
 #include "FXStream.h"
 #include "FXString.h"
+#include "FXSize.h"
+#include "FXPoint.h"
+#include "FXRectangle.h"
 #include "FXObject.h"
 #include "FXDict.h"
+#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXObjectList.h"
@@ -46,7 +52,7 @@
 #include "FXShell.h"
 #include "FXPopup.h"
 #include "FXScrollbar.h"
-#include "FXScrollWindow.h"
+#include "FXScrollArea.h"
 #include "FXTreeList.h"
 #include "FXTreeListBox.h"
 
@@ -65,6 +71,9 @@
     by the TreeList's item height...
   - Perhaps may add some access API's to FXTreeItem?
   - The default height of the treelist box is not good yet.
+  - Perhaps use TWO tree lists, one in the pane, and one in the box;
+    we can then rest assured that the metrics are always computed properly.
+  - Still need some code to make sure always one item shows in box.
 */
 
 #define TREELISTBOX_MASK       (0)
@@ -72,11 +81,11 @@
 
 
 /*******************************************************************************/
-  
+
 // Map
 FXDEFMAP(FXTreeListBox) FXTreeListBoxMap[]={
-//   FXMAPFUNC(SEL_FOCUS_UP,0,FXTreeListBox::onFocusUp),
-//   FXMAPFUNC(SEL_FOCUS_DOWN,0,FXTreeListBox::onFocusDown),
+  FXMAPFUNC(SEL_FOCUS_UP,0,FXTreeListBox::onFocusUp),
+  FXMAPFUNC(SEL_FOCUS_DOWN,0,FXTreeListBox::onFocusDown),
   FXMAPFUNC(SEL_CHANGED,0,FXTreeListBox::onChanged),
   FXMAPFUNC(SEL_COMMAND,0,FXTreeListBox::onCommand),
   FXMAPFUNC(SEL_UPDATE,FXTreeListBox::ID_TREE,FXTreeListBox::onUpdFmTree),
@@ -97,10 +106,10 @@ FXTreeListBox::FXTreeListBox(FXComposite *p,FXint nvis,FXObject* tgt,FXSelector
   target=tgt;
   message=sel;
   field=new FXButton(this," ",NULL,this,FXTreeListBox::ID_FIELD,ICON_BEFORE_TEXT|JUSTIFY_LEFT, 0,0,0,0, pl,pr,pt,pb);
-  field->setBackColor(getApp()->backColor);
+  field->setBackColor(getApp()->getBackColor());
   pane=new FXPopup(this,FRAME_LINE);
   tree=new FXTreeList(pane,nvis,this,FXTreeListBox::ID_TREE,TREELIST_BROWSESELECT|TREELIST_AUTOSELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y|SCROLLERS_TRACK|HSCROLLING_OFF);
-  tree->setIndent(8);
+  tree->setIndent(0);
   button=new FXMenuButton(this,NULL,NULL,pane,FRAME_RAISED|FRAME_THICK|MENUBUTTON_DOWN|MENUBUTTON_ATTACH_RIGHT, 0,0,0,0, 0,0,0,0);
   button->setXOffset(border);
   button->setYOffset(border);
@@ -108,14 +117,20 @@ FXTreeListBox::FXTreeListBox(FXComposite *p,FXint nvis,FXObject* tgt,FXSelector
   }
 
 
-// Create X window
+// Create window
 void FXTreeListBox::create(){
   FXPacker::create();
   pane->create();
   }
 
+// Detach window
+void FXTreeListBox::detach(){
+  pane->detach();
+  FXPacker::detach();
+  }
 
-// Destroy X window
+
+// Destroy window
 void FXTreeListBox::destroy(){
   pane->destroy();
   FXPacker::destroy();
@@ -126,6 +141,7 @@ void FXTreeListBox::destroy(){
 void FXTreeListBox::enable(){
   if(!(flags&FLAG_ENABLED)){
     FXPacker::enable();
+    field->setBackColor(getApp()->getBackColor());
     field->enable();
     button->enable();
     }
@@ -136,6 +152,7 @@ void FXTreeListBox::enable(){
 void FXTreeListBox::disable(){
   if(flags&FLAG_ENABLED){
     FXPacker::disable();
+    field->setBackColor(getApp()->getBaseColor());
     field->disable();
     button->disable();
     }
@@ -205,7 +222,7 @@ long FXTreeListBox::onTreeClicked(FXObject*,FXSelector,void* ptr){
   FXTreeItem *item=(FXTreeItem*)ptr;
   button->handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);    // Unpost the list
   if(item){
-    field->setText(tree->getItemText(item)); 
+    field->setText(tree->getItemText(item));
     field->setIcon(tree->getItemClosedIcon(item));
     handle(this,MKUINT(0,SEL_COMMAND),(void*)item);
     }
@@ -220,31 +237,38 @@ long FXTreeListBox::onFieldButton(FXObject*,FXSelector,void*){
   }
 
 
-// // Select upper item
-// long FXTreeListBox::onFocusUp(FXObject*,FXSelector,void*){
-//   FXTreeItem *item=getCurrentItem();
-//   if(item && item->prev) item=item->prev;
-//   if(!item) item=getLastItem();
-//   if(item){
-//     setCurrentItem(item);
-//     handle(this,MKUINT(0,SEL_COMMAND),(void*)item->label.text());
-//     }
-//   return 1;
-//   }
-// 
-// 
-// // Select lower item
-// long FXTreeListBox::onFocusDown(FXObject*,FXSelector,void*){
-//   FXTreeItem *item=getCurrentItem();
-//   if(item && item->next) item=item->next;
-//   if(!item) item=getFirstItem();
-//   if(item){
-//     setCurrentItem(item);
-//     handle(this,MKUINT(0,SEL_COMMAND),(void*)item->label.text());
-//     }
-//   return 1;
-//   }
-// 
+// Select upper item
+long FXTreeListBox::onFocusUp(FXObject*,FXSelector,void*){
+  FXTreeItem *item=getCurrentItem();
+  if(!item){
+    for(item=getLastItem(); item->getLast(); item=item->getLast());
+    }
+  else if(item->getAbove()){
+    item=item->getAbove();
+    }
+  if(item){
+    setCurrentItem(item);
+    handle(this,MKUINT(0,SEL_COMMAND),(void*)item);
+    }
+  return 1;
+  }
+
+
+// Select lower item
+long FXTreeListBox::onFocusDown(FXObject*,FXSelector,void*){
+  FXTreeItem *item=getCurrentItem();
+  if(!item){
+    item=getFirstItem();
+    }
+  else if(item->getBelow()){
+    item=item->getBelow();
+    }
+  if(item){
+    setCurrentItem(item);
+    handle(this,MKUINT(0,SEL_COMMAND),(void*)item);
+    }
+  return 1;
+  }
 
 
 // Set font
@@ -258,13 +282,13 @@ void FXTreeListBox::setFont(FXFont* fnt){
 
 // Obtain font
 FXFont* FXTreeListBox::getFont() const {
-  return field->getFont(); 
+  return field->getFont();
   }
 
 
 // Get number of items
 FXint FXTreeListBox::getNumItems() const {
-  return tree->getNumItems(); 
+  return tree->getNumItems();
   }
 
 
@@ -281,14 +305,14 @@ void FXTreeListBox::setNumVisible(FXint nvis){
 
 
 // Get first item
-FXTreeItem* FXTreeListBox::getFirstItem() const { 
-  return tree->getFirstItem(); 
+FXTreeItem* FXTreeListBox::getFirstItem() const {
+  return tree->getFirstItem();
   }
 
 
 // Get last item
-FXTreeItem* FXTreeListBox::getLastItem() const { 
-  return tree->getLastItem(); 
+FXTreeItem* FXTreeListBox::getLastItem() const {
+  return tree->getLastItem();
   }
 
 
@@ -309,14 +333,14 @@ FXTreeItem* FXTreeListBox::addItemLast(FXTreeItem* p,FXTreeItem* item){
 // Add item after another
 FXTreeItem* FXTreeListBox::addItemAfter(FXTreeItem* other,FXTreeItem* item){
   recalc();
-  return tree->addItemAfter(other,item);  
+  return tree->addItemAfter(other,item);
   }
 
 
 // Add item before another
 FXTreeItem* FXTreeListBox::addItemBefore(FXTreeItem* other,FXTreeItem* item){
   recalc();
-  return tree->addItemBefore(other,item); 
+  return tree->addItemBefore(other,item);
   }
 
 
@@ -324,7 +348,7 @@ FXTreeItem* FXTreeListBox::addItemBefore(FXTreeItem* other,FXTreeItem* item){
 FXTreeItem* FXTreeListBox::addItemFirst(FXTreeItem* p,const FXString& text,FXIcon* oi,FXIcon* ci,void* ptr){
   FXTreeItem *item=tree->addItemFirst(p,text,oi,ci,ptr);
   recalc();
-  return item; 
+  return item;
   }
 
 
@@ -332,29 +356,29 @@ FXTreeItem* FXTreeListBox::addItemFirst(FXTreeItem* p,const FXString& text,FXIco
 FXTreeItem* FXTreeListBox::addItemLast(FXTreeItem* p,const FXString& text,FXIcon* oi,FXIcon* ci,void* ptr){
   FXTreeItem *item=tree->addItemLast(p,text,oi,ci,ptr);
   recalc();
-  return item; 
+  return item;
   }
 
 
 // Add item after another
 FXTreeItem* FXTreeListBox::addItemAfter(FXTreeItem* other,const FXString& text,FXIcon* oi,FXIcon* ci,void* ptr){
-  FXTreeItem *item=tree->addItemAfter(other,text,oi,ci,ptr);  
+  FXTreeItem *item=tree->addItemAfter(other,text,oi,ci,ptr);
   recalc();
-  return item; 
+  return item;
   }
 
 
 // Add item before another
 FXTreeItem* FXTreeListBox::addItemBefore(FXTreeItem* other,const FXString& text,FXIcon* oi,FXIcon* ci,void* ptr){
-  FXTreeItem *item=tree->addItemBefore(other,text,oi,ci,ptr); 
+  FXTreeItem *item=tree->addItemBefore(other,text,oi,ci,ptr);
   recalc();
-  return item; 
+  return item;
   }
 
 
 // Remove given item
-void FXTreeListBox::removeItem(FXTreeItem* item){ 
-  tree->removeItem(item); 
+void FXTreeListBox::removeItem(FXTreeItem* item){
+  tree->removeItem(item);
   recalc();
   }
 
@@ -367,12 +391,12 @@ void FXTreeListBox::removeItems(FXTreeItem* fm,FXTreeItem* to){
 
 
 // Remove all items
-void FXTreeListBox::removeAllItems(){
-  tree->removeAllItems();
+void FXTreeListBox::clearItems(){
+  tree->clearItems();
   recalc();
   }
 
-  
+
 // Is item current
 FXbool FXTreeListBox::isItemCurrent(const FXTreeItem* item) const {
   return tree->isItemCurrent(item);
@@ -398,8 +422,8 @@ void FXTreeListBox::sortItems(){
 
 
 // Change current item
-void FXTreeListBox::setCurrentItem(FXTreeItem* item){
-  tree->setCurrentItem(item);
+void FXTreeListBox::setCurrentItem(FXTreeItem* item,FXbool notify){
+  tree->setCurrentItem(item,notify);
   if(item){
     field->setIcon(tree->getItemClosedIcon(item));
     field->setText(tree->getItemText(item));
@@ -458,7 +482,7 @@ void FXTreeListBox::setItemData(FXTreeItem* item,void* ptr) const {
   }
 
 
-// Get item data 
+// Get item data
 void* FXTreeListBox::getItemData(const FXTreeItem* item) const {
   if(item==NULL){ fxerror("%s::getItemData: item is NULL\n",getClassName()); }
   return tree->getItemData(item);
@@ -474,7 +498,7 @@ FXTreeListSortFunc FXTreeListBox::getSortFunc() const {
 // Change sort function
 void FXTreeListBox::setSortFunc(FXTreeListSortFunc func){
   tree->setSortFunc(func);
-  }  
+  }
 
 
 // Change list style
@@ -485,7 +509,7 @@ void FXTreeListBox::setListStyle(FXuint mode){
     recalc();
     }
   }
-  
+
 
 // Get list style
 FXuint FXTreeListBox::getListStyle() const {
@@ -534,7 +558,7 @@ void FXTreeListBox::load(FXStream& store){
   store >> button;
   store >> tree;
   store >> pane;
-  }  
+  }
 
 
 // Delete it
