@@ -9,22 +9,23 @@
 *  probably my fault.                                                           *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: fxregex.cpp,v 1.1 1999/09/14 13:41:24 jeroen Exp $                        *
+* $Id: fxregex.cpp,v 1.6 2001/01/25 19:48:37 jeroen Exp $                       *
 ********************************************************************************/
 #include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "fxregex.h"
 
@@ -32,7 +33,7 @@
  * The first byte of the regexp internal "program" is actually this magic
  * number; the start node begins in the second byte.
  */
-const char MAGIC = (const char) 0234;
+const unsigned char MAGIC = 0234;
 
 /*
  * regcomp and regexec -- regsub and regerror are elsewhere
@@ -144,9 +145,9 @@ const char MAGIC = (const char) 0234;
  * Using two bytes for the "next" pointer is vast overkill for most things,
  * but allows patterns to get big without disasters.
  */
-#define	OP(p)	(*(p))
-#define	NEXT(p)	(((*((p)+1)&0377)<<8) + *((p)+2)&0377)
-#define	OPERAND(p)	((p) + 3)
+#define	OP(p)	    (*(p))
+#define	NEXT(p)	    (((*((p)+1)&0377)<<8) + *((p)+2)&0377)
+#define	OPERAND(p)  ((p) + 3)
 
 /*
  * See regmagic.h for one further detail of program structure.
