@@ -3,25 +3,26 @@
 *                          U t i l i t y   F u n c t i o n s                    *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
+* modify it under the terms of the GNU Lesser General Public                    *
 * License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
+* version 2.1 of the License, or (at your option) any later version.            *
 *                                                                               *
 * This library is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
+* Lesser General Public License for more details.                               *
 *                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
+* You should have received a copy of the GNU Lesser General Public              *
+* License along with this library; if not, write to the Free Software           *
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: fxutils.cpp,v 1.5 1999/11/17 07:31:08 jeroen Exp $                      *
+* $Id: fxutils.cpp,v 1.57 2001/07/27 22:20:58 jeroen Exp $                      *
 ********************************************************************************/
 #include "xincs.h"
+#include "fxver.h"
 #include "fxdefs.h"
 #include "fxkeys.h"
 
@@ -38,45 +39,6 @@
 
 /*******************************************************************************/
 
-// Bit-reversal
-const FXuchar fxbitreverse[256]={
-  0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0,
-  0x10, 0x90, 0x50, 0xd0, 0x30, 0xb0, 0x70, 0xf0,
-  0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8,
-  0x18, 0x98, 0x58, 0xd8, 0x38, 0xb8, 0x78, 0xf8,
-  0x04, 0x84, 0x44, 0xc4, 0x24, 0xa4, 0x64, 0xe4,
-  0x14, 0x94, 0x54, 0xd4, 0x34, 0xb4, 0x74, 0xf4,
-  0x0c, 0x8c, 0x4c, 0xcc, 0x2c, 0xac, 0x6c, 0xec,
-  0x1c, 0x9c, 0x5c, 0xdc, 0x3c, 0xbc, 0x7c, 0xfc,
-  0x02, 0x82, 0x42, 0xc2, 0x22, 0xa2, 0x62, 0xe2,
-  0x12, 0x92, 0x52, 0xd2, 0x32, 0xb2, 0x72, 0xf2,
-  0x0a, 0x8a, 0x4a, 0xca, 0x2a, 0xaa, 0x6a, 0xea,
-  0x1a, 0x9a, 0x5a, 0xda, 0x3a, 0xba, 0x7a, 0xfa,
-  0x06, 0x86, 0x46, 0xc6, 0x26, 0xa6, 0x66, 0xe6,
-  0x16, 0x96, 0x56, 0xd6, 0x36, 0xb6, 0x76, 0xf6,
-  0x0e, 0x8e, 0x4e, 0xce, 0x2e, 0xae, 0x6e, 0xee,
-  0x1e, 0x9e, 0x5e, 0xde, 0x3e, 0xbe, 0x7e, 0xfe,
-  0x01, 0x81, 0x41, 0xc1, 0x21, 0xa1, 0x61, 0xe1,
-  0x11, 0x91, 0x51, 0xd1, 0x31, 0xb1, 0x71, 0xf1,
-  0x09, 0x89, 0x49, 0xc9, 0x29, 0xa9, 0x69, 0xe9,
-  0x19, 0x99, 0x59, 0xd9, 0x39, 0xb9, 0x79, 0xf9,
-  0x05, 0x85, 0x45, 0xc5, 0x25, 0xa5, 0x65, 0xe5,
-  0x15, 0x95, 0x55, 0xd5, 0x35, 0xb5, 0x75, 0xf5,
-  0x0d, 0x8d, 0x4d, 0xcd, 0x2d, 0xad, 0x6d, 0xed,
-  0x1d, 0x9d, 0x5d, 0xdd, 0x3d, 0xbd, 0x7d, 0xfd,
-  0x03, 0x83, 0x43, 0xc3, 0x23, 0xa3, 0x63, 0xe3,
-  0x13, 0x93, 0x53, 0xd3, 0x33, 0xb3, 0x73, 0xf3,
-  0x0b, 0x8b, 0x4b, 0xcb, 0x2b, 0xab, 0x6b, 0xeb,
-  0x1b, 0x9b, 0x5b, 0xdb, 0x3b, 0xbb, 0x7b, 0xfb,
-  0x07, 0x87, 0x47, 0xc7, 0x27, 0xa7, 0x67, 0xe7,
-  0x17, 0x97, 0x57, 0xd7, 0x37, 0xb7, 0x77, 0xf7,
-  0x0f, 0x8f, 0x4f, 0xcf, 0x2f, 0xaf, 0x6f, 0xef,
-  0x1f, 0x9f, 0x5f, 0xdf, 0x3f, 0xbf, 0x7f, 0xff
-  };
-
-
-/*******************************************************************************/
-
 
 // Global flag which controls tracing level
 unsigned int fxTraceLevel=0;
@@ -84,6 +46,14 @@ unsigned int fxTraceLevel=0;
 
 /*******************************************************************************/
 
+
+// Thread-safe, linear congruential random number generator from Knuth & Lewis.
+FXuint fxrandom(FXuint& seed){
+  seed=1664525UL*seed+1013904223UL;
+  return seed;
+  }
+
+
 // Allocate memory
 FXint fxmalloc(void** ptr,unsigned long size){
   FXASSERT(ptr);
@@ -104,7 +74,19 @@ FXint fxcalloc(void** ptr,unsigned long size){
 FXint fxresize(void** ptr,unsigned long size){
   FXASSERT(ptr);
   void *p=realloc(*ptr,size);
-  if(p){ *ptr=p; return TRUE; }
+  if(!size || p){ *ptr=p; return TRUE; }
+  return FALSE;
+  }
+
+
+// Allocate and initialize memory
+FXint fxmemdup(void** ptr,unsigned long size,const void* src){
+  FXASSERT(ptr);
+  *ptr=NULL;
+  if(src){
+    *ptr=malloc(size);
+    if(*ptr){ memcpy(*ptr,src,size); return TRUE; }
+    }
   return FALSE;
   }
 
@@ -125,21 +107,98 @@ FXchar *fxstrdup(const FXchar* str){
   return copy;
   }
 
+#ifdef WIN32
 
-// Case independent compare
-int fxstrcasecmp(const FXchar* s1,const FXchar* s2){
-  register FXint c1,c2;
-  if(s1 && s2){
-    do{
-      c1=toupper(*s1++);
-      c2=toupper(*s2++);
+// Missing in CYGWIN
+#ifndef IMAGE_SUBSYSTEM_NATIVE_WINDOWS
+#define IMAGE_SUBSYSTEM_NATIVE_WINDOWS 8
+#endif
+
+#ifndef IMAGE_SUBSYSTEM_WINDOWS_CE_GUI
+#define IMAGE_SUBSYSTEM_WINDOWS_CE_GUI 9
+#endif
+
+
+FXbool fxisconsole(const FXchar *path){
+  IMAGE_OPTIONAL_HEADER optional_header;
+  IMAGE_FILE_HEADER     file_header;
+  IMAGE_DOS_HEADER      dos_header;
+  DWORD                 dwCoffHeaderOffset;
+  DWORD                 dwNewOffset;
+  DWORD                 dwMoreDosHeader[16];
+  ULONG                 ulNTSignature;
+  HANDLE                hImage;
+  DWORD                 dwBytes;
+  FXbool                flag=MAYBE;
+
+  // Open the application file.
+  hImage=CreateFile(path,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
+  if(hImage!=INVALID_HANDLE_VALUE){
+
+    // Read MS-Dos image header.
+    if(ReadFile(hImage,&dos_header,sizeof(IMAGE_DOS_HEADER),&dwBytes,NULL)==0) goto x;
+
+    // Test bytes read
+    if(dwBytes!=sizeof(IMAGE_DOS_HEADER)) goto x;
+
+    // Test signature
+    if(dos_header.e_magic!=IMAGE_DOS_SIGNATURE) goto x;
+
+    // Read more MS-Dos header.
+    if(ReadFile(hImage,dwMoreDosHeader,sizeof(dwMoreDosHeader),&dwBytes,NULL)==0) goto x;
+
+    // Test bytes read
+    if(dwBytes!=sizeof(dwMoreDosHeader)) goto x;
+
+    // Move the file pointer to get the actual COFF header.
+    dwNewOffset=SetFilePointer(hImage,dos_header.e_lfanew,NULL,FILE_BEGIN);
+    dwCoffHeaderOffset=dwNewOffset+sizeof(ULONG);
+    if(dwCoffHeaderOffset==0xFFFFFFFF) goto x;
+
+    // Read NT signature of the file.
+    if(ReadFile(hImage,&ulNTSignature,sizeof(ULONG),&dwBytes,NULL)==0) goto x;
+
+    // Test bytes read
+    if(dwBytes!=sizeof(ULONG)) goto x;
+
+    // Test NT signature
+    if(ulNTSignature!=IMAGE_NT_SIGNATURE) goto x;
+
+    if(ReadFile(hImage,&file_header,IMAGE_SIZEOF_FILE_HEADER,&dwBytes,NULL)==0) goto x;
+
+    // Test bytes read
+    if(dwBytes!=IMAGE_SIZEOF_FILE_HEADER) goto x;
+
+    // Read the optional header of file.
+    if(ReadFile(hImage,&optional_header,IMAGE_SIZEOF_NT_OPTIONAL_HEADER,&dwBytes,NULL)==0) goto x;
+
+    // Test bytes read
+    if(dwBytes!=IMAGE_SIZEOF_NT_OPTIONAL_HEADER) goto x;
+
+    // Switch on systems
+    switch(optional_header.Subsystem){
+      case IMAGE_SUBSYSTEM_WINDOWS_GUI:     // Windows GUI (2)
+      case IMAGE_SUBSYSTEM_WINDOWS_CE_GUI:  // Windows CE GUI (9)
+        flag=FALSE;
+        break;
+      case IMAGE_SUBSYSTEM_WINDOWS_CUI:     // Windows Console (3)
+      case IMAGE_SUBSYSTEM_OS2_CUI:         // OS/2 Console (5)
+      case IMAGE_SUBSYSTEM_POSIX_CUI:       // Posix Console (7)
+        flag=TRUE;
+        break;
+      case IMAGE_SUBSYSTEM_NATIVE:          // Native (1)
+      case IMAGE_SUBSYSTEM_NATIVE_WINDOWS:  // Native Win9x (8)
+      case IMAGE_SUBSYSTEM_UNKNOWN:         // Unknown (0)
+      default:
+        break;
       }
-    while(c1 && (c1==c2));
-    return c1-c2;
+x:  CloseHandle(hImage);
     }
-  return 0;
+  return flag;
   }
 
+#endif
+
 
 // Assert failed routine
 void fxassert(const char* expression,const char* filename,unsigned int lineno){
@@ -160,22 +219,25 @@ void fxassert(const char* expression,const char* filename,unsigned int lineno){
 
 // Log message to [typically] stderr
 void fxmessage(const char* format,...){
-#ifndef WIN32
+#ifndef WIN32                         // Unix
   va_list arguments;
   va_start(arguments,format);
   vfprintf(stderr,format,arguments);
   va_end(arguments);
 #else
-  char msg[256];
+#ifdef _WINDOWS
+  char msg[512];
   va_list arguments;
   va_start(arguments,format);
-  vsprintf(msg,format,arguments);
+  vsnprintf(msg,sizeof(msg),format,arguments);
   va_end(arguments);
-#ifdef _WINDOWS
   OutputDebugString(msg);
   fprintf(stderr,"%s",msg); // if a console is available
 #else
-  fprintf(stderr,"%s",msg);
+  va_list arguments;
+  va_start(arguments,format);
+  vfprintf(stderr,format,arguments);
+  va_end(arguments);
 #endif
 #endif
   }
@@ -188,21 +250,24 @@ void fxerror(const char* format,...){
   va_start(arguments,format);
   vfprintf(stderr,format,arguments);
   va_end(arguments);
-  exit(1);
+  abort();
 #else
-  char msg[256];
+#ifdef _WINDOWS
+  char msg[512];
   va_list arguments;
   va_start(arguments,format);
-  vsprintf(msg,format,arguments);
+  vsnprintf(msg,sizeof(msg),format,arguments);
   va_end(arguments);
-#ifdef _WINDOWS
   OutputDebugString(msg);
   fprintf(stderr,"%s",msg); // if a console is available
   MessageBox(NULL,msg,NULL,MB_OK|MB_ICONEXCLAMATION|MB_APPLMODAL);
   DebugBreak();
 #else
-  fprintf(stderr,"%s",msg);
-  exit(1);
+  va_list arguments;
+  va_start(arguments,format);
+  vfprintf(stderr,format,arguments);
+  va_end(arguments);
+  abort();
 #endif
 #endif
   }
@@ -216,17 +281,20 @@ void fxwarning(const char* format,...){
   vfprintf(stderr,format,arguments);
   va_end(arguments);
 #else
-  char msg[256];
+#ifdef _WINDOWS
+  char msg[512];
   va_list arguments;
   va_start(arguments,format);
-  vsprintf(msg,format,arguments);
+  vsnprintf(msg,sizeof(msg),format,arguments);
   va_end(arguments);
-#ifdef _WINDOWS
   OutputDebugString(msg);
   fprintf(stderr,"%s",msg); // if a console is available
   MessageBox(NULL,msg,NULL,MB_OK|MB_ICONINFORMATION|MB_APPLMODAL);
 #else
-  fprintf(stderr,"%s",msg);
+  va_list arguments;
+  va_start(arguments,format);
+  vfprintf(stderr,format,arguments);
+  va_end(arguments);
 #endif
 #endif
   }
@@ -241,16 +309,19 @@ void fxtrace(unsigned int level,const char* format,...){
     vfprintf(stderr,format,arguments);
     va_end(arguments);
 #else
-    char msg[256];
+#ifdef _WINDOWS
+    char msg[512];
     va_list arguments;
     va_start(arguments,format);
-    vsprintf(msg,format,arguments);
+    vsnprintf(msg,sizeof(msg),format,arguments);
     va_end(arguments);
-#ifdef _WINDOWS
     OutputDebugString(msg);
     fprintf(stderr,"%s",msg); // if a console is available
 #else
-    fprintf(stderr,"%s",msg);
+    va_list arguments;
+    va_start(arguments,format);
+    vfprintf(stderr,format,arguments);
+    va_end(arguments);
 #endif
 #endif
     }
@@ -270,7 +341,7 @@ void fxsleep(unsigned int n){
   value.tv_sec = n/1000000;
   nanosleep(&value,NULL);
 #else
-#ifndef BROKEN_SELECT  
+#ifndef BROKEN_SELECT
   struct timeval value;
   value.tv_usec = n % 1000000;
   value.tv_sec = n / 1000000;
@@ -291,7 +362,7 @@ void fxsleep(unsigned int n){
 FXchar fxkeyval(FXuint keysym,FXuint state){
   if((state&CONTROLMASK) && 0x60<=keysym && keysym<=0x7f) return keysym-0x60;
   if(state&0x2000){   // Patch from mohme@hgb-leipzig.de
-    switch(keysym){ 
+    switch(keysym){
       case 'q': return '@';
       case '7': return '{';
       case '8': return '[';
@@ -320,36 +391,54 @@ FXchar fxkeyval(FXuint keysym,FXuint state){
 
 
 // Parse for accelerator key codes in a string
-FXHotKey fxparseaccel(const FXchar* s){
-  FXuint code=0,mods=0;
+FXHotKey fxparseaccel(const FXchar* string){
+  register const FXuchar *s=(const FXuchar *)string;
+  register FXuint code=0,mods=0;
   if(s){
     while(s[0] && s[0]!='\t' && s[0]!='\n'){
       if(ISSEP(s[0])){
         s++;
         continue;
         }
-      if((toupper(s[0])=='C')&&(toupper(s[1])=='T')&&(toupper(s[2])=='L')&&ISSEP(s[3])){
+      if((tolower(s[0])=='c')&&(tolower(s[1])=='t')&&(tolower(s[2])=='l')&&ISSEP(s[3])){
         mods|=CONTROLMASK;
         s+=4;
         continue;
         }
-      if((toupper(s[0])=='A')&&(toupper(s[1])=='L')&&(toupper(s[2])=='T')&&ISSEP(s[3])){
+      if((tolower(s[0])=='c')&&(tolower(s[1])=='t')&&(tolower(s[2])=='r')&&(tolower(s[3])=='l')&&ISSEP(s[4])){
+        mods|=CONTROLMASK;
+        s+=5;
+        continue;
+        }
+      if((tolower(s[0])=='a')&&(tolower(s[1])=='l')&&(tolower(s[2])=='t')&&ISSEP(s[3])){
         mods|=ALTMASK;
         s+=4;
         continue;
         }
-      if((toupper(s[0])=='S')&&(toupper(s[1])=='H')&&(toupper(s[2])=='I')&&(toupper(s[3])=='F')&&(toupper(s[4])=='T')&&ISSEP(s[5])){
+      if((tolower(s[0])=='s')&&(tolower(s[1])=='h')&&(tolower(s[2])=='i')&&(tolower(s[3])=='f')&&(tolower(s[4])=='t')&&ISSEP(s[5])){
         mods|=SHIFTMASK;
         s+=6;
         continue;
         }
-      if(isalnum(s[0]) && (s[1]=='\0' || s[1]=='\t' || s[1]=='\n')){
-        if(mods&SHIFTMASK) 
+
+      // One-digit function key
+      if(tolower(s[0])=='f' && isdigit(s[1]) && (s[2]=='\0' || s[2]=='\t' || s[2]=='\n')){
+        code=KEY_F1+s[1]-'1';
+        }
+
+      // Two digit function key
+      else if(tolower(s[0])=='f' && isdigit(s[1]) && isdigit(s[2]) && (s[3]=='\0' || s[3]=='\t' || s[3]=='\n')){
+        code=KEY_F1+10*(s[1]-'0')+(s[2]-'0')-1;
+        }
+
+      // One final character
+      else if(s[0] && (s[1]=='\0' || s[1]=='\t' || s[1]=='\n')){
+        if(mods&SHIFTMASK)
           code=toupper(s[0])+KEY_space-' ';
         else
           code=tolower(s[0])+KEY_space-' ';
         }
-      FXTRACE((150,"fxparseaccel: code = %04x mods=%04x\n",code,mods));
+      FXTRACE((150,"fxparseaccel(%s): %08x code = %04x mods=%04x\n",string,MKUINT(code,mods),code,mods));
       return MKUINT(code,mods);
       }
     }
@@ -358,7 +447,8 @@ FXHotKey fxparseaccel(const FXchar* s){
 
 
 // Parse for hot key in a string
-FXHotKey fxparsehotkey(const FXchar* s){
+FXHotKey fxparsehotkey(const FXchar* string){
+  register const FXuchar *s=(const FXuchar*)string;
   FXuint code,mods;
   if(s){
     while(s[0] && s[0]!='\t'){
@@ -367,7 +457,7 @@ FXHotKey fxparsehotkey(const FXchar* s){
           if(isalnum(s[1])){
             mods=ALTMASK;
             code=tolower(s[1])+KEY_space-' ';
-            FXTRACE((150,"fxparsehotkey: code = %04x mods=%04x\n",code,mods));
+            FXTRACE((150,"fxparsehotkey(%s): %08x code = %04x mods=%04x\n",string,MKUINT(code,mods),code,mods));
             return MKUINT(code,mods);
             }
           break;
@@ -397,130 +487,6 @@ FXint fxfindhotkeyoffset(const FXchar* s){
   }
 
 
-#define FOLD(c)	  ((flags&FILEMATCH_CASEFOLD)&&islower(c)?toupper(c):(c))
-
-
-// Match a file name with a pattern; returning 1 if it matches, 0 if not.
-FXint fxfilematch(const char *pattern,const char *string,FXuint flags){
-  if(pattern && string){
-    register const char *p=pattern,*n=string;
-    register char c,c1,cstart,cend;
-    register int neg;
-    
-    // Loop over characters of pattern
-    while((c=*p++)!='\0'){
-      c=FOLD(c);
-      switch(c){
-
-        // Single character wild card
-        case '?':
-	  if(*n=='\0') return 0;
-	  else if((flags&FILEMATCH_FILE_NAME) && *n==PATHSEP) return 0;
-	  else if((flags&FILEMATCH_PERIOD) && *n=='.' && (n==string || ((flags&FILEMATCH_FILE_NAME) && n[-1]==PATHSEP))) return 0;
-	  break;
-
-        // Many character wild card
-        case '*':
-	  if((flags&FILEMATCH_PERIOD) && *n=='.' && (n==string || ((flags&FILEMATCH_FILE_NAME) && n[-1]==PATHSEP))) return 0;
-
-	  for(c=*p++; c=='?' || c=='*'; c=*p++,n++){
-	    if(((flags&FILEMATCH_FILE_NAME) && *n==PATHSEP) || (c=='?' && *n=='\0')) return 0;
-            }
-
-	  if(c=='\0') return 1;
-
-	  c1=(!(flags&FILEMATCH_NOESCAPE) && c=='\\') ? *p : c;
-	  c1=FOLD(c1);
-          for(--p; *n!='\0'; n++){
-	    if((c=='[' || FOLD(*n)==c1) && fxfilematch(p,n,flags&~FILEMATCH_PERIOD)) return 1;
-            }
-          return 0;
-
-        // Character range wild card
-        case '[':
-	  if(*n=='\0') return 0;
-
-	  if((flags&FILEMATCH_PERIOD) && *n=='.' && (n==string || ((flags&FILEMATCH_FILE_NAME) && n[-1]==PATHSEP))) return 0;
-
-	  // Nonzero if the sense of the character class is inverted.
-	  neg=(*p=='!' || *p=='^');
-
-	  if(neg) ++p;
-
-	  c=*p++;
-	  for(;;){
-	    cstart=c;
-            cend=c;
-
-	    if(!(flags&FILEMATCH_NOESCAPE) && c=='\\') cstart=cend=*p++;
-
-	    cstart=cend=FOLD(cstart);
-
-	    // [ (unterminated) loses.
-	    if(c=='\0') return 0;
-
-	    c=*p++;
-	    c=FOLD(c);
-
-	    // [/] can never match.
-	    if((flags&FILEMATCH_FILE_NAME) && c==PATHSEP) return 0;
-
-	    if(c=='-' && *p!=']'){
-	      cend = *p++;
-	      if(!(flags&FILEMATCH_NOESCAPE) && cend=='\\') cend=*p++;
-	      if(cend=='\0') return 0;
-	      cend=FOLD(cend);
-	      c=*p++;
-	      }
-
-	    if(FOLD(*n)>=cstart && FOLD(*n)<=cend) goto matched;
-
-	    if(c==']') break;
-	    }
-	  if(!neg) return 0;
-	  break;
-
-matched:  // Skip the rest of the [...] that already matched.
-	  while(c!=']'){
-
-	    // [... (unterminated) loses.
-	    if(c=='\0') return 0;
-
-	    c=*p++;
-
-	    // XXX 1003.2d11 is unclear if this is right.
-	    if(!(flags&FILEMATCH_NOESCAPE) && c=='\\') ++p;
-	    }
-	  if(neg) return 0;
-	  break;
-
-        // Escaped character
-        case '\\':
-	  if(!(flags&FILEMATCH_NOESCAPE)){
-	    c=*p++;
-	    c=FOLD(c);
-	    }
-          // Fall through...
-
-        // Normal character
-        default:
-	  if(FOLD(*n)!=c) return 0;
-	  break;
-        }
-
-      ++n;
-      }
-
-    if(*n=='\0') return 1;
-
-    // The FILEMATCH_LEADING_DIR flag says that "foo*" matches "foobar/frobozz".
-    if((flags&FILEMATCH_LEADING_DIR) && *n==PATHSEP) return 1;
-    }
-
-  return 0;
-  }
-
-
 
 // Get highlight color
 FXColor makeHiliteColor(FXColor clr){
@@ -559,12 +525,13 @@ FXchar* fxgetusername(FXchar* result,FXuint uid){
   if(!result){fxerror("fxgetusername: NULL result argument.\n");}
 #ifndef WIN32
   struct passwd *pwd;
-  if((pwd=getpwuid(uid))!=NULL) 
-    strcpy(result,pwd->pw_name); 
-  else 
+  if((pwd=getpwuid(uid))!=NULL)
+    strcpy(result,pwd->pw_name);
+  else
     sprintf(result,"%d",uid);
 #else
-  sprintf(result,"%d",uid);
+//  sprintf(result,"%d",uid);
+  strcpy(result,"user");
 #endif
   return result;
   }
@@ -575,12 +542,13 @@ FXchar* fxgetgroupname(FXchar* result,FXuint gid){
   if(!result){fxerror("fxgetgroupname: NULL result argument.\n");}
 #ifndef WIN32
   struct group *grp;
-  if((grp=getgrgid(gid))!=NULL) 
-    strcpy(result,grp->gr_name); 
-  else 
+  if((grp=getgrgid(gid))!=NULL)
+    strcpy(result,grp->gr_name);
+  else
     sprintf(result,"%d",gid);
 #else
-  sprintf(result,"%d",gid);
+  //sprintf(result,"%d",gid);
+  strcpy(result,"group");
 #endif
   return result;
   }
@@ -635,7 +603,7 @@ FXbool fxsuffix(const FXchar* s, const FXchar* t){
 
 
 // Expand ~ in filenames; using HOME environment variable
-// Note "~name" expands to "~name", "~/name" expands to "$HOME/name", 
+// Note "~name" expands to "~name", "~/name" expands to "$HOME/name",
 // and "~" expands to "$HOME"
 FXchar* fxexpand(FXchar* result,const FXchar* name){
   if(!result){fxerror("fxexpand: NULL result argument.\n");}
@@ -651,7 +619,7 @@ FXchar* fxexpand(FXchar* result,const FXchar* name){
     }
   return result;
   }
-        
+
 
 // Construct a full pathname from the given directory and file name
 FXchar* fxpathname(FXchar* result,const FXchar* dirname,const FXchar* filename){
@@ -664,20 +632,14 @@ FXchar* fxpathname(FXchar* result,const FXchar* dirname,const FXchar* filename){
   }
 
 
-// Return the directory part of pathname
+// Return the directory part of pathname (incl drive name)
 FXchar* fxdirpart(FXchar* result,const FXchar* pathname){
   FXchar *p;
   if(!result){fxerror("fxdirpart: NULL result argument.\n");}
-  fxexpand(result,pathname);
+  strcpy(result,pathname);
   if((p=strrchr(result,PATHSEP))){      // Fix by sancelot@usa.net
-    if(fxexists(result)){
-      if(fxisfile(result)){
-        if(p==result) p++;
-        *p='\0';
-        }
-      }
-    else{
-      if(p==result) p++;
+    if(!fxisdir(result)){
+      if((p==result) || (p==result+2 && isalpha(result[0]) && result[1]==':')) p++;
       *p='\0';
       }
     }
@@ -694,7 +656,8 @@ FXchar* fxfiletitle(FXchar* result,const FXchar* pathname){
   if(!result){fxerror("fxfiletitle: NULL result argument.\n");}
   result[0]='\0';
   if(pathname){
-    p=(FXchar*)strrchr(pathname,PATHSEP); // Borland C++ compiler says strrchr() returns a const char*
+    if(isalpha(pathname[0]) && pathname[1]==':') pathname+=2;   // Skip drive name part
+    p=(FXchar*)strrchr(pathname,PATHSEP);                       // Borland C++ compiler says strrchr() returns a const char*
     if(p) pathname=p+1;
     strcpy(result,pathname);
     p=strrchr(result,'.');
@@ -708,6 +671,7 @@ FXchar* fxfiletitle(FXchar* result,const FXchar* pathname){
 FXchar* fxfilepart(FXchar* result,const FXchar* pathname){
   const FXchar *p; // Borland C++ compiler says strrchr() returns const char*
   if(!result){fxerror("fxfilepart: NULL result argument.\n");}
+  if(isalpha(pathname[0]) && pathname[1]==':') pathname+=2;   // Skip drive name part
   p=strrchr(pathname,PATHSEP);
   if(p) pathname=p+1;
   strcpy(result,pathname);
@@ -729,37 +693,44 @@ FXchar* fxfileext(FXchar* result,const FXchar* pathname){
 
 // Check whether a file exists
 FXbool fxexists(const FXchar *name){
-  struct stat info;
   if(!name){fxerror("fxexists: NULL name argument.\n");}
+#ifndef WIN32
+  struct stat info;
   return stat(name,&info)==0;
+#else
+  DWORD atts=GetFileAttributes(name);
+  return (atts!=0xFFFFFFFF);
+#endif
   }
 
 
 // Check whether its a directory
 FXbool fxisdir(const FXchar *name){
-  struct stat info;
   if(!name){fxerror("fxisdir: NULL name argument.\n");}
 #ifndef WIN32
+  struct stat info;
   return (stat(name,&info)==0) && S_ISDIR(info.st_mode);
 #else
-  return (stat(name,&info)==0) && (GetFileAttributes(name)&FILE_ATTRIBUTE_DIRECTORY);
+  DWORD atts=GetFileAttributes(name);
+  return (atts!=0xFFFFFFFF) && (atts&FILE_ATTRIBUTE_DIRECTORY);
 #endif
   }
 
 
 // Check whether its a file
 FXbool fxisfile(const FXchar *name){
-  struct stat info;
   if(!name){fxerror("fxisdir: NULL name argument.\n");}
 #ifndef WIN32
+  struct stat info;
   return (stat(name,&info)==0) && S_ISREG(info.st_mode);
 #else
-  return (stat(name,&info)==0) && !(GetFileAttributes(name)&FILE_ATTRIBUTE_DIRECTORY);
+  DWORD atts=GetFileAttributes(name);
+  return (atts!=0xFFFFFFFF) && !(atts&FILE_ATTRIBUTE_DIRECTORY);
 #endif
   }
 
 
-// Create a unique numbered backup file name for the given pathname 
+// Create a unique numbered backup file name for the given pathname
 FXchar* fxbakname(FXchar* result,const FXchar* pathname){
   FXchar* p;
   int n=0;
@@ -769,7 +740,7 @@ FXchar* fxbakname(FXchar* result,const FXchar* pathname){
   strcpy(result,pathname);
   do{
     sprintf(p,".~%d~",++n);
-    } 
+    }
   while(fxexists(result));
   return result;
   }
@@ -824,10 +795,10 @@ FXchar *fxshortestpath(FXchar *result,const FXchar *pathname){
   *cur='\0';
   tmp=path;
   last=NULL;
-  if(*tmp==PATHSEP){ 
+  if(*tmp==PATHSEP){
     *cur++=PATHSEP;
     *cur='\0';
-    tmp++; 
+    tmp++;
     }
   while((part=fxsplit(tmp,PATHSEP))){
     if(strcmp(part,".")==0)
@@ -838,14 +809,14 @@ FXchar *fxshortestpath(FXchar *result,const FXchar *pathname){
       else
         cur=last+1;
       *cur='\0';
-      } 
+      }
     else{
       if(cur>result && *(cur-1)!=PATHSEP) *cur++=PATHSEP;
       strcpy(cur,part);
       cur+=strlen(part);
       }
     }
-  
+
   // Remove trailing backslash(es)
   while(result<cur-1 && *(cur-1)==PATHSEP && *(cur-2)!=':'){
     *--cur='\0';
@@ -858,7 +829,7 @@ FXchar *fxshortestpath(FXchar *result,const FXchar *pathname){
     FXchar *buffer;
     if(FXMALLOC(&buffer,FXchar,dwSize+1)){
       if(0!=GetFullPathName(result,dwSize,buffer,NULL))
-	strcpy(result,buffer);
+        strcpy(result,buffer);
       FXFREE(&buffer);
       }
     }
@@ -871,30 +842,19 @@ FXchar *fxshortestpath(FXchar *result,const FXchar *pathname){
 // Return directory one level above given one
 FXchar* fxupdir(FXchar* result,const FXchar *dirname){
   FXchar *ptr;
-  
-  // Tests
   if(!result){fxerror("fxupdir: NULL result argument.\n");}
   if(!dirname){fxerror("fxupdir: NULL dirname argument.\n");}
-
-  // Copy
   strcpy(result,dirname);
-
-  // Find the last PATHSEP
-  ptr=strrchr(result,PATHSEP);
-  
-  // If dirname had a trailing PATHSEP, lop it off and try again
-  if(ptr && *(ptr+1)=='\0'){*ptr='\0';ptr=strrchr(result,PATHSEP);}
-
-  // Try lop off the last directory
-  if(ptr){
-    
-    // Is this the toplevel directory already?
-    // The test is a bit complicated as we need to take a possible drive letter into account
-    if((result[0]==PATHSEP && ptr==result) || (isalpha(result[0]) && result[1]==':' && result[2]==PATHSEP && ptr==result+2)){
-      *(ptr+1)='\0';
-      }
-    else{
-      *ptr='\0';
+  if(!fxistopdir(result)){
+    ptr=strrchr(result,PATHSEP);
+    if(ptr && *(ptr+1)=='\0'){ *ptr='\0'; ptr=strrchr(result,PATHSEP); }
+    if(ptr){
+      if((result[0]==PATHSEP && ptr==result) || (isalpha(result[0]) && result[1]==':' && result[2]==PATHSEP && ptr==result+2)){
+        *(ptr+1)='\0';
+        }
+      else{
+        *ptr='\0';
+        }
       }
     }
   return result;
@@ -903,7 +863,7 @@ FXchar* fxupdir(FXchar* result,const FXchar *dirname){
 
 // Test if pathname is the toplevel directory
 FXbool fxistopdir(const FXchar* path){
-  return path==NULL || (path[0]==PATHSEP && path[1]=='\0') || (isalpha(path[0]) && path[1]==':' && path[2]==PATHSEP && path[3]=='\0');
+  return (path[0]==PATHSEP && path[1]=='\0') || (isalpha(path[0]) && path[1]==':' && path[2]==PATHSEP && path[3]=='\0');
   }
 
 
@@ -931,9 +891,28 @@ FXint fxmkdir(const FXchar* pathname,FXuint mode){
 
 // Get home directory
 FXchar* fxgethomedir(FXchar *result){
-  static FXchar* home=NULL;
+  static const FXchar* home=NULL;
   if(!result){fxerror("fxgethomedir: NULL result argument.\n");}
-  if(!home){home=getenv("HOME");if(!home) home="";}
+  if(!home) {
+#ifdef WIN32
+    HKEY hKey;
+    char    szEnv[MAXPATHLEN+5];     // "HOME="
+    DWORD   dwSize=MAXPATHLEN;
+    if(RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders",0,KEY_READ,&hKey)==ERROR_SUCCESS){
+      strcpy(szEnv,"HOME=");
+      if(RegQueryValueEx(hKey,"Personal",NULL,NULL,(LPBYTE)szEnv+5,&dwSize)==ERROR_SUCCESS){
+#ifdef _MSC_VER
+        _putenv(szEnv);
+#else
+        putenv(szEnv);
+#endif
+        }
+      RegCloseKey(hKey);
+      }
+#endif
+    home=getenv("HOME");
+    if(!home) home="";
+    }
   strcpy(result,home);
   return result;
   }
@@ -943,13 +922,13 @@ FXchar* fxgethomedir(FXchar *result){
 // and if the resulting pathname is still relative, basename is prepended
 FXchar *fxabspath(FXchar *result,const FXchar *basename,const FXchar *filename){
   char s[MAXPATHLEN+1],t[MAXPATHLEN+1],cwd[MAXPATHLEN+1];
-  
+
   // Test
   if(!result){fxerror("fxabspath: NULL result argument.\n");}
 
   // Expand tilde's
   fxexpand(s,filename);
-  
+
   // Absolute path may be preceded by a drive letter as in C:\blabla\blabla
   // or just \blabla\blabla or on unix just /blabla/blabla
   if(!(s[0]==PATHSEP || (isalpha(s[0]) && s[1]==':' && s[2]==PATHSEP))){
@@ -966,7 +945,7 @@ FXchar *fxabspath(FXchar *result,const FXchar *basename,const FXchar *filename){
 
 // Search path for file name. A ~ in path is expanded. If name is absolute
 // it is returned unchanged. Otherwise the absolute name is returned in
-// result. If name is not found on path, NULL is returned. 
+// result. If name is not found on path, NULL is returned.
 FXchar *fxsearchpath(FXchar *result,const FXchar *path,const FXchar *name){
   FXchar fullname[MAXPATHLEN+1];
   FXchar basedir[MAXPATHLEN+1];
@@ -985,8 +964,8 @@ FXchar *fxsearchpath(FXchar *result,const FXchar *path,const FXchar *name){
     t=strchr(s,PATHLISTSEP);
     if(!t) t=s+strlen(s);
     if(s!=t){
-      if(s[0]=='.'){ 
-        if(t==s+1) s=t; else if(s[1]==PATHSEP) s+=2; 
+      if(s[0]=='.'){
+        if(t==s+1) s=t; else if(s[1]==PATHSEP) s+=2;
         }
       l=t-s;
       strncpy(fullname,s,l);
@@ -1017,19 +996,19 @@ void fxrgb_to_hsv(FXfloat& h,FXfloat& s,FXfloat& v,FXfloat r,FXfloat g,FXfloat b
   v=r; if(g>v) v=g; if(b>v) v=b;
   t=r; if(g<t) t=g; if(b<t) t=b;
   delta=v-t;
-  if(v!=0.0) 
-    s=delta/v; 
-  else 
+  if(v!=0.0)
+    s=delta/v;
+  else
     s=0.0;
   if(s==0.0){
     h=0.0;
     }
   else{
-    if(r==v) 
+    if(r==v)
       h=(g-b)/delta;
-    else if(g==v) 
+    else if(g==v)
       h=2.0f+(b-r)/delta;
-    else if(b==v) 
+    else if(b==v)
       h=4.0f+(r-g)/delta;
     h=h*60.0f;
     if(h<0.0) h=h+360;
@@ -1081,20 +1060,55 @@ FXint fxstrhash(const FXchar* str){
   }
 
 
-// Parse font description
-FXbool fxparsefontdesc(FXFontDesc& fontdesc,const FXchar* string){
-  return string && 7==sscanf(string,"%s %u %u %u %u %u %u",fontdesc.face,&fontdesc.size,&fontdesc.weight,&fontdesc.slant,&fontdesc.encoding,&fontdesc.setwidth,&fontdesc.flags);
+// Classify IEEE 754 floating point number
+FXint fxieeefloatclass(FXfloat number){
+  FXfloat num=number;
+  FXASSERT(sizeof(FXfloat)==sizeof(FXuint));
+  FXuint s=(*((FXuint*)&num)&0x80000000);        // Sign
+  FXuint e=(*((FXuint*)&num)&0x7f800000);        // Exponent
+  FXuint m=(*((FXuint*)&num)&0x007fffff);        // Mantissa
+  FXint result=0;
+  if(e==0x7f800000){
+    if(m==0)
+      result=1;     // Inf
+    else
+      result=2;     // NaN
+    if(s)
+      result=-result;
+    }
+  return result;
   }
 
 
-// Unparse font description
-FXbool fxunparsefontdesc(FXchar *string,const FXFontDesc& fontdesc){
-  sprintf(string,"%s %u %u %u %u %u %u",fontdesc.face,fontdesc.size,fontdesc.weight,fontdesc.slant,fontdesc.encoding,fontdesc.setwidth,fontdesc.flags);
-  return TRUE;
+// Classify IEEE 754 floating point number
+FXint fxieeedoubleclass(FXdouble number){
+  FXdouble num=number;
+  FXASSERT(sizeof(FXdouble)==2*sizeof(FXuint));
+#if FOX_BIGENDIAN
+  FXuint s=(((FXuint*)&num)[0]&0x80000000);     // Sign
+  FXuint e=(((FXuint*)&num)[0]&0x7ff00000);     // Exponent
+  FXuint h=(((FXuint*)&num)[0]&0x000fffff);     // Mantissa high
+  FXuint l=(((FXuint*)&num)[1]);                // Mantissa low
+#else
+  FXuint s=(((FXuint*)&num)[1]&0x80000000);     // Sign
+  FXuint e=(((FXuint*)&num)[1]&0x7ff00000);     // Exponent
+  FXuint h=(((FXuint*)&num)[1]&0x000fffff);     // Mantissa high
+  FXuint l=(((FXuint*)&num)[0]);                // Mantissa low
+#endif
+  FXint result=0;
+  if(e==0x7ff00000){
+    if(h==0 && l==0)
+      result=1;     // Inf
+    else
+      result=2;     // NaN
+    if(s)
+      result=-result;
+    }
+  return result;
   }
 
 
-#ifdef FX_NATIVE_WIN32
+#ifdef WIN32
 
 // Return the current state of the modifier keys and mouse buttons
 unsigned int fxmodifierkeys(){
@@ -1114,6 +1128,8 @@ unsigned int fxmodifierkeys(){
 
 // Convert font size (in decipoints) to device logical units
 int fxpointsize_to_height(HDC hdc,unsigned size){
+  // The calls to SetGraphicsMode() and ModifyWorldTransform() have no effect on Win95/98
+  // but that's OK.
   SetGraphicsMode(hdc,GM_ADVANCED);
   XFORM xform;
   ModifyWorldTransform(hdc,&xform,MWT_IDENTITY);
@@ -1130,8 +1146,16 @@ int fxpointsize_to_height(HDC hdc,unsigned size){
 
 // Convert logical units to decipoints
 unsigned fxheight_to_pointsize(HDC hdc,int height){
+  // The calls to SetGraphicsMode() and ModifyWorldTransform() have no effect on Win95/98
+  // but that's OK.
   SetGraphicsMode(hdc,GM_ADVANCED);
   XFORM xform;
+  xform.eM11=0.0f;
+  xform.eM12=0.0f;
+  xform.eM21=0.0f;
+  xform.eM22=0.0f;
+  xform.eDx=0.0f;
+  xform.eDy=0.0f;
   ModifyWorldTransform(hdc,&xform,MWT_IDENTITY);
   SetViewportOrgEx(hdc,0,0,NULL);
   SetWindowOrgEx(hdc,0,0,NULL);
