@@ -5,29 +5,17 @@
 *********************************************************************************
 * Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
 *********************************************************************************
-* This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Library General Public                   *
-* License as published by the Free Software Foundation; either                  *
-* version 2 of the License, or (at your option) any later version.              *
-*                                                                               *
-* This library is distributed in the hope that it will be useful,               *
-* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Library General Public License for more details.                              *
-*                                                                               *
-* You should have received a copy of the GNU Library General Public             *
-* License along with this library; if not, write to the Free                    *
-* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
-*********************************************************************************
-* $Id: glviewer.cpp,v 1.3 1999/09/24 18:59:45 jeroen Exp $                     *
+* $Id: glviewer.cpp,v 1.43 2001/07/03 15:16:03 jeroen Exp $                     *
 ********************************************************************************/
 #include "fx.h"
 #include "fx3d.h"
 #include <stdio.h>
 
+#define SHARE_CONTEXT
 
 #ifdef HAVE_OPENGL
 
+
 // Front view
 const unsigned char frontview[]={
   0x47,0x49,0x46,0x38,0x37,0x61,0x10,0x00,0x10,0x00,0xf2,0x00,0x00,0xb2,0xc0,0xdc,
@@ -51,7 +39,7 @@ const unsigned char backview[]={
   0xac,0x58,0x30,0x49,0x04,0xcc,0xa8,0x14,0x2a,0xad,0x16,0xa8,0xd6,0x28,0x36,0x5b,
   0x4d,0x00,0x00,0x3b
   };
-  
+
 // Top view
 const unsigned char topview[]={
   0x47,0x49,0x46,0x38,0x37,0x61,0x10,0x00,0x10,0x00,0xf2,0x00,0x00,0x00,0x80,0x00,
@@ -62,7 +50,7 @@ const unsigned char topview[]={
   0xc3,0x33,0x20,0x72,0xea,0x8a,0xf6,0xa3,0xdc,0x67,0x98,0x5b,0x14,0x8e,0x1a,0x23,
   0x32,0x09,0x38,0x16,0x98,0xcd,0xa5,0x23,0x01,0x00,0x3b
   };
-  
+
 // Bottom view
 const unsigned char bottomview[]={
   0x47,0x49,0x46,0x38,0x37,0x61,0x10,0x00,0x10,0x00,0xf1,0x00,0x00,0x80,0x80,0x80,
@@ -72,7 +60,7 @@ const unsigned char bottomview[]={
   0x94,0xb6,0x2f,0xd6,0xba,0x47,0xc0,0xe5,0xa7,0x73,0x2d,0x13,0xd2,0x33,0xfc,0x7e,
   0xc1,0xe2,0xa1,0x00,0x00,0x3b
   };
-  
+
 // Left view
 const unsigned char leftview[]={
   0x47,0x49,0x46,0x38,0x37,0x61,0x10,0x00,0x10,0x00,0xf2,0x00,0x00,0xb2,0xc0,0xdc,
@@ -84,7 +72,7 @@ const unsigned char leftview[]={
   0x51,0x50,0x38,0x9d,0x0c,0x50,0xc3,0x90,0xa9,0xac,0x56,0xa9,0xd6,0x2c,0x36,0xcb,
   0xed,0x0e,0x13,0x00,0x3b
   };
-  
+
 // Right view
 const unsigned char rightview[]={
   0x47,0x49,0x46,0x38,0x37,0x61,0x10,0x00,0x10,0x00,0xf2,0x00,0x00,0xb2,0xc0,0xdc,
@@ -208,7 +196,7 @@ const unsigned char cut[]={
   0x42,0x9d,0xf9,0x3a,0xed,0x49,0xa1,0x10,0x08,0x21,0x60,0x0a,0xea,0xa4,0xa6,0x86,
   0xa9,0xc0,0x2e,0xba,0xc8,0xf4,0x9d,0xd8,0x28,0x5c,0xf3,0xe7,0x54,0x00,0x00,0x3b
   };
-  
+
 // Copy
 const unsigned char copy[]={
   0x47,0x49,0x46,0x38,0x37,0x61,0x10,0x00,0x10,0x00,0xf1,0x00,0x00,0xb2,0xc0,0xdc,
@@ -218,8 +206,8 @@ const unsigned char copy[]={
   0x49,0x45,0xa0,0x9b,0x32,0xeb,0x48,0xe3,0x71,0x43,0x02,0x8d,0x3a,0x92,0xb8,0x14,
   0xa5,0x9f,0xec,0x71,0x33,0x21,0x67,0x33,0x8b,0xf3,0x99,0x28,0x00,0x00,0x3b
   };
-  
-  
+
+
 // Paste
 const unsigned char paste[]={
   0x47,0x49,0x46,0x38,0x37,0x61,0x10,0x00,0x10,0x00,0xf2,0x00,0x00,0xb2,0xc0,0xdc,
@@ -232,8 +220,8 @@ const unsigned char paste[]={
   0x59,0x3a,0x2b,0x57,0x6c,0xb3,0xd1,0x2b,0x8f,0x27,0x93,0x04,0x00,0x3b
   };
 
-  
-// Property  
+
+// Property
 const unsigned char prop[]={
   0x47,0x49,0x46,0x38,0x37,0x61,0x10,0x00,0x10,0x00,0xf2,0x00,0x00,0xb2,0xc0,0xdc,
   0x00,0x00,0x00,0x00,0x00,0x80,0xc0,0xc0,0xc0,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
@@ -246,7 +234,7 @@ const unsigned char prop[]={
   };
 
 // Delete
-const unsigned char kill[]={
+const unsigned char killobject[]={
   0x47,0x49,0x46,0x38,0x37,0x61,0x10,0x00,0x10,0x00,0xf0,0x00,0x00,0xb2,0xc0,0xdc,
   0xff,0x00,0x00,0x2c,0x00,0x00,0x00,0x00,0x10,0x00,0x10,0x00,0x00,0x02,0x24,0x84,
   0x8f,0x79,0xc1,0xba,0xcb,0x82,0x6a,0x28,0x4a,0xe8,0x40,0x34,0x34,0xdb,0x9b,0x71,
@@ -275,7 +263,7 @@ const unsigned char light[]={
   0xcd,0xb3,0xcb,0x3a,0x78,0xae,0x18,0xb2,0x1e,0x2d,0x24,0xf4,0xdd,0x8a,0xc4,0xe2,
   0x30,0x88,0x6c,0x41,0x12,0x00,0x3b
   };
-  
+
 // Smooth shading
 const unsigned char smoothlight[]={
   0x47,0x49,0x46,0x38,0x37,0x61,0x10,0x00,0x10,0x00,0xf2,0x00,0x00,0xb2,0xc0,0xdc,
@@ -297,7 +285,7 @@ const unsigned char parallel[]={
   0x1b,0x4a,0xa3,0xc9,0x6e,0x69,0x00,0x8b,0x0d,0x07,0xdb,0x08,0x2d,0x73,0xbd,0x22,
   0x9e,0x9f,0xb0,0x58,0x2c,0x00,0x00,0x3b
   };
-  
+
 // Perspective projection
 const unsigned char perspective[]={
   0x47,0x49,0x46,0x38,0x37,0x61,0x10,0x00,0x10,0x00,0xf1,0x00,0x00,0xb2,0xc0,0xdc,
@@ -317,7 +305,7 @@ const unsigned char newfolder[]={
   0xf5,0xb5,0x2d,0xb9,0xba,0xf2,0xb8,0xd9,0xfa,0x55,0xcb,0x22,0xa3,0x9b,0x31,0x4e,
   0x44,0xde,0x24,0x51,0x00,0x00,0x3b
   };
-  
+
 const unsigned char penguin[]={
   0x47,0x49,0x46,0x38,0x37,0x61,0x10,0x00,0x12,0x00,0xf2,0x00,0x00,0xb2,0xc0,0xdc,
   0x80,0x80,0x80,0x00,0x00,0x00,0xc0,0xc0,0xc0,0x10,0x10,0x10,0xff,0xff,0xff,0xe0,
@@ -342,8 +330,8 @@ const unsigned char winapp[]={
   0x60,0xc9,0x6c,0x3a,0x07,0x8e,0xe8,0x22,0x01,0x00,0x3b
   };
 
-  
-// Camera picture  
+
+// Camera picture
 const unsigned char camera[]={
   0x47,0x49,0x46,0x38,0x37,0x61,0x10,0x00,0x10,0x00,0xf2,0x00,0x00,0x00,0x00,0x00,
   0xc0,0xc0,0xc0,0xff,0x00,0x00,0xff,0xff,0x00,0x00,0x00,0xff,0x00,0x00,0x00,0x00,
@@ -360,26 +348,35 @@ const unsigned char camera[]={
 class GLViewWindow : public FXMainWindow {
   FXDECLARE(GLViewWindow)
 private:
-  FXHorizontalFrame *toolbar;                 // Tool bar
-  FXMenuBar         *menubar;                 // Menu bar
+  FXToolbar         *toolbar;                 // Tool bar
+  FXMenubar         *menubar;                 // Menu bar
   FXMDIClient       *mdiclient;               // MDI Client area
-  FXGLViewer        *viewer;                  // GL Viewer to draw into
   FXStatusbar       *statusbar;               // Status bar
   FXGLGroup         *scene;                   // Scene to watch
   FXGIFIcon         *mdiicon;                 // MDI Child window icon
   FXMenuPane        *mdimenu;                 // MDI Window Menu
   FXGLVisual        *glvisual;                // Visual for OpenGL
+  FXMenuPane        *filemenu;
+  FXMenuPane        *editmenu;
+  FXMenuPane        *viewmenu;
+  FXMenuPane        *rendermenu;
+  FXMenuPane        *windowmenu;
+  FXMenuPane        *helpmenu;
+#ifdef SHARE_CONTEXT
+  FXGLContext       *glcontext;               // Context for OpenGL
+#endif
 protected:
   GLViewWindow(){}
 public:
-  
+
   // We define additional ID's, starting from the last one used by the base class+1.
   // This way, we know the ID's are all unique for this particular target.
   enum{
     ID_ABOUT=FXMainWindow::ID_LAST,
     ID_OPEN,
     ID_NEWVIEWER,
-    ID_QUERY_MODE
+    ID_QUERY_MODE,
+    ID_GLVIEWER
     };
 
   // Message handlers
@@ -387,20 +384,21 @@ public:
   long onCmdNewViewer(FXObject*,FXSelector,void*);
   long onCmdAbout(FXObject*,FXSelector,void*);
   long onUpdMode(FXObject*,FXSelector,void*);
+  long onQueryMenu(FXObject*,FXSelector,void*);
 
 public:
 
   // GLViewWindow constructor
   GLViewWindow(FXApp* a);
-  
+
   // Initialize
   virtual void create();
-  
+
   // GLViewWindow destructor
   virtual ~GLViewWindow();
   };
 
-  
+
 /*******************************************************************************/
 
 
@@ -412,6 +410,7 @@ FXDEFMAP(GLViewWindow) GLViewWindowMap[]={
   FXMAPFUNC(SEL_COMMAND,     GLViewWindow::ID_OPEN,       GLViewWindow::onCmdOpen),
   FXMAPFUNC(SEL_COMMAND,     GLViewWindow::ID_NEWVIEWER,  GLViewWindow::onCmdNewViewer),
   FXMAPFUNC(SEL_UPDATE,      GLViewWindow::ID_QUERY_MODE, GLViewWindow::onUpdMode),
+  FXMAPFUNC(SEL_COMMAND,     FXWindow::ID_QUERY_MENU,	  GLViewWindow::onQueryMenu)
   };
 
 
@@ -419,40 +418,39 @@ FXDEFMAP(GLViewWindow) GLViewWindowMap[]={
 // Macro for the GLViewWindow class hierarchy implementation
 FXIMPLEMENT(GLViewWindow,FXMainWindow,GLViewWindowMap,ARRAYNUMBER(GLViewWindowMap))
 
-  
-/*******************************************************************************/
 
+/*******************************************************************************/
 
 // Construct a GLViewWindow
 GLViewWindow::GLViewWindow(FXApp* a):FXMainWindow(a,"OpenGL Test Application",NULL,NULL,DECOR_ALL,0,0,800,600){
   FXIcon *peng=new FXGIFIcon(getApp(),penguin);
-  
+
   setIcon(peng);
 
   FXColorDialog *colordlg=new FXColorDialog(this,"Color Dialog",DECOR_TITLE|DECOR_BORDER);
-  
+
   // Menubar
-  menubar=new FXMenuBar(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
-  
+  menubar=new FXMenubar(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
+
   // Tool bar
   new FXHorizontalSeparator(this,LAYOUT_SIDE_TOP|SEPARATOR_GROOVE|LAYOUT_FILL_X);
-  toolbar=new FXHorizontalFrame(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X,0,0,0,0, 4,4,0,0, 0,0);
-  
+  toolbar=new FXToolbar(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X,0,0,0,0, 4,4,4,4, 0,0);
+
   // Make status bar
   statusbar=new FXStatusbar(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|STATUSBAR_WITH_DRAGCORNER);
-  
+
   // The good old penguin, what would we be without it?
   new FXButton(statusbar,"\tHello, I'm Tux...\nThe symbol for the Linux Operating System.\nAnd all it stands for.",new FXGIFIcon(getApp(),penguin),NULL,0,LAYOUT_RIGHT);
-  
+
   // Contents
   FXHorizontalFrame *frame=new FXHorizontalFrame(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0, 0,0);
-  
+
   // Nice sunken box around GL viewer
   FXVerticalFrame *box=new FXVerticalFrame(frame,FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0);
-  
+
   // MDI Client
   mdiclient=new FXMDIClient(box,LAYOUT_FILL_X|LAYOUT_FILL_Y);
-  
+
   // MDI buttons in menu:- note the message ID's!!!!!
   // Normally, MDI commands are simply sensitized or desensitized;
   // Under the menubar, however, they're hidden if the MDI Client is
@@ -464,25 +462,25 @@ GLViewWindow::GLViewWindow(FXApp* a):FXMainWindow(a,"OpenGL Test Application",NU
 
   // Icon for MDI Child
   mdiicon=new FXGIFIcon(getApp(),winapp);
-  
+
   // Make MDI Window Menu
   mdimenu=new FXMDIMenu(this,mdiclient);
-  
+
   // Make an MDI Child
   FXMDIChild* mdichild=new FXMDIChild(mdiclient,"FOX GL Viewer",mdiicon,mdimenu,MDI_NORMAL,30,30,300,200);
 
-  //FXDebugTarget *dbg=new FXDebugTarget;
-  
   // A visual to drag OpenGL in double-buffered mode; note the glvisual is
   // shared between all windows which need the same depths and numbers of buffers
   // Thus, while the first visual may take some time to initialize, each subsequent
   // window can be created very quickly; we need to determine grpaphics hardware
   // characteristics only once.
   glvisual=new FXGLVisual(getApp(),VISUAL_DOUBLEBUFFER);
-  
+
+  // Make it active
+  mdiclient->setActiveChild(mdichild);
+
   // Drawing gl canvas
-  //viewer=new FXGLViewer(mdichild,dbg,1,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT);
-  viewer=new FXGLViewer(mdichild,glvisual,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT);
+  FXGLViewer *viewer=new FXGLViewer(mdichild,glvisual,this,ID_GLVIEWER,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT);
 
   // Tab book with switchable panels
   FXTabBook* panels=new FXTabBook(frame);
@@ -490,43 +488,61 @@ GLViewWindow::GLViewWindow(FXApp* a):FXMainWindow(a,"OpenGL Test Application",NU
   new FXTabItem(panels,"Angles\tCamera Angles\tSwitch to camera angles panel.");
 
   // Angles
-  FXMatrix *angles=new FXMatrix(panels,2,FRAME_THICK|FRAME_RAISED|MATRIX_BY_COLUMNS|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,10,10);
+  FXMatrix *angles=new FXMatrix(panels,3,FRAME_THICK|FRAME_RAISED|MATRIX_BY_COLUMNS|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,10,10);
   new FXLabel(angles,"X:");
-  FXDial* x_dial=new FXDial(angles,mdiclient,FXGLViewer::ID_DIAL_X,FRAME_SUNKEN|FRAME_THICK|DIAL_CYCLIC|DIAL_HORIZONTAL|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_CENTER_Y,0,0,150,14,0,0,0,0);
+  new FXTextField(angles,6,mdiclient,FXGLViewer::ID_ROLL,TEXTFIELD_INTEGER|JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK);
+  FXDial* x_dial=new FXDial(angles,mdiclient,FXGLViewer::ID_DIAL_X,FRAME_SUNKEN|FRAME_THICK|DIAL_CYCLIC|DIAL_HORIZONTAL|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_CENTER_Y,0,0,160,14,0,0,0,0);
   x_dial->setTipText("Rotate about X");
-  
+  x_dial->setNotchOffset(900);
+
   new FXLabel(angles,"Y:");
-  FXDial* y_dial=new FXDial(angles,mdiclient,FXGLViewer::ID_DIAL_Y,FRAME_SUNKEN|FRAME_THICK|DIAL_CYCLIC|DIAL_HORIZONTAL|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_CENTER_Y,0,0,150,14,0,0,0,0);
+  new FXTextField(angles,6,mdiclient,FXGLViewer::ID_PITCH,TEXTFIELD_INTEGER|JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK);
+  FXDial* y_dial=new FXDial(angles,mdiclient,FXGLViewer::ID_DIAL_Y,FRAME_SUNKEN|FRAME_THICK|DIAL_CYCLIC|DIAL_HORIZONTAL|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_CENTER_Y,0,0,160,14,0,0,0,0);
   y_dial->setTipText("Rotate about Y");
-  
+  y_dial->setNotchOffset(900);
+
   new FXLabel(angles,"Z:");
-  FXDial* z_dial=new FXDial(angles,mdiclient,FXGLViewer::ID_DIAL_Z,FRAME_SUNKEN|FRAME_THICK|DIAL_CYCLIC|DIAL_HORIZONTAL|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_CENTER_Y,0,0,150,14,0,0,0,0);
+  new FXTextField(angles,6,mdiclient,FXGLViewer::ID_YAW,TEXTFIELD_INTEGER|JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK);
+  FXDial* z_dial=new FXDial(angles,mdiclient,FXGLViewer::ID_DIAL_Z,FRAME_SUNKEN|FRAME_THICK|DIAL_CYCLIC|DIAL_HORIZONTAL|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_CENTER_Y,0,0,160,14,0,0,0,0);
   z_dial->setTipText("Rotate about Z");
-  
+  z_dial->setNotchOffset(900);
+
   new FXLabel(angles,"FOV:");
-  FXTextField* fov=new FXTextField(angles,10,mdiclient,FXGLViewer::ID_FOV,JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK);
+  FXTextField* fov=new FXTextField(angles,5,mdiclient,FXGLViewer::ID_FOV,JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK);
+  new FXFrame(angles,0);
   fov->setTipText("Field of view");
-  
+
   new FXLabel(angles,"Zoom:");
-  FXTextField* zz=new FXTextField(angles,10,mdiclient,FXGLViewer::ID_ZOOM,JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK);
+  FXTextField* zz=new FXTextField(angles,5,mdiclient,FXGLViewer::ID_ZOOM,JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK);
+  new FXFrame(angles,0);
   zz->setTipText("Zooming");
-  
+
+  new FXLabel(angles,"Scale X:");
+  new FXTextField(angles,5,mdiclient,FXGLViewer::ID_SCALE_X,JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK);
+  new FXFrame(angles,0);
+  new FXLabel(angles,"Scale Y:");
+  new FXTextField(angles,5,mdiclient,FXGLViewer::ID_SCALE_Y,JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK);
+  new FXFrame(angles,0);
+  new FXLabel(angles,"Scale Z:");
+  new FXTextField(angles,5,mdiclient,FXGLViewer::ID_SCALE_Z,JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK);
+  new FXFrame(angles,0);
+
   // Colors
   new FXTabItem(panels,"Colors\tColors\tSwitch to color panel.");
   FXMatrix *colors=new FXMatrix(panels,2,MATRIX_BY_COLUMNS|FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_Y|LAYOUT_CENTER_X|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,10,10);
   new FXLabel(colors,"Background:",NULL,LAYOUT_RIGHT|LAYOUT_CENTER_Y|JUSTIFY_RIGHT);
-  new FXColorWell(colors,0,mdiclient,FXGLViewer::ID_BACK_COLOR,LAYOUT_TOP|LAYOUT_LEFT, 0,0,0,0 ,0,0,0,0);
+  new FXColorWell(colors,0,mdiclient,FXGLViewer::ID_BACK_COLOR,COLORWELL_OPAQUEONLY|LAYOUT_TOP|LAYOUT_LEFT, 0,0,0,0 ,0,0,0,0);
   new FXLabel(colors,"Ambient:",NULL,LAYOUT_RIGHT|LAYOUT_CENTER_Y|JUSTIFY_RIGHT);
-  new FXColorWell(colors,0,mdiclient,FXGLViewer::ID_AMBIENT_COLOR,LAYOUT_TOP|LAYOUT_LEFT, 0,0,0,0 ,0,0,0,0);
-  
+  new FXColorWell(colors,0,mdiclient,FXGLViewer::ID_AMBIENT_COLOR,COLORWELL_OPAQUEONLY|LAYOUT_TOP|LAYOUT_LEFT, 0,0,0,0 ,0,0,0,0);
+
 
   new FXLabel(colors,"Light Amb:",NULL,LAYOUT_RIGHT|LAYOUT_CENTER_Y|JUSTIFY_RIGHT);
-  new FXColorWell(colors,0,mdiclient,FXGLViewer::ID_LIGHT_AMBIENT,LAYOUT_TOP|LAYOUT_LEFT, 0,0,0,0 ,0,0,0,0);
+  new FXColorWell(colors,0,mdiclient,FXGLViewer::ID_LIGHT_AMBIENT,COLORWELL_OPAQUEONLY|LAYOUT_TOP|LAYOUT_LEFT, 0,0,0,0 ,0,0,0,0);
   new FXLabel(colors,"Light Diff:",NULL,LAYOUT_RIGHT|LAYOUT_CENTER_Y|JUSTIFY_RIGHT);
-  new FXColorWell(colors,0,mdiclient,FXGLViewer::ID_LIGHT_DIFFUSE,LAYOUT_TOP|LAYOUT_LEFT, 0,0,0,0 ,0,0,0,0);
+  new FXColorWell(colors,0,mdiclient,FXGLViewer::ID_LIGHT_DIFFUSE,COLORWELL_OPAQUEONLY|LAYOUT_TOP|LAYOUT_LEFT, 0,0,0,0 ,0,0,0,0);
   new FXLabel(colors,"Light Spec:",NULL,LAYOUT_RIGHT|LAYOUT_CENTER_Y|JUSTIFY_RIGHT);
-  new FXColorWell(colors,0,mdiclient,FXGLViewer::ID_LIGHT_SPECULAR,LAYOUT_TOP|LAYOUT_LEFT, 0,0,0,0 ,0,0,0,0);
-  
+  new FXColorWell(colors,0,mdiclient,FXGLViewer::ID_LIGHT_SPECULAR,COLORWELL_OPAQUEONLY|LAYOUT_TOP|LAYOUT_LEFT, 0,0,0,0 ,0,0,0,0);
+
   // Switches
   new FXTabItem(panels,"Settings\tSettings\tSwitche to settings panel.");
   FXVerticalFrame *settings=new FXVerticalFrame(panels,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_Y|LAYOUT_CENTER_X|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,10,10);
@@ -535,65 +551,64 @@ GLViewWindow::GLViewWindow(FXApp* a):FXMainWindow(a,"OpenGL Test Application",NU
   new FXCheckButton(settings,"Dither",mdiclient,FXGLViewer::ID_DITHER,ICON_BEFORE_TEXT);
   new FXCheckButton(settings,"Lock",mdiclient,FXGLViewer::ID_LOCK,ICON_BEFORE_TEXT);
   new FXCheckButton(settings,"Turbo",mdiclient,FXGLViewer::ID_TURBO,ICON_BEFORE_TEXT);
- 
-  
+
+
   FXIcon *newdoc=new FXGIFIcon(getApp(),filenew);
   FXIcon *opendoc=new FXGIFIcon(getApp(),fileopen);
   FXIcon *savedoc=new FXGIFIcon(getApp(),filesave);
   FXIcon *saveasdoc=new FXGIFIcon(getApp(),filesaveas,0,IMAGE_ALPHAGUESS);
 
   // File Menu
-  FXMenuPane *filemenu=new FXMenuPane(this);
+  filemenu=new FXMenuPane(this);
   new FXMenuTitle(menubar,"&File",NULL,filemenu);
   new FXMenuCommand(filemenu,"&New...\tCtl-N\tCreate new document.",newdoc);
   new FXMenuCommand(filemenu,"&Open...\tCtl-O\tOpen document file.",opendoc,this,ID_OPEN);
   new FXMenuCommand(filemenu,"&Save\tCtl-S\tSave document.",savedoc);
   new FXMenuCommand(filemenu,"Save &As...\t\tSave document to another file.",saveasdoc);
-  new FXMenuCommand(filemenu,"&Print Image...\t\tPrint snapshot image.",NULL,mdiclient,FXGLViewer::ID_PRINT_IMAGE);
-  new FXMenuCommand(filemenu,"&Print Vector...\t\tPrint geometry.",NULL,mdiclient,FXGLViewer::ID_PRINT_VECTOR);
+  new FXMenuCommand(filemenu,"&Print Image...\t\tPrint snapshot image.",NULL,mdiclient,FXGLViewer::ID_PRINT_IMAGE,MENU_AUTOGRAY);
+  new FXMenuCommand(filemenu,"&Print Vector...\t\tPrint geometry.",NULL,mdiclient,FXGLViewer::ID_PRINT_VECTOR,MENU_AUTOGRAY);
   new FXMenuCommand(filemenu,"&Dump...\t\tDump widgets.",NULL,getApp(),FXApp::ID_DUMP);
   new FXMenuCommand(filemenu,"&Quit\tCtl-Q\tQuit the application.",NULL,getApp(),FXApp::ID_QUIT,MENU_DEFAULT);
-  
+
   // Edit Menu
-  FXMenuPane *editmenu=new FXMenuPane(this);
+  editmenu=new FXMenuPane(this);
   new FXMenuTitle(menubar,"&Edit",NULL,editmenu);
-  new FXMenuCommand(editmenu,"Undo");
-  new FXMenuCommand(editmenu,"Copy",NULL,mdiclient,FXGLViewer::ID_COPY_SEL,MENU_AUTOGRAY);
-  new FXMenuCommand(editmenu,"Cut",NULL,mdiclient,FXGLViewer::ID_CUT_SEL,MENU_AUTOGRAY);
-  new FXMenuCommand(editmenu,"Paste",NULL,mdiclient,FXGLViewer::ID_PASTE_SEL,MENU_AUTOGRAY);
+  new FXMenuCommand(editmenu,"Copy\tCtl-C",NULL,mdiclient,FXGLViewer::ID_COPY_SEL,MENU_AUTOGRAY);
+  new FXMenuCommand(editmenu,"Cut\tCtl-X",NULL,mdiclient,FXGLViewer::ID_CUT_SEL,MENU_AUTOGRAY);
+  new FXMenuCommand(editmenu,"Paste\tCtl-V",NULL,mdiclient,FXGLViewer::ID_PASTE_SEL,MENU_AUTOGRAY);
   new FXMenuCommand(editmenu,"Delete",NULL,mdiclient,FXGLViewer::ID_DELETE_SEL,MENU_AUTOGRAY);
-  
+
   // File manipulation
   new FXButton(toolbar,"\tNew\tCreate new document.",newdoc,NULL,0,FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
   new FXButton(toolbar,"\tOpen\tOpen document file.",opendoc,this,ID_OPEN,FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
   new FXButton(toolbar,"\tSave\tSave document.",savedoc,NULL,0,FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
   new FXButton(toolbar,"\tSave As\tSave document to another file.",saveasdoc,this,ID_OPEN,FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
   new FXButton(toolbar,"\tNew Folder\tNo comment",new FXGIFIcon(getApp(),newfolder),NULL,0,FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-  
+
   // Print
   new FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,4,20);
-  new FXButton(toolbar,"\tPrint Image\tPrint shapshot image.",new FXGIFIcon(getApp(),print),mdiclient,FXGLViewer::ID_PRINT_IMAGE,FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-  
+  new FXButton(toolbar,"\tPrint Image\tPrint shapshot image.",new FXGIFIcon(getApp(),print),mdiclient,FXGLViewer::ID_PRINT_IMAGE,BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+
   // Editing
   new FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,4,20);
   new FXButton(toolbar,"\tCut",new FXGIFIcon(getApp(),cut),mdiclient,FXGLViewer::ID_CUT_SEL,BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
   new FXButton(toolbar,"\tCopy",new FXGIFIcon(getApp(),copy),mdiclient,FXGLViewer::ID_COPY_SEL,BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
   new FXButton(toolbar,"\tPaste",new FXGIFIcon(getApp(),paste),mdiclient,FXGLViewer::ID_PASTE_SEL,BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-  
+
   // Projections
   new FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,8,20);
   new FXButton(toolbar,"\tPerspective\tSwitch to perspective projection.",new FXGIFIcon(getApp(),perspective),mdiclient,FXGLViewer::ID_PERSPECTIVE,BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
   new FXButton(toolbar,"\tParallel\tSwitch to parallel projection.",new FXGIFIcon(getApp(),parallel),mdiclient,FXGLViewer::ID_PARALLEL,BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-  
+
   // Shading model
   new FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,8,20);
   new FXButton(toolbar,"\tNo shading\tTurn light sources off.",new FXGIFIcon(getApp(),nolight),mdiclient,FXGLShape::ID_SHADEOFF,BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
   new FXButton(toolbar,"\tFlat shading\tTurn on faceted (flat) shading.",new FXGIFIcon(getApp(),light),mdiclient,FXGLShape::ID_SHADEON,BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
   new FXButton(toolbar,"\tSmooth shading\tTurn on smooth shading.",new FXGIFIcon(getApp(),smoothlight),mdiclient,FXGLShape::ID_SHADESMOOTH,BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-  
+
   new FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,8,20);
   new FXToggleButton(toolbar,"\tToggle Light\tToggle light source.",NULL,new FXGIFIcon(getApp(),nolight),new FXGIFIcon(getApp(),light),NULL,0,FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-  
+
   // View orientation
   new FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,8,20);
   new FXButton(toolbar,"\tFront View\tView objects from the front.",new FXGIFIcon(getApp(),frontview),mdiclient,FXGLViewer::ID_FRONT,BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
@@ -602,35 +617,37 @@ GLViewWindow::GLViewWindow(FXApp* a):FXMainWindow(a,"OpenGL Test Application",NU
   new FXButton(toolbar,"\tRight View\tView objects from the right.",new FXGIFIcon(getApp(),rightview),mdiclient,FXGLViewer::ID_RIGHT,BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
   new FXButton(toolbar,"\tTop View\tView objects from the top.",new FXGIFIcon(getApp(),topview,0,IMAGE_OPAQUE),mdiclient,FXGLViewer::ID_TOP,BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
   new FXButton(toolbar,"\tBottom View\tView objects from below.",new FXGIFIcon(getApp(),bottomview,0,IMAGE_OPAQUE),mdiclient,FXGLViewer::ID_BOTTOM,BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-  
+
   // Miscellaneous buttons
   new FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,8,20);
   new FXButton(toolbar,(FXchar*)0,new FXGIFIcon(getApp(),zoom),NULL,0,FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
   new FXButton(toolbar,"\tColors\tDisplay color dialog.",new FXGIFIcon(getApp(),colorpal),colordlg,FXWindow::ID_SHOW,FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
   new FXButton(toolbar,(FXchar*)0,new FXGIFIcon(getApp(),camera,0,IMAGE_OPAQUE),NULL,0,FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
   new FXButton(toolbar,(FXchar*)0,new FXGIFIcon(getApp(),foxicon),NULL,0,FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-  
+
   // Dangerous delete a bit on the side
   new FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,10,20);
-  new FXButton(toolbar,"\tDelete\tDelete the selected object.",new FXGIFIcon(getApp(),kill),mdiclient,FXGLViewer::ID_DELETE_SEL,BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-  
+  new FXButton(toolbar,"\tDelete\tDelete the selected object.",new FXGIFIcon(getApp(),killobject),mdiclient,FXGLViewer::ID_DELETE_SEL,BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+
   // View menu
-  FXMenuPane *viewmenu=new FXMenuPane(this);
+  viewmenu=new FXMenuPane(this);
   new FXMenuTitle(menubar,"&View",NULL,viewmenu);
   new FXMenuCommand(viewmenu,"Parallel\t\tSwitch to parallel projection.",NULL,mdiclient,FXGLViewer::ID_PARALLEL,MENU_AUTOGRAY);
   new FXMenuCommand(viewmenu,"Perspective\t\tSwitch to perspective projection.",NULL,mdiclient,FXGLViewer::ID_PERSPECTIVE,MENU_AUTOGRAY);
-  new FXMenuCommand(viewmenu,"&Front\t\tFront view.",NULL,mdiclient,FXGLViewer::ID_FRONT,MENU_AUTOGRAY);
-  new FXMenuCommand(viewmenu,"&Back\t\tBack view.",NULL,mdiclient,FXGLViewer::ID_BACK,MENU_AUTOGRAY);
-  new FXMenuCommand(viewmenu,"&Left\t\tLeft view.",NULL,mdiclient,FXGLViewer::ID_LEFT,MENU_AUTOGRAY);
-  new FXMenuCommand(viewmenu,"&Right\t\tRight view.",NULL,mdiclient,FXGLViewer::ID_RIGHT,MENU_AUTOGRAY);
-  new FXMenuCommand(viewmenu,"&Top\t\tTop view.",NULL,mdiclient,FXGLViewer::ID_TOP,MENU_AUTOGRAY);
-  new FXMenuCommand(viewmenu,"&Bottom\t\tBottom view.",NULL,mdiclient,FXGLViewer::ID_BOTTOM,MENU_AUTOGRAY);
+  new FXMenuCommand(viewmenu,"&Front\tCtl-F\tFront view.",NULL,mdiclient,FXGLViewer::ID_FRONT,MENU_AUTOGRAY);
+  new FXMenuCommand(viewmenu,"&Back\tCtl-B\tBack view.",NULL,mdiclient,FXGLViewer::ID_BACK,MENU_AUTOGRAY);
+  new FXMenuCommand(viewmenu,"&Left\tCtl-L\tLeft view.",NULL,mdiclient,FXGLViewer::ID_LEFT,MENU_AUTOGRAY);
+  new FXMenuCommand(viewmenu,"&Right\tCtl-R\tRight view.",NULL,mdiclient,FXGLViewer::ID_RIGHT,MENU_AUTOGRAY);
+  new FXMenuCommand(viewmenu,"&Top\tCtl-T\tTop view.",NULL,mdiclient,FXGLViewer::ID_TOP,MENU_AUTOGRAY);
+  new FXMenuCommand(viewmenu,"&Bottom\tCtl-K\tBottom view.",NULL,mdiclient,FXGLViewer::ID_BOTTOM,MENU_AUTOGRAY);
   new FXMenuCommand(viewmenu,"F&it\t\tFit to view.",NULL,mdiclient,FXGLViewer::ID_FITVIEW,MENU_AUTOGRAY);
-  new FXMenuCommand(viewmenu,"R&eset\t\tReset all viewing parameters",NULL,mdiclient,FXGLViewer::ID_RESETVIEW,MENU_AUTOGRAY);
+  new FXMenuCommand(viewmenu,"R&eset\tCtl-G\tReset all viewing parameters",NULL,mdiclient,FXGLViewer::ID_RESETVIEW,MENU_AUTOGRAY);
+  new FXMenuCommand(viewmenu,"Zoom\t\tZoom in on area",NULL,mdiclient,FXGLViewer::ID_LASSO_ZOOM,MENU_AUTOGRAY);
+  new FXMenuCommand(viewmenu,"Select\t\tZoom in on area",NULL,mdiclient,FXGLViewer::ID_LASSO_SELECT,MENU_AUTOGRAY);
   new FXMenuCommand(viewmenu,"Lock\t\tLock view orientation",NULL,mdiclient,FXGLViewer::ID_LOCK,MENU_AUTOGRAY);
-  
+
   // Rendering menu
-  FXMenuPane *rendermenu=new FXMenuPane(this);
+  rendermenu=new FXMenuPane(this);
   new FXMenuTitle(menubar,"&Rendering",NULL,rendermenu);
   new FXMenuCommand(rendermenu,"Points\t\tRender as points.",NULL,mdiclient,FXGLShape::ID_STYLE_POINTS,MENU_AUTOGRAY);
   new FXMenuCommand(rendermenu,"Wire Frame\t\tRender as wire frame.",NULL,mdiclient,FXGLShape::ID_STYLE_WIREFRAME,MENU_AUTOGRAY);
@@ -638,65 +655,81 @@ GLViewWindow::GLViewWindow(FXApp* a):FXMainWindow(a,"OpenGL Test Application",NU
   new FXMenuCommand(rendermenu,"Bounding Box\t\tRender bounding box only.",NULL,mdiclient,FXGLShape::ID_STYLE_BOUNDINGBOX,MENU_AUTOGRAY);
 
   // Window menu
-  FXMenuPane *windowmenu=new FXMenuPane(this);
+  windowmenu=new FXMenuPane(this);
   new FXMenuTitle(menubar,"&Windows",NULL,windowmenu);
   new FXMenuCommand(windowmenu,"New Viewer\t\tCreate new viewer window.",NULL,this,ID_NEWVIEWER);
-  new FXMenuCommand(windowmenu,"Tile Horizontally\t\tTile windows horizontally.",NULL,mdiclient,FXWindow::ID_MDI_TILEHORIZONTAL);
-  new FXMenuCommand(windowmenu,"Tile Vertically\t\tTile windows vertically.",NULL,mdiclient,FXWindow::ID_MDI_TILEVERTICAL);
-  new FXMenuCommand(windowmenu,"Cascade\t\tCascade windows.",NULL,mdiclient,FXWindow::ID_MDI_CASCADE);
+  new FXMenuCommand(windowmenu,"Tile Horizontally\t\tTile windows horizontally.",NULL,mdiclient,FXMDIClient::ID_MDI_TILEHORIZONTAL);
+  new FXMenuCommand(windowmenu,"Tile Vertically\t\tTile windows vertically.",NULL,mdiclient,FXMDIClient::ID_MDI_TILEVERTICAL);
+  new FXMenuCommand(windowmenu,"Cascade\t\tCascade windows.",NULL,mdiclient,FXMDIClient::ID_MDI_CASCADE);
   new FXMenuCommand(windowmenu,"Toolbar",NULL,toolbar,FXWindow::ID_TOGGLESHOWN);
   new FXMenuCommand(windowmenu,"Control panel",NULL,panels,FXWindow::ID_TOGGLESHOWN);
   new FXMenuCommand(windowmenu,"Delete\t\tDelete current viewer window.",NULL,mdiclient,FXMDIClient::ID_MDI_CLOSE);
+  FXMenuSeparator* sep1=new FXMenuSeparator(windowmenu);
+  sep1->setTarget(mdiclient);
+  sep1->setSelector(FXMDIClient::ID_MDI_ANY);
+  new FXMenuCommand(windowmenu,NULL,NULL,mdiclient,FXMDIClient::ID_MDI_1);
+  new FXMenuCommand(windowmenu,NULL,NULL,mdiclient,FXMDIClient::ID_MDI_2);
+  new FXMenuCommand(windowmenu,NULL,NULL,mdiclient,FXMDIClient::ID_MDI_3);
+  new FXMenuCommand(windowmenu,NULL,NULL,mdiclient,FXMDIClient::ID_MDI_4);
 
   // Help menu
-  FXMenuPane* helpmenu=new FXMenuPane(this);
+  helpmenu=new FXMenuPane(this);
   new FXMenuTitle(menubar,"&Help",NULL,helpmenu,LAYOUT_RIGHT);
   new FXMenuCommand(helpmenu,"&About FOX...\t\tDisplay FOX about panel.",NULL,this,ID_ABOUT,0);
-  
+
   // Make a tool tip
   new FXTooltip(getApp(),0);
-  
+
   // The status bar shows our mode
   statusbar->getStatusline()->setTarget(this);
   statusbar->getStatusline()->setSelector(ID_QUERY_MODE);
-    
+
   // Make a scene!
   scene=new FXGLGroup;
-  scene->append(new FXGLPoint(2,0,0));
-  scene->append(new FXGLPoint(0,2,0));
-  scene->append(new FXGLPoint(2,2,0));
-  scene->append(new FXGLPoint(0,0,0));
-  scene->append(new FXGLLine(0,0,0, 1,0,0));
-  scene->append(new FXGLLine(0,0,0, 0,1,0));
-  scene->append(new FXGLLine(0,0,0, 0,0,1));
-  scene->append(new FXGLLine(0,0,0, 1,1,1));
+//   scene->append(new FXGLPoint(2,0,0));
+//   scene->append(new FXGLPoint(0,2,0));
+//   scene->append(new FXGLPoint(2,2,0));
+//   scene->append(new FXGLPoint(0,0,0));
+//   scene->append(new FXGLLine(0,0,0, 1,0,0));
+//   scene->append(new FXGLLine(0,0,0, 0,1,0));
+//   scene->append(new FXGLLine(0,0,0, 0,0,1));
+//   scene->append(new FXGLLine(0,0,0, 1,1,1));
   FXGLGroup *gp2=new FXGLGroup;
   scene->append(gp2);
   FXGLSphere *sphere=new FXGLSphere(1.0, 1.0, 0.0, 0.5);
+  FXGLSphere *sphere2=new FXGLSphere(0.0, 0.0, 0.0, 0.8);
   sphere->setTipText("Sphere");
-  gp2->append(new FXGLCube(-1.0, 0.0, 0.0,  0.5, 0.5, 0.5));
-  gp2->append(new FXGLCube( 1.0, 0.0, 0.0,  0.5, 0.5, 0.5));
-  gp2->append(new FXGLCube( 0.0,-1.0, 0.0,  0.5, 0.5, 0.5));
-  gp2->append(new FXGLCube( 0.0, 1.0, 0.0,  0.5, 0.5, 1.5));
+  gp2->append(new FXGLCube(-1.0, 0.0, 0.0,  1.0, 1.0, 1.0));
+  gp2->append(new FXGLCube( 1.0, 0.0, 0.0,  1.0, 1.0, 1.0));
+  gp2->append(new FXGLCube( 0.0,-1.0, 0.0,  1.0, 1.0, 1.0));
+  gp2->append(new FXGLCube( 0.0, 1.0, 0.0,  1.0, 1.0, 1.0));
   gp2->append(new FXGLCone(1.0,-1.5, 0.0, 1.0, 0.5));
   gp2->append(new FXGLCylinder(-1.0, 0.5, 0.0, 1.0, 0.5));
   gp2->append(sphere);
-  
+  gp2->append(sphere2);
+
   // Add scene to GL viewer
   viewer->setScene(scene);
+
   }
-    
+
 
 // Destructor
 GLViewWindow::~GLViewWindow(){
+  delete filemenu;
+  delete editmenu;
+  delete viewmenu;
+  delete rendermenu;
+  delete windowmenu;
+  delete helpmenu;
   }
 
- 
 
-// Create and initialize 
+
+// Create and initialize
 void GLViewWindow::create(){
   FXMainWindow::create();
-  show();
+  show(PLACEMENT_SCREEN);
   }
 
 
@@ -727,19 +760,21 @@ long GLViewWindow::onCmdOpen(FXObject*,FXSelector,void*){
 
 // Make new viewer
 long GLViewWindow::onCmdNewViewer(FXObject*,FXSelector,void*){
-  
+  static int no=1;
+
   // Make new MDI child widget
-  FXMDIChild* mdichild=new FXMDIChild(mdiclient,"FOX GL Viewer",mdiicon,mdimenu,MDI_NORMAL,30,30,300,200);
+  FXMDIChild* mdichild=new FXMDIChild(mdiclient,FXStringFormat("GL Viewer #%d",no),mdiicon,mdimenu,MDI_NORMAL,30,30,300,200);
+
+  FXGLViewer *view=new FXGLViewer(mdichild,glvisual,this,ID_GLVIEWER);
 
-  // Drawing gl canvas
-  FXGLViewer *view=new FXGLViewer(mdichild,glvisual,NULL,0);
-  
   // Change of scenery
   view->setScene(scene);
-  
+
   // Create the new window
   mdichild->create();
-  
+
+  // Next number
+  no++;
   return 1;
   }
 
@@ -753,25 +788,52 @@ long GLViewWindow::onUpdMode(FXObject* sender,FXSelector,void*){
   }
 
 
+/*
+*  When the user right-clicks in the GLViewer background, the viewer first sends
+*  a SEL_COMMAND message with identifier FXWindow::ID_QUERY_MENU to the selected
+*  GLObject (if any). If that message isn't handled, it tries to send it to the
+*  GLViewer's target (which in our case is the main window).
+*/
+long GLViewWindow::onQueryMenu(FXObject* sender,FXSelector,void* ptr){
+  FXEvent *event=(FXEvent*)ptr;
+  FXMenuPane pane(this);
+  new FXMenuCommand(&pane,"Parallel\t\tSwitch to parallel projection.",NULL,sender,FXGLViewer::ID_PARALLEL);
+  new FXMenuCommand(&pane,"Perspective\t\tSwitch to perspective projection.",NULL,sender,FXGLViewer::ID_PERSPECTIVE);
+  new FXMenuSeparator(&pane);
+  new FXMenuCommand(&pane,"&Front\t\tFront view.",NULL,sender,FXGLViewer::ID_FRONT);
+  new FXMenuCommand(&pane,"&Back\t\tBack view.",NULL,sender,FXGLViewer::ID_BACK);
+  new FXMenuCommand(&pane,"&Left\t\tLeft view.",NULL,sender,FXGLViewer::ID_LEFT);
+  new FXMenuCommand(&pane,"&Right\t\tRight view.",NULL,sender,FXGLViewer::ID_RIGHT);
+  new FXMenuCommand(&pane,"&Top\t\tTop view.",NULL,sender,FXGLViewer::ID_TOP);
+  new FXMenuCommand(&pane,"&Bottom\t\tBottom view.",NULL,sender,FXGLViewer::ID_BOTTOM);
+  new FXMenuSeparator(&pane);
+  new FXMenuCommand(&pane,"F&it\t\tFit to view.",NULL,sender,FXGLViewer::ID_FITVIEW);
+  new FXMenuCommand(&pane,"R&eset\t\tReset all viewing parameters",NULL,sender,FXGLViewer::ID_RESETVIEW);
+  new FXMenuCommand(&pane,"Lock\t\tLock view orientation",NULL,sender,FXGLViewer::ID_LOCK);
+  pane.create();
+  pane.popup(NULL,event->root_x,event->root_y);
+  getApp()->runModalWhileShown(&pane);
+  return 1;
+  }
+
+
 // Here we begin
 int main(int argc,char *argv[]){
 
   // Make application
-  FXApp* application=new FXApp("GLViewer","Test");
-  
+  FXApp application("GLViewer","FoxTest");
+
   // Open the display
-  application->init(argc,argv);
+  application.init(argc,argv);
 
   // Make window
-  new GLViewWindow(application);
-  
+  new GLViewWindow(&application);
+
   // Create the application's windows
-  application->create();
-  
+  application.create();
+
   // Run the application
-  application->run();
-  
-  return 0;
+  return application.run();
   }
 
 
