@@ -5,7 +5,7 @@
 *********************************************************************************
 * Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
 *********************************************************************************
-* $Id: groupbox.cpp,v 1.6 1999/11/15 17:14:54 jeroen Exp $                      *
+* $Id: groupbox.cpp,v 1.41 2001/06/28 22:38:25 jeroen Exp $                     *
 ********************************************************************************/
 #include "fx.h"
 #include <stdio.h>
@@ -13,7 +13,6 @@
 #include <string.h>
 
 
-
 /*******************************************************************************/
 
 
@@ -24,9 +23,14 @@ protected:
     
   // Member data
   FXTooltip*         tooltip;
-  FXMenuBar*         menubar;
+  FXMenubar*         menubar;
   FXMenuPane*        filemenu;
+  FXMenuPane*        popupmenu;
   FXMenuPane*        helpmenu;
+  FXMenuPane*        editmenu;
+  FXMenuPane*        submenu1;
+  FXPopup*           pop;
+  FXPopup*           coolpop;
   FXHorizontalFrame* contents;
   FXPacker*          group1;
   FXGroupBox*        group2;
@@ -39,7 +43,11 @@ protected:
 public:
   
   // Message handlers
-  long onCmdOpen(FXObject*,FXSelector,void*);
+  long onCmdFileDlgAny(FXObject*,FXSelector,void*);
+  long onCmdFileDlgExisting(FXObject*,FXSelector,void*);
+  long onCmdFileDlgMultiple(FXObject*,FXSelector,void*);
+  long onCmdFileDlgDirectory(FXObject*,FXSelector,void*);
+  long onCmdDirDlg(FXObject*,FXSelector,void*);
   long onCmdDownSize(FXObject*,FXSelector,void*);
   long onCmdDelete(FXObject*,FXSelector,void*);
   long onCmdAbout(FXObject*,FXSelector,void*);
@@ -56,7 +64,11 @@ public:
     ID_POPUP,
     ID_ABOUT,
     ID_DELETE,
-    ID_OPEN,
+    ID_FILEDLG_ANY,
+    ID_FILEDLG_EXISTING,
+    ID_FILEDLG_MULTIPLE,
+    ID_FILEDLG_DIRECTORY,
+    ID_DIRDLG,
     ID_OPTION1,
     ID_OPTION2,
     ID_OPTION3,
@@ -69,6 +81,7 @@ public:
 public:
   GroupWindow(FXApp* a);
   virtual void create();
+  virtual ~GroupWindow();
   };
 
 
@@ -85,7 +98,11 @@ FXDEFMAP(GroupWindow) GroupWindowMap[]={
   FXMAPFUNCS(SEL_UPDATE,  GroupWindow::ID_RADIO1,GroupWindow::ID_RADIO3,     GroupWindow::onUpdRadio),
   FXMAPFUNC(SEL_COMMAND,  GroupWindow::ID_POPUP,                             GroupWindow::onCmdPopup),
   FXMAPFUNC(SEL_COMMAND,  GroupWindow::ID_ABOUT,                             GroupWindow::onCmdAbout),
-  FXMAPFUNC(SEL_COMMAND,  GroupWindow::ID_OPEN,                              GroupWindow::onCmdOpen),
+  FXMAPFUNC(SEL_COMMAND,  GroupWindow::ID_FILEDLG_ANY,                       GroupWindow::onCmdFileDlgAny),
+  FXMAPFUNC(SEL_COMMAND,  GroupWindow::ID_FILEDLG_EXISTING,                  GroupWindow::onCmdFileDlgExisting),
+  FXMAPFUNC(SEL_COMMAND,  GroupWindow::ID_FILEDLG_MULTIPLE,                  GroupWindow::onCmdFileDlgMultiple),
+  FXMAPFUNC(SEL_COMMAND,  GroupWindow::ID_FILEDLG_DIRECTORY,                 GroupWindow::onCmdFileDlgDirectory),
+  FXMAPFUNC(SEL_COMMAND,  GroupWindow::ID_DIRDLG,                            GroupWindow::onCmdDirDlg),
   FXMAPFUNCS(SEL_COMMAND, GroupWindow::ID_OPTION1,GroupWindow::ID_OPTION4,   GroupWindow::onCmdOption),
   };
 
@@ -130,8 +147,8 @@ const unsigned char minifolderclosed[]={
 
   
 // Make some windows
-GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DECOR_ALL,0,0,800,600){
-  tooltip=new FXTooltip(getApp(),0,100,100);
+GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DECOR_ALL,0,0,0,0){
+  //tooltip=new FXTooltip(getApp(),0,100,100);
 
   
   FXIcon *doc=new FXGIFIcon(getApp(),minidoc1);
@@ -139,22 +156,26 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
   FXIcon *folder_closed=new FXGIFIcon(getApp(),minifolderclosed);
 
   // Menubar
-  menubar=new FXMenuBar(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
+  menubar=new FXMenubar(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
   filemenu=new FXMenuPane(this);
-    new FXMenuCommand(filemenu,"&Open\tCtl-O\tOpen a file.",NULL,this,ID_OPEN,MENU_DEFAULT);
+    new FXMenuCommand(filemenu,"Open any",folder_open,this,ID_FILEDLG_ANY);
+    new FXMenuCommand(filemenu,"Open existing",folder_open,this,ID_FILEDLG_EXISTING);
+    new FXMenuCommand(filemenu,"Open multiple",folder_open,this,ID_FILEDLG_MULTIPLE);
+    new FXMenuCommand(filemenu,"Open directory",folder_open,this,ID_FILEDLG_DIRECTORY);
+    new FXMenuCommand(filemenu,"Open directory dialog",folder_open,this,ID_DIRDLG);
     new FXMenuCommand(filemenu,"Radio&1",NULL,this,ID_RADIO1,0);
     new FXMenuCommand(filemenu,"Radio&2",NULL,this,ID_RADIO2,0);
     new FXMenuCommand(filemenu,"Radio&3",NULL,this,ID_RADIO3,0);
     new FXMenuCommand(filemenu,"Delete\tCtl-X",NULL,this,ID_DELETE,0);
-    new FXMenuCommand(filemenu,"Downsize",NULL,this,ID_DOWNSIZE,0);
+    new FXMenuCommand(filemenu,"Downsize\tF5\tResize to minimum",NULL,this,ID_DOWNSIZE,0);
     new FXMenuCommand(filemenu,"&Size",NULL,this,ID_DOWNSIZE,0);
     new FXMenuCommand(filemenu,"Dump Widgets",NULL,getApp(),FXApp::ID_DUMP);
 
     // Make edit popup menu
-    FXMenuPane *editmenu=new FXMenuPane(this);
+    editmenu=new FXMenuPane(this);
       new FXMenuCommand(editmenu,"Undo");
       new FXMenuCommand(editmenu,"Cut");
-        FXMenuPane *submenu1=new FXMenuPane(this);
+        submenu1=new FXMenuPane(this);
           new FXMenuCommand(submenu1,"&One");
           new FXMenuCommand(submenu1,"&Two");
           new FXMenuCommand(submenu1,"Th&ree");
@@ -169,7 +190,11 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
     new FXMenuCommand(helpmenu,"&About FOX...",NULL,this,ID_ABOUT,0);
   new FXMenuTitle(menubar,"&Help",NULL,helpmenu,LAYOUT_RIGHT);
 
+  popupmenu=new FXMenuPane(this);
+    FXTextField* poptext=new FXTextField(popupmenu,10,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP,0,0,0,0);
+    poptext->setText("Popup with text");
   
+    
   // Status bar
   FXStatusbar *status=new FXStatusbar(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|STATUSBAR_WITH_DRAGCORNER);
   
@@ -185,16 +210,16 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
   FXLabel *testlabel=new FXLabel(group1,"&This is a multi-line\nlabel widget\nwith a big font",NULL,LAYOUT_CENTER_X|JUSTIFY_CENTER_X);
   testlabel->setFont(new FXFont(getApp(),"helvetica",24,FONTWEIGHT_BOLD,FONTSLANT_ITALIC,FONTENCODING_DEFAULT));
   
-  new FXButton(group1,"Small &Button",NULL,NULL,0,LAYOUT_BOTTOM|FRAME_RAISED|FRAME_THICK);
+  new FXButton(group1,"Small &Button",NULL,NULL,0,FRAME_RAISED|FRAME_THICK);
   new FXButton(group1,"Big Fat Wide Button\nComprising\nthree lines",NULL,NULL,0,FRAME_RAISED|FRAME_THICK);
   new FXToggleButton(group1,"C&losed\tTooltip for closed\tHelp for closed","O&pen\nState\tTooltip for open\tHelp for open",folder_closed,folder_open,NULL,0,ICON_BEFORE_TEXT|JUSTIFY_LEFT|FRAME_RAISED|FRAME_THICK);
 
-  FXPopup *pop=new FXPopup(this);
+  pop=new FXPopup(this);
   
-  new FXOption(pop,"First\tTip #1\tHelp first",NULL,this,ID_OPTION1,JUSTIFY_HZ_APART|ICON_AFTER_TEXT);
-  new FXOption(pop,"Second\tTip #2\tHelp second",NULL,this,ID_OPTION2,JUSTIFY_HZ_APART|ICON_AFTER_TEXT);
-  new FXOption(pop,"Third\tTip #3\tHelp third",NULL,this,ID_OPTION3,JUSTIFY_HZ_APART|ICON_AFTER_TEXT);
-  new FXOption(pop,"Fourth\tTip #4\tHelp fourth",NULL,this,ID_OPTION4,JUSTIFY_HZ_APART|ICON_AFTER_TEXT);
+  new FXOption(pop,"&First\tTip #1\tHelp first",NULL,this,ID_OPTION1,JUSTIFY_HZ_APART|ICON_AFTER_TEXT);
+  new FXOption(pop,"&Second\tTip #2\tHelp second",NULL,this,ID_OPTION2,JUSTIFY_HZ_APART|ICON_AFTER_TEXT);
+  new FXOption(pop,"Th&ird\tTip #3\tHelp third",NULL,this,ID_OPTION3,JUSTIFY_HZ_APART|ICON_AFTER_TEXT);
+  new FXOption(pop,"F&ourth\tTip #4\tHelp fourth",NULL,this,ID_OPTION4,JUSTIFY_HZ_APART|ICON_AFTER_TEXT);
   
   new FXOptionMenu(group1,pop,LAYOUT_TOP|FRAME_RAISED|FRAME_THICK|JUSTIFY_HZ_APART|ICON_AFTER_TEXT);
 
@@ -205,7 +230,7 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
   new FXMenuButton(group1,"&Menu",NULL,filemenu,MENUBUTTON_ATTACH_BOTH|MENUBUTTON_DOWN|JUSTIFY_HZ_APART|LAYOUT_TOP|FRAME_RAISED|FRAME_THICK|ICON_AFTER_TEXT);
   new FXMenuButton(group1,"&Menu",NULL,filemenu,MENUBUTTON_UP|LAYOUT_TOP|FRAME_RAISED|FRAME_THICK|ICON_AFTER_TEXT);
 
-  FXPopup *coolpop=new FXPopup(this,POPUP_HORIZONTAL);
+  coolpop=new FXPopup(this,POPUP_HORIZONTAL);
   new FXButton(coolpop,"A\tTipA",NULL,NULL,0,FRAME_THICK|FRAME_RAISED|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,30,30);
   new FXButton(coolpop,"B\tTipB",NULL,NULL,0,FRAME_THICK|FRAME_RAISED|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,30,30);
   new FXButton(coolpop,"C\tTipC",NULL,NULL,0,FRAME_THICK|FRAME_RAISED|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,30,30);
@@ -218,7 +243,7 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
   new FXButton(matrix,"&Wide button",NULL,NULL,0,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X);
   new FXButton(matrix,"A",NULL,NULL,0,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X);
   
-  new FXButton(matrix,"BB",NULL,NULL,0,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN);
+  new FXButton(matrix,"BBBB",NULL,NULL,0,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN);
   new FXButton(matrix,"B",NULL,NULL,0,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_COLUMN);
   new FXButton(matrix,"BB",NULL,NULL,0,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_COLUMN);
   
@@ -245,6 +270,7 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
   new FXSlider(frame,NULL,0,LAYOUT_FIX_HEIGHT|SLIDER_VERTICAL|SLIDER_ARROW_RIGHT,0,0,30,200);  
   new FXSlider(frame,NULL,0,LAYOUT_FIX_HEIGHT|SLIDER_VERTICAL|SLIDER_ARROW_LEFT,0,0,30,200);
   new FXSlider(frame,NULL,0,LAYOUT_FIX_HEIGHT|SLIDER_VERTICAL|SLIDER_INSIDE_BAR,0,0,20,200);
+  new FXScrollbar(frame,NULL,0,SCROLLBAR_VERTICAL|LAYOUT_FIX_HEIGHT|LAYOUT_FIX_WIDTH,0,0,20,300);
 
   FXVerticalFrame *vframe1=new FXVerticalFrame(frame,LAYOUT_FILL_X|LAYOUT_FILL_Y);  
   new FXArrowButton(vframe1,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED|FRAME_THICK|ARROW_UP);
@@ -270,7 +296,7 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
   new FXCheckButton(vv,"Radio Stad Amsterdam",NULL,0,ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP);
   
   FXSpinner *spinner=new FXSpinner(group3,20,NULL,0,SPIN_NORMAL|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP);
-  spinner->setMinMax(1,20);
+  spinner->setRange(1,20);
   
   FXComboBox* combobox=new FXComboBox(group3,5,5,NULL,0,COMBOBOX_INSERT_LAST|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP);
   combobox->appendItem("Very Wide Item");
@@ -317,9 +343,13 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
   textfield4->setText("Grayed out");
   textfield4->disable();
   
-  new FXTextField(group3,20,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP|LAYOUT_FIX_HEIGHT,0,0,0,30);
+  FXTextField* realnumber=new FXTextField(group3,20,NULL,0,TEXTFIELD_REAL|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP|LAYOUT_FIX_HEIGHT,0,0,0,30);
+  realnumber->setText("1.0E+3");
+  FXTextField* intnumber=new FXTextField(group3,20,NULL,0,TEXTFIELD_INTEGER|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP|LAYOUT_FIX_HEIGHT,0,0,0,30);
+  intnumber->setText("1000");
   
   FXDial *dial2=new FXDial(group3,NULL,0,DIAL_CYCLIC|DIAL_HAS_NOTCH|DIAL_HORIZONTAL|LAYOUT_FILL_X|FRAME_RAISED|FRAME_THICK,0,0,120,0);
+  new FXScrollbar(group3,NULL,0,SCROLLBAR_HORIZONTAL|LAYOUT_FIX_HEIGHT|LAYOUT_FIX_WIDTH,0,0,300,20);
   
   FXProgressBar *pbar=new FXProgressBar(group3,NULL,0,LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK|PROGRESSBAR_PERCENTAGE);
   pbar->setProgress(48);
@@ -333,22 +363,67 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
   dial2->setTarget(pbar);
   dial2->setSelector(FXWindow::ID_SETVALUE);
   choice=0;
+  
   }
   
+GroupWindow::~GroupWindow(){
+  delete filemenu;
+  delete helpmenu;
+  delete editmenu;
+  delete submenu1;
+  delete pop;
+  delete coolpop;
+  delete popupmenu;
+  }
+
+
+
+static const FXchar sourcefiles[]="All Files (*)\nC++ Source Files (*.cpp,*.cxx,*.cc)\nC Source Files (*.c)\nC++ Header Files (*.hpp,*.hxx,*.hh,*.h)\n*.o\nAny Extension (*.*)\nThree Letter (*.???)\nREADME*";
+
+
+// Open any file
+long GroupWindow::onCmdFileDlgAny(FXObject*,FXSelector,void*){
+  FXString file=FXFileDialog::getSaveFilename(this,"Save file","../tests/groupbox.cpp",sourcefiles,1);
+  fxmessage("File=\"%s\"\n",file.text());
+  return 1;
+  }
+
+
+// Open existing file
+long GroupWindow::onCmdFileDlgExisting(FXObject*,FXSelector,void*){
+  FXString file=FXFileDialog::getOpenFilename(this,"Open file","../tests/dippy.h",sourcefiles,3);
+  fxmessage("File=\"%s\"\n",file.text());
+  return 1;
+  }
+
+
+// Open multiple
+long GroupWindow::onCmdFileDlgMultiple(FXObject*,FXSelector,void*){
+  FXString* files=FXFileDialog::getOpenFilenames(this,"Open file","../tests/groupbox.cpp",sourcefiles);
+  if(files){
+    for(int i=0; !files[i].empty(); i++){
+      fxmessage("Files=\"%s\"\n",files[i].text());
+      }
+    delete [] files;
+    }
+  return 1;
+  }
+
+
+// Open existing directory
+long GroupWindow::onCmdFileDlgDirectory(FXObject*,FXSelector,void*){
+  FXString dir=FXFileDialog::getOpenDirectory(this,"Open directory","../tests");
+  fxmessage("Dir=\"%s\"\n",dir.text());
+  return 1;
+  }
+
 
 // Open
-long GroupWindow::onCmdOpen(FXObject*,FXSelector,void*){
-  const FXchar *patterns[]={
-    "All Files",          "*",
-    "C++ Source Files",   "*.[Cc][Pp][Pp]",
-    "C++ Header Files",   "*.[Hh]",
-    "Object Files",       "*.o",
-    "HTML Header Files",  "*.[Hh][Tt][Mm][Ll]",NULL};
-  FXFileDialog open(this,"Open some file");
-  open.setPatternList(patterns);
-  if(open.execute()){
-    fxmessage("File=%s\n",open.getFilename().text());
-    }
+long GroupWindow::onCmdDirDlg(FXObject*,FXSelector,void*){
+   FXDirDialog open(this,"Open some file");
+   if(open.execute()){
+     fxmessage("Dir=%s\n",open.getDirectory().text());
+     }
   return 1;
   }
 
@@ -379,7 +454,7 @@ long GroupWindow::onCmdDelete(FXObject*,FXSelector,void*){
 long GroupWindow::onCmdPopup(FXObject*,FXSelector,void*){
   FXint x,y; FXuint buttons;
   getRoot()->getCursorPosition(x,y,buttons);
-  filemenu->popup(NULL,x,y);
+  popupmenu->popup(NULL,x,y);
   return 1;
   }
 
@@ -409,7 +484,7 @@ long GroupWindow::onCmdAbout(FXObject*,FXSelector,void*){
 // Start
 void GroupWindow::create(){
   FXMainWindow::create();
-  show();
+  show(PLACEMENT_SCREEN);
   }
 
 
@@ -418,21 +493,21 @@ void GroupWindow::create(){
 
 // Start the whole thing
 int main(int argc,char *argv[]){
-
+  
   // Make application
-  FXApp* application=new FXApp("Groupbox","Test");
+  FXApp application("Groupbox","FoxTest");
   
   // Open display
-  application->init(argc,argv);
+  application.init(argc,argv);
   
   // Make window
-  new GroupWindow(application);
+  new GroupWindow(&application);
   
   // Create app
-  application->create();
+  application.create();
   
   // Run
-  application->run();
+  return application.run();
   }
 
 
