@@ -5,7 +5,7 @@
 *********************************************************************************
 * Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
 *********************************************************************************
-* $Id: switcher.cpp,v 1.2 1999/10/29 07:58:41 jeroen Exp $                      *
+* $Id: switcher.cpp,v 1.8 2001/02/21 21:05:40 jeroen Exp $                      *
 ********************************************************************************/
 #include "fx.h"
 #include <stdio.h>
@@ -23,7 +23,7 @@
 class SwitcherTest : public FXMainWindow {
   FXDECLARE(SwitcherTest)
 protected:
-  FXMenuBar*         menubar;
+  FXMenubar*         menubar;
   FXMenuPane*        filemenu;
   FXHorizontalFrame* contents;
   FXVerticalFrame*   buttons;
@@ -35,7 +35,8 @@ protected:
   SwitcherTest(){}
 public:
   SwitcherTest(FXApp *a);
-  void create();
+  virtual void create();
+  virtual ~SwitcherTest();
   };
 
 
@@ -57,7 +58,7 @@ SwitcherTest::SwitcherTest(FXApp *a):FXMainWindow(a,"Switcher Test",NULL,NULL,DE
   new FXTooltip(getApp());
   
   // Menubar
-  menubar=new FXMenuBar(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
+  menubar=new FXMenubar(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
   
   // Separator
   new FXHorizontalSeparator(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|SEPARATOR_GROOVE);
@@ -102,10 +103,15 @@ SwitcherTest::SwitcherTest(FXApp *a):FXMainWindow(a,"Switcher Test",NULL,NULL,DE
   }
   
 
+SwitcherTest::~SwitcherTest(){
+  delete filemenu;
+  }
+
+
 // Start
 void SwitcherTest::create(){
   FXMainWindow::create();
-  show();
+  show(PLACEMENT_SCREEN);
   }
 
 
@@ -116,18 +122,18 @@ void SwitcherTest::create(){
 int main(int argc,char *argv[]){
 
   // Make application
-  FXApp* application=new FXApp("Switcher","Test");
+  FXApp application("Switcher","FoxTest");
   
   // Open display
-  application->init(argc,argv);
+  application.init(argc,argv);
   
-  new SwitcherTest(application);
+  new SwitcherTest(&application);
   
   // Create app
-  application->create();
+  application.create();
   
   // Run
-  application->run();
+  return application.run();
   }
 
 
